package com.bocloud.pms.booter.controller;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.GridBean;
import com.bocloud.common.model.Pager;
import com.bocloud.common.model.RequestContext;
import com.bocloud.common.utils.Common;
import com.bocloud.pms.entity.Ass;
import com.bocloud.pms.interfaces.middleware.AssService;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

@Api(value = "应用服务器", tags = "应用服务器")
@RestController
@RequestMapping("/v1/ass")
public class AssController {

    @Autowired
    private AssService assService;

    /**
     * 查询应用服务器列表
     *
     * @param pager
     * @param context
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "查询应用服务器列表", response = BocloudResult.class, httpMethod = "GET",
        produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping
    public BocloudResult<GridBean<Ass>> list(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return assService.list(pager, context);
    }

    /**
     * 详情
     *
     * @param id
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "查看应用服务器详情", response = BocloudResult.class, httpMethod = "GET",
        produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/{id}")
    public BocloudResult<Ass> detail(@PathVariable(value = Common.ID) Long id) {
        return assService.detail(id);
    }

    /**
     * 删除
     *
     * @param id
     * @param context
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "删除应用服务器", response = BocloudResult.class, httpMethod = "DELETE",
        produces = MediaType.APPLICATION_JSON_VALUE)
    @DeleteMapping(value = "/{id}")
    public BocloudResult remove(@PathVariable(value = Common.ID) Long id,
        @Value(Common.REQ_CONTEXT) RequestContext context) {
        return assService.remove(id, context);
    }

}
