package com.bocloud.pms.booter.controller;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.GridBean;
import com.bocloud.common.model.RequestContext;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.ResultTools;
import com.bocloud.pms.entity.CmsBinding;
import com.bocloud.pms.entity.CmsLog;
import com.bocloud.pms.interfaces.middleware.CmsBindingsService;
import com.bocloud.pms.interfaces.middleware.CmsService;
import com.bocloud.service.utils.IpTool;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.Date;

/**
 * Created by liuyuanyuan on 2020/3/17 15:14
 */
@Api(value = "消息队列绑定关系", tags = "绑定关系")
@RestController
@RequestMapping("/v1/cms")
public class CmsBindingsController {
    @Autowired
    private CmsService cmsService;
    @Autowired
    private CmsBindingsService cmsBindingsService;

    /**
     * 添加消息队列绑定关系
     *
     * @param mqId
     * @param mqBinding
     * @param context
     * @param request
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "添加消息队列绑定关系", response = BocloudResult.class, httpMethod = "POST", produces = MediaType.APPLICATION_JSON_VALUE)
    @PostMapping("/{id}/bindings")
    public BocloudResult createBindingsEq(@PathVariable(value = Common.ID) Long mqId,
                                          @RequestBody CmsBinding mqBinding,
                                          @Value(Common.REQ_CONTEXT) RequestContext context, HttpServletRequest request) {
        if (null != mqBinding && null != mqId) {
            long start = System.currentTimeMillis();
            BocloudResult result = cmsBindingsService.create(mqId, mqBinding);
            if (null != result.getData()) {
                CmsLog log = new CmsLog("bindings", "添加消息队列绑定关系", JSONObject.toJSONString(result), null,
                        IpTool.getIP(request), new Date(), System.currentTimeMillis() - start, (Long) result.getData(),
                        context.getTarget());
                cmsService.saveLog(log);
            }
            return result;
        }
        return ResultTools.formatErrResult();
    }

    /**
     * 删除消息队列绑定关系
     *
     * @param mqId
     * @param cmsBinding
     * @param context
     * @param request
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "删除消息队列绑定关系", response = BocloudResult.class, httpMethod = "DELETE", produces = MediaType.APPLICATION_JSON_VALUE)
    @DeleteMapping("/{id}/bindings")
    public BocloudResult remove(@PathVariable(value = Common.ID) Long mqId,
                                @RequestBody CmsBinding cmsBinding,
                                @Value(Common.REQ_CONTEXT) RequestContext context, HttpServletRequest request) {
        if (null != mqId && null != cmsBinding) {
            long start = System.currentTimeMillis();
            BocloudResult result = cmsBindingsService.remove(mqId, cmsBinding);
            if (null != result.getData()) {
                CmsLog log = new CmsLog("bindings", "删除绑定", JSONObject.toJSONString(result), null,
                        IpTool.getIP(request), new Date(), System.currentTimeMillis() - start, (Long) result.getData(),
                        context.getTarget());
                cmsService.saveLog(log);
            }
            return result;
        }
        return ResultTools.formatErrResult();
    }

    /**
     * 查询消息队列绑定关系列表
     *
     * @param mqId
     * @param page
     * @param rows
     * @param mqBinding
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "查询消息队列绑定关系列表", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/{id}/bindings")
    public BocloudResult<GridBean<CmsBinding>> list(@PathVariable(value = Common.ID) Long mqId,
                                                    @RequestParam(value = Common.PAGE, required = false) Integer page,
                                                    @RequestParam(value = Common.ROWS, required = false) Integer rows,
                                                    @RequestBody CmsBinding mqBinding) {
        if (null != mqId && null != mqBinding) {
            return cmsBindingsService.list(mqId, page, rows, mqBinding);
        }
        return ResultTools.formatErrResult();
    }

    /**
     * 查询消息队列绑定详情
     *
     * @param mqId
     * @param cmsBinding
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "查询消息队列绑定详情", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/{id}/bindings/info")
    public BocloudResult<CmsBinding> detail(@PathVariable(value = Common.ID) Long mqId,
                                            @RequestBody CmsBinding cmsBinding) {
        if (null != mqId && null != cmsBinding) {
            return cmsBindingsService.detail(mqId, cmsBinding);
        }
        return ResultTools.formatErrResult();
    }
}
