package com.bocloud.pms.booter.controller;

import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.GridBean;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.ResultTools;
import com.bocloud.pms.entity.CmsChannel;
import com.bocloud.pms.entity.resource.model.ChannelModel;
import com.bocloud.pms.interfaces.middleware.CmsChannelService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.*;

/**
 * Created by liuyuanyuan on 2020/3/18 17:22
 */
@Api(value = "消息队列channels", tags = "channels")
@RestController
@RequestMapping("/v1/cms")
public class CmsChannelController {
    @Autowired
    private CmsChannelService cmsChannelService;

    /**
     * 消息队列channels列表
     *
     * @param mqId
     * @param page
     * @param rows
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "消息队列channels列表", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/{id}/channels")
    public BocloudResult<GridBean<CmsChannel>> list(@PathVariable(value = Common.ID) Long mqId,
                                                    @RequestParam(value = Common.PAGE, required = false) Integer page,
                                                    @RequestParam(value = Common.ROWS, required = false) Integer rows) {
        return cmsChannelService.list(mqId, page, rows);
    }

    /**
     * 查看消息队列channels详情
     *
     * @param mqId
     * @param name
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "查看消息队列channels详情", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/{id}/channels/{name}")
    public BocloudResult<ChannelModel> detail(@PathVariable(value = Common.ID) Long mqId,
                                              @PathVariable(value = Common.NAME) String name) {
        name = name.replaceAll(" ", "%20").replaceAll(":", "%3A").replaceAll(">", "%3E");
        if (null != mqId && null != name) {
            return cmsChannelService.detail(mqId, name);
        }
        return ResultTools.formatErrResult();
    }
}
