package com.bocloud.pms.booter.controller;

import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.GridBean;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.ResultTools;
import com.bocloud.pms.entity.CmsConnection;
import com.bocloud.pms.entity.resource.model.ConnectionModel;
import com.bocloud.pms.interfaces.middleware.CmsConnectionService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.*;

/**
 * Created by liuyuanyuan on 2020/3/30 10:25
 */
@Api(value = "消息队列连接", tags = "消息队列连接")
@RestController
@RequestMapping("/v1/cms")
public class CmsConnectionController {
    @Autowired
    private CmsConnectionService cmsConnectionService;

    /**
     * 查询消息队列连接列表
     *
     * @param mqId
     * @param page
     * @param rows
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "查询消息队列连接列表", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/{id}/connections")
    public BocloudResult<GridBean<CmsConnection>> list(@PathVariable(value = Common.ID) Long mqId,
                                                       @RequestParam(value = Common.PAGE, required = false) Integer page,
                                                       @RequestParam(value = Common.ROWS, required = false) Integer rows) {
        return cmsConnectionService.list(mqId, page, rows);
    }

    /**
     * 查看连接详情
     *
     * @param mqId
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "查看消息队列连接详情", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/{id}/connections/{name}")
    public BocloudResult<ConnectionModel> detail(@PathVariable(value = Common.ID) Long mqId,
                                                 @PathVariable(value = Common.NAME) String name) {
        name = name.replaceAll(" ", "%20").replaceAll(":", "%3A").replaceAll(">", "%3E");
        if (null != mqId && null != name) {
            return cmsConnectionService.detail(mqId, name);
        }
        return ResultTools.formatErrResult();
    }

    /**
     * 关闭消息队列连接详情
     *
     * @param mqId
     * @param name
     * @param xReason
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "关闭消息队列连接详情", response = BocloudResult.class, httpMethod = "DELETE", produces = MediaType.APPLICATION_JSON_VALUE)
    @DeleteMapping("/{id}/connections/{name}")
    public BocloudResult close(@PathVariable(value = Common.ID) Long mqId,
                               @PathVariable(value = Common.NAME) String name,
                               @RequestParam(value = "xReason", required = false) String xReason) {
        name = name.replaceAll(" ", "%20").replaceAll(":", "%3A").replaceAll(">", "%3E");
        if (null != mqId && null != name) {
            return cmsConnectionService.close(mqId, name, xReason);
        }
        return ResultTools.formatErrResult();
    }
}
