package com.bocloud.pms.booter.controller;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.GridBean;
import com.bocloud.common.model.Pager;
import com.bocloud.common.model.RequestContext;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.ResultTools;
import com.bocloud.pms.entity.Cms;
import com.bocloud.pms.entity.CmsLog;
import com.bocloud.pms.interfaces.middleware.CmsNodeService;
import com.bocloud.pms.interfaces.middleware.CmsService;
import com.bocloud.service.utils.IpTool;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.Date;

@Api(value = "消息队列", tags = "消息队列")
@RestController
@RequestMapping("/v1/cms")
public class CmsController {

    @Autowired
    private CmsService cmsService;
    @Autowired
    private CmsNodeService cmsNodeService;

    /**
     * 消息队列实例加入回收站
     *
     * @param id
     * @param context
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "消息队列实例加入回收站", response = BocloudResult.class, httpMethod = "PATCH", produces = MediaType.APPLICATION_JSON_VALUE)
    @PatchMapping("/{id}/recycle")
    public BocloudResult recycle(@PathVariable(value = Common.ID) Long id,
                                 @Value(Common.REQ_CONTEXT) RequestContext context) {
        return cmsService.recycle(id, context);
    }

    /**
     * 消息队列实例回收站还原
     *
     * @param id
     * @param context
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "消息队列实例回收站还原", response = BocloudResult.class, httpMethod = "PATCH", produces = MediaType.APPLICATION_JSON_VALUE)
    @PatchMapping("/{id}/restore")
    public BocloudResult restore(@PathVariable(value = Common.ID) Long id,
                                 @Value(Common.REQ_CONTEXT) RequestContext context) {
        return cmsService.restore(id, context);
    }

    /**
     * 查询消息队列列表
     *
     * @param pager
     * @param context
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "查询消息队列列表", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping
    public BocloudResult<GridBean<Cms>> list(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return cmsService.list(pager, context);
    }

    /**
     * 详情
     *
     * @param id
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "查看消息队列详情", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/{id}")
    public BocloudResult<Cms> detail(@PathVariable(value = Common.ID) Long id) {
        return cmsService.detail(id);
    }

    /**
     * 查看消息队列基础信息
     *
     * @param id
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "查看消息队列基础信息", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/{id}/basics")
    public BocloudResult<Cms> basics(@PathVariable(value = Common.ID) Long id) {
        return cmsService.basics(id);
    }

    /**
     * 删除
     *
     * @param id
     * @param context
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "删除消息队列", response = BocloudResult.class, httpMethod = "DELETE", produces = MediaType.APPLICATION_JSON_VALUE)
    @DeleteMapping(value = "/{id}")
    public BocloudResult remove(@PathVariable(value = Common.ID) Long id,
                                @Value(Common.REQ_CONTEXT) RequestContext context) {
        return cmsService.remove(id, context);
    }

    /**
     * 查询消息队列日志列表
     *
     * @param pager
     * @param context
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "查询消息队列日志列表", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/logs")
    public BocloudResult<GridBean<CmsLog>> logs(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return cmsService.logs(pager, context);
    }

    /**
     * 备份列表
     *
     * @param id
     * @param context
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "查询消息队列备份列表", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/{id}/backups")
    public BocloudResult backups(@PathVariable(value = Common.ID) Long id,
                                 @Value(Common.REQ_CONTEXT) RequestContext context) {
        return cmsService.backups(id, context);
    }

    /**
     * 消息队列备份恢复
     *
     * @param id
     * @param snapshotId
     * @param context
     * @param request
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "消息队列备份恢复", response = BocloudResult.class, httpMethod = "PATCH", produces = MediaType.APPLICATION_JSON_VALUE)
    @PatchMapping(value = "/{id}/revert")
    public BocloudResult action(@PathVariable(value = Common.ID) Long id,
                                @RequestParam(value = "snapshotId", required = false) Long snapshotId,
                                @Value(Common.REQ_CONTEXT) RequestContext context, HttpServletRequest request) {
        if (null != id && null != snapshotId) {
            long start = System.currentTimeMillis();
            BocloudResult result = cmsService.revert(id, snapshotId, context);
            CmsLog log = new CmsLog("消息队列", "消息队列恢复", JSONObject.toJSONString(result), null, IpTool.getIP(request),
                    new Date(), System.currentTimeMillis() - start, id, context.getTarget());
            cmsService.saveLog(log);
            return result;
        }
        return ResultTools.formatErrResult();
    }

    /**
     * 消息队列备份操作
     *
     * @param id
     * @param context
     * @param request
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "消息队列备份操作", response = BocloudResult.class, httpMethod = "PATCH", produces = MediaType.APPLICATION_JSON_VALUE)
    @PatchMapping(value = "/{id}/backup")
    public BocloudResult action(@PathVariable(value = Common.ID) Long id,
                                @Value(Common.REQ_CONTEXT) RequestContext context, HttpServletRequest request) {
        long start = System.currentTimeMillis();
        BocloudResult result = cmsService.backup(id, context);
        CmsLog log = new CmsLog("消息队列", "消息队列备份", JSONObject.toJSONString(result), null, IpTool.getIP(request),
                new Date(), System.currentTimeMillis() - start, id, context.getTarget());
        cmsService.saveLog(log);
        return result;
    }

    /**
     * 删除备份
     *
     * @param id
     * @param context
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "删除消息队列备份", response = BocloudResult.class, httpMethod = "DELETE", produces = MediaType.APPLICATION_JSON_VALUE)
    @DeleteMapping(value = "/backups/{id}")
    public BocloudResult removeBackup(@PathVariable(value = Common.ID) Long id,
                                      @Value(Common.REQ_CONTEXT) RequestContext context) {
        return cmsService.removeBackup(id, context);
    }

    /**
     * 安全列表
     *
     * @param id
     * @param context
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "查询消息队列安全列表", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/{id}/securities")
    public BocloudResult securities(@PathVariable(value = Common.ID) Long id,
                                    @Value(Common.REQ_CONTEXT) RequestContext context) {
        return cmsService.securities(id, context);
    }
}
