package com.bocloud.pms.booter.controller;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.GridBean;
import com.bocloud.common.model.RequestContext;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.ResultTools;
import com.bocloud.pms.entity.CmsExchange;
import com.bocloud.pms.entity.CmsLog;
import com.bocloud.pms.interfaces.middleware.CmsExchangeService;
import com.bocloud.pms.interfaces.middleware.CmsService;
import com.bocloud.service.utils.IpTool;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.Date;

/**
 * Created by liuyuanyuan on 2020/3/10 18:07
 */
@Api(value = "交换机", tags = "交换机")
@RestController
@RequestMapping("/v1/cms")
public class CmsExchangeController {
    @Autowired
    private CmsExchangeService cmsExchangeService;
    @Autowired
    private CmsService cmsService;

    /**
     * 查询消息队列交换机列表
     *
     * @param mqId
     * @param page
     * @param rows
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "查询消息队列交换机列表", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/{id}/exchanges")
    public BocloudResult<GridBean<CmsExchange>> list(@PathVariable(value = Common.ID) Long mqId,
                                                     @RequestParam(value = Common.PAGE, required = false) Integer page,
                                                     @RequestParam(value = Common.ROWS, required = false) Integer rows) {
        return cmsExchangeService.list(mqId, page, rows);
    }

    /**
     * 查看消息队列交换机详情
     *
     * @param mqId
     * @param exchangeName
     * @param vhost
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "查看消息队列交换机详情", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/{id}/exchanges/{vhost}/{name}")
    public BocloudResult<CmsExchange> detail(@PathVariable(value = Common.ID) Long mqId,
                                             @PathVariable(value = Common.NAME) String exchangeName, @PathVariable(value = "vhost") String vhost) {
        if (null != mqId && null != exchangeName && null != vhost) {
            return cmsExchangeService.detail(mqId, exchangeName, vhost);
        }
        return ResultTools.formatErrResult();
    }

    /**
     * 添加消息队列交换机
     *
     * @param mqId
     * @param cmsExchange
     * @param context
     * @param request
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "添加消息队列交换机", response = BocloudResult.class, httpMethod = "POST", produces = MediaType.APPLICATION_JSON_VALUE)
    @PostMapping("/{id}/exchanges")
    public BocloudResult create(@PathVariable(value = Common.ID) Long mqId,
                                @RequestBody CmsExchange cmsExchange,
                                @Value(Common.REQ_CONTEXT) RequestContext context, HttpServletRequest request) {
        if (null != cmsExchange && null != mqId) {
            long start = System.currentTimeMillis();
            BocloudResult result = cmsExchangeService.create(mqId, cmsExchange, context);
            if (null != result.getData()) {
                CmsLog log = new CmsLog("交换机", "创建交换机", JSONObject.toJSONString(result), null, IpTool.getIP(request),
                        new Date(), System.currentTimeMillis() - start, (Long) result.getData(), context.getTarget());
                cmsService.saveLog(log);
            }
            return result;
        }
        return ResultTools.formatErrResult();
    }

    /**
     * 删除消息队列交换机
     *
     * @param mqId
     * @param name
     * @param vhost
     * @param context
     * @param request
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "删除消息队列交换机", response = BocloudResult.class, httpMethod = "DELETE", produces = MediaType.APPLICATION_JSON_VALUE)
    @DeleteMapping("/{id}/exchanges/{vhost}/{name}")
    public BocloudResult remove(@PathVariable(value = Common.ID) Long mqId,
                                @PathVariable(value = Common.NAME) String name, @PathVariable(value = "vhost") String vhost,
                                @Value(Common.REQ_CONTEXT) RequestContext context, HttpServletRequest request) {
        if (null != mqId && null != name && null != vhost) {
            long start = System.currentTimeMillis();
            BocloudResult result = cmsExchangeService.remove(vhost, mqId, name);
            if (null != result.getData()) {
                CmsLog log = new CmsLog("交换机", "删除交换机", JSONObject.toJSONString(result), null, IpTool.getIP(request),
                        new Date(), System.currentTimeMillis() - start, (Long) result.getData(), context.getTarget());
                cmsService.saveLog(log);
            }
            return result;
        }
        return ResultTools.formatErrResult();
    }

}
