package com.bocloud.pms.booter.controller;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.annotations.Log;
import com.bocloud.common.enums.BoCloudService;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.GridBean;
import com.bocloud.common.model.Pager;
import com.bocloud.common.model.RequestContext;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.ResultTools;
import com.bocloud.pms.entity.CmsLog;
import com.bocloud.pms.entity.CmsUser;
import com.bocloud.pms.entity.CmsVhost;
import com.bocloud.pms.interfaces.middleware.CmsManageService;
import com.bocloud.pms.interfaces.middleware.CmsService;
import com.bocloud.service.utils.IpTool;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.Date;
import java.util.List;

/**
 * Created by liuyuanyuan on 2020/3/11 17:50
 */
@Api(value = "manage", tags = "manage")
@RestController
@RequestMapping("/v1/cms")
public class CmsManageController {
    @Autowired
    private CmsService cmsService;
    @Autowired
    private CmsManageService cmsManageService;
    private final String key = "vhost";

    /**
     * 查询消息队列vhost列表
     *
     * @param id
     * @param pager
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "查询消息队列vhost列表", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/{id}/vhosts")
    public BocloudResult<GridBean<CmsVhost>> listOfVhost(@PathVariable(value = Common.ID) Long id,
                                                         Pager pager) {
        return cmsManageService.listOfVhost(id, pager);
    }

    /**
     * 查看消息队列vhost详情
     *
     * @param mqId
     * @param name
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "查看消息队列vhost详情", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/{id}/vhosts/{name}")
    public BocloudResult<CmsVhost> detailOfVhost(@PathVariable(value = Common.ID) Long mqId,
                                                 @PathVariable(value = "name") String name) {
        if (null != mqId && null != name) {
            return cmsManageService.detailOfVhost(mqId, name);
        }
        return ResultTools.formatErrResult();
    }

    /**
     * 消息队列vhost权限列表
     *
     * @param mqId
     * @param vhostName
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "消息队列vhost权限列表", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/{id}/vhosts/{name}/permissions")
    public BocloudResult<List<CmsVhost>> permissionOfVhostOfList(@PathVariable(value = Common.ID) Long mqId,
                                                                 @PathVariable(value = "name") String vhostName) {
        if (null != mqId && null != vhostName) {
            return cmsManageService.permissionOfVhostOfList(mqId, vhostName);
        }
        return ResultTools.formatErrResult();
    }

    /**
     * 消息队列vhost设置权限
     *
     * @param mqId
     * @param vhostName
     * @param cmsVhost
     * @param context
     * @param request
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "消息队列vhost设置权限", response = BocloudResult.class, httpMethod = "POST", produces = MediaType.APPLICATION_JSON_VALUE)
    @PostMapping("/{id}/vhosts/{name}/permissions")
    public BocloudResult createPermissionOfVhost(@PathVariable(value = Common.ID) Long mqId,
                                                 @PathVariable(value = "name") String vhostName,
                                                 @RequestBody CmsVhost cmsVhost,
                                                 @Value(Common.REQ_CONTEXT) RequestContext context, HttpServletRequest request) {
        if (null != mqId && null != vhostName) {
            long start = System.currentTimeMillis();
            BocloudResult result = cmsManageService.createPermissionOfVhost(mqId, vhostName, cmsVhost);
            if (null != result.getData()) {
                CmsLog log = new CmsLog(key, "vhost设置权限", JSONObject.toJSONString(result), null,
                        IpTool.getIP(request), new Date(), System.currentTimeMillis() - start, (Long) result.getData(),
                        context.getTarget());
                cmsService.saveLog(log);
            }
            return result;
        }
        return ResultTools.formatErrResult();
    }

    /**
     * 消息队列vhost权限清除
     *
     * @param mqId
     * @param vhostName
     * @param context
     * @param request
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "消息队列vhost权限清除", response = BocloudResult.class, httpMethod = "DELETE", produces = MediaType.APPLICATION_JSON_VALUE)
    @DeleteMapping("/{id}/vhosts/{name}/permissions")
    public BocloudResult deletePermissionOfVhost(@PathVariable(value = Common.ID) Long mqId,
                                                 @PathVariable(value = "name") String vhostName,
                                                 @RequestParam(value = "user", required = false) String userName,
                                                 @Value(Common.REQ_CONTEXT) RequestContext context, HttpServletRequest request) {
        if (null != mqId && null != vhostName && null != userName) {
            long start = System.currentTimeMillis();
            BocloudResult result = cmsManageService.deletePermissionOfVhost(mqId, vhostName, userName);
            if (null != result.getData()) {
                CmsLog log = new CmsLog(key, "vhost权限清除", JSONObject.toJSONString(result), null,
                        IpTool.getIP(request), new Date(), System.currentTimeMillis() - start, (Long) result.getData(),
                        context.getTarget());
                cmsService.saveLog(log);
            }
            return result;
        }
        return ResultTools.formatErrResult();
    }

    /**
     * 查看消息队列user详情
     *
     * @param mqId
     * @param userName
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "查看消息队列user详情", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/{id}/users/{name}/permissions")
    public BocloudResult<List<CmsUser>> permissionOfUserOfList(@PathVariable(value = Common.ID) Long mqId,
                                                               @PathVariable(value = "name") String userName) {
        if (null != mqId && null != userName) {
            return cmsManageService.permissionOfUserOfList(mqId, userName);
        }
        return ResultTools.formatErrResult();
    }

    /**
     * 消息队列用户权限设置
     *
     * @param mqId
     * @param userName
     * @param cmsUser
     * @param context
     * @param request
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "消息队列用户权限设置", response = BocloudResult.class, httpMethod = "POST", produces = MediaType.APPLICATION_JSON_VALUE)
    @PostMapping("/{id}/users/{name}/permissions")
    public BocloudResult createPermissionOfUser(@PathVariable(value = Common.ID) Long mqId,
                                                @PathVariable(value = "name") String userName,
                                                @RequestBody CmsUser cmsUser,
                                                @Value(Common.REQ_CONTEXT) RequestContext context, HttpServletRequest request) {
        if (null != mqId && null != userName) {
            long start = System.currentTimeMillis();
            BocloudResult result = cmsManageService.createPermissionOfUser(mqId, userName, cmsUser);
            if (null != result.getData()) {
                CmsLog log = new CmsLog("user", "user设置权限", JSONObject.toJSONString(result), null,
                        IpTool.getIP(request), new Date(), System.currentTimeMillis() - start, (Long) result.getData(),
                        context.getTarget());
                cmsService.saveLog(log);
            }
            return result;
        }
        return ResultTools.formatErrResult();
    }

    /**
     * 消息队列user权限清除
     *
     * @param mqId
     * @param userName
     * @param context
     * @param request
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "消息队列user权限清除", response = BocloudResult.class, httpMethod = "DELETE", produces = MediaType.APPLICATION_JSON_VALUE)
    @DeleteMapping("/{id}/users/{name}/permissions")
    public BocloudResult deletePermissionOfUser(@PathVariable(value = Common.ID) Long mqId,
                                                @PathVariable(value = "name") String userName,
                                                @RequestParam(value = "vhost", required = false) String vhostName,
                                                @Value(Common.REQ_CONTEXT) RequestContext context, HttpServletRequest request) {
        if (null != mqId && null != userName && null != vhostName) {
            long start = System.currentTimeMillis();
            BocloudResult result = cmsManageService.deletePermissionOfUser(mqId, userName, vhostName);
            if (null != result.getData()) {
                CmsLog log = new CmsLog("user", "user权限清除", JSONObject.toJSONString(result), null,
                        IpTool.getIP(request), new Date(), System.currentTimeMillis() - start, (Long) result.getData(),
                        context.getTarget());
                cmsService.saveLog(log);
            }
            return result;
        }
        return ResultTools.formatErrResult();
    }

    /**
     * 添加消息队列vhost
     *
     * @param mqId
     * @param cmsVhost
     * @param context
     * @param request
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "添加消息队列vhost", response = BocloudResult.class, httpMethod = "POST", produces = MediaType.APPLICATION_JSON_VALUE)
    @PostMapping("/{id}/vhosts")
    public BocloudResult createVhost(@PathVariable(value = Common.ID) Long mqId,
                                     @RequestBody CmsVhost cmsVhost,
                                     @Value(Common.REQ_CONTEXT) RequestContext context, HttpServletRequest request) {
        if (null != cmsVhost && null != mqId) {
            long start = System.currentTimeMillis();
            BocloudResult result = cmsManageService.create(cmsVhost, mqId);
            if (null != result.getData()) {
                CmsLog log = new CmsLog(key, "添加vhost", JSONObject.toJSONString(result), null,
                        IpTool.getIP(request), new Date(), System.currentTimeMillis() - start, (Long) result.getData(),
                        context.getTarget());
                cmsService.saveLog(log);
            }
            return result;
        }
        return ResultTools.formatErrResult();
    }

    /**
     * 删除消息队列vhost
     *
     * @param mqId
     * @param vhostName
     * @param context
     * @param request
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "删除消息队列vhost", response = BocloudResult.class, httpMethod = "DELETE", produces = MediaType.APPLICATION_JSON_VALUE)
    @DeleteMapping("/{id}/vhosts/{name}")
    @Log(module = BoCloudService.PMS, target = "vhost", operation = "删除消息队列vhost")
    public BocloudResult removeVhost(@PathVariable(value = Common.ID) Long mqId,
                                     @PathVariable(value = "name") String vhostName, @Value(Common.REQ_CONTEXT) RequestContext context,
                                     HttpServletRequest request) {
        if (null != mqId && null != vhostName) {
            long start = System.currentTimeMillis();
            BocloudResult result = cmsManageService.removeVhost(mqId, vhostName);
            if (null != result.getData()) {
                CmsLog log = new CmsLog(key, "删除vhost", JSONObject.toJSONString(result), null,
                        IpTool.getIP(request), new Date(), System.currentTimeMillis() - start, (Long) result.getData(),
                        context.getTarget());
                cmsService.saveLog(log);
            }
            return result;
        }
        return ResultTools.formatErrResult();
    }

    /**
     * 查询消息队列user列表
     *
     * @param id
     * @param pager
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "查询消息队列user列表", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/{id}/users")
    public BocloudResult<GridBean<CmsUser>> listOfUser(@PathVariable(value = Common.ID) Long id,
                                                       Pager pager) {
        return cmsManageService.listOfUser(id, pager);
    }

    /**
     * 查看消息队列user详情
     *
     * @param mqId
     * @param name
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "查看消息队列user详情", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/{id}/users/{name}")
    public BocloudResult<CmsUser> detailOfUser(@PathVariable(value = Common.ID) Long mqId,
                                               @PathVariable(value = "name") String name) {
        if (null != mqId && null != name) {
            return cmsManageService.detailOfUser(mqId, name);
        }
        return ResultTools.formatErrResult();
    }

    /**
     * 添加消息队列user
     *
     * @param cmsUser
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "添加消息队列user", response = BocloudResult.class, httpMethod = "POST", produces = MediaType.APPLICATION_JSON_VALUE)
    @PostMapping("/{id}/users")
    public BocloudResult createUser(@PathVariable(value = Common.ID) Long mqId,
                                    @RequestBody CmsUser cmsUser,
                                    @Value(Common.REQ_CONTEXT) RequestContext context, HttpServletRequest request) {
        if (null != cmsUser && null != mqId) {
            long start = System.currentTimeMillis();
            BocloudResult result = cmsManageService.create(mqId, cmsUser);
            if (null != result.getData()) {
                CmsLog log = new CmsLog("user", "添加user", JSONObject.toJSONString(result), null, IpTool.getIP(request),
                        new Date(), System.currentTimeMillis() - start, (Long) result.getData(), context.getTarget());
                cmsService.saveLog(log);
            }
            return result;
        }
        return ResultTools.formatErrResult();
    }

    /**
     * 删除消息队列user
     *
     * @param mqId
     * @param userName
     * @param context
     * @param request
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "删除消息队列user", response = BocloudResult.class, httpMethod = "DELETE", produces = MediaType.APPLICATION_JSON_VALUE)
    @DeleteMapping("/{id}/users/{name}")
    public BocloudResult removeUser(@PathVariable(value = Common.ID) Long mqId,
                                    @PathVariable(value = "name") String userName, @Value(Common.REQ_CONTEXT) RequestContext context,
                                    HttpServletRequest request) {
        if (null != mqId && null != userName) {
            long start = System.currentTimeMillis();
            BocloudResult result = cmsManageService.removeUser(mqId, userName);
            if (null != result.getData()) {
                CmsLog log = new CmsLog("user", "删除user", JSONObject.toJSONString(result), null, IpTool.getIP(request),
                        new Date(), System.currentTimeMillis() - start, (Long) result.getData(), context.getTarget());
                cmsService.saveLog(log);
            }
            return result;
        }
        return ResultTools.formatErrResult();
    }

    /**
     * 更新消息队列user
     *
     * @param mqId
     * @param cmsUser
     * @param context
     * @param request
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "更新消息队列user", response = BocloudResult.class, httpMethod = "PUT", produces = MediaType.APPLICATION_JSON_VALUE)
    @PutMapping("/{id}/users/{name}")
    public BocloudResult updateUser(@PathVariable(value = Common.ID) Long mqId,
                                    @PathVariable(value = "name") String userName,
                                    @RequestBody CmsUser cmsUser,
                                    @Value(Common.REQ_CONTEXT) RequestContext context, HttpServletRequest request) {
        if (null != cmsUser && null != mqId) {
            long start = System.currentTimeMillis();
            cmsUser.setName(userName);
            BocloudResult result = cmsManageService.updateUser(mqId, cmsUser);
            if (null != result.getData()) {
                CmsLog log = new CmsLog("user", "更新user", JSONObject.toJSONString(result), null, IpTool.getIP(request),
                        new Date(), System.currentTimeMillis() - start, (Long) result.getData(), context.getTarget());
                cmsService.saveLog(log);
            }
            return result;
        }
        return ResultTools.formatErrResult();
    }

}
