package com.bocloud.pms.booter.controller;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.GridBean;
import com.bocloud.common.model.RequestContext;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.ResultTools;
import com.bocloud.pms.entity.CmsLog;
import com.bocloud.pms.entity.CmsQueue;
import com.bocloud.pms.interfaces.middleware.CmsQueueService;
import com.bocloud.pms.interfaces.middleware.CmsService;
import com.bocloud.service.utils.IpTool;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.Date;

/**
 * Created by liuyuanyuan on 2020/3/11 16:59
 */
@Api(value = "消息队列", tags = "消息队列")
@RestController
@RequestMapping("/v1/cms")
public class CmsQueueController {
    @Autowired
    private CmsService cmsService;
    @Autowired
    private CmsQueueService cmsQueueService;

    /**
     * 查询消息队列列表
     *
     * @param mqId
     * @param page
     * @param rows
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "查询消息队列列表", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/{id}/queues")
    public BocloudResult<GridBean<CmsQueue>> list(@PathVariable(value = Common.ID) Long mqId,
                                                  @RequestParam(value = Common.PAGE, required = false) Integer page,
                                                  @RequestParam(value = Common.ROWS, required = false) Integer rows) {
        return cmsQueueService.list(mqId, page, rows);
    }

    /**
     * 查看消息队列详情
     *
     * @param mqId
     * @param queueName
     * @param vhost
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "查看消息队列详情", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/{id}/queues/{vhost}/{name}")
    public BocloudResult<CmsQueue> detail(@PathVariable(value = Common.ID) Long mqId,
                                          @PathVariable(value = Common.NAME) String queueName, @PathVariable(value = "vhost") String vhost) {
        if (null != mqId && null != queueName && null != vhost) {
            return cmsQueueService.detail(mqId, queueName, vhost);
        }
        return ResultTools.formatErrResult();
    }

    /**
     * 添加
     *
     * @param mqId
     * @param cmsQueue
     * @param context
     * @param request
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "添加消息队列", response = BocloudResult.class, httpMethod = "POST", produces = MediaType.APPLICATION_JSON_VALUE)
    @PostMapping("/{id}/queues")
    public BocloudResult create(@PathVariable(value = Common.ID) Long mqId,
                                @RequestBody CmsQueue cmsQueue,
                                @Value(Common.REQ_CONTEXT) RequestContext context, HttpServletRequest request) {
        if (null != cmsQueue && null != mqId) {
            long start = System.currentTimeMillis();
            BocloudResult result = cmsQueueService.create(mqId, cmsQueue, context);
            if (null != result.getData()) {
                CmsLog log = new CmsLog("消息队列", "创建消息队列", JSONObject.toJSONString(result), null, IpTool.getIP(request),
                        new Date(), System.currentTimeMillis() - start, (Long) result.getData(), context.getTarget());
                cmsService.saveLog(log);
            }
            return result;
        }
        return ResultTools.formatErrResult();
    }

    /**
     * 删除
     *
     * @param mqId
     * @param vhost
     * @param name
     * @param context
     * @param request
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "删除消息队列", response = BocloudResult.class, httpMethod = "DELETE", produces = MediaType.APPLICATION_JSON_VALUE)
    @DeleteMapping("/{id}/queues/{vhost}/{name}")
    public BocloudResult remove(@PathVariable(value = Common.ID) Long mqId, @PathVariable(value = "vhost") String vhost,
                                @PathVariable(value = Common.NAME) String name, @Value(Common.REQ_CONTEXT) RequestContext context,
                                HttpServletRequest request) {
        if (null != mqId && null != name && null != vhost) {
            long start = System.currentTimeMillis();
            BocloudResult result = cmsQueueService.remove(mqId, vhost, name);
            if (null != result.getData()) {
                CmsLog log = new CmsLog("消息队列", "删除消息队列", JSONObject.toJSONString(result), null, IpTool.getIP(request),
                        new Date(), System.currentTimeMillis() - start, (Long) result.getData(), context.getTarget());
                cmsService.saveLog(log);
            }
            return result;
        }
        return ResultTools.formatErrResult();
    }
}
