package com.bocloud.pms.booter.controller;

import javax.servlet.http.HttpServletRequest;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PatchMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.GridBean;
import com.bocloud.common.model.Pager;
import com.bocloud.common.model.RequestContext;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.ResultTools;
import com.bocloud.pms.entity.CmsTopic;
import com.bocloud.pms.interfaces.middleware.CmsTopicService;
import com.bocloud.service.utils.IpTool;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

/**
 * Created by liuyuanyuan on 2020/3/12 16:57
 */
@Api(value = "主题", tags = "主题")
@RestController
@RequestMapping("/v1/cms")
public class CmsTopicController {
    @Autowired
    private CmsTopicService cmsTopicService;

    /**
     * 查询消息队列主题列表
     * 
     * @param mqId
     * @param pager
     * @param context
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "查询消息队列主题列表", response = BocloudResult.class, httpMethod = "GET",
        produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/{id}/topics")
    public BocloudResult<GridBean<CmsTopic>> list(@PathVariable(value = Common.ID) Long mqId, Pager pager,
        @Value(Common.REQ_CONTEXT) RequestContext context) {
        return cmsTopicService.list(mqId, pager, context);
    }

    /**
     * 查看消息队列主题详情
     *
     * @param mqId
     * @param queueName
     * @param vhost
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "查看消息队列主题详情", response = BocloudResult.class, httpMethod = "GET",
        produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/{id}/topics/{topicId}")
    public BocloudResult<CmsTopic> detail(@PathVariable(value = Common.ID) Long mqId,
        @PathVariable(value = "topicId") Long topicId, @PathVariable(value = Common.NAME) String queueName,
        @PathVariable(value = "vhost") String vhost) {
        if (null != mqId && null != queueName && null != vhost) {
            return cmsTopicService.detail(mqId, topicId);
        }
        return ResultTools.formatErrResult();
    }

    /**
     * 添加消息队列主题
     *
     * @param cmsTopic
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "添加消息队列主题", response = BocloudResult.class, httpMethod = "POST",
        produces = MediaType.APPLICATION_JSON_VALUE)
    @PostMapping("/{id}/topics")
    public BocloudResult create(@RequestBody CmsTopic cmsTopic, @Value(Common.REQ_CONTEXT) RequestContext context,
        HttpServletRequest request) {
        if (null != cmsTopic) {
            return cmsTopicService.create(cmsTopic, context, IpTool.getIP(request));
        }
        return ResultTools.formatErrResult();
    }

    /**
     * 删除消息队列主题
     *
     * @param id
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "删除消息队列主题", response = BocloudResult.class, httpMethod = "DELETE",
        produces = MediaType.APPLICATION_JSON_VALUE)
    @DeleteMapping("/{id}/topics/{topicId}")
    public BocloudResult remove(@PathVariable(value = Common.ID) Long id, @PathVariable(value = "topicId") Long topicId,
        @Value(Common.REQ_CONTEXT) RequestContext context, HttpServletRequest request) {
        if (null != id && null != topicId) {
            return cmsTopicService.remove(id, topicId, context, IpTool.getIP(request));
        }
        return ResultTools.formatErrResult();
    }

    /**
     * 新增消息队列分区
     *
     * @param id
     * @param context
     * @param request
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "新增消息队列分区", response = BocloudResult.class, httpMethod = "POST",
        produces = MediaType.APPLICATION_JSON_VALUE)
    @PatchMapping("/{id}/topics/{topicId}")
    public BocloudResult edit(@PathVariable(value = Common.ID) Long id, @PathVariable(value = "topicId") Long topicId,
        @RequestBody CmsTopic cmsTopic, @Value(Common.REQ_CONTEXT) RequestContext context, HttpServletRequest request) {
        if (null != topicId && null != id || null != cmsTopic) {
            return cmsTopicService.edit(id, topicId, cmsTopic, context, IpTool.getIP(request));
        }
        return ResultTools.formatErrResult();
    }

}
