package com.bocloud.pms.booter.controller;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.GridBean;
import com.bocloud.common.model.Pager;
import com.bocloud.common.model.RequestContext;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.ResultTools;
import com.bocloud.pms.entity.Kvs;
import com.bocloud.pms.entity.KvsLog;
import com.bocloud.pms.entity.Parameter;
import com.bocloud.pms.interfaces.middleware.KvsNodeService;
import com.bocloud.pms.interfaces.middleware.KvsService;
import com.bocloud.service.utils.IpTool;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.Date;
import java.util.List;

@Api(value = "非关系数据库", tags = "非关系数据库")
@RestController
@RequestMapping("/v1/kvs")
public class KvsController {

    @Autowired
    private KvsService kvsService;
    @Autowired
    private KvsNodeService kvsNodeService;

    /**
     * 非关系数据库加入回收站
     *
     * @param id
     * @param context
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "非关系数据库加入回收站", response = BocloudResult.class, httpMethod = "PATCH",
            produces = MediaType.APPLICATION_JSON_VALUE)
    @PatchMapping("/{id}/recycle")
    public BocloudResult recycle(@PathVariable(value = Common.ID) Long id,
                                 @Value(Common.REQ_CONTEXT) RequestContext context) {
        return kvsService.recycle(id, context);
    }

    /**
     * 非关系数据库回收站还原
     *
     * @param id
     * @param context
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "非关系数据库回收站还原", response = BocloudResult.class, httpMethod = "PATCH",
            produces = MediaType.APPLICATION_JSON_VALUE)
    @PatchMapping("/{id}/restore")
    public BocloudResult restore(@PathVariable(value = Common.ID) Long id,
                                 @Value(Common.REQ_CONTEXT) RequestContext context) {
        return kvsService.restore(id, context);
    }

    /**
     * 查询非关系数据库列表
     *
     * @param pager
     * @param context
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "查询非关系数据库列表", response = BocloudResult.class, httpMethod = "GET",
            produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping
    public BocloudResult<GridBean<Kvs>> list(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return kvsService.list(pager, context);
    }

    /**
     * 详情
     *
     * @param id
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "查看非关系数据库详情", response = BocloudResult.class, httpMethod = "GET",
            produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/{id}")
    public BocloudResult<Kvs> detail(@PathVariable(value = Common.ID) Long id) {
        return kvsService.detail(id);
    }

    /**
     * 查看非关系数据库基础信息
     *
     * @param id
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "查看非关系数据库基础信息", response = BocloudResult.class, httpMethod = "GET",
            produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/{id}/basics")
    public BocloudResult<Kvs> basics(@PathVariable(value = Common.ID) Long id) {
        return kvsService.basics(id);
    }

    /**
     * 删除
     *
     * @param id
     * @param context
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "删除非关系数据库", response = BocloudResult.class, httpMethod = "DELETE",
            produces = MediaType.APPLICATION_JSON_VALUE)
    @DeleteMapping(value = "/{id}")
    public BocloudResult remove(@PathVariable(value = Common.ID) Long id,
                                @Value(Common.REQ_CONTEXT) RequestContext context) {
        return kvsService.remove(id, context);
    }

    /**
     * 备份列表
     *
     * @param id
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "查询非关系数据库备份列表", response = BocloudResult.class, httpMethod = "GET",
            produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/{id}/backups")
    public BocloudResult backups(@PathVariable(value = Common.ID) Long id,
                                 @Value(Common.REQ_CONTEXT) RequestContext context) {
        return kvsService.backups(id, context);
    }

    /**
     * 非关系数据库数据库备份操作
     *
     * @param id
     * @param context
     * @param request
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "非关系数据库数据库备份操作", response = BocloudResult.class, httpMethod = "PATCH",
            produces = MediaType.APPLICATION_JSON_VALUE)
    @PatchMapping(value = "/{id}/backup")
    public BocloudResult backup(@PathVariable(value = Common.ID) Long id,
                                @Value(Common.REQ_CONTEXT) RequestContext context, HttpServletRequest request) {
        long start = System.currentTimeMillis();
        BocloudResult result = kvsService.backup(id, context);
        if (null != result.getData()) {
            KvsLog log = new KvsLog("数据库", "数据库备份", JSONObject.toJSONString(result), null, IpTool.getIP(request),
                    new Date(), System.currentTimeMillis() - start, id, context.getTarget());
            kvsService.saveLog(log);
        }
        return result;
    }

    /**
     * 非关系数据库备份恢复操作
     *
     * @param id
     * @param snapshotId
     * @param context
     * @param request
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "非关系数据库备份恢复操作", response = BocloudResult.class, httpMethod = "PATCH",
            produces = MediaType.APPLICATION_JSON_VALUE)
    @PatchMapping(value = "/{id}/revert")
    public BocloudResult revert(@PathVariable(value = Common.ID) Long id,
                                @RequestParam(value = "snapshotId", required = false) Long snapshotId,
                                @Value(Common.REQ_CONTEXT) RequestContext context, HttpServletRequest request) {
        if (null != id && null != snapshotId) {
            long start = System.currentTimeMillis();
            BocloudResult result = kvsService.revert(id, snapshotId, context);
            if (null != result.getData()) {
                KvsLog log = new KvsLog("数据库", "数据库恢复", JSONObject.toJSONString(result), null, IpTool.getIP(request),
                        new Date(), System.currentTimeMillis() - start, id, context.getTarget());
                kvsService.saveLog(log);
            }
            return result;
        }
        return ResultTools.formatErrResult();
    }

    /**
     * 非关系数据库密码重置操作
     *
     * @param id
     * @param password
     * @param newPassword
     * @param context
     * @param request
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "非关系数据库密码重置操作", response = BocloudResult.class, httpMethod = "PATCH",
            produces = MediaType.APPLICATION_JSON_VALUE)
    @PatchMapping(value = "/{id}/reset")
    public BocloudResult reset(@PathVariable(value = Common.ID) Long id,
                               @RequestParam(value = "password", required = false) String password,
                               @RequestParam(value = "newPassword", required = false) String newPassword,
                               @Value(Common.REQ_CONTEXT) RequestContext context, HttpServletRequest request) {
        if (null != password && null != newPassword) {
            long start = System.currentTimeMillis();
            BocloudResult result = kvsService.reset(id, password, newPassword, IpTool.getIP(request), context);
            if (null != result.getData()) {
                KvsLog log = new KvsLog("数据库", "数据库密码重置", JSONObject.toJSONString(result), null, IpTool.getIP(request),
                        new Date(), System.currentTimeMillis() - start, id, context.getTarget());
                kvsService.saveLog(log);
            }
            return result;
        }
        return ResultTools.formatErrResult();
    }

    @ApiOperation(tags = {"CMC", "CSC"}, value = "非关系数据库密码重置操作", response = BocloudResult.class, httpMethod = "PATCH",
            produces = MediaType.APPLICATION_JSON_VALUE)
    @PatchMapping(value = "/{id}")
    public BocloudResult action(@PathVariable(value = Common.ID) Long id, @RequestBody String content,
                                @Value(Common.REQ_CONTEXT) RequestContext context, HttpServletRequest request) {
        long start = System.currentTimeMillis();
        JSONObject object = JSONObject.parseObject(content);
        if ("reset".equals(object.getString("action"))) {
            BocloudResult result = kvsService.reset(id, object.getString("password"), object.getString("newPassword"),
                    IpTool.getIP(request), context);
            if (null != result.getData()) {
                KvsLog log = new KvsLog("数据库", "数据库密码重置", JSONObject.toJSONString(result), null, IpTool.getIP(request),
                        new Date(), System.currentTimeMillis() - start, id, context.getTarget());
                kvsService.saveLog(log);
            }
            return result;
        }
        return ResultTools.formatErrResult();
    }

    /**
     * 删除备份
     *
     * @param id
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "删除非关系数据库备份", response = BocloudResult.class, httpMethod = "DELETE",
            produces = MediaType.APPLICATION_JSON_VALUE)
    @DeleteMapping("/backups/{id}")
    public BocloudResult removeBackup(@PathVariable(value = Common.ID) Long id,
                                      @Value(Common.REQ_CONTEXT) RequestContext context) {
        return kvsService.removeBackup(id, context);
    }

    /**
     * 安全列表
     *
     * @param id
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "查询非关系数据库安全列表", response = BocloudResult.class, httpMethod = "GET",
            produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/{id}/securities")
    public BocloudResult securities(@PathVariable(value = Common.ID) Long id,
                                    @Value(Common.REQ_CONTEXT) RequestContext context) {
        return kvsService.securities(id, context);
    }

    /**
     * 参数列表
     *
     * @param id
     * @param context
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "查询非关系数据库参数列表", response = BocloudResult.class, httpMethod = "GET",
            produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/{id}/parameters")
    public BocloudResult<List<JSONObject>> parameters(@PathVariable(value = Common.ID) Long id,
                                                      @Value(Common.REQ_CONTEXT) RequestContext context) {
        return kvsService.parameters(id, context);
    }

    /**
     * 非关系数据库参数设置
     *
     * @param parameter
     * @param context
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "非关系数据库参数设置", response = BocloudResult.class, httpMethod = "PUT",
            produces = MediaType.APPLICATION_JSON_VALUE)
    @PutMapping("/parameters/setting")
    public BocloudResult parameters(@RequestBody Parameter parameter,
                                    @Value(Common.REQ_CONTEXT) RequestContext context) {
        return kvsService.setting(parameter.getName(), parameter.getOldValue(), parameter.getValue(), parameter.getId(),
                parameter.getType(), context);
    }

    /**
     * 查询非关系数据库日志列表
     *
     * @param pager
     * @param context
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "查询非关系数据库日志列表", response = BocloudResult.class, httpMethod = "GET",
            produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/parameters/logs")
    public BocloudResult<GridBean> paramLogs(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return kvsService.paramLogs(pager, context);
    }

    /**
     * 查询非关系数据库日志列表
     *
     * @param pager
     * @param context
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "查询非关系数据库日志列表", response = BocloudResult.class, httpMethod = "GET",
            produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/logs")
    public BocloudResult<GridBean<KvsLog>> logs(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return kvsService.logs(pager, context);
    }
}
