package com.bocloud.pms.booter.controller;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.GridBean;
import com.bocloud.common.model.Pager;
import com.bocloud.common.model.RequestContext;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.ResultTools;
import com.bocloud.pms.entity.KvsLog;
import com.bocloud.pms.entity.KvsUser;
import com.bocloud.pms.interfaces.middleware.KvsService;
import com.bocloud.pms.interfaces.middleware.KvsUserService;
import com.bocloud.service.utils.IpTool;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.Date;

@Api(value = "非关系数据库", tags = "非关系数据库")
@RestController
@RequestMapping("/v1/kvs/users")
public class KvsUserController {

    @Autowired
    private KvsUserService kvsUserService;
    @Autowired
    private KvsService kvsService;
    private final String key = "数据库用户";

    /**
     * 列表
     *
     * @param pager
     * @param context
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "查询非关系数据库用户列表", response = BocloudResult.class, httpMethod = "GET",
            produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping
    public BocloudResult<GridBean<KvsUser>> list(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return kvsUserService.list(pager, context);
    }

    /**
     * 添加
     *
     * @param kvsUser
     * @param context
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "添加非关系数据库用户", response = BocloudResult.class, httpMethod = "POST",
            produces = MediaType.APPLICATION_JSON_VALUE)
    @PostMapping
    public BocloudResult create(@RequestBody KvsUser kvsUser, @Value(Common.REQ_CONTEXT) RequestContext context,
                                HttpServletRequest request) {
        if (null != kvsUser) {
            long start = System.currentTimeMillis();
            BocloudResult result = kvsUserService.create(kvsUser, context);
            if (null != result.getData()) {
                KvsLog log = new KvsLog(key, "创建用户", JSONObject.toJSONString(result), null, IpTool.getIP(request),
                        new Date(), System.currentTimeMillis() - start, (Long) result.getData(), context.getTarget());
                kvsService.saveLog(log);
            }
            return result;
        }
        return ResultTools.formatErrResult();
    }

    /**
     * 删除
     *
     * @param id
     * @param context
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "删除非关系数据库用户", response = BocloudResult.class, httpMethod = "DELETE",
            produces = MediaType.APPLICATION_JSON_VALUE)
    @DeleteMapping("/{id}")
    public BocloudResult remove(@PathVariable(value = Common.ID) Long id,
                                @Value(Common.REQ_CONTEXT) RequestContext context, HttpServletRequest request) {
        long start = System.currentTimeMillis();
        BocloudResult result = kvsUserService.remove(id, context);
        if (null != result.getData()) {
            KvsLog log = new KvsLog(key, "删除用户", JSONObject.toJSONString(result), null, IpTool.getIP(request),
                    new Date(), System.currentTimeMillis() - start, (Long) result.getData(), context.getTarget());
            kvsService.saveLog(log);
        }
        return result;
    }

    /**
     * 详情
     *
     * @param id
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "查看非关系数据库用户详情", response = BocloudResult.class, httpMethod = "GET",
            produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/{id}")
    public BocloudResult<KvsUser> detail(@PathVariable(value = Common.ID) Long id) {
        return kvsUserService.detail(id);
    }

    /**
     * 数据库用户操作
     *
     * @param kvsUser
     * @param context
     * @param request
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "非关系数据库用户操作", response = BocloudResult.class, httpMethod = "PATCH",
            produces = MediaType.APPLICATION_JSON_VALUE)
    @PatchMapping(value = "/{id}")
    public BocloudResult action(@RequestBody KvsUser kvsUser, @Value(Common.REQ_CONTEXT) RequestContext context,
                                HttpServletRequest request) {
        long start = System.currentTimeMillis();
        if ("reset".equals(kvsUser.getAction())) {
            BocloudResult result = kvsUserService.reset(kvsUser, context);
            if (null != result.getData()) {
                KvsLog log = new KvsLog(key, "重置用户密码", JSONObject.toJSONString(result), null, IpTool.getIP(request),
                        new Date(), System.currentTimeMillis() - start, (Long) result.getData(), context.getTarget());
                kvsService.saveLog(log);
            }
            return result;
        }
        return ResultTools.formatErrResult();
    }

}
