package com.bocloud.pms.booter.controller;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PatchMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.GridBean;
import com.bocloud.common.model.Pager;
import com.bocloud.common.model.RequestContext;
import com.bocloud.common.utils.Common;
import com.bocloud.pms.entity.Lbs;
import com.bocloud.pms.interfaces.middleware.LbsService;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

@Api(value = "负载均衡", tags = "负载均衡")
@RestController
@RequestMapping("/v1/lbs")
public class LbsController {

    @Autowired
    private LbsService lbsService;

    /**
     * 查询负载均衡列表
     *
     * @param pager
     * @param context
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "查询负载均衡列表", response = BocloudResult.class, httpMethod = "GET",
        produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping
    public BocloudResult<GridBean<Lbs>> list(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return lbsService.list(pager, context);
    }

    /**
     * 加入回收站
     *
     * @param id
     * @param context
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "负载均衡加入回收站", response = BocloudResult.class, httpMethod = "PATCH",
        produces = MediaType.APPLICATION_JSON_VALUE)
    @PatchMapping("/{id}/recycle")
    public BocloudResult recycle(@PathVariable(value = Common.ID) Long id,
        @Value(Common.REQ_CONTEXT) RequestContext context) {
        return lbsService.recycle(id, context);
    }

    /**
     * 回收站还原
     *
     * @param id
     * @param context
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "负载均衡回收站还原", response = BocloudResult.class, httpMethod = "PATCH",
        produces = MediaType.APPLICATION_JSON_VALUE)
    @PatchMapping("/{id}/restore")
    public BocloudResult restore(@PathVariable(value = Common.ID) Long id,
        @Value(Common.REQ_CONTEXT) RequestContext context) {
        return lbsService.restore(id, context);
    }

    /**
     * 详情
     *
     * @param id
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "查看负载均衡详情", response = BocloudResult.class, httpMethod = "GET",
        produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/{id}")
    public BocloudResult<Lbs> detail(@PathVariable(value = Common.ID) Long id) {
        return lbsService.detail(id);
    }

    /**
     * 删除
     *
     * @param id
     * @param context
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "删除负载均衡", response = BocloudResult.class, httpMethod = "DELETE",
        produces = MediaType.APPLICATION_JSON_VALUE)
    @DeleteMapping(value = "/{id}")
    public BocloudResult remove(@PathVariable(value = Common.ID) Long id,
        @Value(Common.REQ_CONTEXT) RequestContext context) {
        return lbsService.remove(id, context);
    }

}
