package com.bocloud.pms.booter.controller;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.GridBean;
import com.bocloud.common.model.Pager;
import com.bocloud.common.model.RequestContext;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.ResultTools;
import com.bocloud.pms.entity.*;
import com.bocloud.pms.entity.resource.Snapshot;
import com.bocloud.pms.interfaces.middleware.RdsNodeService;
import com.bocloud.pms.interfaces.middleware.RdsService;
import com.bocloud.service.utils.IpTool;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.Date;
import java.util.List;

@Api(value = "关系数据库", tags = "关系数据库")
@RestController
@RequestMapping("/v1/rds")
public class RdsController {

    @Autowired
    private RdsService rdsService;
    @Autowired
    private RdsNodeService rdsNodeService;

    /**
     * 查询关系数据库列表
     *
     * @param pager
     * @param context
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "查询关系数据库列表", response = BocloudResult.class, httpMethod = "GET",
            produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping
    public BocloudResult<GridBean<Rds>> list(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return rdsService.list(pager, context);
    }

    /**
     * 详情
     *
     * @param id
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "查看关系数据库详情", response = BocloudResult.class, httpMethod = "GET",
            produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/{id}")
    public BocloudResult<Rds> detail(@PathVariable(value = Common.ID) Long id) {
        return rdsService.detail(id);
    }

    /**
     * 查看非关系数据库基础信息
     *
     * @param id
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "查看关系数据库基础信息", response = BocloudResult.class, httpMethod = "GET",
            produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/{id}/basics")
    public BocloudResult<Rds> basics(@PathVariable(value = Common.ID) Long id) {
        return rdsService.basics(id);
    }

    /**
     * 删除
     *
     * @param id
     * @param context
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "删除关系数据库", response = BocloudResult.class, httpMethod = "DELETE",
            produces = MediaType.APPLICATION_JSON_VALUE)
    @DeleteMapping(value = "/{id}")
    public BocloudResult remove(@PathVariable(value = Common.ID) Long id,
                                @Value(Common.REQ_CONTEXT) RequestContext context) {
        return rdsService.remove(id, context);
    }

    /**
     * 查询关系数据库连接列表
     *
     * @param pager
     * @param context
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "查询关系数据库连接列表", response = BocloudResult.class, httpMethod = "GET",
            produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/connections")
    public BocloudResult<GridBean<RdsDb>> connections(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return rdsService.connections(pager, context);
    }

    /**
     * 加入回收站
     *
     * @param id
     * @param context
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "关系数据库加入回收站", response = BocloudResult.class, httpMethod = "PATCH",
            produces = MediaType.APPLICATION_JSON_VALUE)
    @PatchMapping("/{id}/recycle")
    public BocloudResult recycle(@PathVariable(value = Common.ID) Long id,
                                 @Value(Common.REQ_CONTEXT) RequestContext context) {
        return rdsService.recycle(id, context);
    }

    /**
     * 回收站还原
     *
     * @param id
     * @param context
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "关系数据库回收站还原", response = BocloudResult.class, httpMethod = "PATCH",
            produces = MediaType.APPLICATION_JSON_VALUE)
    @PatchMapping("/{id}/restore")
    public BocloudResult restore(@PathVariable(value = Common.ID) Long id,
                                 @Value(Common.REQ_CONTEXT) RequestContext context) {
        return rdsService.restore(id, context);
    }

    /**
     * 查询关系数据库日志列表
     *
     * @param pager
     * @param context
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "查询关系数据库日志列表", response = BocloudResult.class, httpMethod = "GET",
            produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/logs")
    public BocloudResult<GridBean<RdsLog>> logs(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return rdsService.logs(pager, context);
    }

    /**
     * 备份列表
     *
     * @param id
     * @param context
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "查询关系数据库备份列表", response = BocloudResult.class, httpMethod = "GET",
            produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/{id}/backups")
    public BocloudResult<List<Snapshot>> backups(@PathVariable(value = Common.ID) Long id,
                                                 @Value(Common.REQ_CONTEXT) RequestContext context) {
        return rdsService.backups(id, context);
    }

    /**
     * 删除备份
     *
     * @param id
     * @param context
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "删除关系数据库备份", response = BocloudResult.class, httpMethod = "DELETE",
            produces = MediaType.APPLICATION_JSON_VALUE)
    @DeleteMapping("/backups/{id}")
    public BocloudResult removeBackup(@PathVariable(value = Common.ID) Long id,
                                      @Value(Common.REQ_CONTEXT) RequestContext context) {
        return rdsService.removeBackup(id, context);
    }

    /**
     * 关系数据库备份操作
     *
     * @param id
     * @param context
     * @param request
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "关系数据库备份操作", response = BocloudResult.class, httpMethod = "PATCH",
            produces = MediaType.APPLICATION_JSON_VALUE)
    @PatchMapping(value = "/{id}/backup")
    public BocloudResult backup(@PathVariable(value = Common.ID) Long id,
                                @Value(Common.REQ_CONTEXT) RequestContext context, HttpServletRequest request) {
        long start = System.currentTimeMillis();
        BocloudResult result = rdsService.backup(id, context);
        RdsLog log = new RdsLog("数据库", "数据库备份", JSONObject.toJSONString(result), null, IpTool.getIP(request),
                new Date(), System.currentTimeMillis() - start, id, context.getTarget());
        rdsService.saveLog(log);
        return result;
    }

    /**
     * 关系数据库备份恢复操作
     *
     * @param id
     * @param snapshotId
     * @param context
     * @param request
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "关系数据库备份恢复操作", response = BocloudResult.class, httpMethod = "PATCH",
            produces = MediaType.APPLICATION_JSON_VALUE)
    @PatchMapping(value = "/{id}/revert")
    public BocloudResult revert(@PathVariable(value = Common.ID) Long id,
                                @RequestParam(value = "snapshotId", required = false) Long snapshotId,
                                @Value(Common.REQ_CONTEXT) RequestContext context, HttpServletRequest request) {
        if (null != id && null != snapshotId) {
            long start = System.currentTimeMillis();
            BocloudResult result = rdsService.revert(id, snapshotId, context);
            RdsLog log = new RdsLog("数据库", "数据库恢复", JSONObject.toJSONString(result), null, IpTool.getIP(request),
                    new Date(), System.currentTimeMillis() - start, id, context.getTarget());
            rdsService.saveLog(log);
            return result;
        }
        return ResultTools.formatErrResult();
    }

    /**
     * 安全列表
     *
     * @param id
     * @param context
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "查询关系数据库安全列表", response = BocloudResult.class, httpMethod = "GET",
            produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/{id}/securities")
    public BocloudResult securities(@PathVariable(value = Common.ID) Long id,
                                    @Value(Common.REQ_CONTEXT) RequestContext context) {
        return rdsService.securities(id, context);
    }

    /**
     * 参数列表
     *
     * @param id
     * @param context
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "查询关系数据库参数列表", response = BocloudResult.class, httpMethod = "GET",
            produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/{id}/parameters")
    public BocloudResult<List<JSONObject>> parameters(@PathVariable(value = Common.ID) Long id,
                                                      @Value(Common.REQ_CONTEXT) RequestContext context) {
        return rdsService.parameters(id, context);
    }

    /**
     * 参数设置
     *
     * @param context
     * @param context
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "关系数据库参数设置", response = BocloudResult.class, httpMethod = "PUT",
            produces = MediaType.APPLICATION_JSON_VALUE)
    @PutMapping("/parameters/setting")
    public BocloudResult parameters(@RequestBody Parameter parameter,
                                    @Value(Common.REQ_CONTEXT) RequestContext context) {
        return rdsService.setting(parameter.getName(), parameter.getOldValue(), parameter.getValue(), parameter.getId(),
                parameter.getType(), context);
    }

    /**
     * 查询关系数据库日志列表
     *
     * @param pager
     * @param context
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "查询关系数据库日志列表", response = BocloudResult.class, httpMethod = "GET",
            produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/parameters/logs")
    public BocloudResult<GridBean<RdsParamLog>> paramLogs(Pager pager,
                                                          @Value(Common.REQ_CONTEXT) RequestContext context) {
        return rdsService.paramLogs(pager, context);
    }
}
