package com.bocloud.pms.booter.controller;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.GridBean;
import com.bocloud.common.model.Pager;
import com.bocloud.common.model.RequestContext;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.ResultTools;
import com.bocloud.pms.entity.RdsDb;
import com.bocloud.pms.entity.RdsLog;
import com.bocloud.pms.interfaces.middleware.RdsDBService;
import com.bocloud.pms.interfaces.middleware.RdsService;
import com.bocloud.service.utils.IpTool;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.Date;

@Api(value = "关系数据库", tags = "关系数据库")
@RestController
@RequestMapping("/v1/rds/dbs")
public class RdsDbController {

    @Autowired
    private RdsDBService rdsDbService;
    @Autowired
    private RdsService rdsService;

    /**
     * 列表
     *
     * @param pager
     * @param context
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "查询关系数据库实例列表", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping
    public BocloudResult<GridBean<RdsDb>> list(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return rdsDbService.list(pager, context);
    }

    /**
     * 添加
     *
     * @param rdsDb
     * @param context
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "添加关系数据库实例", response = BocloudResult.class, httpMethod = "POST", produces = MediaType.APPLICATION_JSON_VALUE)
    @PostMapping
    public BocloudResult create(@RequestBody RdsDb rdsDb,
                                @Value(Common.REQ_CONTEXT) RequestContext context, HttpServletRequest request) {
        if (null != rdsDb) {
            long start = System.currentTimeMillis();
            BocloudResult result = rdsDbService.create(rdsDb, context);
            RdsLog log = new RdsLog("数据库实例", "创建数据库实例", JSONObject.toJSONString(result), null, IpTool.getIP(request),
                    new Date(), System.currentTimeMillis() - start, rdsDb.getRdsId(), context.getTarget());
            rdsService.saveLog(log);
            return result;
        }
        return ResultTools.formatErrResult();
    }

    /**
     * 删除
     *
     * @param id
     * @param context
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "删除关系数据库实例", response = BocloudResult.class, httpMethod = "DELETE", produces = MediaType.APPLICATION_JSON_VALUE)
    @DeleteMapping("/{id}")
    public BocloudResult remove(@PathVariable(value = Common.ID) Long id,
                                @Value(Common.REQ_CONTEXT) RequestContext context, HttpServletRequest request) {
        long start = System.currentTimeMillis();
        BocloudResult result = rdsDbService.remove(id, context);
        if (null != result.getData()) {
            RdsLog log = new RdsLog("数据库实例", "删除关系数据库实例", JSONObject.toJSONString(result), null, IpTool.getIP(request),
                    new Date(), System.currentTimeMillis() - start, (Long) result.getData(), context.getTarget());
            rdsService.saveLog(log);
        }
        return result;
    }

    /**
     * 详情
     *
     * @param id
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "查看关系数据库实例详情", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/{id}")
    public BocloudResult<RdsDb> detail(@PathVariable(value = Common.ID) Long id) {
        return rdsDbService.detail(id);
    }

}
