package com.bocloud.pms.booter.controller;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.GridBean;
import com.bocloud.common.model.Pager;
import com.bocloud.common.model.RequestContext;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.ResultTools;
import com.bocloud.pms.entity.RdsLog;
import com.bocloud.pms.entity.RdsUser;
import com.bocloud.pms.interfaces.middleware.RdsService;
import com.bocloud.pms.interfaces.middleware.RdsUserService;
import com.bocloud.service.utils.IpTool;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.Date;

@Api(value = "关系数据库", tags = "关系数据库")
@RestController
@RequestMapping("/v1/rds/users")
public class RdsUserController {

    @Autowired
    private RdsUserService rdsUserService;
    @Autowired
    private RdsService rdsService;
    private final String key = "数据库用户";

    /**
     * 列表
     *
     * @param pager
     * @param context
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "查询关系数据库用户列表", response = BocloudResult.class, httpMethod = "GET",
            produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping
    public BocloudResult<GridBean<RdsUser>> list(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return rdsUserService.list(pager, context);
    }

    /**
     * 添加
     *
     * @param rdsUser
     * @param context
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "添加关系数据库用户", response = BocloudResult.class, httpMethod = "POST",
            produces = MediaType.APPLICATION_JSON_VALUE)
    @PostMapping
    public BocloudResult create(@RequestBody RdsUser rdsUser, @Value(Common.REQ_CONTEXT) RequestContext context,
                                HttpServletRequest request) {
        if (null != rdsUser) {
            long start = System.currentTimeMillis();
            BocloudResult result = rdsUserService.create(rdsUser, context);
            if (null != result.getData()) {
                RdsLog log = new RdsLog(key, "创建用户", JSONObject.toJSONString(result), null, IpTool.getIP(request),
                        new Date(), System.currentTimeMillis() - start, (Long) result.getData(), context.getTarget());
                rdsService.saveLog(log);
            }
            return result;
        }
        return ResultTools.formatErrResult();
    }

    /**
     * 删除
     *
     * @param id
     * @param context
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "删除关系数据库用户", response = BocloudResult.class, httpMethod = "DELETE",
            produces = MediaType.APPLICATION_JSON_VALUE)
    @DeleteMapping("/{id}")
    public BocloudResult remove(@PathVariable(value = Common.ID) Long id,
                                @Value(Common.REQ_CONTEXT) RequestContext context, HttpServletRequest request) {
        long start = System.currentTimeMillis();
        BocloudResult result = rdsUserService.remove(id, context);
        if (null != result.getData()) {
            RdsLog log = new RdsLog(key, "删除用户", JSONObject.toJSONString(result), null, IpTool.getIP(request),
                    new Date(), System.currentTimeMillis() - start, (Long) result.getData(), context.getTarget());
            rdsService.saveLog(log);
        }
        return result;
    }

    /**
     * 详情
     *
     * @param id
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "查看关系数据库用户详情", response = BocloudResult.class, httpMethod = "GET",
            produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/{id}")
    public BocloudResult<RdsUser> detail(@PathVariable(value = Common.ID) Long id) {
        return rdsUserService.detail(id);
    }

    /**
     * 关系数据库用户密码重置操作
     *
     * @param id
     * @param rdsUser
     * @param context
     * @param request
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "关系数据库用户密码重置操作", response = BocloudResult.class, httpMethod = "PATCH",
            produces = MediaType.APPLICATION_JSON_VALUE)
    @PatchMapping(value = "/{id}/reset")
    public BocloudResult reset(@PathVariable(value = Common.ID) Long id, @RequestBody RdsUser rdsUser,
                               @Value(Common.REQ_CONTEXT) RequestContext context, HttpServletRequest request) {
        if (null != rdsUser) {
            long start = System.currentTimeMillis();
            BocloudResult result = rdsUserService.reset(rdsUser, context);
            if (null != result.getData()) {
                RdsLog log = new RdsLog(key, "重置用户密码", JSONObject.toJSONString(result), null, IpTool.getIP(request),
                        new Date(), System.currentTimeMillis() - start, (Long) result.getData(), context.getTarget());
                rdsService.saveLog(log);
            }
            return result;
        }
        return ResultTools.formatErrResult();
    }

    /**
     * 关系数据库用户授权操作
     *
     * @param id
     * @param rdsUser
     * @param context
     * @param request
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "关系数据库用户授权操作", response = BocloudResult.class, httpMethod = "PATCH",
            produces = MediaType.APPLICATION_JSON_VALUE)
    @PatchMapping(value = "/{id}/grant")
    public BocloudResult grant(@PathVariable(value = Common.ID) Long id, @RequestBody RdsUser rdsUser,
                               @Value(Common.REQ_CONTEXT) RequestContext context, HttpServletRequest request) {
        long start = System.currentTimeMillis();
        BocloudResult result = rdsUserService.grant(rdsUser, context);
        if (null != result.getData()) {
            RdsLog log = new RdsLog(key, "用户授权", JSONObject.toJSONString(result), null, IpTool.getIP(request),
                    new Date(), System.currentTimeMillis() - start, (Long) result.getData(), context.getTarget());
            rdsService.saveLog(log);
        }
        return result;
    }

    /**
     * 关系数据库用户权限回收操作
     *
     * @param id
     * @param context
     * @param request
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "关系数据库用户权限回收操作", response = BocloudResult.class, httpMethod = "PATCH",
            produces = MediaType.APPLICATION_JSON_VALUE)
    @PatchMapping(value = "/{id}/revoke")
    public BocloudResult revoke(@PathVariable(value = Common.ID) Long id,
                                @Value(Common.REQ_CONTEXT) RequestContext context, HttpServletRequest request) {
        long start = System.currentTimeMillis();
        BocloudResult result = rdsUserService.revoke(id, context);
        if (null != result.getData()) {
            RdsLog log = new RdsLog(key, "回收用户权限", JSONObject.toJSONString(result), null, IpTool.getIP(request),
                    new Date(), System.currentTimeMillis() - start, (Long) result.getData(), context.getTarget());
            rdsService.saveLog(log);
        }
        return result;
    }

    @ApiOperation(tags = {"CMC", "CSC"}, value = "关系数据库用户操作", response = BocloudResult.class, httpMethod = "PATCH",
            produces = MediaType.APPLICATION_JSON_VALUE)
    @PatchMapping(value = "/{id}")
    public BocloudResult action(@PathVariable(value = Common.ID) Long id, @RequestBody String content,
                                @Value(Common.REQ_CONTEXT) RequestContext context, HttpServletRequest request) {
        long start = System.currentTimeMillis();
        RdsUser rdsUser = JSONObject.parseObject(content, RdsUser.class);
        if ("reset".equals(rdsUser.getAction())) {
            BocloudResult result = rdsUserService.reset(rdsUser, context);
            if (null != result.getData()) {
                RdsLog log = new RdsLog(key, "重置用户密码", JSONObject.toJSONString(result), null, IpTool.getIP(request),
                        new Date(), System.currentTimeMillis() - start, (Long) result.getData(), context.getTarget());
                rdsService.saveLog(log);
            }
            return result;
        } else if ("grant".equals(rdsUser.getAction())) {
            BocloudResult result = rdsUserService.grant(rdsUser, context);
            if (null != result.getData()) {
                RdsLog log = new RdsLog(key, "用户授权", JSONObject.toJSONString(result), null, IpTool.getIP(request),
                        new Date(), System.currentTimeMillis() - start, (Long) result.getData(), context.getTarget());
                rdsService.saveLog(log);
            }
            return result;
        } else if ("revoke".equals(rdsUser.getAction())) {
            BocloudResult result = rdsUserService.revoke(id, context);
            if (null != result.getData()) {
                RdsLog log = new RdsLog(key, "回收用户权限", JSONObject.toJSONString(result), null, IpTool.getIP(request),
                        new Date(), System.currentTimeMillis() - start, (Long) result.getData(), context.getTarget());
                rdsService.saveLog(log);
            }
            return result;
        }
        return ResultTools.formatErrResult();
    }

}
