package com.bocloud.pms.booter.listener;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.utils.JSONTools;
import com.bocloud.pms.entity.resource.enums.ResourceCategory;
import com.bocloud.pms.interfaces.middleware.*;
import com.rabbitmq.client.Channel;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.amqp.core.Message;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

/**
 * 节点角色变更监听 Created by liuyuanyuan on 2020/4/28 10:35
 */
@Component("nodeRoleChangeListener")
public class NodeRoleChangeListener {
    private static Logger logger = LoggerFactory.getLogger(NodeRoleChangeListener.class);
    private static final String SECURITY_QUEUE = "software.node.event";
    @Autowired
    private KvsService kvsService;
    @Autowired
    private RdsService rdsService;
    @Autowired
    private CmsService cmsService;
    @Autowired
    private CcsService ccsService;
    @Autowired
    private LbsService lbsService;
    @Autowired
    private AssService assService;

    @RabbitListener(queues = SECURITY_QUEUE, admin = "rabbitAdmin", autoStartup = "true")
    public void process(Message message, Channel channel) throws Exception {
        channel.basicAck(message.getMessageProperties().getDeliveryTag(), false);
        String content = new String(message.getBody());
        processDebugLog("get message:{}", content);
        if (StringUtils.isBlank(content)) {
            // 消息没有内容，直接确认收到 淹没处理
            return;
        }
        JSONArray objects = JSONArray.parseArray(content);
        if (objects.isEmpty()) {
            // 消息参数格式错误，直接确认收到淹没处理
            return;
        }
        logger.info("接收服务节点变更信息：{}", objects.toJSONString());
        for (Object object : objects) {
            JSONObject data = JSONTools.isJSONObj(String.valueOf(object));
            switch (ResourceCategory.valueOf(data.getString("category"))) {
                case SqlServer:
                case PostgreSQL:
                case MySQL:
                case Oracle:
                    rdsService.synchronousRoleState(data);
                    break;
                case MongoDB:
                case Redis:
                case MemCache:
                    kvsService.synchronousRoleState(data);
                    break;
                case Kafka:
                case RabbitMQ:
                case ActiveMQ:
                case RocketMQ:
                    cmsService.synchronousRoleState(data);
                    break;
                case Zookeeper:
                case Etcd:
                    ccsService.synchronousRoleState(data);
                    break;
                case Nginx:
                    lbsService.synchronousRoleState(data);
                    break;
                case Tomcat:
                case Apache:
                case Weblogic:
                case Websphere:
                    assService.synchronousRoleState(data);
                    break;
                default:
                    break;
            }
        }
    }

    private void processDebugLog(String format, Object... objects) {
        if (logger.isDebugEnabled()) {
            logger.debug(format, objects);
        }
    }
}
