package com.bocloud.pms.booter.listener;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.model.Result;
import com.bocloud.pms.entity.resource.OrderMessage;
import com.bocloud.pms.entity.resource.enums.ResourceCategory;
import com.bocloud.pms.entity.resource.model.ApplicationParam;
import com.bocloud.pms.interfaces.ResourceOperationService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.List;

/**
 * 资源申请监听器
 *
 * @author wangyu
 * @version 1.0
 * @since 2018年1月8日
 */
@Component
public class ResourceApplyListener {

    private static Logger logger = LoggerFactory.getLogger(ResourceApplyListener.class);
    @Autowired
    private ResourceOperationService resourceOperationService;
    private static final String CREATION_QUEUE = "paas.resource.creation.queue";

    @RabbitListener(queues = CREATION_QUEUE, admin = "rabbitAdmin", autoStartup = "true")
    public void process(List<ApplicationParam> params) {
        try {
            for (ApplicationParam param : params) {
                if (null == param) {
                    return;
                }
                logger.info("接收到资源申请创建请求：{}", JSONObject.toJSONString(param));
                // 申请类别
                JSONObject configs = param.getConfigs();
                String category = configs.getString("category");
                // 当前申请类型是本服务需要处理的
                Result processResult = resourceOperationService.process(ResourceCategory.valueOf(category), param,
                        OrderMessage.Operation.Application);
                logger.info("资源申请处理结果：{}", processResult);
            }
        } catch (Exception e) {
            logger.error("Get error message:{}", e.getMessage());
        }
    }

}
