package com.bocloud.pms.booter.listener;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.pms.entity.resource.OrderMessage;
import com.bocloud.pms.entity.resource.enums.ResourceCategory;
import com.bocloud.pms.entity.resource.model.ExtensionParam;
import com.bocloud.pms.interfaces.ResourceOperationService;

/**
 * 资源延期监听器
 *
 * @author wangyu
 * @version 1.0
 * @since 2020年4月15日
 */
@Component
public class ResourceExtensionListener {

    private static Logger logger = LoggerFactory.getLogger(ResourceExtensionListener.class);
    @Autowired
    private ResourceOperationService resourceOperationService;
    private static final String EXTENSION_QUEUE = "paas.resource.extension.queue";

    @RabbitListener(queues = EXTENSION_QUEUE, admin = "rabbitAdmin", autoStartup = "true")
    public void process(ExtensionParam param) throws Exception {
        try {
            if (null == param) {
                return;
            }
            logger.info("接收到资源延期消息：{}", JSONObject.toJSONString(param));
            String category = param.getCategory();
            resourceOperationService.process(ResourceCategory.valueOf(category), param,
                OrderMessage.Operation.Extension);
        } catch (Exception e) {
            logger.error("Get error message:{}", e.getMessage());
        }
    }

}
