package com.bocloud.pms.booter.listener;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.pms.entity.resource.OrderMessage;
import com.bocloud.pms.entity.resource.enums.ResourceCategory;
import com.bocloud.pms.entity.resource.model.CancelationParam;
import com.bocloud.pms.interfaces.ResourceOperationService;

/**
 * 资源退订监听器
 *
 * @author wangyu
 * @version 1.0
 * @since 2018年1月8日
 */
@Component
public class ResourceUnsubscribeListener {

    private static Logger logger = LoggerFactory.getLogger(ResourceUnsubscribeListener.class);
    @Autowired
    private ResourceOperationService resourceOperationService;
    private static final String DELETION_QUEUE = "paas.resource.deletion.queue";

    @RabbitListener(queues = DELETION_QUEUE, admin = "rabbitAdmin", autoStartup = "true")
    public void process(CancelationParam param) throws Exception {
        try {
            if (null == param) {
                return;
            }
            logger.info("接收到资源退订消息：{}", JSONObject.toJSONString(param));
            String category = param.getCategory();
            resourceOperationService.process(ResourceCategory.valueOf(category), param,
                OrderMessage.Operation.Cancelation);
        } catch (Exception e) {
            logger.error("Get error message:{}", e.getMessage());
        }
    }

}
