package com.bocloud.pms.booter.listener;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.utils.JSONTools;
import com.bocloud.pms.entity.resource.enums.ResourceCategory;
import com.bocloud.pms.interfaces.middleware.CmsService;
import com.bocloud.pms.interfaces.middleware.KvsService;
import com.bocloud.pms.interfaces.middleware.RdsService;
import com.rabbitmq.client.Channel;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.amqp.core.Message;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

/**
 * 服务任务结果监听器
 *
 * @author weiwei
 */
@Component("serviceTaskResultListener")
public class ServiceTaskResultListener {

    private static Logger logger = LoggerFactory.getLogger(ServiceTaskResultListener.class);
    private static final String SECURITY_QUEUE = "service.task.result";
    @Autowired
    private KvsService kvsService;
    @Autowired
    private RdsService rdsService;
    @Autowired
    private CmsService cmsService;

    @RabbitListener(queues = SECURITY_QUEUE, admin = "rabbitAdmin", autoStartup = "true")
    public void process(Message message, Channel channel) throws Exception {
        channel.basicAck(message.getMessageProperties().getDeliveryTag(), false);
        String content = new String(message.getBody());
        processDebugLog("get message:{}", content);
        if (StringUtils.isBlank(content)) {
            // 消息没有内容，直接确认收到 淹没处理
            return;
        }
        JSONObject object = JSONTools.isJSONObj(content);
        if (null == object) {
            // 消息参数格式错误，直接确认收到淹没处理
            return;
        }
        logger.info("接收服务任务结果信息：" + object.toJSONString());
        JSONObject service = object.getJSONObject("service");
        if (null == service) {
            return;
        }
        switch (ResourceCategory.valueOf(service.getString("category"))) {
            case SqlServer:
            case PostgreSQL:
            case MySQL:
                rdsService.updateStatus(content);
                break;
            case MongoDB:
            case Redis:
                kvsService.updateStatus(content);
                break;
            case Kafka:
            case RabbitMQ:
                cmsService.updateStatus(content);
                break;
            default:
                break;
        }
        return;
    }

    private void processDebugLog(String format, Object... objects) {
        if (logger.isDebugEnabled()) {
            logger.debug(format, objects);
        }
    }

}
