package com.bocloud.pms.booter.listener;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.pms.entity.resource.TaskNode;
import com.bocloud.pms.interfaces.middleware.*;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.amqp.core.Message;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

/**
 * 软件安装监听器
 *
 * @author weiwei
 */
@Component("softInstallListener")
public class SoftInstallListener {

    @Autowired
    private RdsService rdsService;
    @Autowired
    private KvsService kvsService;
    @Autowired
    private CmsService cmsService;
    @Autowired
    private AssService assService;
    @Autowired
    private CcsService ccsService;
    @Autowired
    private LbsService lbsService;

    private static Logger logger = LoggerFactory.getLogger(SoftInstallListener.class);

    @RabbitListener(queues = "software.install.queue", admin = "rabbitAdmin", autoStartup = "true")
    public void process(Message message) throws Exception {
        String content = new String(message.getBody());
        processDebugLog("get bocloud message:{}", content);
        if (StringUtils.isBlank(content)) {
            // 消息没有内容，直接确认收到 淹没处理
            return;
        }
        logger.info("接收软件安装信息：" + content);
        TaskNode node = JSONObject.parseObject(content, TaskNode.class);
        if (null == node) {
            return;
        }
        switch (node.getCode()) {
            case "task.software.mysql":
            case "task.software.oracle":
            case "task.software.sqlserver":
            case "task.software.postgresql":
                rdsService.install(content, node.getContext());
                break;
            case "task.software.redis":
            case "task.software.mongodb":
            case "task.software.memcache":
                kvsService.install(content, node.getContext());
                break;
            case "task.software.rocketmq":
            case "task.software.activemq":
            case "task.software.rabbitmq":
            case "task.software.kafka":
                cmsService.install(content, node.getContext());
                break;
            case "task.software.nginx":
                lbsService.install(content, node.getContext());
                break;
            case "task.software.zookeeper":
            case "task.software.etcd":
                ccsService.install(content, node.getContext());
                break;
            case "task.software.tomcat":
            case "task.software.apache":
            case "task.software.weblogic":
            case "task.software.websphere":
                assService.install(content, node.getContext());
                break;
            default:
                break;
        }

    }

    private void processDebugLog(String format, Object... objects) {
        if (logger.isDebugEnabled()) {
            logger.debug(format, objects);
        }
    }

}
