package com.bocloud.pms.booter.listener;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.utils.JSONTools;
import com.bocloud.common.utils.ListTool;
import com.bocloud.common.utils.MapTools;
import com.bocloud.pms.entity.resource.Server;
import com.bocloud.pms.repository.resource.ServerRepository;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.amqp.core.AmqpTemplate;
import org.springframework.amqp.core.Message;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.Map;

/**
 * 任务接口监听器
 *
 * @author wangyu
 * @version 1.0
 * @since 2018年1月8日
 */
@Component("taskResultListener")
public class TaskResultListener {

    private static Logger logger = LoggerFactory.getLogger(TaskResultListener.class);
    @Autowired
    private ServerRepository serverRepository;
    @Autowired
    private AmqpTemplate amqpTemplate;

    @RabbitListener(queues = "task.result.queue", admin = "rabbitAdmin", autoStartup = "true")
    public void process(Message message) throws Exception {
        String content = new String(message.getBody());
        processDebugLog("get bocloud message:{}", content);
        if (StringUtils.isBlank(content)) {
            // 消息没有内容，直接确认收到 淹没处理
            return;
        }
        JSONObject object = JSONTools.isJSONObj(content);
        if (null == object) {
            // 消息参数格式错误，直接确认收到淹没处理
            return;
        }
        logger.info("接收任务结果信息：" + object.toJSONString());
        // 申请类别
        String service = object.getString("service");
        if ("NodeExporter".equals(service)) {
            JSONObject instance = object.getJSONObject("instance");
            if (null == instance || instance.isEmpty()) {
                return;
            }
            List<JSONObject> nodes = JSONArray.parseArray(instance.getString("nodes"), JSONObject.class);
            if (null == nodes || nodes.isEmpty()) {
                return;
            }
            for (JSONObject node : nodes) {
                if ("task.software.exporter".equals(node.getString("code"))) {
                    JSONObject input = JSONObject.parseObject(node.getString("input"));
                    if (null == input || null == input.getJSONObject("target")) {
                        continue;
                    }
                    List<JSONObject> datas = JSONArray.parseArray(input.getJSONObject("target").getString("data"),
                            JSONObject.class);
                    if (ListTool.isEmpty(datas)) {
                        continue;
                    }
                    Long id = datas.get(0).getLong("id");
                    if (null == id) {
                        continue;
                    }
                    Server server = serverRepository.query(id);
                    if (null == server) {
                        continue;
                    }
                    JSONObject output = JSONObject.parseObject(node.getString("output"));
                    boolean success = false;
                    if (null != output) {
                        success = (null == output.getBoolean("success") ? false : output.getBoolean("success"));
                    }
                    server.setExporterInstalled(success);
                    serverRepository.update(server);
                    if (success) {
                        Map<String, Object> map = MapTools.simpleMap("serverId", id);
                        JSONObject context = instance.getJSONObject("context");
                        if (null == context) {
                            context = instance.getJSONObject("props");
                        }
                        map.put("userId", context.getLong("target"));
                        amqpTemplate.convertAndSend("exporter.install.event", map);
                    }
                }
            }
        } else {
            // 当前申请消息是云管服务不需要处理的，直接拒绝
            logger.info("This message category [{}] should not process in this service, send it to mq again!", service);
            amqpTemplate.convertAndSend("task.result.queue", message);
        }
    }

    private void processDebugLog(String format, Object... objects) {
        if (logger.isDebugEnabled()) {
            logger.debug(format, objects);
        }
    }

}
