package com.bocloud.pms.entity.resource;

import com.bocloud.common.utils.DaySerializer;
import com.bocloud.entity.annotations.Column;
import com.bocloud.entity.annotations.IgnoreAll;
import com.bocloud.entity.annotations.PK;
import com.bocloud.entity.annotations.Table;
import com.bocloud.entity.bean.GenericEntity;
import com.bocloud.entity.meta.PKStrategy;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import org.springframework.format.annotation.DateTimeFormat;

import java.util.Date;
import java.util.List;

/**
 * 镜像实体类
 *
 * @author DZG
 * @since V1.0 2016年9月13日
 */
@Table("image")
public class Image extends GenericEntity {

	@PK(value = PKStrategy.AUTO)
	private Long id; // ID
	@Column("os_type")
	private String osType;// 镜像格式
	@Column("os_name")
	private String osName; // OS名称
	@Column("os_version")
	private String osVersion; // OS版本名称
	@Column("os_category")
	private String osCategory;// 操作系统
	@Column("size")
	private Double size;// 容量
	@Column("min_disk")
	private Long minDisk;// 作为系统的容量
	@Column("kernel_version")
	private String kernelVersion;// 内核版本
	@Column("vendor_id")
	private Long vendorId; // 云供应商ID
	@Column("vendor_type")
	private String vendorType;// 镜像类型
	@Column("image_uuid")
	private String imageUuid; // 镜像ID
	@Column("architecture")
	private String architecture; // 系统位数
	@Column("template_id")
	private Long templateId;// 模板ID
	@Column("catalog")
	private String catalog;// 分类
	@Column("icon_id")
	private Long iconId;
	@Column("file_path")
	private String filePath;// 本地上传路径
	@Column("qga_installed")
	private Boolean qgaInstalled;// 是否安装了hw_qemu_guest_agent
	@Column("store_id")
	private Long storeId;// 数据存储ID
	@Column("default_user")
	private String defaultUser;// 默认用户
	@Column("tenant_id")
	private Long tenantId;// 租户ID
	@Column("project_id")
	private Long projectId;
	@Column("region_id")
	private String regionId;
	@Column("region")
	private Long region;
	@Column("image_label")
	private String imageLabel;// 镜像标签
	@Column("image_type")
	private String imageType;
	@Column("iso_name")
	private String isoName;// 物理镜像的文件名
	@Column("azure_resource_group_name")
	private String azureResourceGroupName; // azure 资源组名称
	@IgnoreAll
	private String diskFormat;
	@IgnoreAll
	private CloudServer server;
	@IgnoreAll
	private String tenantName;// 租户名称
	@IgnoreAll
	private String projectName;// 项目名称
	@IgnoreAll
	private String projectVisibility;
	@IgnoreAll
	private String targetName;// 租户用户
	@IgnoreAll
	@JsonSerialize(using = DaySerializer.class)
	@DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
	private Date expiredTime;// 过期时间
	@Column("tenant_uuid")
	private String tenantUuid;// openstack底层租户id
	@IgnoreAll
	private String osTenantName;// OpenStack底层租户名称
	@IgnoreAll
	private Long serverId;// 云主机ID
	@IgnoreAll
	private String vendorName; // 云供应商名称
	@IgnoreAll
	private String icon;// 应用图标
	@IgnoreAll
	private String regionName; // 镜像所在区域名称
	@IgnoreAll
	private String instanceId;
	@IgnoreAll
	private List<Long> volumeIds;
	@IgnoreAll
	private List<Volume> volumes; // 镜像挂载的卷
	@IgnoreAll
	private String imageSource; // 判断镜像是通过哪种方式创建的，云主机制作镜像和上传镜像

	public Image() {
		super();
	}

	public Image(String name, String osType, String osVersion, Double size, String isoName, String status,
                 String remark) {
		super();
		this.setName(name);
		this.osType = osType;
		this.osVersion = osVersion;
		this.size = size;
		this.setStatus(status);
		this.isoName = isoName;
		this.setRemark(remark);
	}

	public Long getId() {
		return id;
	}

	public void setId(Long id) {
		this.id = id;
	}

	public String getOsType() {
		return osType;
	}

	public void setOsType(String osType) {
		this.osType = osType;
	}

	public String getOsName() {
		return osName;
	}

	public void setOsName(String osName) {
		this.osName = osName;
	}

	public String getOsVersion() {
		return osVersion;
	}

	public void setOsVersion(String osVersion) {
		this.osVersion = osVersion;
	}

	public String getOsCategory() {
		return osCategory;
	}

	public void setOsCategory(String osCategory) {
		this.osCategory = osCategory;
	}

	public Double getSize() {
		return size;
	}

	public void setSize(Double size) {
		this.size = size;
	}

	public Long getMinDisk() {
		return minDisk;
	}

	public void setMinDisk(Long minDisk) {
		this.minDisk = minDisk;
	}

	public String getKernelVersion() {
		return kernelVersion;
	}

	public void setKernelVersion(String kernelVersion) {
		this.kernelVersion = kernelVersion;
	}

	public Long getVendorId() {
		return vendorId;
	}

	public void setVendorId(Long vendorId) {
		this.vendorId = vendorId;
	}

	public String getVendorType() {
		return vendorType;
	}

	public void setVendorType(String vendorType) {
		this.vendorType = vendorType;
	}

	public String getImageUuid() {
		return imageUuid;
	}

	public void setImageUuid(String imageUuid) {
		this.imageUuid = imageUuid;
	}

	public String getArchitecture() {
		return architecture;
	}

	public void setArchitecture(String architecture) {
		this.architecture = architecture;
	}

	public Long getTemplateId() {
		return templateId;
	}

	public void setTemplateId(Long templateId) {
		this.templateId = templateId;
	}

	public String getCatalog() {
		return catalog;
	}

	public void setCatalog(String catalog) {
		this.catalog = catalog;
	}

	public Long getIconId() {
		return iconId;
	}

	public void setIconId(Long iconId) {
		this.iconId = iconId;
	}

	public String getFilePath() {
		return filePath;
	}

	public void setFilePath(String filePath) {
		this.filePath = filePath;
	}

	public Boolean getQgaInstalled() {
		return qgaInstalled;
	}

	public void setQgaInstalled(Boolean qgaInstalled) {
		this.qgaInstalled = qgaInstalled;
	}

	public Long getStoreId() {
		return storeId;
	}

	public void setStoreId(Long storeId) {
		this.storeId = storeId;
	}

	public String getDefaultUser() {
		return defaultUser;
	}

	public void setDefaultUser(String defaultUser) {
		this.defaultUser = defaultUser;
	}

	public Long getTenantId() {
		return tenantId;
	}

	public void setTenantId(Long tenantId) {
		this.tenantId = tenantId;
	}

	public String getRegionId() {
		return regionId;
	}

	public void setRegionId(String regionId) {
		this.regionId = regionId;
	}

	public Long getRegion() {
		return region;
	}

	public void setRegion(Long region) {
		this.region = region;
	}

	public String getImageLabel() {
		return imageLabel;
	}

	public void setImageLabel(String imageLabel) {
		this.imageLabel = imageLabel;
	}

	public CloudServer getServer() {
		return server;
	}

	public void setServer(CloudServer server) {
		this.server = server;
	}

	public String getTenantName() {
		return tenantName;
	}

	public void setTenantName(String tenantName) {
		this.tenantName = tenantName;
	}

	public String getProjectName() {
		return projectName;
	}

	public void setProjectName(String projectName) {
		this.projectName = projectName;
	}

	public String getTargetName() {
		return targetName;
	}

	public void setTargetName(String targetName) {
		this.targetName = targetName;
	}

	public String getIsoName() {
		return isoName;
	}

	public void setIsoName(String isoName) {
		this.isoName = isoName;
	}

	public Date getExpiredTime() {
		return expiredTime;
	}

	public void setExpiredTime(Date expiredTime) {
		this.expiredTime = expiredTime;
	}

	public Long getServerId() {
		return serverId;
	}

	public void setServerId(Long serverId) {
		this.serverId = serverId;
	}

	public String getVendorName() {
		return vendorName;
	}

	public void setVendorName(String vendorName) {
		this.vendorName = vendorName;
	}

	public String getIcon() {
		return icon;
	}

	public void setIcon(String icon) {
		this.icon = icon;
	}

	public String getRegionName() {
		return regionName;
	}

	public void setRegionName(String regionName) {
		this.regionName = regionName;
	}

	public void preSave(Long creatorId, Long menderId, Long vendorId, Long tenantId, Long projectId) {
		this.setCreatorId(creatorId);
		this.setMenderId(menderId);
		this.setVendorId(vendorId);
		this.setTenantId(tenantId);
		this.setProjectId(projectId);
	}

	public String getImageType() {
		return imageType;
	}

	public void setImageType(String imageType) {
		this.imageType = imageType;
	}

	public String getInstanceId() {
		return instanceId;
	}

	public void setInstanceId(String instanceId) {
		this.instanceId = instanceId;
	}

	public List<Long> getVolumeIds() {
		return volumeIds;
	}

	public void setVolumeIds(List<Long> volumeIds) {
		this.volumeIds = volumeIds;
	}

	public String getProjectVisibility() {
		return projectVisibility;
	}

	public void setProjectVisibility(String projectVisibility) {
		this.projectVisibility = projectVisibility;
	}

	public List<Volume> getVolumes() {
		return volumes;
	}

	public void setVolumes(List<Volume> volumes) {
		this.volumes = volumes;
	}

	public String getAzureResourceGroupName() {
		return azureResourceGroupName;
	}

	public void setAzureResourceGroupName(String azureResourceGroupName) {
		this.azureResourceGroupName = azureResourceGroupName;
	}

	public String getTenantUuid() {
		return tenantUuid;
	}

	public void setTenantUuid(String tenantUuid) {
		this.tenantUuid = tenantUuid;
	}

	public String getOsTenantName() {
		return osTenantName;
	}

	public void setOsTenantName(String osTenantName) {
		this.osTenantName = osTenantName;
	}

	public Long getProjectId() {
		return projectId;
	}

	public void setProjectId(Long projectId) {
		this.projectId = projectId;
	}

	public String getDiskFormat() {
		return diskFormat;
	}

	public void setDiskFormat(String diskFormat) {
		this.diskFormat = diskFormat;
	}

	public String getImageSource() {
		return imageSource;
	}

	public void setImageSource(String imageSource) {
		this.imageSource = imageSource;
	}
}
