package com.bocloud.pms.entity.resource;

import com.bocloud.common.utils.DateDeserializer;
import com.bocloud.common.utils.DateSerializer;
import com.bocloud.entity.annotations.*;
import com.bocloud.entity.bean.Generic;
import com.bocloud.entity.meta.PKStrategy;
import com.fasterxml.jackson.databind.annotation.JsonDeserialize;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.NoArgsConstructor;
import org.springframework.format.annotation.DateTimeFormat;

import java.util.Date;

/**
 * 
 * @description IP管理
 * @version 5.1.0
 * @author wangyu
 * @date: 2020-11-30
 *
 */
@Data
@EqualsAndHashCode(callSuper = true)
@NoArgsConstructor
@Table("ip")
public class Ip extends Generic {

	@PK(value = PKStrategy.AUTO)
	private Long id; // ID
	@Column("ip")
	private String ip;// IP地址
	@Column("gateway")
	private String gateway; // 网关
	@Column("dns")
	private String dns; // DNS
	@Column("mask")
	private String mask; // DNS
	@Column("vendor_id")
	private Long vendorId;// 平台ID
	@Column("pool_id")
	private Long poolId;// IP池ID
	@Column("resource_id")
	private String resourceId;// 资源ID
	@Column("target")
	private Long target;// 目标ID
	@Column("catalog")
	private String catalog;// 目标类型
	@Column("network_id")
	private Long networkId; // 网络ID
	@Column("subnet_id")
	private Long subnetId; // 子网id
	@Column("status")
	private IpStatus status;// IP状态
	@Column("remark")
	private String remark;//备注
	@IgnoreUpdate
	@Column(value = "gmt_create", nullable = false)
	@JsonSerialize(using = DateSerializer.class)
	@JsonDeserialize(using = DateDeserializer.class)
	@DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
	private Date gmtCreate;// 创建时间
	@IgnoreAll
	private String networkName;// 网络名称
	@IgnoreAll
	private String subnetName;// 子网名称
	@IgnoreAll
	private String vendorName;// 平台名称
	@IgnoreAll
	private String targetName;// ip绑定目标名称


	public Ip(String ip, String mask, String gateway, String dns, Long vendorId, String resourceId, Long poolId,
              IpStatus status) {
		super();
		this.ip = ip;
		this.mask = mask;
		this.gateway = gateway;
		this.dns = dns;
		this.vendorId = vendorId;
		this.resourceId = resourceId;
		this.poolId = poolId;
		this.status = status;
	}

	/**
	 * 
	 * @description IP状态枚举
	 * @version 5.1.0
	 * @author wangyu
	 * @date: 2020-11-30
	 *
	 */
	public enum IpStatus {
		free, // 空闲
		used, // 已使用
		locked// 已锁定
	}
}
