package com.bocloud.pms.entity.resource;

import com.bocloud.common.utils.DateDeserializer;
import com.bocloud.common.utils.DateSerializer;
import com.bocloud.entity.annotations.Column;
import com.bocloud.entity.annotations.IgnoreAll;
import com.bocloud.entity.annotations.PK;
import com.bocloud.entity.annotations.Table;
import com.bocloud.entity.bean.Generic;
import com.bocloud.entity.meta.PKStrategy;
import com.bocloud.pms.entity.resource.enums.OrderItemStatus;
import com.fasterxml.jackson.databind.annotation.JsonDeserialize;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import lombok.AllArgsConstructor;
import lombok.Getter;
import lombok.NoArgsConstructor;
import lombok.Setter;
import org.springframework.format.annotation.DateTimeFormat;

import java.math.BigDecimal;
import java.util.Date;
import java.util.List;

/**
 * 订单明细
 *
 * @author wangyu
 * @version 1.0
 * @since 2020年3月10日
 */
@NoArgsConstructor
@AllArgsConstructor
@Table("order_item")
public class OrderItem extends Generic {

    /**
     * id
     */
    @Setter
    @Getter
    @PK(PKStrategy.AUTO)
    private Long id;
    /**
     * 子订单编号
     */
    @Setter
    @Getter
    @Column("sn")
    private String sn;
    /**
     * 订单id
     */
    @Setter
    @Getter
    @Column("order_id")
    private Long orderId;
    /**
     * 订单类别(complex|single)
     */
    @Setter
    @Getter
    @Column("catalog")
    private Catalog catalog;
    /**
     * sku id
     */
    @Setter
    @Getter
    @Column("sku_id")
    private Long skuId;
    /**
     * 资源状态
     */
    @Setter
    @Getter
    @Column("status")
    private OrderItemStatus status;
    /**
     * 基础价格
     */
    @Setter
    @Getter
    @Column("basic_price")
    private BigDecimal basicPrice;
    /**
     * 最终价格
     */
    @Setter
    @Getter
    @Column("final_price")
    private BigDecimal finalPrice;
    /**
     * 订购时长信息 {"mode":"Hour|Month","month":12}
     */
    @Setter
    @Getter
    @Column("duration")
    private String duration;
    /**
     * 是否删除
     */
    @Setter
    @Getter
    @Column("is_deleted")
    private Boolean deleted;
    /**
     * 错误消息
     */
    @Setter
    @Getter
    @Column("error_msg")
    private String errorMsg;
    /**
     * 变更者ID
     */
    @Setter
    @Getter
    @Column("changer_id")
    private Long changerId;
    /**
     * 稽核者ID
     */
    @Setter
    @Getter
    @Column("auditor_id")
    private Long auditorId;
    /**
     * 稽核状态
     */
    @Setter
    @Getter
    @Column("audit_status")
    private String auditStatus;
    /**
     * 稽核时间
     */
    @Setter
    @Getter
    @JsonSerialize(using = DateSerializer.class)
    @JsonDeserialize(using = DateDeserializer.class)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @Column("audit_time")
    private Date auditTime;
    /**
     * 生效时间
     */
    @Setter
    @Getter
    @JsonSerialize(using = DateSerializer.class)
    @JsonDeserialize(using = DateDeserializer.class)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @Column("effective_time")
    private Date effectiveTime;
    /**
     * 失效时间
     */
    @Setter
    @Getter
    @JsonSerialize(using = DateSerializer.class)
    @JsonDeserialize(using = DateDeserializer.class)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @Column("expired_time")
    private Date expiredTime;
    /**
     * 回收站移出时间
     */
    @Setter
    @Getter
    @JsonSerialize(using = DateSerializer.class)
    @JsonDeserialize(using = DateDeserializer.class)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @Column("move_out_time")
    private Date moveOutTime;
    /**
     * 资源信息{"name":"name","uuid":"uuid","code":"ecs"}
     */
    @Setter
    @Getter
    @Column("resources")
    private String resources;
    /**
     * 规格信息
     * {"skuId":1,"priceId":1,"count":1,"ratio":0.8,"serviceCode":"serviceCode","specs":{"cpu":1,"memory":1,"disk":100}}
     */
    @Setter
    @Getter
    @Column("elements")
    private String elements;
    /**
     * 服务编码
     */
    @Setter
    @Getter
    @Column("service_code")
    private String serviceCode;
    /**
     * 配置信息
     */
    @Setter
    @Getter
    @Column("inventory")
    private String inventory;
    /**
     * 订单时间
     */
    @Setter
    @Getter
    @JsonSerialize(using = DateSerializer.class)
    @JsonDeserialize(using = DateDeserializer.class)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @IgnoreAll
    private Date orderTime;
    /**
     * 服务名称
     */
    @Setter
    @Getter
    @IgnoreAll
    private String serviceName;
    /**
     * 规格详情
     */
    @Setter
    @Getter
    @IgnoreAll
    private String elementNames;
    /**
     * 规格详情
     */
    @Setter
    @Getter
    @IgnoreAll
    private List<String> elementList;
    /**
     * 折扣列表
     */
    @Setter
    @Getter
    @IgnoreAll
    private List<String> discountList;
    @Setter
    @Getter
    @IgnoreAll
    private Long categoryId;

    @Setter
    @Getter
    @IgnoreAll
    private Long tenantId;
    @Setter
    @Getter
    @IgnoreAll
    private Long vendorId;
    // 系统盘类型
    @Setter
    @Getter
    @IgnoreAll
    private Long systemDiskCategoryId;

    /**
     * 订单类别
     *
     * @author wangyu
     * @version 1.0
     * @since 2020年4月13日
     */
    public enum Catalog {
        complex, // 复合
        single// 单一
    }

}
