package com.bocloud.pms.entity.resource;

import com.bocloud.common.model.RequestContext.Catalog;
import com.bocloud.common.utils.DateSerializer;
import com.bocloud.common.utils.DaySerializer;
import com.bocloud.entity.annotations.Column;
import com.bocloud.entity.annotations.IgnoreAll;
import com.bocloud.entity.annotations.PK;
import com.bocloud.entity.annotations.Table;
import com.bocloud.entity.bean.GenericEntity;
import com.bocloud.entity.meta.PKStrategy;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import org.springframework.format.annotation.DateTimeFormat;

import java.util.Date;

/**
 * 快照实体类
 *
 * @author dongkai
 */
@Table("snapshot")
public class Snapshot extends GenericEntity {

	@PK(value = PKStrategy.AUTO)
	private Long id; // ID
	@Column("snapshot_uuid")
	private String snapshotUuid; // 快照uuid
	@Column("snapshot_val")
	private String snapshotVal; // 快照value
	@Column("volume_uuid")
	private String volumeUuid;// 块设备uuid
	@Column("size")
	private Integer size; // 大小
	@Column("size_byte")
	private Long sizeByte; // 大小，byte
	@Column("rollback_status")
	private String rollbackStatus;// 回滚状态
	@Column("vendor_id")
	private Long vendorId; // 云供应商ID
	@Column("region")
	private Long region; // 镜像所在区域
	@Column("progress")
	private String progress;
	@Column("vm_id")
	private Long vmId;
	@Column("create_time")
	@JsonSerialize(using = DateSerializer.class)
	@DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
	private Date createTime; // 快照创建时间
	@Column("volume_id")
	private Long volumeId; // 块设备ID
	@Column("tenant_id")
	private Long tenantId;// 租户ID
	@Column("project_id")
	private Long projectId;// 租户ID
	@IgnoreAll
	private String vendorName; // 云供应商名称
	@IgnoreAll
	private String regionName; // 镜像所在区域名称
	@IgnoreAll
	private String vmName;// 所在云主机名称
	@IgnoreAll
	private String volumeName;// 块设备名称
	@IgnoreAll
	private String tenantName;// 所属租户
	@IgnoreAll
	private String projectVisibility;
	@IgnoreAll
	private String targetName;// 租户用户
	@IgnoreAll
	@JsonSerialize(using = DaySerializer.class)
	@DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
	private Date expiredTime;// 过期时间
	@IgnoreAll
	private Boolean bootVolume;// 所属卷是否为启动盘
	@IgnoreAll
	private String vendorType;// 平台类型
	@Column("region_id")
	private String regionId;
	@Column("zone")
	private String zone;
	@Column("azure_resource_group_name")
	private String azureResourceGroupName; // azure 资源组名称
	@Column("tenant_uuid")
	private String tenantUuid;// 底层租户id
	@IgnoreAll
	private String zoneName;
	@Column("allocated_size")
	private Double allocatedSize; // 已分配的容量
	@Column("pool_id")
	private Long poolId; // 快照所属存储池id
	@Column("snapshot_group_id")
	private Long snapshotGroupId; // 所属组快照id
	@Column("snapshot_group_name")
	@IgnoreAll
	private String snapshotGroupName; // 所属组快照id
	@Column("catalog")
	private Catalog catalog;// 账号类别
	@Column("pool_name")
	@IgnoreAll
	private String poolName;
	@IgnoreAll
	private String osTenantName;

	@Column("block_device_mapping")
	private String blockDeviceMapping;
	@Column("snapshot_type")
	private String snapshotType;
	@Column("key_path")
	private String keyPath;
	@Column("setting_key_path")
	private String settingKeyPath;
	@Column("storage_id")
	private Long storageId; // 所属存储设备id
	/**
	 * @return the id
	 */
	public Long getId() {
		return id;
	}

	/**
	 * @param id
	 *            the id to set
	 */
	public void setId(Long id) {
		this.id = id;
	}

	/**
	 * @return the snapshotUuid
	 */
	public String getSnapshotUuid() {
		return snapshotUuid;
	}

	/**
	 * @param snapshotUuid
	 *            the snapshotUuid to set
	 */
	public void setSnapshotUuid(String snapshotUuid) {
		this.snapshotUuid = snapshotUuid;
	}

	/**
	 * @return the snapshotVal
	 */
	public String getSnapshotVal() {
		return snapshotVal;
	}

	/**
	 * @param snapshotVal
	 *            the snapshotVal to set
	 */
	public void setSnapshotVal(String snapshotVal) {
		this.snapshotVal = snapshotVal;
	}

	/**
	 * @return the volumeUuid
	 */
	public String getVolumeUuid() {
		return volumeUuid;
	}

	/**
	 * @param volumeUuid
	 *            the volumeUuid to set
	 */
	public void setVolumeUuid(String volumeUuid) {
		this.volumeUuid = volumeUuid;
	}

	/**
	 * @return the size
	 */
	public Integer getSize() {
		return size;
	}

	/**
	 * @param size
	 *            the size to set
	 */
	public void setSize(Integer size) {
		this.size = size;
	}

	/**
	 * @return the rollbackStatus
	 */
	public String getRollbackStatus() {
		return rollbackStatus;
	}

	/**
	 * @param rollbackStatus
	 *            the rollbackStatus to set
	 */
	public void setRollbackStatus(String rollbackStatus) {
		this.rollbackStatus = rollbackStatus;
	}

	/**
	 * @return the vendorId
	 */
	public Long getVendorId() {
		return vendorId;
	}

	/**
	 * @param vendorId
	 *            the vendorId to set
	 */
	public void setVendorId(Long vendorId) {
		this.vendorId = vendorId;
	}

	/**
	 * @return the region
	 */
	public Long getRegion() {
		return region;
	}

	/**
	 * @param region
	 *            the region to set
	 */
	public void setRegion(Long region) {
		this.region = region;
	}

	/**
	 * @return the progress
	 */
	public String getProgress() {
		return progress;
	}

	/**
	 * @param progress
	 *            the progress to set
	 */
	public void setProgress(String progress) {
		this.progress = progress;
	}

	/**
	 * @return the vmId
	 */
	public Long getVmId() {
		return vmId;
	}

	/**
	 * @param vmId
	 *            the vmId to set
	 */
	public void setVmId(Long vmId) {
		this.vmId = vmId;
	}

	/**
	 * @return the createTime
	 */
	public Date getCreateTime() {
		return createTime;
	}

	/**
	 * @param createTime
	 *            the createTime to set
	 */
	public void setCreateTime(Date createTime) {
		this.createTime = createTime;
	}

	/**
	 * @return the volumeId
	 */
	public Long getVolumeId() {
		return volumeId;
	}

	/**
	 * @param volumeId
	 *            the volumeId to set
	 */
	public void setVolumeId(Long volumeId) {
		this.volumeId = volumeId;
	}

	/**
	 * @return the vendorName
	 */
	public String getVendorName() {
		return vendorName;
	}

	/**
	 * @param vendorName
	 *            the vendorName to set
	 */
	public void setVendorName(String vendorName) {
		this.vendorName = vendorName;
	}

	/**
	 * @return the regionName
	 */
	public String getRegionName() {
		return regionName;
	}

	/**
	 * @param regionName
	 *            the regionName to set
	 */
	public void setRegionName(String regionName) {
		this.regionName = regionName;
	}

	/**
	 * @return the vmName
	 */
	public String getVmName() {
		return vmName;
	}

	/**
	 * @param vmName
	 *            the vmName to set
	 */
	public void setVmName(String vmName) {
		this.vmName = vmName;
	}

	/**
	 * @return the volumeName
	 */
	public String getVolumeName() {
		return volumeName;
	}

	/**
	 * @param volumeName
	 *            the volumeName to set
	 */
	public void setVolumeName(String volumeName) {
		this.volumeName = volumeName;
	}

	/**
	 * @return the tenantId
	 */
	public Long getTenantId() {
		return tenantId;
	}

	/**
	 * @param tenantId
	 *            the tenantId to set
	 */
	public void setTenantId(Long tenantId) {
		this.tenantId = tenantId;
	}

	/**
	 * @return the tenantName
	 */
	public String getTenantName() {
		return tenantName;
	}

	/**
	 * @param tenantName
	 *            the tenantName to set
	 */
	public void setTenantName(String tenantName) {
		this.tenantName = tenantName;
	}

	/**
	 * @return the expiredTime
	 */
	public Date getExpiredTime() {
		return expiredTime;
	}

	/**
	 * @param expiredTime
	 *            the expiredTime to set
	 */
	public void setExpiredTime(Date expiredTime) {
		this.expiredTime = expiredTime;
	}

	public String getTargetName() {
		return targetName;
	}

	public void setTargetName(String targetName) {
		this.targetName = targetName;
	}

	/**
	 * @return the bootVolume
	 */
	public Boolean getBootVolume() {
		return bootVolume;
	}

	/**
	 * @param bootVolume
	 *            the bootVolume to set
	 */
	public void setBootVolume(Boolean bootVolume) {
		this.bootVolume = bootVolume;
	}

	public String getVendorType() {
		return vendorType;
	}

	public void setVendorType(String vendorType) {
		this.vendorType = vendorType;
	}

	public Snapshot() {
		super();
	}

	public Snapshot(String snapshotUuid, String volumeUuid, Integer size, Long vendorId, Long region) {
		this.snapshotUuid = snapshotUuid;
		this.volumeUuid = volumeUuid;
		this.size = size;
		this.vendorId = vendorId;
		this.region = region;
	}

	public void preSave(Long creatorId, Long menderId, Long vendorId, Long vmId, Long tenantId) {
		this.setCreatorId(creatorId);
		this.setMenderId(menderId);
		this.setVendorId(vendorId);
		this.setVmId(vmId);
		this.setTenantId(tenantId);
	}

	public String getRegionId() {
		return regionId;
	}

	public void setRegionId(String regionId) {
		this.regionId = regionId;
	}

	public String getZone() {
		return zone;
	}

	public void setZone(String zone) {
		this.zone = zone;
	}

	public String getZoneName() {
		return zoneName;
	}

	public void setZoneName(String zoneName) {
		this.zoneName = zoneName;
	}

	public String getProjectVisibility() {
		return projectVisibility;
	}

	public void setProjectVisibility(String projectVisibility) {
		this.projectVisibility = projectVisibility;
	}

	public String getAzureResourceGroupName() {
		return azureResourceGroupName;
	}

	public void setAzureResourceGroupName(String azureResourceGroupName) {
		this.azureResourceGroupName = azureResourceGroupName;
	}

	public String getTenantUuid() {
		return tenantUuid;
	}

	public void setTenantUuid(String tenantUuid) {
		this.tenantUuid = tenantUuid;
	}

	public Double getAllocatedSize() {
		return allocatedSize;
	}

	public void setAllocatedSize(Double allocatedSize) {
		this.allocatedSize = allocatedSize;
	}

	public Long getPoolId() {
		return poolId;
	}

	public void setPoolId(Long poolId) {
		this.poolId = poolId;
	}

	public Long getSnapshotGroupId() {
		return snapshotGroupId;
	}

	public void setSnapshotGroupId(Long snapshotGroupId) {
		this.snapshotGroupId = snapshotGroupId;
	}

	public String getSnapshotGroupName() {
		return snapshotGroupName;
	}

	public void setSnapshotGroupName(String snapshotGroupName) {
		this.snapshotGroupName = snapshotGroupName;
	}

	public String getPoolName() {
		return poolName;
	}

	public void setPoolName(String poolName) {
		this.poolName = poolName;
	}

	public Long getSizeByte() {
		return sizeByte;
	}

	public void setSizeByte(Long sizeByte) {
		this.sizeByte = sizeByte;
	}

	public String getOsTenantName() {
		return osTenantName;
	}

	public void setOsTenantName(String osTenantName) {
		this.osTenantName = osTenantName;
	}

	public Catalog getCatalog() {
		return catalog;
	}

	public void setCatalog(Catalog catalog) {
		this.catalog = catalog;
	}

	public void preSave(Long creatorId, Long menderId, Long vendorId, Long vmId, Long tenantId, Long projectId) {
		this.setCreatorId(creatorId);
		this.setMenderId(menderId);
		this.setVendorId(vendorId);
		this.setVmId(vmId);
		this.setTenantId(tenantId);
		this.setProjectId(projectId);
	}

	public Long getProjectId() {
		return projectId;
	}

	public void setProjectId(Long projectId) {
		this.projectId = projectId;
	}

	public String getBlockDeviceMapping() {
		return blockDeviceMapping;
	}

	public void setBlockDeviceMapping(String blockDeviceMapping) {
		this.blockDeviceMapping = blockDeviceMapping;
	}

	public String getSnapshotType() {
		return snapshotType;
	}

	public void setSnapshotType(String snapshotType) {
		this.snapshotType = snapshotType;
	}

	public String getKeyPath() {
		return keyPath;
	}

	public void setKeyPath(String keyPath) {
		this.keyPath = keyPath;
	}

	public String getSettingKeyPath() {
		return settingKeyPath;
	}

	public void setSettingKeyPath(String settingKeyPath) {
		this.settingKeyPath = settingKeyPath;
	}

	public Long getStorageId() {
		return storageId;
	}

	public void setStorageId(Long storageId) {
		this.storageId = storageId;
	}
}
