package com.bocloud.pms.entity.resource.model;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.enums.OSCategory;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.pms.entity.resource.NodeFormatException;
import org.apache.commons.lang3.StringUtils;

import java.util.List;

/**
 * Worker请求目标封装类
 *
 * @author dmw
 */
public class WorkerHost {

    private Long id;
    private String host;// 主机IP
    private String user;// 操作账号
    private String pasd;// 账号密码
    private Integer port;// SSH端口，默认22
    private String task;// 关联的任务名称
    private String rawName;// 主机名称
    private String category = OSCategory.LINUX.name();// 操作系统类型，默认Linux
    private String connection;

    public WorkerHost() {
        super();
    }

    public WorkerHost(String host, String connection) {
        super();
        this.host = host;
        this.connection = connection;
    }

    public WorkerHost(String host, String user, String pasd, Integer port, OSCategory category) {
        super();
        this.host = host;
        this.user = user;
        this.pasd = pasd;
        this.category = category.name();
        if (null == port || 0 >= port) {
            switch (category) {
                case WINDOWS:
                    this.port = 5986;
                    break;
                case LINUX:
                    this.port = 22;
                    break;
                case UNIX:
                    this.port = 22;
                    break;
                default:
                    this.port = 22;
                    break;
            }
        } else {
            this.port = port;
        }
    }

    public WorkerHost(Long id, String host, String user, String pasd, Integer port, OSCategory category) {
        super();
        this.id = id;
        this.host = host;
        this.user = user;
        this.pasd = pasd;
        this.category = category.name();
        if (null == port || 0 >= port) {
            switch (category) {
                case WINDOWS:
                    this.port = 5986;
                    break;
                case LINUX:
                    this.port = 22;
                    break;
                case UNIX:
                    this.port = 22;
                    break;
                default:
                    this.port = 22;
                    break;
            }
        } else {
            this.port = port;
        }
    }

    public static BocloudResult<WorkerHost> parseTarget(String content) {
        BocloudResult<WorkerHost> result = null;
        if (StringUtils.isEmpty(content)) {
            result = new BocloudResult<>(false, "参数为空！");
        } else {
            try {
                WorkerHost target = JSONObject.parseObject(content, WorkerHost.class);
                result = new BocloudResult<WorkerHost>(true, target, "success");
            } catch (Exception e) {
                result = new BocloudResult<WorkerHost>(false, e.getLocalizedMessage());
            }
        }
        return result;
    }

    public static List<WorkerHost> parseTargets(String content) throws NodeFormatException {
        if (StringUtils.isEmpty(content)) {
            throw new NodeFormatException("参数为空！");
        } else {
            try {
                List<WorkerHost> targets = JSONArray.parseArray(content, WorkerHost.class);
                return targets;
            } catch (Exception e) {
                throw new NodeFormatException("目标机器参数类型错误！", e);
            }
        }
    }

    /**
     * @return the id
     */
    public Long getId() {
        return id;
    }

    /**
     * @param id the id to set
     */
    public void setId(Long id) {
        this.id = id;
    }

    public String getHost() {
        return host;
    }

    public void setHost(String host) {
        this.host = host;
    }

    public String getUser() {
        return user;
    }

    public void setUser(String user) {
        this.user = user;
    }

    public String getPasd() {
        return pasd;
    }

    public void setPasd(String pasd) {
        this.pasd = pasd;
    }

    public Integer getPort() {
        return port;
    }

    public void setPort(Integer port) {
        this.port = port;
    }

    public String getCategory() {
        return category;
    }

    public void setCategory(String category) {
        this.category = category;
    }

    /**
     * @return the rawName
     */
    public String getRawName() {
        return rawName;
    }

    /**
     * @param rawName the rawName to set
     */
    public void setRawName(String rawName) {
        this.rawName = rawName;
    }

    public String getTask() {
        return task;
    }

    public void setTask(String task) {
        this.task = task;
    }

    public String getConnection() {
        return connection;
    }

    public void setConnection(String connection) {
        this.connection = connection;
    }
}
