package com.bocloud.pms.interfaces;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Param;
import com.bocloud.common.model.RequestContext;
import com.bocloud.pms.entity.resource.Ip;

import java.util.List;
import java.util.Map;

public interface IpService {

	/**
	 * 列表展示
	 * 
	 * @param page
	 * @param rows
	 * @param params
	 * @param sorter
	 * @param simple
	 * @param condition
	 * @param context
	 * @return
	 */
	public BocloudResult list(Integer page, Integer rows, List<Param> params, Map<String, String> sorter,
			boolean simple, String condition, RequestContext context);

	/**
	 * 详情
	 *
	 * @param id
	 * @return
	 */
	public BocloudResult detail(Long id);

	/**
	 * 根据平台ID以及请求IP数量获取IP
	 *
	 * @param vendorId
	 * @param count
	 * @param version
	 * @return
	 */
	public BocloudResult<List<Ip>> achieveIps(Long vendorId, Long count, String version);

	/**
	 * 从指定IP池分配IP
	 * 
	 * @param poolId
	 * @param count
	 * @param version
	 * @return
	 */
	public BocloudResult<List<Ip>> achievePoolIps(Long poolId, Long count, String version);

	/**
	 * 根据IP池ID以及请求IP数量获取IP(随机分配，不按照顺序分配)
	 * 
	 * @param poolId
	 * @param count
	 * @param version
	 * @return
	 */
	public BocloudResult<List<Ip>> achieveDhcpIps(Long poolId, Long count, String version);

	/**
	 * 处理平台垃圾下的IP垃圾数据
	 *
	 * @param vendorId
	 * @return
	 */
	public BocloudResult processRubbishIp(Long vendorId);

	/**
	 * 从网络资源池下的IP池中分配IP地址
	 *
	 * @param vendorId
	 * @param poolGroupId
	 * @param count
	 * @return
	 */
	public BocloudResult<List<Ip>> achieveIps(Long vendorId, Long poolGroupId, Integer count);

	/**
	 * 从nsx路由器端口下子网IP池中分配
	 * 
	 * @param routerPortId
	 * @param count
	 * @param version
	 * @return
	 */
	public BocloudResult<List<Ip>> achieveNsxIps(Long routerPortId, Integer count, String version);

	public BocloudResult alarmIps(Long vendorId);

	/**
	 * 根据子网id删除
	 * 
	 * @param ip
	 * @param subnetId
	 * @return
	 */
	public BocloudResult deleteBySubnetId(String ip, Long subnetId);

	/**
	 * 批量保存
	 * 
	 * @param object
	 * @param context
	 * @return
	 */
	BocloudResult batchSave(JSONObject object, RequestContext context);

	/**
	 * 删除IP
	 * 
	 * @param id
	 * @return
	 */
	BocloudResult remove(Long id);

	/**
	 * ip扫描
	 * 
	 * @param params
	 * @param context
	 * @return
	 */
	BocloudResult scan(String params, RequestContext context);

	/**
	 * 预分配IP地址
	 * 
	 * @param poolId
	 * @param count
	 * @param context
	 * @return
	 */
	BocloudResult preAllocation(Long poolId, Integer count, RequestContext context);

	/**
	 * 检查IP是否被占用
	 * 
	 * @param poolId
	 * @param ips
	 * @param context
	 * @return
	 */
	BocloudResult checkIp(Long poolId, List<String> ips, RequestContext context);
}
