package com.bocloud.pms.interfaces.middleware;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.GridBean;
import com.bocloud.common.model.Pager;
import com.bocloud.common.model.RequestContext;
import com.bocloud.pms.entity.Cms;
import com.bocloud.pms.entity.CmsLog;
import com.bocloud.pms.entity.resource.OrderMessage;
import com.bocloud.pms.entity.resource.enums.ResourceCategory;

public interface CmsService {

    /**
     * 列表展示
     *
     * @param pager
     * @param context
     * @return
     */
    public BocloudResult<GridBean<Cms>> list(Pager pager, RequestContext context);

    /**
     * 申请
     *
     * @param params
     * @param category
     * @param operation
     * @param context
     * @return
     */
    public BocloudResult apply(Object param, ResourceCategory category, OrderMessage.Operation operation,
        RequestContext context);

    /**
     * 详情
     *
     * @param id
     * @return
     */
    public BocloudResult<Cms> detail(Long id);

    /**
     * 基础信息
     *
     * @param id
     * @return
     */
    public BocloudResult<Cms> basics(Long id);

    /**
     * 删除
     *
     * @param id
     * @param context
     * @return
     */
    public BocloudResult remove(Long id, RequestContext context);

    public void updateStatus(String content);

    /**
     * 备份
     *
     * @param id
     * @param context
     * @return
     */
    public BocloudResult backup(Long id, RequestContext context);

    /**
     * 删除备份
     *
     * @param snapshotId
     * @param context
     * @return
     */
    public BocloudResult removeBackup(Long snapshotId, RequestContext context);

    /**
     * 恢复
     *
     * @param id
     * @param snapshotId
     * @param context
     * @return
     */
    public BocloudResult revert(Long id, Long snapshotId, RequestContext context);

    /**
     * 操作日志列表
     *
     * @param pager
     * @param context
     * @return
     */
    public BocloudResult<GridBean<CmsLog>> logs(Pager pager, RequestContext context);

    /**
     * 备份列表
     *
     * @param id
     * @param context
     * @return
     */
    public BocloudResult backups(Long id, RequestContext context);

    /**
     * 安全列表
     *
     * @param id
     * @param context
     * @return
     */
    public BocloudResult securities(Long id, RequestContext context);

    /**
     * 保存日志
     *
     * @param log
     */
    public void saveLog(CmsLog log);

    /**
     * 同步角色状态
     *
     * @param content
     */
    public void synchronousRoleState(JSONObject content);

    /**
     * 记录安装
     *
     * @param params
     * @param context
     * @return
     */
    public void install(String params, RequestContext context);

    /**
     * 加入回收站
     *
     * @param id
     * @param context
     * @return
     */
    public BocloudResult recycle(Long id, RequestContext context);

    /**
     * 回收站还原
     *
     * @param id
     * @param context
     * @return
     */
    public BocloudResult restore(Long id, RequestContext context);

}
