package com.bocloud.pms.interfaces.middleware;

import java.util.List;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.GridBean;
import com.bocloud.common.model.Pager;
import com.bocloud.common.model.RequestContext;
import com.bocloud.pms.entity.Kvs;
import com.bocloud.pms.entity.KvsLog;
import com.bocloud.pms.entity.resource.OrderMessage;
import com.bocloud.pms.entity.resource.enums.ResourceCategory;

public interface KvsService {

    /**
     * 列表展示
     *
     * @param pager
     * @param context
     * @return
     */
    public BocloudResult<GridBean<Kvs>> list(Pager pager, RequestContext context);

    /**
     * 申请
     *
     * @param params
     * @param category
     * @param operation
     * @param context
     * @return
     */
    public BocloudResult apply(Object param, ResourceCategory category, OrderMessage.Operation operation,
        RequestContext context);

    /**
     * 详情
     *
     * @param id
     * @return
     */
    public BocloudResult<Kvs> detail(Long id);

    /**
     * 基础信息
     *
     * @param id
     * @return
     */
    public BocloudResult<Kvs> basics(Long id);

    /**
     * 删除
     *
     * @param id
     * @param context
     * @return
     */
    public BocloudResult remove(Long id, RequestContext context);

    public void updateStatus(String content);

    /**
     * 备份
     *
     * @param id
     * @param context
     * @return
     */
    public BocloudResult backup(Long id, RequestContext context);

    /**
     * 删除备份
     *
     * @param snapshotId
     * @param context
     * @return
     */
    public BocloudResult removeBackup(Long snapshotId, RequestContext context);

    /**
     * 恢复
     *
     * @param id
     * @param snapshotId
     * @param context
     * @return
     */
    public BocloudResult revert(Long id, Long snapshotId, RequestContext context);

    /**
     * 操作日志列表
     *
     * @param pager
     * @param context
     * @return
     */
    public BocloudResult<GridBean<KvsLog>> logs(Pager pager, RequestContext context);

    /**
     * 备份列表
     *
     * @param id
     * @param context
     * @return
     */
    public BocloudResult backups(Long id, RequestContext context);

    /**
     * 安全列表
     *
     * @param id
     * @param context
     * @return
     */
    public BocloudResult securities(Long id, RequestContext context);

    /**
     * 参数列表
     *
     * @param id
     * @param context
     * @return
     */
    public BocloudResult<List<JSONObject>> parameters(Long id, RequestContext context);

    /**
     * 参数设置
     *
     * @param name
     * @param oldValue
     * @param value
     * @param id
     * @param type
     * @param context
     * @return
     */
    public BocloudResult setting(String name, String oldValue, String value, Long id, String type,
        RequestContext context);

    /**
     * 参数修改日志
     *
     * @param pager
     * @param context
     * @return
     */
    public BocloudResult<GridBean> paramLogs(Pager pager, RequestContext context);

    /**
     * 保存日志
     *
     * @param log
     */
    public void saveLog(KvsLog log);

    /**
     * 修改密码
     *
     * @param id
     * @param password
     * @param newPassword
     * @param requestIp
     * @param context
     * @return
     */
    public BocloudResult reset(Long id, String password, String newPassword, String requestIp, RequestContext context);

    /**
     * 同步角色状态
     *
     * @param content
     */
    public void synchronousRoleState(JSONObject content);

    /**
     * 记录安装
     *
     * @param params
     * @param context
     * @return
     */
    public void install(String params, RequestContext context);

    /**
     * 加入回收站
     *
     * @param id
     * @param context
     * @return
     */
    public BocloudResult recycle(Long id, RequestContext context);

    /**
     * 回收站还原
     *
     * @param id
     * @param context
     * @return
     */
    public BocloudResult restore(Long id, RequestContext context);

}
