package com.bocloud.pms.repository;

import java.util.Date;
import java.util.List;
import java.util.Map;

import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import com.bocloud.common.model.Pager;
import com.bocloud.common.model.Param;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.MapTools;
import com.bocloud.database.core.intf.impl.BasicGenericDao;
import com.bocloud.database.utils.QueryBuilder;
import com.bocloud.pms.entity.Ass;

/**
 * 应用服务Repository接口实现类
 *
 * @author weiwei
 */
@Repository("assRepository")
public class AssRepository extends BasicGenericDao<Ass, Long> {

    public AssRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate) {
        super(jdbcTemplate, npJdbcTemplate);
    }

    public List<Ass> list(int page, int rows, List<Param> params, Map<String, String> sorter) throws Exception {
        String sql =
            "select a.*,c.name category_name from ass a left join service_category c on a.category_id = c.id where a.is_deleted = 0";
        sql = QueryBuilder.buildRaw(sql, new Pager(page, rows, params, sorter), "a");
        return this.list(Ass.class, sql);
    }

    public int count(List<Param> params) throws Exception {
        String sql = "select count(*) from ass a where a.is_deleted = 0";
        sql = QueryBuilder.buildRaw(sql, params, null, "a");
        Map<String, Object> param = QueryBuilder.getParam(params);
        return this.countQuery(sql, param).intValue();
    }

    public boolean remove(Long id, Long userId) throws Exception {
        String sql = "update ass set is_deleted = 1, gmt_modify = :gmtModify, mender_id = :menderId where id = :id";
        Map<String, Object> params = MapTools.simpleMap("id", id);
        params.put("gmtModify", new Date());
        params.put("menderId", userId);
        return this.execute(sql, params) > 0;
    }

    public Ass query(Long id) throws Exception {
        String sql = "select a.*,c.name category_name from ass a left join service_category c on a.category_id = c.id"
            + " where a.is_deleted = 0 and a.id = :id";
        Map<String, Object> paramMap = MapTools.simpleMap("id", id);
        List<Ass> list = this.list(Ass.class, sql, paramMap);
        return list.isEmpty() ? null : list.get(0);
    }

    public boolean updateStatus(Long id, String status, Long userId) throws Exception {
        String sql = "update ass set status = :status, gmt_modify = :gmtModify, mender_id = :menderId where id = :id";
        Map<String, Object> params = MapTools.simpleMap("id", id);
        params.put("gmtModify", new Date());
        params.put("menderId", userId);
        params.put("status", status);
        return this.execute(sql, params) > 0;
    }

    public Ass get(String uuid) throws Exception {
        String sql = "select a.*,c.name category_name from ass a"
            + " left join service_category c on a.category_id = c.id where a.is_deleted = 0 and a.uuid = :uuid";
        Map<String, Object> params = MapTools.simpleMap(Common.UUID, uuid);
        List<Ass> list = this.list(Ass.class, sql, params);
        return list.isEmpty() ? null : list.get(0);
    }

}
