package com.bocloud.pms.repository;

import com.bocloud.common.model.Pager;
import com.bocloud.common.model.Param;
import com.bocloud.common.utils.MapTools;
import com.bocloud.database.core.intf.impl.BasicGenericDao;
import com.bocloud.database.utils.QueryBuilder;
import com.bocloud.pms.entity.CmsTopic;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * Created by liuyuanyuan on 2020/3/12 18:01
 */
@Repository("cmsTopicRepository")
public class CmsTopicRepository extends BasicGenericDao<CmsTopic, Long> {

    public CmsTopicRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate) {
        super(jdbcTemplate, npJdbcTemplate);
    }

    public List<CmsTopic> list(Long mqId, int page, int rows, List<Param> params, Map<String, String> sorter)
            throws Exception {
        String sql = "select a.* from cms_topic a where a.mq_id = :mqId and a.is_deleted = 0";
        sql = QueryBuilder.buildRaw(sql, new Pager(page, rows, params, sorter), "a");
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        paramMap.put("mqId", mqId);
        return this.list(CmsTopic.class, sql, paramMap);
    }

    public int count(List<Param> params) throws Exception {
        String sql = "select count(*) from cms_topic a where a.is_deleted = 0";
        sql = QueryBuilder.buildRaw(sql, params, null, "");
        Map<String, Object> param = QueryBuilder.getParam(params);
        return this.countQuery(sql, param).intValue();
    }

    public boolean remove(Long id, Long userId) throws Exception {
        String sql = "update cms_topic set is_deleted = 1, gmt_modify = :gmtModify, mender_id = :menderId where id = :id";
        Map<String, Object> params = MapTools.simpleMap("id", id);
        params.put("gmtModify", new Date());
        params.put("menderId", userId);
        return this.execute(sql, params) > 0;
    }

    public CmsTopic query(Long id) throws Exception {
        String sql = "select a.* from cms_topic a where a.is_deleted = 0 and a.id = :id";
        List<CmsTopic> list = this.list(CmsTopic.class, sql, MapTools.simpleMap("id", id));
        return list.isEmpty() ? null : list.get(0);
    }

    public boolean deleteByMq(Long cmsId) throws Exception {
        String sql = "update cms_topic set is_deleted = 1, gmt_modify = :gmtModify where mq_id = :id";
        Map<String, Object> params = MapTools.simpleMap("id", cmsId);
        params.put("gmtModify", new Date());
        return this.execute(sql, params) > 0;
    }
}
