package com.bocloud.pms.repository;

import com.bocloud.common.model.Pager;
import com.bocloud.common.model.Param;
import com.bocloud.common.utils.MapTools;
import com.bocloud.database.core.intf.impl.BasicGenericDao;
import com.bocloud.database.utils.QueryBuilder;
import com.bocloud.pms.entity.CmsUser;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * 缓存用户Repository接口实现
 *
 * @author weiwei
 */
@Repository("cmsUserRepository")
public class CmsUserRepository extends BasicGenericDao<CmsUser, Long> {

	public CmsUserRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate) {
		super(jdbcTemplate, npJdbcTemplate);
	}

	public List<CmsUser> list(int page, int rows, List<Param> params, Map<String, String> sorter) throws Exception {
		String sql = "select a.* from cms_user a where a.is_deleted = 0";
		sql = QueryBuilder.buildRaw(sql, new Pager(page, rows, params, sorter), "a");
		Map<String, Object> paramMap = QueryBuilder.getParam(params);
		return this.list(CmsUser.class, sql, paramMap);
	}

	public int count(List<Param> params) throws Exception {
		String sql = "select count(*) from cms_user a where a.is_deleted = 0";
		sql = QueryBuilder.buildRaw(sql, params, null, "");
		Map<String, Object> param = QueryBuilder.getParam(params);
		return this.countQuery(sql, param).intValue();
	}

	public boolean remove(Long id, Long userId) throws Exception {
		String sql = "update cms_user set is_deleted = 1, gmt_modify = :gmtModify, mender_id = :menderId where id = :id";
		Map<String, Object> params = MapTools.simpleMap("id", id);
		params.put("gmtModify", new Date());
		params.put("menderId", userId);
		return this.execute(sql, params) > 0;
	}

	public CmsUser query(Long id) throws Exception {
		String sql = "select a.* from cms_user a where a.is_deleted = 0 and a.id = :id";
		List<CmsUser> list = this.list(CmsUser.class, sql, MapTools.simpleMap("id", id));
		return list.isEmpty() ? null : list.get(0);
	}

	public List<CmsUser> list(Long mqId) throws Exception {
		String sql = "select * from cms_user where is_deleted = 0 and mq_id = :mqId";
		return this.list(CmsUser.class, sql, MapTools.simpleMap("mqId", mqId));
	}

	public boolean deleteByMq(Long mqId) throws Exception {
		String sql = "delete from cms_user where mq_id = :mqId";
		return this.execute(sql, MapTools.simpleMap("mqId", mqId)) > 0;
	}

}
