package com.bocloud.pms.repository;

import com.bocloud.common.utils.MapTools;
import com.bocloud.database.core.intf.impl.BasicGenericDao;
import com.bocloud.pms.entity.KvsNode;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.List;
import java.util.Map;

/**
 * 缓存节点Repository接口实现
 *
 * @author weiwei
 */
@Repository("kvsNodeRepository")
public class KvsNodeRepository extends BasicGenericDao<KvsNode, Long> {

    public KvsNodeRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate) {
        super(jdbcTemplate, npJdbcTemplate);
    }

    public List<KvsNode> list(Long kvsId) throws Exception {
        String sql = "select * from kvs_node where kvs_id = :kvsId";
        Map<String, Object> params = MapTools.simpleMap("kvsId", kvsId);
        return this.list(KvsNode.class, sql, params);
    }

    public boolean deleteByKvs(Long kvsId) throws Exception {
        String sql = "delete from kvs_node where kvs_id = :kvsId";
        Map<String, Object> params = MapTools.simpleMap("kvsId", kvsId);
        return this.execute(sql, params) > 0;
    }

    public KvsNode query(Long id) throws Exception {
        String sql = "select * from kvs_node where id = :id";
        Map<String, Object> params = MapTools.simpleMap("id", id);
        List<KvsNode> list = this.list(KvsNode.class, sql, params);
        if (list.isEmpty()) {
            return null;
        }
        return list.get(0);
    }

    public boolean batchUpdate(List<KvsNode> kvsNodes) throws Exception {
        return this.batchUpdate(kvsNodes, kvsNodes.size());
    }
}
