package com.bocloud.pms.repository;

import com.bocloud.common.model.Pager;
import com.bocloud.common.model.Param;
import com.bocloud.common.utils.MapTools;
import com.bocloud.database.core.intf.impl.BasicGenericDao;
import com.bocloud.database.utils.QueryBuilder;
import com.bocloud.pms.entity.KvsUser;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * 缓存用户Repository接口实现
 *
 * @author weiwei
 */
@Repository("kvsUserRepository")
public class KvsUserRepository extends BasicGenericDao<KvsUser, Long> {

    public KvsUserRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate) {
        super(jdbcTemplate, npJdbcTemplate);
    }

    public List<KvsUser> list(int page, int rows, List<Param> params, Map<String, String> sorter) throws Exception {
        String sql = "select a.* from kvs_user a where a.is_deleted = 0";
        sql = QueryBuilder.buildRaw(sql, new Pager(page, rows, params, sorter), "a");
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        return this.list(KvsUser.class, sql, paramMap);
    }

    public int count(List<Param> params) throws Exception {
        String sql = "select count(*) from kvs_user a where a.is_deleted = 0";
        sql = QueryBuilder.buildRaw(sql, params, null, "");
        Map<String, Object> param = QueryBuilder.getParam(params);
        return this.countQuery(sql, param).intValue();
    }

    public boolean remove(Long id, Long userId) throws Exception {
        String sql = "delete from kvs_user where id = :id";
        Map<String, Object> params = MapTools.simpleMap("id", id);
        return this.execute(sql, params) > 0;
    }

    public KvsUser query(Long id) throws Exception {
        String sql = "select a.* from kvs_user a where a.id = :id";
        Map<String, Object> params = MapTools.simpleMap("id", id);
        List<KvsUser> list = this.list(KvsUser.class, sql, params);
        if (list.isEmpty()) {
            return null;
        }
        return list.get(0);
    }

    public List<KvsUser> list(Long kvsId) throws Exception {
        String sql = "select * from kvs_user where kvs_id = :kvsId";
        Map<String, Object> params = MapTools.simpleMap("kvsId", kvsId);
        List<KvsUser> list = this.list(KvsUser.class, sql, params);
        List<KvsUser> users = new ArrayList<>();
        for (Object object : list) {
            users.add((KvsUser) object);
        }
        return users;
    }

    public boolean deleteByKvs(Long kvsId) throws Exception {
        String sql = "delete from kvs_user where kvs_id = :kvsId";
        return this.execute(sql, MapTools.simpleMap("kvsId", kvsId)) > 0;
    }

}
