package com.bocloud.pms.repository;

import java.util.Date;
import java.util.List;
import java.util.Map;

import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import com.bocloud.common.model.Pager;
import com.bocloud.common.model.Param;
import com.bocloud.common.utils.MapTools;
import com.bocloud.database.core.intf.impl.BasicGenericDao;
import com.bocloud.database.utils.QueryBuilder;
import com.bocloud.pms.entity.RdsUser;

/**
 * 数据库用户Repository接口实现
 *
 * @author weiwei
 */
@Repository("rdsUserRepository")
public class RdsUserRepository extends BasicGenericDao<RdsUser, Long> {

    public RdsUserRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate) {
        super(jdbcTemplate, npJdbcTemplate);
    }

    public List<RdsUser> list(Pager pager) throws Exception {
        String sql = "select a.* from rds_user a where a.is_deleted = 0";
        sql = QueryBuilder.buildRaw(sql, pager, "a");
        Map<String, Object> paramMap = QueryBuilder.getParam(pager.getParams());
        return this.list(RdsUser.class, sql, paramMap);
    }

    public int count(List<Param> params) throws Exception {
        String sql = "select count(*) from rds_user a where a.is_deleted = 0";
        sql = QueryBuilder.buildRaw(sql, params, null, "a");
        Map<String, Object> param = QueryBuilder.getParam(params);
        return this.countQuery(sql, param).intValue();
    }

    public boolean remove(Long id, Long userId) throws Exception {
        String sql =
            "update rds_user set is_deleted = 1, gmt_modify = :gmtModify, mender_id = :menderId where id = :id";
        Map<String, Object> params = MapTools.simpleMap("id", id);
        params.put("gmtModify", new Date());
        params.put("menderId", userId);
        return this.execute(sql, params) > 0;
    }

    public RdsUser query(Long id) throws Exception {
        String sql = "select a.* from rds_user a where a.is_deleted = 0 and a.id = :id";
        List<RdsUser> list = this.list(RdsUser.class, sql, MapTools.simpleMap("id", id));
        if (list.isEmpty()) {
            return null;
        }
        return list.get(0);
    }

    public List<RdsUser> list(Long rdsId) throws Exception {
        String sql = "select * from rds_user where is_deleted = 0 and rds_id = :rdsId";
        return this.list(RdsUser.class, sql, MapTools.simpleMap("rdsId", rdsId));
    }

    public boolean deleteByRds(Long rdsId) throws Exception {
        String sql = "delete from rds_user where rds_id = :rdsId";
        return this.execute(sql, MapTools.simpleMap("rdsId", rdsId)) > 0;
    }

}
