package com.bocloud.pms.repository.resource;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.apache.commons.lang3.StringUtils;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import com.bocloud.common.model.Pager;
import com.bocloud.common.model.Param;
import com.bocloud.common.model.SimpleBean;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.ListTool;
import com.bocloud.common.utils.MapTools;
import com.bocloud.database.core.intf.impl.BasicGenericDao;
import com.bocloud.database.utils.QueryBuilder;
import com.bocloud.pms.entity.resource.CloudServer;
import com.bocloud.pms.entity.resource.enums.ResourceCategory;

/**
 * 云主机数据访问
 *
 * @author wangyu
 * @version 1.0
 * @since 2020年5月15日
 */
@Repository("cloudServerRepository")
public class CloudServerRepository extends BasicGenericDao<CloudServer, Long> {

    public CloudServerRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate) {
        super(jdbcTemplate, npJdbcTemplate);
    }

    public CloudServer get(Long id) {
        String sql = "select a.* from cloud_server a where a.is_deleted = 0 and a.id = :id";
        Map<String, Object> params = MapTools.simpleMap("id", id);
        List<CloudServer> list = this.list(CloudServer.class, sql, params);
        return list.isEmpty() ? null : list.get(0);
    }

    public CloudServer get(String instanceId) {
        String sql = "select a.* from cloud_server a where a.is_deleted = 0 and a.instance_id = :instanceId";
        Map<String, Object> params = MapTools.simpleMap("instanceId", instanceId);
        List<CloudServer> list = this.list(CloudServer.class, sql, params);
        return list.isEmpty() ? null : list.get(0);
    }

    public List<CloudServer> list(int page, int rows, List<Param> params, Map<String, String> sorter, Long labelId,
        List<Long> projectIds, List<Long> vendorids) {
        String sql = null;
        if (null != labelId && 0L != labelId) {
            sql = "select DISTINCT a.*,b.name vendor_name,t.name from cloud_server a "
                + " left join cloud_vendor b on b.id = a.vendor_id" + " left join tenant t on a.tenant_id = t.id"
                + " where a.is_deleted = 0 and a.id in (select b.target from server_label b where b.type = 'VM' and b.label_id = "
                + labelId + " )";
        } else {
            sql =
                "select a.*,b.name vendor_name, b.type vendor_type , t.name tenant_name,c.cluster_id,u.name target_name, region.name region_name, zone.name zone_name, p.name project_name from cloud_server a "
                    + " left join tenant t on a.tenant_id = t.id left join cloud_vendor b on a.vendor_id = b.id left join server c on a.host_id = c.id left join object_relation o on o.res_id = a.id and o.category = 'Computer' left join users u on u.id = o.target "
                    + " left join region region on a.region_id = region.region_id and a.vendor_id = region.vendor_id "
                    + " left join zone zone on a.zone = zone.zone_Id and a.vendor_id = zone.vendor_id "
                    + " left join project p on a.project_id = p.id " + " where a.is_deleted = 0 ";
        }
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        if (!ListTool.isEmpty(projectIds)) {
            sql += " and a.project_id in (:projectIds) ";
            paramMap.put("projectIds", projectIds);
        }
        if (!ListTool.isEmpty(vendorids)) {
            sql += " and a.vendor_id in (:vendorids) ";
            paramMap.put("vendorids", vendorids);
        }
        sql = QueryBuilder.buildRaw(sql, new Pager(page, rows, params, sorter), "a");
        return this.list(CloudServer.class, sql, paramMap);
    }

    public List<SimpleBean> list(List<Param> params, Map<String, String> sorter, List<Long> projectIds,
        List<Long> vendorIds) {
        String sql = "select a.id,a.name,a.instance_id from cloud_server a where a.is_deleted = 0";
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        if (!ListTool.isEmpty(projectIds)) {
            sql += " and a.project_id in (:projectIds) ";
            paramMap.put("projectIds", projectIds);
        }
        if (!ListTool.isEmpty(vendorIds)) {
            sql += " and a.vendor_id in (:vendorIds) ";
            paramMap.put("vendorIds", vendorIds);
        }
        sql = QueryBuilder.build(sql, new Pager(1, Integer.MAX_VALUE, params, sorter), "a");
        List<CloudServer> vms = this.list(CloudServer.class, sql, paramMap);
        List<SimpleBean> beans = new ArrayList<>();
        for (CloudServer vm : vms) {
            SimpleBean simpleBean = new SimpleBean(vm.getId(), vm.getName());
            simpleBean.setUuid(vm.getInstanceId());
            beans.add(simpleBean);
        }
        return beans;
    }

    public int count(List<Param> params, Long subnetId) {
        String sql = "select count(a.id) from cloud_server a where a.is_deleted = 0 ";
        sql = QueryBuilder.buildRaw(sql, params, null, "a");
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        return this.countQuery(sql, paramMap).intValue();
    }

    public int count(List<Param> params, Long labelId, List<Long> projectIds, List<Long> vendorIds) {
        String sql = null;
        if (null != labelId && 0L != labelId) {
            sql =
                "select count(a.id) from cloud_server a where a.is_deleted = 0 and a.id in (select b.target from server_label b where b.type = 'VM' and b.label_id = "
                    + labelId + ")";
        } else {
            sql = "select count(a.id) from cloud_server a where a.is_deleted = 0 ";
        }
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        if (!ListTool.isEmpty(projectIds)) {
            sql += " and a.project_id in (:projectIds) ";
            paramMap.put("projectIds", projectIds);
        }
        if (!ListTool.isEmpty(vendorIds)) {
            sql += " and a.vendor_id in (:vendorIds) ";
            paramMap.put("vendorIds", vendorIds);
        }
        sql = QueryBuilder.buildRaw(sql, params, null, "a");
        return this.countQuery(sql, paramMap).intValue();
    }

    public CloudServer query(Long id) {
        String sql =
            "select a.*,b.name vendor_name,t.name tenant_name,pj.name project_name, r.name region_name from cloud_server a "
                + " left join tenant t on a.tenant_id = t.id "
                + " left join region r on a.region_id = r.region_id and r.vendor_id = a.vendor_id "
                + " left join cloud_vendor b on a.vendor_id = b.id "
                + " left join object_relation o on o.res_id = a.id and o.category = 'Computer' "
                + " left join project pj on pj.id = a.project_id " + " where a.is_deleted = 0 and a.id = :id";
        Map<String, Object> params = MapTools.simpleMap(Common.ID, id);
        List<CloudServer> list = this.list(CloudServer.class, sql, params);
        return list.isEmpty() ? null : list.get(0);
    }

    public int count(List<Param> params, String type) {
        String sql =
            "select count(a.id) from cloud_server a,cloud_vendor b where a.is_deleted = 0 and a.vendor_id = b.id";
        if (!StringUtils.isEmpty(type)) {
            sql += " and b.type = '" + type + "'";
        }
        sql = QueryBuilder.buildRaw(sql, params, null, "a");
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        return this.countQuery(sql, paramMap).intValue();
    }

    public int count(String category, Long targetId, List<Param> params) {
        String sql = null;
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        if (category.equals(ResourceCategory.Storage.name())) {
            sql =
                "select count(a.id) from cloud_server a,store_host b where a.is_deleted = 0 and a.is_template = false and a.id = b.vhost_id and b.category = 'VM' and b.store_id = :targetId";
            paramMap.put("targetId", targetId);
        } else if (category.equals(ResourceCategory.Network.name())) {
            sql =
                "select count(a.id) from cloud_server a,network_host b where a.is_deleted = 0 and a.is_template = false and a.id = b.vhost_id and b.category = 'VM' and b.network_id = :targetId";
            paramMap.put("targetId", targetId);
        } else {
            sql = "select count(a.id) from cloud_server a where a.is_deleted = 0";
        }
        sql = QueryBuilder.buildRaw(sql, params, null, "a");
        return this.countQuery(sql, paramMap).intValue();
    }

    public List<CloudServer> list(int page, int rows, List<Param> params, Map<String, String> sorter, Long subnetId,
        Long tenantId, List<Long> projectIds, List<String> vendorTypes) {
        String sql = null;
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        if (null != subnetId && 0L != subnetId) {
            if (null != tenantId && 0L != tenantId) {
                sql =
                    "select a.*,v.name vendor_name,b.expired_time,t.name tenant_name, r.name region_name,z.name zone_name"
                        + " from cloud_server a  "
                        + " left join region r on r.region_id = a.region_id and r.vendor_id = a.vendor_id "
                        + " left join zone z on z.zone_id = a.zone and z.vendor_id = a.vendor_id "
                        + " left join port p on a.instance_id = p.device_uuid "
                        + " left join port_fixedip pf on p.id = pf.port_id  "
                        + " left join tenant t on a.tenant_id = t.id "
                        + " left join cloud_vendor v on a.vendor_id = v.id,object_relation b "
                        + " where a.is_deleted = 0 and a.id = b.res_id and b.category = 'Computer' and a.tenant_id = "
                        + tenantId + " and pf.subnet_id = " + subnetId;
            } else {
                sql =
                    "select a.*,b.expired_time,project.visibility project_visibility,v.name vendor_name,t.name tenant_name , r.name region_name,z.name zone_name "
                        + " from cloud_server a "
                        + " left join region r on r.region_id = a.region_id and r.vendor_id = a.vendor_id "
                        + " left join zone z on z.zone_id = a.zone and z.vendor_id = a.vendor_id "
                        + " left join project project on a.project_id = project.id "
                        + " left join port p on a.instance_id = p.device_uuid left join port_fixedip pf on p.id = pf.port_id  left join tenant t on a.tenant_id = t.id left join cloud_vendor v on a.vendor_id = v.id left join object_relation b on a.id = b.res_id and b.category = 'Computer' where a.is_deleted = 0 and pf.subnet_id = "
                        + subnetId;
            }
        } else {
            if (null != tenantId && 0L != tenantId) {
                sql =
                    "select a.*,v.name vendor_name,b.expired_time,t.name tenant_name , r.name region_name,z.name zone_name"
                        + " from cloud_server a "
                        + " left join region r on r.region_id = a.region_id and r.vendor_id = a.vendor_id "
                        + " left join zone z on z.zone_id = a.zone and z.vendor_id = a.vendor_id "
                        + " left join tenant t on a.tenant_id = t.id left join cloud_vendor v on a.vendor_id = v.id，object_relation b "
                        + " where a.id = b.res_id and b.category = 'Computer' and a.is_deleted = 0 and a.tenant_id = "
                        + tenantId;
            } else {
                sql =
                    "select a.*,v.name vendor_name,b.expired_time,t.name tenant_name , r.name region_name,z.name zone_name "
                        + " from cloud_server a "
                        + " left join region r on r.region_id = a.region_id and r.vendor_id = a.vendor_id "
                        + " left join zone z on z.zone_id = a.zone and z.vendor_id = a.vendor_id "
                        + " left join project project on a.project_id = project.id "
                        + " left join tenant t on a.tenant_id = t.id left join cloud_vendor v on a.vendor_id = v.id,object_relation b "
                        + " where a.is_deleted = 0 and a.id = b.res_id and b.category = 'Computer'";
            }
        }
        if (!ListTool.isEmpty(projectIds)) {
            sql += " and a.project_id in (:projectIds) ";
            paramMap.put("projectIds", projectIds);
        }
        if (!ListTool.isEmpty(vendorTypes)) {
            sql += " and a.vendor_type in (:vendorTypes) ";
            paramMap.put("vendorTypes", vendorTypes);
        }
        sql = QueryBuilder.buildRaw(sql, new Pager(page, rows, params, sorter), "a");
        return this.list(CloudServer.class, sql, paramMap);
    }

    public int count(List<Param> params, Long subnetId, Long tenantId, List<Long> projectIds, List<String> vendorTypes)
        throws Exception {
        String sql = null;
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        if (null != subnetId && 0L != subnetId) {
            if (null != tenantId && 0L != tenantId) {
                sql = "select count(a.id) " + " from cloud_server a  "
                    + " left join region r on r.region_id = a.region_id and r.vendor_id = a.vendor_id "
                    + " left join zone z on z.zone_id = a.zone and z.vendor_id = a.vendor_id "
                    + " left join port p on a.instance_id = p.device_uuid "
                    + " left join port_fixedip pf on p.id = pf.port_id  " + " left join tenant t on a.tenant_id = t.id "
                    + " left join cloud_vendor v on a.vendor_id = v.id,object_relation b "
                    + " where a.is_deleted = 0 and a.id = b.res_id and b.category = 'Computer' and a.tenant_id = "
                    + tenantId + " and pf.subnet_id = " + subnetId;
            } else {
                sql = "select count(a.id) " + " from cloud_server a "
                    + " left join region r on r.region_id = a.region_id and r.vendor_id = a.vendor_id "
                    + " left join zone z on z.zone_id = a.zone and z.vendor_id = a.vendor_id "
                    + " left join project project on a.project_id = project.id "
                    + " left join port p on a.instance_id = p.device_uuid left join port_fixedip pf on p.id = pf.port_id  left join tenant t on a.tenant_id = t.id left join cloud_vendor v on a.vendor_id = v.id left join object_relation b on a.id = b.res_id and b.category = 'Computer' where a.is_deleted = 0 and pf.subnet_id = "
                    + subnetId;
            }
        } else {
            if (null != tenantId && 0L != tenantId) {
                sql = "select count(a.id) " + " from cloud_server a "
                    + " left join region r on r.region_id = a.region_id and r.vendor_id = a.vendor_id "
                    + " left join zone z on z.zone_id = a.zone and z.vendor_id = a.vendor_id "
                    + " left join tenant t on a.tenant_id = t.id left join cloud_vendor v on a.vendor_id = v.id,object_relation b "
                    + " where a.id = b.res_id and b.category = 'Computer' and a.is_deleted = 0 and a.tenant_id = "
                    + tenantId;
            } else {
                sql = "select count(a.id) " + " from cloud_server a "
                    + " left join region r on r.region_id = a.region_id and r.vendor_id = a.vendor_id "
                    + " left join zone z on z.zone_id = a.zone and z.vendor_id = a.vendor_id "
                    + " left join project project on a.project_id = project.id "
                    + " left join tenant t on a.tenant_id = t.id left join cloud_vendor v on a.vendor_id = v.id,object_relation b"
                    + " where a.is_deleted = 0 and a.id = b.res_id and b.category = 'Computer'";
            }
        }
        if (!ListTool.isEmpty(projectIds)) {
            sql += " and a.project_id in (:projectIds) ";
            paramMap.put("projectIds", projectIds);
        }
        if (!ListTool.isEmpty(vendorTypes)) {
            sql += " and a.vendor_type in (:vendorTypes) ";
            paramMap.put("vendorTypes", vendorTypes);
        }
        sql = QueryBuilder.buildRaw(sql, params, null, "a");
        return this.countQuery(sql, paramMap).intValue();
    }

    public CloudServer getByInstanceId(String instanceId) {
        return getByInstanceId(instanceId, null);
    }

    public CloudServer getByInstanceId(String instanceId, Long vendorId) {
        String sql = "select a.*,t.name tenant_name " + "from cloud_server a left join tenant t on a.tenant_id = t.id "
            + "where a.is_deleted = 0 and a.instance_id = :instanceId";
        Map<String, Object> params = MapTools.simpleMap("instanceId", instanceId);
        if (vendorId != null) {
            sql += " and a.vendor_id = :vendorId ";
            params.put(Common.VENDORID, vendorId);
        }
        List<CloudServer> list = this.list(CloudServer.class, sql, params);
        if (list.isEmpty()) {
            return null;
        }
        return list.get(0);
    }

    public CloudServer getByUuid(String uuid) {
        String sql = "select a.* from cloud_server a where a.is_deleted = 0 and a.uuid = :uuid";
        Map<String, Object> params = MapTools.simpleMap("uuid", uuid);
        List<CloudServer> list = this.list(CloudServer.class, sql, params);
        if (list.isEmpty()) {
            return null;
        }
        return list.get(0);
    }

    public List<CloudServer> list(int page, int rows, List<Param> params, Map<String, String> sorter) {
        String sql =
            "select a.*,t.name tenant_name, b.type vendor_type,b.name vendor_name, p.name project_name from cloud_server a "
                + " left join tenant t on a.tenant_id = t.id " + " left join cloud_vendor b on a.vendor_id = b.id "
                + " left join project p on a.project_id = p.id " + " where a.is_deleted = 0 and a.is_template = 0 ";
        sql = QueryBuilder.buildRaw(sql, new Pager(page, rows, params, sorter), "a");
        return this.list(CloudServer.class, sql, null);
    }

    public int count(List<Param> params) {
        String sql = " select count(1) from cloud_server a where a.is_deleted = 0 and a.is_template = 0 ";
        sql = QueryBuilder.buildRaw(sql, params, null, "a");
        return this.countQuery(sql, null).intValue();
    }

    public List<CloudServer> listByInstanceId(List<String> instanceIds) {
        String sql = "select * from cloud_server a where is_deleted = 0 and a.instance_id in (:instanceIds)";
        Map<String, Object> params = MapTools.simpleMap("instanceIds", instanceIds);
        return this.list(CloudServer.class, sql, params);
    }

}
