package com.bocloud.pms.repository.resource;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import com.bocloud.common.exception.BoCloudDBException;
import com.bocloud.common.model.Pager;
import com.bocloud.common.model.Param;
import com.bocloud.common.model.SimpleBean;
import com.bocloud.common.utils.IDFactory;
import com.bocloud.common.utils.ListTool;
import com.bocloud.common.utils.MapTools;
import com.bocloud.database.core.intf.impl.BasicGenericDao;
import com.bocloud.database.utils.QueryBuilder;
import com.bocloud.pms.entity.resource.CloudVendor;

/**
 * 云平台数据访问
 *
 * @author wangyu
 * @version 1.0
 * @since 2020年5月15日
 */
@Repository("cloudVendorRepository")
public class CloudVendorRepository extends BasicGenericDao<CloudVendor, Long> {

    public CloudVendorRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate) {
        super(jdbcTemplate, npJdbcTemplate);
    }

    @Override
    public void save(CloudVendor vendor) throws BoCloudDBException {
        vendor.setGuid(IDFactory.instance().uuid());
        super.save(vendor);
    }

    public List<CloudVendor> list(int page, int rows, List<Param> params, Map<String, String> sorter,
        List<Long> vendorIds) {
        String sql = "SELECT " + "a.*, b. NAME env_name,c. NAME dc_name,d.instance,d. ONLINE,d.offline,d.host_count,"
            + "d.data_store_count,d.template_count,d.cluster_count,d.image_count,d.cpu_count,"
            + "d.mon_count,d.osd_count,d.mds_count,d.cpu_total,d.cpu_used,d.cpu_unused,d.mem_total,"
            + "d.mem_used,d.mem_unused,d.disk_total,d.disk_used,d.disk_unused,d.tenant_count,"
            + "d.network_count,d.vs_count,d.pool_count,d.node_count,d.profile_count,d.monitor_count,"
            + "d.vpc_count,d.lpar_count,d.vios_count,d.mdisk_count,d.flash_copy_count,d.fabric_count,d.zone_count,"
            + "d.ns_count,d.pod_count,d.service_count,d.deployment_count,d.alias_count,"
            + "d.access_path_count AS access_path_count,d.edge_cluster_count,d.group_count,d.transport_zone_count,"
            + "d.firewall_count,d.router_count,d.switch_count,d.object_user_count,d.object_bucket_count,"
            + "d.object_router_count,d.desktop_pools,d.farms,d.sessions,d.users,d.vcenters,"
            + "d.file_user_count , d.file_client_count , d.snapshot_count, d.file_system_count, d.logic_port_count,"
            + "d.vas_count, d.app_count, d.session_count, d.db_count " + "FROM cloud_vendor a "
            + "LEFT JOIN environment b ON a.env_id = b.id " + "LEFT JOIN data_center c ON c.id = a.dc_id "
            + "LEFT JOIN cloud_vendor_statistics d ON d.vendor_id = a.id " + "WHERE " + "a.is_deleted = 0 "
            + "AND a.type IS NOT NULL";
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        if (!ListTool.isEmpty(vendorIds)) {
            sql += " and a.id in (:vendorIds) ";
            paramMap.put("vendorIds", vendorIds);
        }
        sql = QueryBuilder.buildRaw(sql, new Pager(page, rows, params, sorter), "a");
        return this.list(CloudVendor.class, sql, paramMap);
    }

    public List<CloudVendor> list(int page, int rows, List<Param> params, Map<String, String> sorter) {
        String sql = "SELECT "
            + "a.*, b. NAME env_name,c. NAME dc_name,d.instance,d. ONLINE,d.offline,d.host_count,d.data_store_count,d.template_count,d.cluster_count,d.image_count,d.cpu_count,d.mon_count,d.osd_count,d.mds_count,d.cpu_total,d.cpu_used,d.cpu_unused,d.mem_total,d.mem_used,d.mem_unused,d.disk_total,d.disk_used,d.disk_unused,d.tenant_count,d.network_count,d.vs_count,d.pool_count,d.node_count,d.profile_count,d.monitor_count,d.vpc_count,d.lpar_count,d.vios_count,d.mdisk_count,d.flash_copy_count,d.fabric_count,d.zone_count,d.ns_count,d.pod_count,d.service_count,d.deployment_count,d.alias_count,d.access_path_count AS access_path_count,d.edge_cluster_count,d.group_count,d.transport_zone_count,d.firewall_count,d.router_count,d.switch_count,d.object_user_count,d.object_bucket_count,d.object_router_count,d.desktop_pools,d.farms,d.sessions,d.users,d.vcenters"
            + " , d.file_user_count , d.file_client_count , d.snapshot_count, d.file_system_count, d.logic_port_count, d.vas_count "
            + "FROM " + "cloud_vendor a " + "LEFT JOIN environment b ON a.env_id = b.id "
            + "LEFT JOIN data_center c ON c.id = a.dc_id "
            + "LEFT JOIN cloud_vendor_statistics d ON d.vendor_id = a.id " + "WHERE " + "a.is_deleted = 0 "
            + "AND a.type IS NOT NULL";
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        sql = QueryBuilder.buildRaw(sql, new Pager(page, rows, params, sorter), "a");
        return this.list(CloudVendor.class, sql, paramMap);
    }

    public List<SimpleBean> list(List<Param> params, Map<String, String> sorter, List<Long> vendorIds) {
        String sql = "select a.id,a.name,a.uuid from cloud_vendor a where a.is_deleted = 0";
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        if (!ListTool.isEmpty(vendorIds)) {
            sql += " and a.id in (:vendorIds) ";
            paramMap.put("vendorIds", vendorIds);
        }
        sql = QueryBuilder.build(sql, new Pager(1, Integer.MAX_VALUE, params, sorter), "a");
        List<CloudVendor> vendors = this.list(CloudVendor.class, sql, paramMap);
        List<SimpleBean> beans = new ArrayList<>();
        for (CloudVendor vendor : vendors) {
            SimpleBean bean = new SimpleBean(vendor.getId(), vendor.getName());
            bean.setUuid(vendor.getUuid());
            beans.add(bean);
        }
        return beans;
    }

    public List<SimpleBean> list(List<Param> params, Map<String, String> sorter) {
        String sql = "select a.id,a.name,a.uuid from cloud_vendor a where a.is_deleted = 0";
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        sql = QueryBuilder.build(sql, new Pager(1, Integer.MAX_VALUE, params, sorter), "a");
        List<CloudVendor> vendors = this.list(CloudVendor.class, sql, paramMap);
        List<SimpleBean> beans = new ArrayList<>();
        for (CloudVendor vendor : vendors) {
            SimpleBean bean = new SimpleBean(vendor.getId(), vendor.getName());
            bean.setUuid(vendor.getUuid());
            beans.add(bean);
        }
        return beans;
    }

    public int count(List<Param> params, List<Long> vendorIds) {
        String sql = "select count(1) from cloud_vendor a where a.is_deleted = 0 and a.type is not null";
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        if (!ListTool.isEmpty(vendorIds)) {
            sql += " and a.id in (:vendorIds) ";
            paramMap.put("vendorIds", vendorIds);
        }
        sql = QueryBuilder.build(sql, params, null, "a");
        return this.countQuery(sql, paramMap).intValue();
    }

    public int count(List<Param> params) {
        String sql = "select count(1) from cloud_vendor a where a.is_deleted = 0 and a.type is not null";
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        sql = QueryBuilder.build(sql, params, null, "a");
        return this.countQuery(sql, paramMap).intValue();
    }

    public CloudVendor query(Long id) {
        String sql =
            "select a.*,b.name env_name,c.name dc_name,d.instance,d.online,d.offline,d.host_count,d.data_store_count,d.template_count,d.cluster_count,d.image_count,d.cpu_count,d.mon_count,d.osd_count,d.mds_count,d.cpu_total,d.cpu_used,d.cpu_unused,d.mem_total,d.mem_used,d.mem_unused,d.disk_total,d.disk_used,d.disk_unused,d.tenant_count,d.network_count,d.vs_count,d.pool_count,d.node_count,d.profile_count,d.monitor_count,d.vpc_count,d.lpar_count,d.vios_count,d.mdisk_count,d.flash_copy_count,d.fabric_count,d.zone_count,d.ns_count,d.pod_count,d.service_count,d.deployment_count,d.alias_count, d.access_path_count as access_path_count,d.edge_cluster_count,d.group_count,d.transport_zone_count,d.firewall_count,d.router_count,d.switch_count,d.desktop_pools,d.farms,d.sessions,d.users,d.vcenters from cloud_vendor a left join environment b on a.env_id = b.id left join data_center c on c.id = a.dc_id left join cloud_vendor_statistics d on d.vendor_id = a.id where a.is_deleted = 0 and a.id = :id";
        Map<String, Object> paramMap = MapTools.simpleMap("id", id);
        List<CloudVendor> list = this.list(CloudVendor.class, sql, paramMap);
        if (list.isEmpty()) {
            return null;
        }
        return list.get(0);
    }

    public CloudVendor query(String uuid) {
        String sql =
            "select a.*,b.name env_name from cloud_vendor a left join environment b on a.env_id = b.id where a.uuid =:uuid";
        Map<String, Object> paramMap = MapTools.simpleMap("uuid", uuid);
        List<CloudVendor> list = this.list(CloudVendor.class, sql, paramMap);
        if (list.isEmpty()) {
            return null;
        }
        return list.get(0);
    }

    public List<CloudVendor> list(List<Param> params) {
        String sql =
            "select a.*,b.name env_name from cloud_vendor a left join environment b on a.env_id = b.id where a.is_deleted = 0";
        sql = QueryBuilder.build(sql, new Pager(1, Integer.MAX_VALUE, params, new HashMap<>()), "a");
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        return this.list(CloudVendor.class, sql, paramMap);
    }

}
