package com.bocloud.pms.repository.resource;

import java.util.List;
import java.util.Map;

import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import com.bocloud.common.model.Pager;
import com.bocloud.common.model.Param;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.ListTool;
import com.bocloud.common.utils.MapTools;
import com.bocloud.database.core.intf.impl.BasicGenericDao;
import com.bocloud.database.utils.QueryBuilder;
import com.bocloud.pms.entity.resource.Ip;

/**
 * @author wangyu
 * @version 5.1.0
 * @description IP数据访问
 * @date: 2020-11-30
 */
@Repository("ipRepository")
public class IpRepository extends BasicGenericDao<Ip, Long> {

    public IpRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate) {
        super(jdbcTemplate, npJdbcTemplate);
    }

    public List<Ip> list(int page, int rows, List<Param> params, Map<String, String> sorter) {
        String sql =
            "select a.*,b.name vendor_name,n.name network_name from ip a left join cloud_vendor b on a.vendor_id=b.id left join network n on n.id = a.network_id where 1 = 1 ";
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        sql = QueryBuilder.buildRaw(sql, new Pager(page, rows, params, sorter), "a");
        return this.list(Ip.class, sql, paramMap);
    }

    public int count(List<Param> params) {
        String sql = "select count(id) from ip where 1 = 1 ";
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        sql = QueryBuilder.buildRaw(sql, params, null, "");
        return this.countQuery(sql, paramMap).intValue();
    }

    public Ip query(Long id) {
        String sql =
            "select a.*,b.name vendor_name from ip a left join cloud_vendor b on a.vendor_id=b.id where b.is_deleted=0 and a.id = :id";
        Map<String, Object> params = MapTools.simpleMap(Common.ID, id);
        List<Ip> list = this.list(Ip.class, sql, params);
        return ListTool.isEmpty(list) ? null : list.get(0);
    }

    public Boolean delete(Long id) {
        String sql = "delete from ip where id = :id";
        Map<String, Object> params = MapTools.simpleMap(Common.ID, id);
        return this.execute(sql, params) > 0;
    }

    public int count(Long subnetId, Long vendorId) {
        String sql = "select count(1) from ip where subnet_id = :subnetId and vendor_id = :vendorId";
        Map<String, Object> params = MapTools.simpleMap(Common.VENDORID, vendorId);
        params.put("subnetId", subnetId);
        return this.countQuery(sql, params).intValue();
    }

}
