package com.bocloud.pms.repository.resource;

import java.util.List;
import java.util.Map;

import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import com.bocloud.common.model.Pager;
import com.bocloud.common.model.Param;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.MapTools;
import com.bocloud.database.core.intf.impl.BasicGenericDao;
import com.bocloud.database.utils.QueryBuilder;
import com.bocloud.pms.entity.resource.Resource;

/**
 * 资源抽血数据访问
 *
 * @author wangyu
 * @version 1.0
 * @since 2020年5月15日
 */
@Repository("resourceRepository")
public class ResourceRepository extends BasicGenericDao<Resource, Long> {

    public ResourceRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate) {
        super(jdbcTemplate, npJdbcTemplate);
    }

    public List<Resource> list(int page, int rows, List<Param> params, Map<String, String> sorter) {
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        String sql = "select res.*,v.name vendor_name,t.name tenant_name,g.name group_name, " + " u.name owner_name "
            + " from resource res " + " left join cloud_vendor v on res.vendor_id = v.id"
            + " left join tenant t on res.tenant_id = t.id " + " left join pool_group g on res.group_id = g.id "
            + " left join users u on res.owner_id = u.id " + " where res.is_deleted = 0";
        sql = QueryBuilder.buildRaw(sql, new Pager(page, rows, params, sorter), "res");
        return this.list(Resource.class, sql, paramMap);
    }

    public List<Resource> list(int page, int rows, List<Param> params, Map<String, String> sorter, Long tenantId,
        Long userId) {
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        String sql = "select * from resource r where r.is_deleted = 0 and r.target in ("
            + "select a.id from cloud_server a,object_relation b where a.is_deleted = 0 and a.id = b.res_id and b.category = 'Computer'";
        if (null != userId) {
            sql += " and b.catalog = 'User' and b.target = " + userId;
        }
        if (null != tenantId) {
            sql += " and a.tenant_id = " + tenantId;
        }
        sql += ")";
        sql = QueryBuilder.buildRaw(sql, new Pager(page, rows, params, sorter), "r");
        return this.list(Resource.class, sql, paramMap);
    }

    public int count(List<Param> params, Long tenantId, Long userId) {
        Map<String, Object> param = QueryBuilder.getParam(params);
        String sql = "select count(1) from resource r where r.is_deleted = 0 and r.target in ("
            + "select a.id from cloud_server a,object_relation b where a.is_deleted = 0 and a.id = b.res_id and b.category = 'Computer'";
        if (null != userId) {
            sql += " and b.catalog = 'User' and b.target = " + userId;
        }
        if (null != tenantId) {
            sql += " and a.tenant_id = " + tenantId;
        }
        sql += ")";
        sql = QueryBuilder.buildRaw(sql, params, null, "r");
        return this.countQuery(sql, param).intValue();
    }

    public int count(List<Param> params) {
        Map<String, Object> param = QueryBuilder.getParam(params);
        String sql = "select count(1) from resource r where r.is_deleted = 0";
        sql = QueryBuilder.buildRaw(sql, params, null, "r");
        return this.countQuery(sql, param).intValue();
    }

    public Resource query(Long id) {
        String sql = " select res.*, v.name vendor_name,t.name tenant_name,g.name group_name, " + " u.name owner_name "
            + " from resource res " + " left join cloud_vendor v on res.vendor_id = v.id"
            + " left join tenant t on res.tenant_id = t.id " + " left join pool_group g on res.group_id = g.id "
            + " left join users u on res.owner_id = u.id " + " where res.is_deleted = 0 and res.id = :id ";
        Map<String, Object> params = MapTools.simpleMap(Common.ID, id);
        List<Resource> list = this.list(Resource.class, sql, params);
        if (list.isEmpty()) {
            return null;
        }
        return list.get(0);
    }

    public Resource query(String category, String catalog, Long target) {
        String sql =
            "select * from resource where is_deleted = 0 and category = :category and catalog = :catalog and target = :target";
        Map<String, Object> params = MapTools.simpleMap("category", category);
        params.put("catalog", catalog);
        params.put("target", target);
        List<Resource> list = this.list(Resource.class, sql, params);
        if (list.isEmpty()) {
            return null;
        }
        return list.get(0);
    }

    public Resource query(String category, String catalog, Long target, Long vendorId) {
        String sql = "select * from resource where is_deleted = 0"
            + " and category = :category and catalog = :catalog and target = :target and vendor_id = :vendorId";
        Map<String, Object> params = MapTools.simpleMap("category", category);
        params.put("catalog", catalog);
        params.put("target", target);
        params.put(Common.VENDORID, vendorId);
        List<Resource> list = this.list(Resource.class, sql, params);
        if (list.isEmpty()) {
            return null;
        }
        return list.get(0);
    }

}
