package com.bocloud.pms.repository.resource;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.lang3.StringUtils;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import com.bocloud.common.model.Pager;
import com.bocloud.common.model.Param;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.ListTool;
import com.bocloud.common.utils.MapTools;
import com.bocloud.database.core.intf.impl.BasicGenericDao;
import com.bocloud.database.utils.QueryBuilder;
import com.bocloud.pms.entity.resource.Server;
import com.bocloud.pms.entity.resource.enums.ResourceCategory;

/**
 * 物理服务器数据访问
 *
 * @author wangyu
 * @version 1.0
 * @since 2020年5月15日
 */
@Repository("serverRepository")
public class ServerRepository extends BasicGenericDao<Server, Long> {

    public ServerRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate) {
        super(jdbcTemplate, npJdbcTemplate);
    }

    public Server query(Long id) {
        String sql =
            "select a.*,b.id rack_id,b.name rackname,c.id as room_id,c.name roomname,d.id dc_id,d.name dcname,e.name cluster_name,f.name vdc_name,j.name vendor_name from server a "
                + "left join rack b on a.rack_id = b.id " + "left join room c on a.room_id = c.id "
                + "left join clusters e on a.cluster_id = e.id " + "left join vdc f on a.vdc_id = f.id "
                + "left join cloud_vendor j on a.vendor_id = j.id " + "left join data_center d on d.id = a.dc_id "
                + "where a.is_deleted = 0 and a.id = :id";
        Map<String, Object> params = MapTools.simpleMap("id", id);
        List<Server> list = this.list(Server.class, sql, params);
        if (list.isEmpty()) {
            return null;
        }
        return list.get(0);
    }

    public Server basic(Long id) {
        String sql =
            "select a.name,b.name cluster_name,c.name vdc_name from server a left join clusters b on b.id = a.cluster_id left join vdc c on c.id = a.vdc_id where a.id = :id";
        Map<String, Object> params = MapTools.simpleMap("id", id);
        List<Server> list = this.list(Server.class, sql, params);
        return ListTool.isEmpty(list) ? null : list.get(0);
    }

    public int count(List<Param> params, List<Long> labelIds) {
        String sql = "select count(1) from server a where a.is_deleted = 0";
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        if (!ListTool.isEmpty(labelIds)) {
            sql += " and a.id in (select target from server_label where type = 'PM' and id in (:labelIds))";
            paramMap.put("labelIds", labelIds);
        }
        sql = QueryBuilder.buildRaw(sql, params, null, "a");
        return this.countQuery(sql, paramMap).intValue();
    }

    public boolean exists(String ip) {
        String sql = "select * from server a where a.is_deleted = 0 and a.manage_ip = :ip";
        Map<String, Object> params = MapTools.simpleMap("ip", ip);
        List<Server> list = this.list(Server.class, sql, params);
        return !list.isEmpty();
    }

    public Server getByName(String name, Long vendorId) {
        String sql = "select * from server where is_deleted = 0 and name = :name and vendor_id = :vendorId";
        Map<String, Object> param = MapTools.simpleMap("name", name);
        param.put(Common.VENDORID, vendorId);
        List<Server> list = this.list(Server.class, sql, param);
        if (list.isEmpty()) {
            return null;
        }
        return list.get(0);
    }

    public List<Server> list(int page, int rows, List<Param> params, Map<String, String> sorter, String type)
        throws Exception {
        String sql =
            "select a.*,b.name cluster_name,c.name vdc_name,d.name vendor_name from server a left join clusters b on a.cluster_id = b.id left join vdc c on a.vdc_id = c.id,cloud_vendor d where a.is_deleted = 0 and a.vendor_id = d.id";
        if (!StringUtils.isEmpty(type)) {
            sql += " and d.type = '" + type + "'";
        }
        sql = QueryBuilder.buildRaw(sql, new Pager(page, rows, params, sorter), "a");
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        return this.list(Server.class, sql, paramMap);
    }

    public int count(List<Param> params, String type) {
        String sql = "select count(1) from server a,cloud_vendor b where a.is_deleted = 0 and a.vendor_id = b.id";
        if (!StringUtils.isEmpty(type)) {
            sql += " and b.type = '" + type + "'";
        }
        sql = QueryBuilder.buildRaw(sql, params, null, "a");
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        return this.countQuery(sql, paramMap).intValue();
    }

    public List<Server> list(Long vendorId, String type) {
        String sql =
            "select a.*,b.name vendor_name from server a,cloud_vendor b where a.is_deleted = 0 and a.vendor_id = b.id";
        if (!StringUtils.isEmpty(type)) {
            sql += " and b.type = '" + type + "'";
        }
        Map<String, Object> params = new HashMap<>();
        if (null != vendorId) {
            sql += " and a.vendor_id = :vendorId";
            params.put(Common.VENDORID, vendorId);
        }
        return this.list(Server.class, sql, params);
    }

    public List<Server> listAll() {
        String sql =
            "select a.*,b.name cluster_name,c.name vdc_name,d.name vendor_name from server a left join clusters b on a.cluster_id = b.id left join vdc c on a.vdc_id = c.id,cloud_vendor d where a.is_deleted = 0 and a.vendor_id = d.id";
        return this.list(Server.class, sql);
    }

    public List<Server> listByStore(Long storeId) {
        String sql =
            "select a.* from server a,store_host b where a.is_deleted = 0 and a.id = b.vhost_id and b.category = 'PM' and b.store_id = :storeId";
        Map<String, Object> params = MapTools.simpleMap("storeId", storeId);
        return this.list(Server.class, sql, params);
    }

    public List<Server> listByIds(List<Long> ids) {
        String sql = "select  a.* from server a where a.id in (:ids) and a.is_deleted = 0";
        Map<String, Object> params = MapTools.simpleMap("ids", ids);
        return this.list(Server.class, sql, params);
    }

    public int count(String category, Long targetId, List<Param> params) {
        String sql = null;
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        if (category.equals(ResourceCategory.Storage.name())) {
            sql =
                "select DISTINCT count(a.id) from server a,store_host b where a.is_deleted = 0 and a.id = b.vhost_id and b.category = 'PM' and b.store_id = :targetId";
            paramMap.put("targetId", targetId);
        } else if (category.equals(ResourceCategory.Network.name())) {
            sql =
                "select DISTINCT count(a.id) from server a,network_host b where a.is_deleted = 0 and a.id = b.vhost_id and b.category = 'PM' and b.network_id = :targetId";
            paramMap.put("targetId", targetId);
        } else {
            sql = "select DISTINCT count(a.id) from server a where a.is_deleted = 0";
        }
        sql = QueryBuilder.buildRaw(sql, params, null, "a");
        return this.countQuery(sql, paramMap).intValue();
    }

    public Server getByUuid(String uuid) {
        String sql = "select * from server a where a.is_deleted = 0 and a.uuid = :uuid";
        Map<String, Object> params = MapTools.simpleMap("uuid", uuid);
        List<Server> list = this.list(Server.class, sql, params);
        if (list.isEmpty()) {
            return null;
        }
        return list.get(0);
    }

    public Server getByResourceId(String resourceId) {
        String sql = "select * from server a where a.is_deleted = 0 ";
        Map<String, Object> params = new HashMap<>();
        if (resourceId != null) {
            params.put("resource_id", resourceId);
            sql += " and a.resource_id = :resource_id";
        }
        List<Server> list = this.list(Server.class, sql, params);
        if (list.isEmpty()) {
            return null;
        }
        return list.get(0);
    }

    public Server queryByIp(String ip) {
        String sql = "select * from server a where is_deleted = 0 and manage_ip = :ip ";
        Map<String, Object> params = MapTools.simpleMap("ip", ip);
        List<Server> list = this.list(Server.class, sql, params);
        if (!ListTool.isEmpty(list)) {
            return list.get(0);
        }
        return null;
    }

}
