package com.bocloud.pms.repository.resource;

import java.util.Date;
import java.util.List;
import java.util.Map;

import org.apache.commons.lang3.StringUtils;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import com.bocloud.common.model.Pager;
import com.bocloud.common.model.Param;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.MapTools;
import com.bocloud.database.core.intf.impl.BasicGenericDao;
import com.bocloud.database.utils.QueryBuilder;
import com.bocloud.pms.entity.resource.Snapshot;

/**
 * 快照数据访问
 *
 * @author wangyu
 * @version 1.0
 * @since 2020年5月15日
 */
@Repository("snapshotRepository")
public class SnapshotRepository extends BasicGenericDao<Snapshot, Long> {

    public SnapshotRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate) {
        super(jdbcTemplate, npJdbcTemplate);
    }

    public List<Snapshot> list(int page, int rows, List<Param> params, Map<String, String> sorter) {
        String sql = "select s.*,c.name vm_name,t.name tenant_name,c.id as vendor_id from snapshot s "
            + " left join cloud_server c on s.vm_id = c.id " + " left join tenant t on s.tenant_id = t.id "
            + " where s.is_deleted = 0";
        sql = QueryBuilder.buildRaw(sql, new Pager(page, rows, params, sorter), "s");
        return this.list(Snapshot.class, sql);
    }

    public int count(List<Param> params) {
        String sql = "select count(1) from snapshot s where s.is_deleted = 0";
        sql = QueryBuilder.buildRaw(sql, params, null, "s");
        Map<String, Object> param = QueryBuilder.getParam(params);
        return this.countQuery(sql, param).intValue();
    }

    public boolean remove(Long id, Long userId) {
        String sql =
            "update snapshot set is_deleted = 1, gmt_modify = :gmtModify, mender_id = :menderId where id = :id";
        Map<String, Object> params = MapTools.simpleMap(Common.ID, id);
        params.put("gmtModify", new Date());
        params.put("menderId", userId);
        return this.execute(sql, params) > 0;
    }

    public List<Snapshot> listByVmid(Long vmId) {
        String sql =
            "select a.*,t.name tenant_name from snapshot a left join tenant t on a.tenant_id = t.id where a.is_deleted = 0 and a.vm_id = :vmId";
        Map<String, Object> params = MapTools.simpleMap("vmId", vmId);
        return this.list(Snapshot.class, sql, params);
    }

    public int count(Long vmId) {
        String sql = "select count(id) from snapshot s where s.is_deleted = 0 and s.vm_id = :vmId";
        Map<String, Object> param = MapTools.simpleMap("vmId", vmId);
        return this.countQuery(sql, param).intValue();
    }

    public Snapshot query(Long id) {
        String sql = "select a.*,t.name tenant_name from snapshot a " + " left join tenant t on a.tenant_id = t.id "
            + " left join object_relation o on o.res_id = a.id and o.category = 'Snapshot' where a.is_deleted = 0 and a.id = :id";
        Map<String, Object> params = MapTools.simpleMap(Common.ID, id);
        List<Snapshot> list = this.list(Snapshot.class, sql, params);
        if (list.isEmpty()) {
            return null;
        }
        return list.get(0);
    }

    public int count(List<Param> params, String type) {
        String sql = "select count(1) from snapshot s,cloud_vendor c where s.is_deleted = 0 and s.vendor_id = c.id";
        if (!StringUtils.isEmpty(type)) {
            sql += " and c.type = '" + type + "'";
        }
        sql = QueryBuilder.buildRaw(sql, params, null, "s");
        Map<String, Object> param = QueryBuilder.getParam(params);
        return this.countQuery(sql, param).intValue();
    }

}
