package com.bocloud.pms.service;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.http.HttpClient;
import com.bocloud.common.model.*;
import com.bocloud.common.model.RequestContext.Catalog;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.GridHelper;
import com.bocloud.common.utils.IDFactory;
import com.bocloud.common.utils.ListTool;
import com.bocloud.pms.entity.Ass;
import com.bocloud.pms.entity.AssNode;
import com.bocloud.pms.entity.resource.Resource;
import com.bocloud.pms.entity.resource.TaskNode;
import com.bocloud.pms.entity.resource.enums.BocloudStatus;
import com.bocloud.pms.entity.resource.enums.ResourceCategory;
import com.bocloud.pms.interfaces.middleware.AssService;
import com.bocloud.pms.repository.AssNodeRepository;
import com.bocloud.pms.repository.AssRepository;
import com.bocloud.pms.repository.resource.ResourceRepository;
import com.bocloud.pms.service.utils.MDSUtils;
import com.bocloud.pms.service.utils.ResourceEventPublisher;
import com.bocloud.service.lock.BocloudACLock;
import com.bocloud.service.lock.LockFactory;
import com.google.common.collect.Maps;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.amqp.core.AmqpTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import java.util.*;
import java.util.concurrent.TimeUnit;

/**
 * ass Service接口实现
 *
 * @author weiwei
 */
@Service("assService")
public class AssServiceImpl implements AssService {

    private static Logger logger = LoggerFactory.getLogger(AssServiceImpl.class);
    @Autowired
    private LockFactory lockFactory;
    @Autowired
    private AssRepository assRepository;
    @Autowired
    private AssNodeRepository assNodeRepository;
    @Autowired
    private ResourceRepository resourceRepository;
    @Autowired
    private RedisTemplate<String, String> redisTemplate;
    @Autowired
    private ResourceEventPublisher resourceEventPublisher;
    @Autowired
    private AmqpTemplate amqpTemplate;

    @Override
    public BocloudResult<GridBean<Ass>> list(Pager pager, RequestContext context) {
        try {
            List<Param> params = pager.getParams();
            Integer page = pager.getPage();
            Integer rows = pager.getRows();
            Map<String, String> sorter = pager.getSorter();
            params = Optional.ofNullable(params).orElse(new ArrayList<>());
            sorter = Optional.ofNullable(sorter).orElse(Maps.newHashMap());
            sorter.put("gmtCreate", Common.ONE);
            List<Ass> list = assRepository.list(page, rows, params, sorter);
            for (Ass ass : list) {
                JSONObject specs = JSONObject.parseObject(ass.getSpec());
                if (null != specs) {
                    ass.setSpec(specs.get("cpu") + "核/" + specs.get("memory") + "GB/" + specs.get("disk") + "GB");
                    ass.setDataDisk(specs.getInteger("dataDisk"));
                }
                JSONObject inventory = JSONObject.parseObject(ass.getInventory());
                if (null != inventory) {
                    List<JSONObject> elements = JSONArray.parseArray(inventory.getString("elements"), JSONObject.class);
                    if (!ListTool.isEmpty(elements)) {
                        for (JSONObject object : elements) {
                            if (object.getString("serviceCode").contains("ass." + ass.getType())) {
                                continue;
                            }
                            ass.setDiskSkuId(object.getLong("skuId"));
                        }
                    }
                }
                ass.setTenantName(redisTemplate.opsForValue().get("Tenant_Name_" + ass.getTenantId()));
            }
            int total = assRepository.count(params);
            GridBean gridBean = GridHelper.getBean(page, rows, total, list);
            return new BocloudResult<>(true, gridBean, "查询成功");
        } catch (Exception e) {
            logger.error("list ass failure:", e);
            return new BocloudResult<>(false, "查询失败", null, null);
        }
    }

    @Override
    public BocloudResult create(String params, RequestContext context) {
        return null;
    }

    private void syncNodeStatus(AssNode node, Ass ass, HttpClient client, Result result) {
        switch (ResourceCategory.valueOf(ass.getType())) {
            case Tomcat:
                result = client.get(null, null, "http://" + node.getManageIp() + ":" + node.getPort());
                break;
            case Apache:
                result = client.get(null, null, "http://" + node.getManageIp() + ":" + node.getPort());
                if (result.getMessage().contains("Testing 123")) {
                    result.setSuccess(true);
                }
                break;
            case Weblogic:
                result = client.get(null, null, "http://" + node.getManageIp() + ":" + node.getPort() + "/console");
                break;
            case Websphere:
                result = client.get(null, null, "http://" + node.getManageIp() + ":" + node.getPort() + "/ibm/console");
                break;
            default:
                break;
        }
        if (result.isSuccess()) {
            if (!"RUNNING".equals(node.getStatus())) {
                node.setStatus("RUNNING");
                assNodeRepository.update(node);
                ass.setStatus("RUNNING");
                assRepository.update(ass);
            }
        } else {
            if (!"STOPPED".equals(node.getStatus())) {
                node.setStatus("STOPPED");
                assNodeRepository.update(node);
                ass.setStatus("STOPPED");
                assRepository.update(ass);
            }
        }
    }

    @Override
    public BocloudResult<Ass> detail(Long id) {
        try {
            Ass ass = assRepository.query(id);
            ass = Optional.ofNullable(ass).orElseThrow(() -> new Exception("数据不存在"));
            List<AssNode> nodes = assNodeRepository.list(id);
            HttpClient client = new HttpClient(30000);
            Result result = null;
            for (AssNode node : nodes) {
                syncNodeStatus(node, ass, client, result);
            }
            ass.setNodes(nodes);
            JSONObject specs = JSONObject.parseObject(ass.getSpec());
            if (null != specs) {
                ass.setSpec(specs.get("cpu") + "核/" + specs.get("memory") + "GB/" + specs.get("disk") + "GB");
                ass.setDataDisk(specs.getInteger("dataDisk"));
            }
            JSONObject inventory = JSONObject.parseObject(ass.getInventory());
            if (null != inventory) {
                JSONObject configs = inventory.getJSONObject("configs");
                ass.setDiskCategory(configs.getString("diskCategoryName"));
            }
            return new BocloudResult<>(true, ass, "查询详情成功");
        } catch (Exception e) {
            logger.error("query ass detail fail:", e);
            return new BocloudResult<>(false, "查询详情失败");
        }
    }

    @Override
    public BocloudResult remove(Long id, RequestContext context) {
        String path = Ass.class.getSimpleName() + "_" + id;
        try (BocloudACLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            Ass ass = assRepository.query(id);
            ass = Optional.ofNullable(ass).orElseThrow(() -> new Exception("数据不存在"));
            if (null != ass.getTenantId() && ass.getTenantId() > 0) {
                return new BocloudResult(false, "租户数据不可删除");
            }
            assNodeRepository.deleteByAss(ass.getId());
            assRepository.remove(ass.getId(), context.getTarget());
            MDSUtils.send(amqpTemplate, ass.getId(), ass.getType(), "delete");
            return new BocloudResult(true, ass.getName() + "删除成功");
        } catch (Exception e) {
            logger.error("remove ass failure:", e);
            return new BocloudResult(false, "删除失败");
        }
    }

    @Override
    public void synchronousRoleState(JSONObject content) {
        try {
            Long id = content.getLong("id");
            Ass ass = assRepository.query(id);
            ass = Optional.ofNullable(ass).orElseThrow(() -> new Exception("数据不存在"));
            List<AssNode> nodes = assNodeRepository.list(id);
            HttpClient client = new HttpClient(30000);
            Result result = null;
            for (AssNode node : nodes) {
                syncNodeStatus(node, ass, client, result);
            }
            int statusCount = MDSUtils.statusCount(nodes, BocloudStatus.Server.RUNNING.name());
            if (0 == statusCount) {
                if (!BocloudStatus.Server.STOPPED.name().equals(ass.getStatus())) {
                    ass.setStatus(BocloudStatus.Server.STOPPED.name());
                    assRepository.update(ass);
                }
            } else if (statusCount == nodes.size()) {
                if (!BocloudStatus.Server.RUNNING.name().equals(ass.getStatus())) {
                    ass.setStatus(BocloudStatus.Server.RUNNING.name());
                    assRepository.update(ass);
                }
            } else {
                if (!BocloudStatus.Server.EXCEPTION.name().equals(ass.getStatus())) {
                    ass.setStatus(BocloudStatus.Server.EXCEPTION.name());
                    assRepository.update(ass);
                }
            }
            RequestContext context = new RequestContext();
            context.setTenant(ass.getTenantId());
            context.setTarget(ass.getTenantId());
            context.setCatalog(Catalog.valueOf(ass.getGmtCatalog()));
            resourceEventPublisher
                .send(new OperateResult(true, "节点角色状态变更通知", "node_change_" + ass.getType().toLowerCase(), null,
                    OperateResult.OperateCategory.ServiceEvent, context));
        } catch (Exception e) {
            logger.error("集群角色同步失败：", e.getMessage());
        }
    }

    @Override
    public void install(String content, RequestContext context) {
        TaskNode node = JSONObject.parseObject(content, TaskNode.class);
        JSONObject output = JSONObject.parseObject(node.getOutput());
        if (null == output) {
            return;
        }
        List<JSONObject> hosts = JSONObject.parseArray(output.getString("data"), JSONObject.class);
        if (ListTool.isEmpty(hosts)) {
            return;
        }
        JSONObject input = JSONObject.parseObject(node.getParams());
        if (null == input) {
            return;
        }
        JSONObject target = JSONObject.parseObject(input.getString("target"));
        List<JSONObject> datas = JSONObject.parseArray(target.getString("data"), JSONObject.class);
        if (ListTool.isEmpty(datas)) {
            return;
        }
        Map<String, JSONObject> hostMap = new HashMap<>();
        for (JSONObject host : hosts) {
            if (host.getBoolean("success")) {
                hostMap.put(host.getString("host"), host);
            }
        }
        Integer port = null;
        ResourceCategory category = null;
        JSONObject fronts = JSONObject.parseObject(input.getString("fronts"));
        String version = fronts.getString("version");
        switch (node.getCode()) {
            case "task.software.tomcat":
                category = ResourceCategory.Tomcat;
                JSONObject tomcatConfig = fronts.getJSONObject("software_config");
                port = tomcatConfig.getInteger("tomcat_default_port_connector");
                port = null == port ? 8080 : port;
                break;
            case "task.software.apache":
                category = ResourceCategory.Apache;
                JSONObject apacheConfig = fronts.getJSONObject("software_config");
                port = apacheConfig.getInteger("apache_listen_port");
                port = null == port ? 80 : port;
                version = "2.4.6";
                break;
            case "task.software.weblogic":
                category = ResourceCategory.Weblogic;
                port = 7001;
                break;
            case "task.software.websphere":
                category = ResourceCategory.Websphere;
                port = 9060;
                version = "8.5.5";
                break;
            default:
                return;
        }
        try {
            String status = null;
            if (ResourceCategory.Weblogic.equals(category) || ResourceCategory.Websphere.equals(category)) {
                status = BocloudStatus.Server.STOPPED.name();
            } else {
                status = BocloudStatus.Server.RUNNING.name();
            }
            for (JSONObject data : datas) {
                if (!hostMap.containsKey(data.getString("host"))) {
                    continue;
                }
                Ass ass = new Ass();
                ass.setName(category.name() + System.currentTimeMillis());
                ass.setStatus(status);
                ass.setCreatorId(context.getTarget());
                ass.setTenantId(context.getTenant());
                ass.setInstanceId(IDFactory.instance().uuid());
                ass.setType(category.name());
                ass.setGmtCatalog(context.getCatalog().name());
                ass.setManageIp(data.getString("host"));
                ass.setCategory("standalone");
                if (null == port) {
                    ass.setAddress(data.getString("host"));
                } else {
                    ass.setAddress(data.getString("host") + ":" + port);
                }
                ass.setVersion(version);
                ass.setTaskId(node.getInstanceId());
                assRepository.save(ass);
                AssNode assNode = new AssNode(data.getString("host"), port, "standalone", status, null, ass.getId());
                assNode.setManageIp(data.getString("host"));
                if ("CloudServer".equals(data.getString("catalog"))) {
                    assNode.setServerId(data.getLong("id"));
                } else {
                    Resource resource = resourceRepository.query(data.getLong("id"));
                    if (null != resource) {
                        assNode.setServerId(resource.getTarget());
                    }
                }
                assNodeRepository.save(assNode);
                MDSUtils.send(amqpTemplate, ass.getId(), ass.getType(), "create");
            }
        } catch (Exception e) {
            logger.error("save ass softwart install failure:", e);
        }
    }

}
