package com.bocloud.pms.service;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.http.HttpClient;
import com.bocloud.common.model.*;
import com.bocloud.common.model.RequestContext.Catalog;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.GridHelper;
import com.bocloud.common.utils.IDFactory;
import com.bocloud.common.utils.ListTool;
import com.bocloud.pms.entity.Ccs;
import com.bocloud.pms.entity.CcsNode;
import com.bocloud.pms.entity.resource.Resource;
import com.bocloud.pms.entity.resource.TaskNode;
import com.bocloud.pms.entity.resource.enums.BocloudStatus;
import com.bocloud.pms.entity.resource.enums.ResourceCategory;
import com.bocloud.pms.entity.resource.enums.Status;
import com.bocloud.pms.interfaces.middleware.CcsService;
import com.bocloud.pms.repository.CcsNodeRepository;
import com.bocloud.pms.repository.CcsRepository;
import com.bocloud.pms.repository.resource.ResourceRepository;
import com.bocloud.pms.service.utils.MDSUtils;
import com.bocloud.pms.service.utils.ResourceEventPublisher;
import com.bocloud.service.lock.BocloudACLock;
import com.bocloud.service.lock.LockFactory;
import com.google.common.collect.Maps;
import org.apache.zookeeper.client.FourLetterWordMain;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.amqp.core.AmqpTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import java.io.ByteArrayInputStream;
import java.util.*;
import java.util.concurrent.TimeUnit;

/**
 * ccs Service接口实现
 *
 * @author weiwei
 */
@Service("ccsService")
public class CcsServiceImpl implements CcsService {

    private static Logger logger = LoggerFactory.getLogger(CcsServiceImpl.class);
    @Autowired
    private LockFactory lockFactory;
    @Autowired
    private CcsRepository ccsRepository;
    @Autowired
    private CcsNodeRepository ccsNodeRepository;
    @Autowired
    private ResourceRepository resourceRepository;
    @Autowired
    private RedisTemplate<String, String> redisTemplate;
    @Autowired
    private AmqpTemplate amqpTemplate;
    @Autowired
    private ResourceEventPublisher resourceEventPublisher;

    @Override
    public BocloudResult<GridBean<Ccs>> list(Pager pager, RequestContext context) {
        try {
            List<Param> params = pager.getParams();
            Integer page = pager.getPage();
            Integer rows = pager.getRows();
            Map<String, String> sorter = pager.getSorter();
            params = Optional.ofNullable(params).orElse(new ArrayList<>());
            sorter = Optional.ofNullable(sorter).orElse(Maps.newHashMap());
            sorter.put("gmtCreate", Common.ONE);
            List<Ccs> list = ccsRepository.list(page, rows, params, sorter);
            for (Ccs ccs : list) {
                JSONObject specs = JSONObject.parseObject(ccs.getSpec());
                if (null != specs) {
                    ccs.setSpec(specs.get("cpu") + "核/" + specs.get("memory") + "GB/" + specs.get("disk") + "GB");
                    ccs.setDataDisk(specs.getInteger("dataDisk"));
                }
                JSONObject inventory = JSONObject.parseObject(ccs.getInventory());
                if (null != inventory) {
                    List<JSONObject> elements = JSONArray.parseArray(inventory.getString("elements"), JSONObject.class);
                    if (!ListTool.isEmpty(elements)) {
                        for (JSONObject object : elements) {
                            if (object.getString("serviceCode").contains("ccs." + ccs.getType())) {
                                continue;
                            }
                            ccs.setDiskSkuId(object.getLong("skuId"));
                        }
                    }
                }
                ccs.setTenantName(redisTemplate.opsForValue().get("Tenant_Name_" + ccs.getTenantId()));
            }
            int total = ccsRepository.count(params);
            GridBean gridBean = GridHelper.getBean(page, rows, total, list);
            return new BocloudResult<>(true, gridBean, "查询成功");
        } catch (Exception e) {
            logger.error("list ccs failure:", e);
            return new BocloudResult<>(false, "查询失败", null, null);
        }
    }

    @Override
    public BocloudResult create(String params, RequestContext context) {
        return null;
    }

    @Override
    public void synchronousRoleState(JSONObject content) {
        try {
            Long id = content.getLong("id");
            Ccs ccs = ccsRepository.query(id);
            ccs = Optional.ofNullable(ccs).orElseThrow(() -> new Exception("数据不存在"));
            ResourceCategory category = ResourceCategory.getValue(ccs.getType());
            List<CcsNode> nodes = ccsNodeRepository.list(id);
            for (CcsNode node : nodes) {
                syncNodeStatus(node, category);
            }
            int statusCount = MDSUtils.statusCount(nodes, BocloudStatus.Server.RUNNING.name());
            if (0 == statusCount) {
                if (!BocloudStatus.Server.STOPPED.name().equals(ccs.getStatus())) {
                    ccs.setStatus(BocloudStatus.Server.STOPPED.name());
                    ccsRepository.update(ccs);
                }
            } else if (statusCount == nodes.size()) {
                if (!BocloudStatus.Server.RUNNING.name().equals(ccs.getStatus())) {
                    ccs.setStatus(BocloudStatus.Server.RUNNING.name());
                    ccsRepository.update(ccs);
                }
            } else {
                if (!BocloudStatus.Server.EXCEPTION.name().equals(ccs.getStatus())) {
                    ccs.setStatus(BocloudStatus.Server.EXCEPTION.name());
                    ccsRepository.update(ccs);
                }
            }
            RequestContext context = new RequestContext();
            context.setTenant(ccs.getTenantId());
            context.setTarget(ccs.getTenantId());
            context.setCatalog(Catalog.valueOf(ccs.getGmtCatalog()));
            resourceEventPublisher
                    .send(new OperateResult(true, "节点角色状态变更通知", "node_change_" + ccs.getType().toLowerCase(), null,
                            OperateResult.OperateCategory.ServiceEvent, context));
        } catch (Exception e) {
            logger.error("集群角色同步失败：", e.getMessage());
        }
    }

    @Override
    public BocloudResult<Ccs> basics(Long id) {
        try {
            Ccs ccs = ccsRepository.query(id);
            ccs = Optional.ofNullable(ccs).orElseThrow(() -> new Exception("数据不存在"));
            List<CcsNode> nodes = ccsNodeRepository.list(id);
            ccs.setNodes(nodes);
            return new BocloudResult<>(true, ccs, "查询成功");
        } catch (Exception e) {
            logger.error("query ccs detail fail:", e);
            return new BocloudResult<>(false, "查询详情失败");
        }
    }

    @Override
    public BocloudResult<Ccs> detail(Long id) {
        try {
            Ccs ccs = ccsRepository.query(id);
            ccs = Optional.ofNullable(ccs).orElseThrow(() -> new Exception("数据不存在"));
            ResourceCategory category = ResourceCategory.getValue(ccs.getType());
            List<CcsNode> nodes = ccsNodeRepository.list(id);
            for (CcsNode node : nodes) {
                syncNodeStatus(node, category);
            }
            int statusCount = MDSUtils.statusCount(nodes, BocloudStatus.Server.RUNNING.name());
            if (0 == statusCount) {
                if (!BocloudStatus.Server.STOPPED.name().equals(ccs.getStatus())) {
                    ccs.setStatus(BocloudStatus.Server.STOPPED.name());
                    ccsRepository.update(ccs);
                }
            } else if (statusCount == nodes.size()) {
                if (!BocloudStatus.Server.RUNNING.name().equals(ccs.getStatus())) {
                    ccs.setStatus(BocloudStatus.Server.RUNNING.name());
                    ccsRepository.update(ccs);
                }
            } else {
                if (!BocloudStatus.Server.EXCEPTION.name().equals(ccs.getStatus())) {
                    ccs.setStatus(BocloudStatus.Server.EXCEPTION.name());
                    ccsRepository.update(ccs);
                }
            }
            ccs.setNodes(nodes);
            JSONObject specs = JSONObject.parseObject(ccs.getSpec());
            if (null != specs) {
                ccs.setSpec(specs.get("cpu") + "核/" + specs.get("memory") + "GB/" + specs.get("disk") + "GB");
                ccs.setDataDisk(specs.getInteger("dataDisk"));
            }
            JSONObject inventory = JSONObject.parseObject(ccs.getInventory());
            if (null != inventory) {
                JSONObject configs = inventory.getJSONObject("configs");
                ccs.setDiskCategory(configs.getString("diskCategoryName"));
            }
            return new BocloudResult<>(true, ccs, "查询详情成功");
        } catch (Exception e) {
            logger.error("query ccs detail fail:", e);
            return new BocloudResult<>(false, "查询详情失败");
        }
    }

    private void syncNodeStatus(CcsNode node, ResourceCategory category) {
        if (ResourceCategory.Zookeeper.equals(category)) {
            try {
                String stat =
                        FourLetterWordMain.send4LetterWord(node.getManageIp(), node.getPort(), "stat", false, 5000);
                if (null != stat) {
                    Properties properties = new Properties();
                    ByteArrayInputStream stream = new ByteArrayInputStream(stat.getBytes());
                    properties.load(stream);
                    String mode = String.valueOf(properties.get("Mode"));
                    if (!"RUNNING".equals(node.getStatus()) || !mode.equals(node.getRole())) {
                        node.setStatus("RUNNING");
                        node.setRole(mode);
                        ccsNodeRepository.update(node);
                    }
                }
            } catch (Exception e) {
                logger.error("Zookeeper节点{" + node.getManageIp() + "}连接异常！");
                if (!"STOPPED".equals(node.getStatus()) || !"(not reachable/healthy)".equals(node.getRole())) {
                    node.setStatus("STOPPED");
                    node.setRole("(not reachable/healthy)");
                    ccsNodeRepository.update(node);
                }
            }
        } else if (ResourceCategory.Etcd.equals(category)) {
            HttpClient httpClient = new HttpClient(5000);
            Result result =
                    httpClient.get(null, null, "http://" + node.getManageIp() + ":" + node.getPort() + "/v2/stats/leader");
            if (result.isFailed()) {
                if (result.getMessage().contains("not current leader")) {
                    if (!"RUNNING".equals(node.getStatus()) || !"follower".equals(node.getRole())) {
                        node.setStatus("RUNNING");
                        node.setRole("follower");
                        ccsNodeRepository.update(node);
                    }
                } else {
                    if (!"STOPPED".equals(node.getStatus()) || !"(not reachable/healthy)".equals(node.getRole())) {
                        node.setStatus("STOPPED");
                        node.setRole("(not reachable/healthy)");
                        ccsNodeRepository.update(node);
                    }
                }
            } else {
                if (!"RUNNING".equals(node.getStatus()) || !"leader".equals(node.getRole())) {
                    node.setStatus("RUNNING");
                    node.setRole("leader");
                    ccsNodeRepository.update(node);
                }
            }
        }
    }

    @Override
    public BocloudResult remove(Long id, RequestContext context) {
        String path = Ccs.class.getSimpleName() + "_" + id;
        try (BocloudACLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            Ccs ccs = ccsRepository.query(id);
            ccs = Optional.ofNullable(ccs).orElseThrow(() -> new Exception("数据不存在"));
            if (null != ccs.getTenantId() && ccs.getTenantId() > 0) {
                return new BocloudResult(false, "租户数据不可删除");
            }
            ccsNodeRepository.deleteByCcs(ccs.getId());
            ccsRepository.remove(ccs.getId(), context.getTarget());
            MDSUtils.send(amqpTemplate, ccs.getId(), ccs.getType(), "delete");
            return new BocloudResult(true, ccs.getName() + "删除成功");
        } catch (Exception e) {
            logger.error("remove ccs failure:", e);
            return new BocloudResult(false, "删除失败");
        }
    }

    @Override
    public void install(String content, RequestContext context) {
        TaskNode node = JSONObject.parseObject(content, TaskNode.class);
        JSONObject output = JSONObject.parseObject(node.getOutput());
        if (null == output) {
            return;
        }
        List<JSONObject> hosts = JSONObject.parseArray(output.getString("data"), JSONObject.class);
        if (ListTool.isEmpty(hosts)) {
            return;
        }
        JSONObject input = JSONObject.parseObject(node.getParams());
        if (null == input) {
            return;
        }
        Integer port = null;
        ResourceCategory category = null;
        JSONObject fronts = JSONObject.parseObject(input.getString("fronts"));
        String version = fronts.getString("version");
        JSONObject config = fronts.getJSONObject("software_config");
        switch (node.getCode()) {
            case "task.software.zookeeper":
                category = ResourceCategory.Zookeeper;
                port = config.getInteger("client_port");
                port = null == port ? 2181 : port;
                break;
            case "task.software.etcd":
                category = ResourceCategory.Etcd;
                port = config.getInteger("etcd_client_port");
                port = null == port ? 2379 : port;
                break;
            default:
                return;
        }
        try {
            if (null != fronts.getBoolean("cluster_install") && fronts.getBoolean("cluster_install")) {
                if (Status.Node.FAILED.name().equals(node.getStatus())) {
                    return;
                }
                Ccs ccs = new Ccs();
                ccs.setName(category.name() + System.currentTimeMillis());
                ccs.setStatus(BocloudStatus.Server.RUNNING.name());
                ccs.setCreatorId(context.getTarget());
                ccs.setTenantId(context.getTenant());
                ccs.setInstanceId(IDFactory.instance().uuid());
                ccs.setType(category.name());
                ccs.setGmtCatalog(context.getCatalog().name());
                ccs.setVersion(version);
                ccs.setCategory(hosts.size() > 1 ? "cluster" : "standalone");
                ccs.setTaskId(node.getInstanceId());
                ccsRepository.save(ccs);
                String role = hosts.size() > 1 ? "node" : "standalone";
                String address = "";
                List<CcsNode> nodes = new ArrayList<>();
                for (JSONObject host : hosts) {
                    CcsNode ccsNode = new CcsNode(host.getString("host"), port, role,
                            BocloudStatus.Server.RUNNING.name(), null, ccs.getId());
                    ccsNode.setManageIp(host.getString("host"));
                    if ("CloudServer".equals(host.getString("catalog"))) {
                        ccsNode.setServerId(host.getLong("id"));
                    } else {
                        Resource resource = resourceRepository.query(host.getLong("id"));
                        if (null != resource) {
                            ccsNode.setServerId(resource.getTarget());
                        }
                    }
                    ccsNodeRepository.save(ccsNode);
                    address += host.getString("host") + ":" + port + ",";
                    syncNodeStatus(ccsNode, category);
                    nodes.add(ccsNode);
                }
                ccs.setAddress(address.substring(0, address.length() - 1));
                int statusCount = MDSUtils.statusCount(nodes, BocloudStatus.Server.RUNNING.name());
                if (0 == statusCount) {
                    if (!BocloudStatus.Server.STOPPED.name().equals(ccs.getStatus())) {
                        ccs.setStatus(BocloudStatus.Server.STOPPED.name());
                    }
                } else if (statusCount == nodes.size()) {
                    if (!BocloudStatus.Server.RUNNING.name().equals(ccs.getStatus())) {
                        ccs.setStatus(BocloudStatus.Server.RUNNING.name());
                    }
                } else {
                    if (!BocloudStatus.Server.EXCEPTION.name().equals(ccs.getStatus())) {
                        ccs.setStatus(BocloudStatus.Server.EXCEPTION.name());
                    }
                }
                ccsRepository.update(ccs);
                MDSUtils.send(amqpTemplate, ccs.getId(), ccs.getType(), "create");
                return;
            }
            JSONObject target = JSONObject.parseObject(input.getString("target"));
            List<JSONObject> datas = JSONObject.parseArray(target.getString("data"), JSONObject.class);
            if (ListTool.isEmpty(datas)) {
                return;
            }
            Map<String, JSONObject> hostMap = new HashMap<>();
            for (JSONObject host : hosts) {
                if (host.getBoolean("success")) {
                    hostMap.put(host.getString("host"), host);
                }
            }
            for (JSONObject data : datas) {
                if (!hostMap.containsKey(data.getString("host"))) {
                    continue;
                }
                Ccs ccs = new Ccs();
                ccs.setName(category.name() + System.currentTimeMillis());
                ccs.setStatus(BocloudStatus.Server.RUNNING.name());
                ccs.setCreatorId(context.getTarget());
                ccs.setTenantId(context.getTenant());
                ccs.setInstanceId(IDFactory.instance().uuid());
                ccs.setType(category.name());
                ccs.setGmtCatalog(context.getCatalog().name());
                ccs.setVersion(version);
                ccs.setCategory("standalone");
                ccs.setManageIp(data.getString("host"));
                ccs.setAddress(data.getString("host") + ":" + port);
                ccs.setTaskId(node.getInstanceId());
                ccsRepository.save(ccs);
                CcsNode ccsNode = new CcsNode(data.getString("host"), port, "standalone",
                        BocloudStatus.Server.RUNNING.name(), null, ccs.getId());
                ccsNode.setManageIp(data.getString("host"));
                if ("CloudServer".equals(data.getString("catalog"))) {
                    ccsNode.setServerId(data.getLong("id"));
                } else {
                    Resource resource = resourceRepository.query(data.getLong("id"));
                    if (null != resource) {
                        ccsNode.setServerId(resource.getTarget());
                    }
                }
                ccsNodeRepository.save(ccsNode);
                MDSUtils.send(amqpTemplate, ccs.getId(), ccs.getType(), "create");
            }
        } catch (Exception e) {
            logger.error("save ccs softwart install failure:", e);
        }
    }

}
