package com.bocloud.pms.service;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.GridBean;
import com.bocloud.common.model.Result;
import com.bocloud.common.utils.GridHelper;
import com.bocloud.pms.entity.Cms;
import com.bocloud.pms.entity.CmsBinding;
import com.bocloud.pms.entity.resource.enums.ResourceCategory;
import com.bocloud.pms.interfaces.middleware.CmsBindingsService;
import com.bocloud.pms.repository.CmsRepository;
import com.bocloud.pms.service.utils.MDSUtils;
import com.bocloud.pms.service.utils.RabbitMQHelper;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * Created by liuyuanyuan on 2020/3/17 17:24
 */
@Service("cmsBindingsService")
public class CmsBindingsServiceImpl implements CmsBindingsService {

    private static Logger logger = LoggerFactory.getLogger(CmsBindingsServiceImpl.class);
    @Autowired
    private CmsRepository cmsRepository;

    @Override
    public BocloudResult<GridBean<CmsBinding>> list(Long mqId, int page, int rows, CmsBinding mqBinding) {
        try {
            Cms cms = cmsRepository.query(mqId);
            if (null == cms) {
                logger.warn("cms not exist");
                return new BocloudResult<>(false, "message queue不存在");
            }
            if (ResourceCategory.RabbitMQ.name().equalsIgnoreCase(cms.getType())) {
                String url = RabbitMQHelper.buildBindingUrl(cms.getManageIp(), mqBinding, "list");
                Result result = RabbitMQHelper.get(url, cms.getUsername(), MDSUtils.decrypt(cms.getPassword()));
                if (result.isFailed()) {
                    logger.error("get bindings data is error{}", result.getMessage());
                    return new BocloudResult<>(false, "获取bindings失败");
                }
                List<CmsBinding> mqBindings = JSONArray.parseArray(result.getMessage(), CmsBinding.class);
                List<CmsBinding> list = (List<CmsBinding>) RabbitMQHelper.pageUtil(mqBindings, page, rows);
                GridBean gridBean = GridHelper.getBean(page, rows, mqBindings.size(), list);
                return new BocloudResult<>(true, gridBean, "查询bindings成功");
            }
            return new BocloudResult<>(false, "暂不支持其他消息队列");
        } catch (Exception e) {
            logger.error("get bindings failure:", e);
            return new BocloudResult<>(false, "获取bindings失败");
        }
    }

    @Override
    public BocloudResult create(Long mqId, CmsBinding mqBinding) {
        try {
            Cms cms = cmsRepository.query(mqId);
            if (null == cms) {
                logger.warn("cms not exist");
                return new BocloudResult(false, "message queue不存在");
            }
            if (ResourceCategory.RabbitMQ.name().equalsIgnoreCase(cms.getType())) {
                String url = RabbitMQHelper.buildBindingUrl(cms.getManageIp(), mqBinding, "create");
                Result result = RabbitMQHelper.post(url, RabbitMQHelper.buildBindingsModel(mqBinding),
                        cms.getUsername(), MDSUtils.decrypt(cms.getPassword()));
                if (result.isFailed()) {
                    logger.error("create bindings is error{}", result.getMessage());
                    return new BocloudResult(false, "创建bindings失败");
                }
                return new BocloudResult(true, cms.getId(), "创建bindings成功");
            }
            return new BocloudResult(false, "暂不支持其他消息队列");
        } catch (Exception e) {
            logger.error("create bindings failure:", e);
            return new BocloudResult(false, "创建bindings失败");
        }
    }

    @Override
    public BocloudResult remove(Long mqId, CmsBinding mqBinding) {
        try {
            Cms cms = cmsRepository.query(mqId);
            if (null == cms) {
                logger.warn("cms not exist");
                return new BocloudResult(false, "message queue不存在");
            }
            if (ResourceCategory.RabbitMQ.name().equalsIgnoreCase(cms.getType())) {
                String url = RabbitMQHelper.buildBindingUrl(cms.getManageIp(), mqBinding, "delete");
                Result result = RabbitMQHelper.delete(url, null, cms.getUsername(),
                        MDSUtils.decrypt(cms.getPassword()));
                if (result.isFailed()) {
                    logger.error("delete bindings is error{}", result.getMessage());
                    return new BocloudResult(false, cms.getId(), "删除bindings失败");
                }
                return new BocloudResult(true, cms.getId(), "删除bindings成功");
            }
            return new BocloudResult(false, "暂不支持其他消息队列");
        } catch (Exception e) {
            logger.error("delete bindings failure:", e);
            return new BocloudResult(false, "删除bindings失败");
        }
    }

    @Override
    public BocloudResult<CmsBinding> detail(Long mqId, CmsBinding mqBinding) {
        try {
            Cms cms = cmsRepository.query(mqId);
            if (null == cms) {
                logger.warn("cms not exist");
                return new BocloudResult<>(false, "message queue不存在");
            }
            if (ResourceCategory.RabbitMQ.name().equalsIgnoreCase(cms.getType())) {
                String url = RabbitMQHelper.buildBindingUrl(cms.getManageIp(), mqBinding, "detail");
                Result result = RabbitMQHelper.get(url, cms.getUsername(), MDSUtils.decrypt(cms.getPassword()));
                if (result.isFailed()) {
                    logger.error("get bindings data is error{}", result.getMessage());
                    return new BocloudResult<>(false, "获取bindings失败");
                }
                CmsBinding binding = JSONObject.parseObject(result.getMessage(), CmsBinding.class);
                return new BocloudResult<>(true, binding, "查询bindings成功");
            }
            return new BocloudResult<>(false, "暂不支持其他消息队列");
        } catch (Exception e) {
            logger.error("get bindings failure:", e);
            return new BocloudResult<>(false, "获取bindings失败");
        }
    }

}
