package com.bocloud.pms.service;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.GridBean;
import com.bocloud.common.model.Result;
import com.bocloud.common.utils.GridHelper;
import com.bocloud.pms.entity.Cms;
import com.bocloud.pms.entity.CmsChannel;
import com.bocloud.pms.entity.CmsConsumer;
import com.bocloud.pms.entity.resource.enums.ResourceCategory;
import com.bocloud.pms.entity.resource.model.ChannelModel;
import com.bocloud.pms.interfaces.middleware.CmsChannelService;
import com.bocloud.pms.repository.CmsRepository;
import com.bocloud.pms.service.utils.MDSUtils;
import com.bocloud.pms.service.utils.RabbitMQHelper;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * Created by liuyuanyuan on 2020/3/18 17:28
 */
@Service("cmsChannelService")
public class CmsChannelServiceImpl implements CmsChannelService {
    private static Logger logger = LoggerFactory.getLogger(CmsChannelServiceImpl.class);
    @Autowired
    private CmsRepository cmsRepository;

    @Override
    public BocloudResult<GridBean<CmsChannel>> list(Long mqId, int page, int rows) {
        GridBean gridBean = null;
        try {
            Cms cms = cmsRepository.query(mqId);
            if (null == cms) {
                logger.warn("cms not exist");
                return new BocloudResult<>(false, "message queue不存在");
            }
            if (ResourceCategory.RabbitMQ.name().equalsIgnoreCase(cms.getType())) {
                String url = RabbitMQHelper.buildChannelnUrl(cms.getManageIp(), null, "list");
                Result result = RabbitMQHelper.get(url, cms.getUsername(), MDSUtils.decrypt(cms.getPassword()));
                if (result.isFailed()) {
                    logger.error("get channels data is error{}", result.getMessage());
                    return new BocloudResult<>(false, "获取channels数据失败");
                }
                List<CmsChannel> channels = JSONArray.parseArray(result.getMessage(), CmsChannel.class);
                if (!channels.isEmpty()) {
                    for (CmsChannel channel : channels) {
                        JSONObject messageStats = channel.getMessageStats();
                        if (null == messageStats) {
                            continue;
                        }
                        JSONObject publishDetails = messageStats.getJSONObject("publish_details");
                        if (null != publishDetails) {
                            channel.setPublishRates(publishDetails.getLong("rate"));
                        }
                        JSONObject confirmDetails = messageStats.getJSONObject("confirm_details");
                        if (null != confirmDetails) {
                            channel.setConfirmRates(confirmDetails.getLong("rate"));
                        }
                        JSONObject deliverGetDetails = messageStats.getJSONObject("deliver_get_details");
                        if (null != deliverGetDetails) {
                            channel.setDeliverGetRates(deliverGetDetails.getLong("rate"));
                        }
                        JSONObject ackDetails = messageStats.getJSONObject("ack_details");
                        if (null != ackDetails) {
                            channel.setAckRates(ackDetails.getLong("rate"));
                        }
                    }
                }
                gridBean = GridHelper.getBean(page, rows, channels.size(),
                        RabbitMQHelper.pageUtil(channels, page, rows));
                return new BocloudResult<>(true, gridBean, "获取channels数据成功");
            }
            return new BocloudResult<>(false, "暂不支持其他消息队列");
        } catch (Exception e) {
            logger.error("get channels failure:", e);
            return new BocloudResult<>(false, "获取channels数据失败");
        }
    }

    @Override
    public BocloudResult<ChannelModel> detail(Long mqId, String name) {
        try {
            Cms cms = cmsRepository.query(mqId);
            if (null == cms) {
                logger.warn("cms not exist");
                return new BocloudResult<>(false, "message queue不存在");
            }
            ChannelModel channelModel = new ChannelModel();
            if (ResourceCategory.RabbitMQ.name().equalsIgnoreCase(cms.getType())) {
                // 获取connection Details
                String url = RabbitMQHelper.buildChannelnUrl(cms.getManageIp(), name, "detail");
                Result result = RabbitMQHelper.get(url, cms.getUsername(), MDSUtils.decrypt(cms.getPassword()));
                if (result.isFailed()) {
                    logger.error("get channels details data is error{}", result.getMessage());
                    return new BocloudResult<>(false, "获取channels详情失败");
                }
                CmsChannel mqChannel = JSONObject.parseObject(result.getMessage(), CmsChannel.class);
                JSONObject connectionDetails = mqChannel.getConnectionDetails();
                if (null != connectionDetails) {
                    mqChannel.setConnectionName(
                            StringUtils.substringBefore(connectionDetails.getString("name"), "-").trim());
                }
                if (null != mqChannel) {
                    JSONArray consumerDetails = mqChannel.getConsumerDetails();
                    List<CmsConsumer> consumers = JSONArray.parseArray(consumerDetails.toJSONString(), CmsConsumer.class);
                    channelModel.setDetails(mqChannel);
                    if (!consumers.isEmpty()) {
                        channelModel.setConsumers(consumers);
                    } else {
                        channelModel.setConsumers(null);
                    }
                }
                return new BocloudResult<>(true, channelModel, "获取channels详情成功");
            }
            return new BocloudResult<>(false, "暂不支持其他消息队列");
        } catch (Exception e) {
            logger.error("get channels details failure:", e);
            return new BocloudResult<>(false, "获取channels详情失败");
        }
    }
}
