package com.bocloud.pms.service;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.GridBean;
import com.bocloud.common.model.Result;
import com.bocloud.common.utils.GridHelper;
import com.bocloud.pms.entity.Cms;
import com.bocloud.pms.entity.CmsChannel;
import com.bocloud.pms.entity.CmsClientProperties;
import com.bocloud.pms.entity.CmsConnection;
import com.bocloud.pms.entity.resource.enums.ResourceCategory;
import com.bocloud.pms.entity.resource.model.ConnectionModel;
import com.bocloud.pms.interfaces.middleware.CmsConnectionService;
import com.bocloud.pms.repository.CmsRepository;
import com.bocloud.pms.service.utils.MDSUtils;
import com.bocloud.pms.service.utils.RabbitMQHelper;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;

/**
 * Created by liuyuanyuan on 2020/3/18 11:42
 */
@Service("cmsConnectionService")
public class CmsConnectionServiceImpl implements CmsConnectionService {
    private static Logger logger = LoggerFactory.getLogger(CmsConnectionServiceImpl.class);
    @Autowired
    private CmsRepository cmsRepository;

    @Override
    public BocloudResult close(Long mqId, String name, String xReason) {
        try {
            Cms cms = cmsRepository.query(mqId);
            if (null == cms) {
                logger.warn("cms not exist");
                return new BocloudResult(false, "message queue不存在");
            }
            if (ResourceCategory.RabbitMQ.name().equalsIgnoreCase(cms.getType())) {
                String url = RabbitMQHelper.buildConnectionUrl(cms.getManageIp(), name, "close");
                Result result = RabbitMQHelper.delete(url, RabbitMQHelper.buildConnectionModel(xReason),
                        cms.getUsername(), MDSUtils.decrypt(cms.getPassword()));
                if (result.isFailed()) {
                    logger.error("close connection is error{}", result.getMessage());
                    return new BocloudResult(false, cms.getId(), "关闭连接失败");
                }
                return new BocloudResult(true, cms.getId(), "关闭连接成功");
            }
            return new BocloudResult(false, cms.getId(), "暂不支持其他消息队列");
        } catch (Exception e) {
            logger.error("close connection failure:", e);
            return new BocloudResult(false, "关闭连接失败");
        }
    }

    @Override
    public BocloudResult<GridBean<CmsConnection>> list(Long mqId, int page, int rows) {
        GridBean gridBean = null;
        try {
            Cms cms = cmsRepository.query(mqId);
            if (null == cms) {
                logger.warn("cms not exist");
                return new BocloudResult<>(false, "message queue不存在");
            }
            if (ResourceCategory.RabbitMQ.name().equalsIgnoreCase(cms.getType())) {
                String url = RabbitMQHelper.buildConnectionUrl(cms.getManageIp(), null, "list");
                Result result = RabbitMQHelper.get(url, cms.getUsername(), MDSUtils.decrypt(cms.getPassword()));
                if (result.isFailed()) {
                    logger.error("get connection data is error{}", result.getMessage());
                    return new BocloudResult<>(false, "获取connection数据失败");
                }
                List<CmsConnection> connections = JSONArray.parseArray(result.getMessage(), CmsConnection.class);
                if (!connections.isEmpty()) {
                    for (CmsConnection connection : connections) {
                        JSONObject recvOctDetails = connection.getRecvOctDetails();
                        if (null != recvOctDetails) {
                            connection.setFromClient(recvOctDetails.getLong("rate"));
                        }
                        JSONObject sendOctDetails = connection.getSendOctDetails();
                        if (null != sendOctDetails) {
                            connection.setToClient(sendOctDetails.getLong("rate"));
                        }
                    }
                }
                gridBean = GridHelper.getBean(page, rows, connections.size(),
                        RabbitMQHelper.pageUtil(connections, page, rows));
                return new BocloudResult<>(true, gridBean, "获取connection数据成功");
            }
            return new BocloudResult<>(false, "暂不支持其他消息队列");
        } catch (Exception e) {
            logger.error("get connection failure:", e);
            return new BocloudResult<>(false, "获取connection数据失败");
        }
    }

    @Override
    public BocloudResult<ConnectionModel> detail(Long mqId, String name) {
        try {
            Cms cms = cmsRepository.query(mqId);
            if (null == cms) {
                logger.warn("cms not exist");
                return new BocloudResult<>(false, "message queue不存在");
            }
            if (ResourceCategory.RabbitMQ.name().equalsIgnoreCase(cms.getType())) {
                // 获取connection Details
                String urlDetail = RabbitMQHelper.buildConnectionUrl(cms.getManageIp(), name, "detail");
                Result resultDetail = RabbitMQHelper.get(urlDetail, cms.getUsername(),
                        MDSUtils.decrypt(cms.getPassword()));
                if (resultDetail.isFailed()) {
                    logger.error("get connection details data is error{}", resultDetail.getMessage());
                    return new BocloudResult<>(false, "获取connection详情失败");
                }
                CmsConnection connection = JSONObject.parseObject(resultDetail.getMessage(), CmsConnection.class);
                // 获取connection channels数据
                String urlChannels = RabbitMQHelper.buildConnectionUrl(cms.getManageIp(), name, "getChannels");
                Result resultChannels = RabbitMQHelper.get(urlChannels, cms.getUsername(),
                        MDSUtils.decrypt(cms.getPassword()));
                if (resultChannels.isFailed()) {
                    logger.error("get connection channels data is error{}", resultChannels.getMessage());
                    return new BocloudResult<>(false, "获取connection channels失败");
                }
                List<CmsChannel> channels = JSONArray.parseArray(resultChannels.getMessage(), CmsChannel.class);
                for (CmsChannel channel : channels) {
                    JSONObject messageStats = channel.getMessageStats();
                    if (null != messageStats) {
                        channel.setPublishRates(null == messageStats.getJSONObject("publish_details") ? null : messageStats.getJSONObject("publish_details").getLong("rate"));
                        channel.setConfirmRates(null == messageStats.getJSONObject("confirm_details") ? null : messageStats.getJSONObject("confirm_details").getLong("rate"));
                        channel.setDeliverGetRates(null == messageStats.getJSONObject("deliver_get_details") ? null : messageStats.getJSONObject("deliver_get_details").getLong("rate"));
                        channel.setAckRates(null == messageStats.getJSONObject("ack_details") ? null : messageStats.getJSONObject("ack_details").getLong("rate"));
                    }
                }
                // 整合数据
                ConnectionModel connectionModel = new ConnectionModel();
                // 获取clientProperties
                CmsClientProperties clientProperties = JSONObject
                        .parseObject(String.valueOf(connection.getClientProperties()), CmsClientProperties.class);
                connection.setClientProvidedName(clientProperties.getConnectionName());
                //将时间戳转为时间格式
                SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
                Date date = new Date(new Long(connection.getConnectedAt()));
                connection.setConnectedAt(simpleDateFormat.format(date));
                connectionModel.setDetails(connection);
                connectionModel.setChannels(channels);
                connectionModel.setClientProperties(clientProperties);
                return new BocloudResult<>(true, connectionModel, "查询connection信息成功");
            }
            return new BocloudResult<>(false,"暂不支持其他消息队列");
        } catch (Exception e) {
            logger.error("get connection failure:", e);
            return new BocloudResult<>(false, "获取connection失败");
        }
    }
}
