package com.bocloud.pms.service;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.GridBean;
import com.bocloud.common.model.RequestContext;
import com.bocloud.common.model.Result;
import com.bocloud.common.utils.GridHelper;
import com.bocloud.common.utils.ListTool;
import com.bocloud.pms.entity.Cms;
import com.bocloud.pms.entity.CmsExchange;
import com.bocloud.pms.entity.resource.enums.ResourceCategory;
import com.bocloud.pms.interfaces.middleware.CmsExchangeService;
import com.bocloud.pms.repository.CmsRepository;
import com.bocloud.pms.service.utils.MDSUtils;
import com.bocloud.pms.service.utils.RabbitMQHelper;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * Created by liuyuanyuan on 2020/3/10 18:16
 */
@Service("cmsExchangeService")
public class CmsExchangeServiceImpl implements CmsExchangeService {

    private static Logger logger = LoggerFactory.getLogger(CmsExchangeServiceImpl.class);
    @Autowired
    private CmsRepository cmsRepository;

    @Override
    public BocloudResult<GridBean<CmsExchange>> list(Long mqId, int page, int rows) {
        GridBean gridBean = null;
        try {
            Cms cms = cmsRepository.query(mqId);
            if (null == cms) {
                logger.warn("cms not exist");
                return new BocloudResult<>(false, "message queue不存在");
            }
            if (ResourceCategory.RabbitMQ.name().equalsIgnoreCase(cms.getType())) {
                String url = RabbitMQHelper.buildExchangeUrl(cms.getManageIp(), null, null, "list");
                Result result = RabbitMQHelper.get(url, cms.getUsername(), MDSUtils.decrypt(cms.getPassword()));
                if (result.isFailed()) {
                    logger.error("get exchange data is error{}", result.getMessage());
                    return new BocloudResult<>(false, "获取交换机失败");
                }
                List<CmsExchange> exchanges = JSONArray.parseArray(result.getMessage(), CmsExchange.class);
                if (!ListTool.isEmpty(exchanges)) {
                    for (CmsExchange exchange : exchanges) {
                        if (null == exchange.getName() || "".equals(exchange.getName())) {
                            exchange.setName("amq.default");
                        }
                        JSONObject stats = exchange.getMessageStats();
                        if (null == stats) {
                            continue;
                        }
                        JSONObject inDetails = stats.getJSONObject("publish_in_details");
                        if (null != inDetails) {
                            exchange.setPublishInRate(inDetails.getLong("rate"));
                        }
                        JSONObject outDetails = stats.getJSONObject("publish_out_details");
                        if (null != outDetails) {
                            exchange.setPublishOutRate(outDetails.getLong("rate"));
                        }
                    }
                }
                gridBean = GridHelper.getBean(page, rows, exchanges.size(),
                        RabbitMQHelper.pageUtil(exchanges, page, rows));
                return new BocloudResult<>(true, gridBean, "查询交换机成功");
            }
            return new BocloudResult<>(false, "暂不支持其他消息队列");
        } catch (Exception e) {
            logger.error("get exchange failure:", e);
            return new BocloudResult<>(false, "获取交换机失败");
        }
    }

    @Override
    public BocloudResult create(Long mqId, CmsExchange exchange, RequestContext context) {
        try {
            Cms cms = cmsRepository.query(mqId);
            if (null == cms) {
                logger.warn("cms not exist");
                return new BocloudResult(false, "message queue不存在");
            }
            if (ResourceCategory.RabbitMQ.name().equalsIgnoreCase(cms.getType())) {
                String url = RabbitMQHelper.buildExchangeUrl(cms.getManageIp(), exchange.getVhost(), exchange.getName(),
                        "create");
                Result result = RabbitMQHelper.put(url, RabbitMQHelper.buildExchangeModel(exchange), cms.getUsername(),
                        MDSUtils.decrypt(cms.getPassword()));
                if (result.isFailed()) {
                    logger.error("create exchange is error{}", result.getMessage());
                    return new BocloudResult(false, "创建交换机失败");
                }
                return new BocloudResult(true, cms.getId(), "创建交换机成功");
            }
            return new BocloudResult(false, cms.getId(), "暂不支持其他消息队列");
        } catch (Exception e) {
            logger.error("create exchange failure:", e);
            return new BocloudResult(false, "创建交换机失败");
        }

    }

    @Override
    public BocloudResult remove(String vhost, Long mqId, String exchangeName) {
        try {
            Cms cms = cmsRepository.query(mqId);
            if (null == cms) {
                logger.warn("cms not exist");
                return new BocloudResult(false, "message queue不存在");
            }
            if (ResourceCategory.RabbitMQ.name().equalsIgnoreCase(cms.getType())) {
                String url = RabbitMQHelper.buildExchangeUrl(cms.getManageIp(), vhost, exchangeName, "delete");
                Result result = RabbitMQHelper.delete(url, null, cms.getUsername(),
                        MDSUtils.decrypt(cms.getPassword()));
                if (result.isFailed()) {
                    logger.error("delete exchange is error{}", result.getMessage());
                    return new BocloudResult(false, cms.getId(), "删除交换机失败");
                }
                return new BocloudResult(true, cms.getId(), "删除交换机成功");
            }
            return new BocloudResult(false, cms.getId(), "暂不支持其他消息队列");
        } catch (Exception e) {
            logger.error("delete exchange failure:", e);
            return new BocloudResult(false, "删除交换机失败");
        }
    }

    @Override
    public BocloudResult<CmsExchange> detail(Long mqId, String exchangeName, String vhost) {
        try {
            Cms cms = cmsRepository.query(mqId);
            if (null == cms) {
                logger.warn("cms not exist");
                return new BocloudResult<>(false, "message queue不存在");
            }
            if (ResourceCategory.RabbitMQ.name().equalsIgnoreCase(cms.getType())) {
                String url = RabbitMQHelper.buildExchangeUrl(cms.getManageIp(), vhost, exchangeName, "detail");
                Result result = RabbitMQHelper.get(url, cms.getUsername(), MDSUtils.decrypt(cms.getPassword()));
                if (result.isFailed()) {
                    logger.error("get exchange data is error{}", result.getMessage());
                    return new BocloudResult<>(false, "获取交换机失败");
                }
                CmsExchange exchange = JSONObject.parseObject(result.getMessage(), CmsExchange.class);
                return new BocloudResult<>(true, exchange, "查询交换机成功");
            }
            return new BocloudResult<>(false, "暂不支持其他消息队列");
        } catch (Exception e) {
            logger.error("get exchange failure:", e);
            return new BocloudResult<>(false, "获取交换机失败");
        }
    }

}
