package com.bocloud.pms.service;

import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.GridBean;
import com.bocloud.common.model.Pager;
import com.bocloud.common.model.Result;
import com.bocloud.common.utils.GridHelper;
import com.bocloud.common.utils.ListTool;
import com.bocloud.pms.entity.Cms;
import com.bocloud.pms.entity.CmsUser;
import com.bocloud.pms.entity.CmsVhost;
import com.bocloud.pms.entity.resource.enums.ResourceCategory;
import com.bocloud.pms.interfaces.middleware.CmsManageService;
import com.bocloud.pms.repository.CmsRepository;
import com.bocloud.pms.service.utils.MDSUtils;
import com.bocloud.pms.service.utils.RabbitMQHelper;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;

/**
 * Created by liuyuanyuan on 2020/3/12 10:03
 */
@Service("cmsManageService")
public class CmsManageServiceImpl implements CmsManageService {

    private static Logger logger = LoggerFactory.getLogger(CmsManageServiceImpl.class);
    @Autowired
    private CmsRepository cmsRepository;

    @Override
    public BocloudResult<GridBean<CmsVhost>> listOfVhost(Long mqId, Pager pager) {
        GridBean gridBean = null;
        try {
            Cms cms = cmsRepository.query(mqId);
            if (null == cms) {
                logger.warn("cms not exist");
                return new BocloudResult<>(false, "message queue不存在");
            }
            if (ResourceCategory.RabbitMQ.name().equalsIgnoreCase(cms.getType())) {
                String url = RabbitMQHelper.buildUrl(cms.getManageIp(), null, "vhost", true);
                Result result = RabbitMQHelper.get(url, cms.getUsername(), MDSUtils.decrypt(cms.getPassword()));
                if (result.isFailed()) {
                    logger.error("get vhost data is error{}", result.getMessage());
                    return new BocloudResult<>(false, "获取vhost失败");
                }
                List<CmsVhost> vhosts = JSONArray.parseArray(result.getMessage(), CmsVhost.class);
                Result permissions = RabbitMQHelper.get(RabbitMQHelper.buildPermissionsUrl(cms.getManageIp()),
                    cms.getUsername(), MDSUtils.decrypt(cms.getPassword()));
                if (permissions.isFailed()) {
                    logger.error("get permissions data is error{}", result.getMessage());
                    return new BocloudResult<>(false, "获取数据失败");
                }
                Map<String, List<String>> userMap = Maps.newHashMap();
                List<JSONObject> permissionList =
                    JSONArray.parseArray(permissions.getMessage()).toJavaList(JSONObject.class);
                for (JSONObject jsonObject : permissionList) {
                    List<String> vhostList = userMap.get(jsonObject.getString("vhost"));
                    if (null == vhostList) {
                        vhostList = Lists.newArrayList();
                        vhostList.add(jsonObject.getString("user"));
                        userMap.put(jsonObject.getString("vhost"), vhostList);
                    } else {
                        vhostList.add(jsonObject.getString("user"));
                    }
                }
                if (!vhosts.isEmpty()) {
                    for (CmsVhost vhost : vhosts) {
                        // if (vhost.getName().equals("/")) {
                        // continue;
                        // }
                        List<String> users = userMap.get(vhost.getName());
                        if (!ListTool.isEmpty(users)) {
                            vhost.setUsers(String.valueOf(users));
                        }
                        JSONObject recvOctDetails = vhost.getRecvOctDetails();
                        if (null != recvOctDetails) {
                            vhost.setRecvOctRate(recvOctDetails.getLong("rate"));
                        }
                        JSONObject sendOctDetails = vhost.getSendOctDetails();
                        if (null != sendOctDetails) {
                            vhost.setSendOctRate(sendOctDetails.getLong("rate"));
                        }
                        JSONObject messageStats = vhost.getMessageStats();
                        if (null != messageStats) {
                            JSONObject publishDetails = messageStats.getJSONObject("publish_details");
                            if (null != publishDetails) {
                                vhost.setPublishRate(publishDetails.getLong("rate"));
                            }
                            JSONObject deliverGetDetails = messageStats.getJSONObject("deliver_get_details");
                            if (null != deliverGetDetails) {
                                vhost.setDeliverRate(deliverGetDetails.getLong("rate"));
                            }
                        }
                    }
                    gridBean = GridHelper.getBean(pager.getPage(), pager.getRows(), vhosts.size(),
                        RabbitMQHelper.pageUtil(vhosts, pager.getPage(), pager.getRows()));
                    return new BocloudResult<>(true, gridBean, "查询vhost成功");
                }
            }
            return new BocloudResult<>(false, "暂不支持其他消息队列");
        } catch (Exception e) {
            logger.error("get vhost failure:", e);
            return new BocloudResult<>(false, "获取vhost失败");
        }
    }

    @Override
    public BocloudResult create(CmsVhost mqVhost, Long mqId) {
        try {
            Cms cms = cmsRepository.query(mqId);
            if (null == cms) {
                logger.warn("cms not exist");
                return new BocloudResult(false, "message queue不存在");
            }
            if (ResourceCategory.RabbitMQ.name().equalsIgnoreCase(cms.getType())) {
                String url = RabbitMQHelper.buildUrl(cms.getManageIp(), mqVhost.getName(), "vhost", false);
                Result result = RabbitMQHelper.put(url, RabbitMQHelper.buildVhostModel(), cms.getUsername(),
                    MDSUtils.decrypt(cms.getPassword()));
                if (result.isFailed()) {
                    logger.error("create vhost is error{}", result.getMessage());
                    return new BocloudResult(false, "创建vhost失败");
                }
                // 为cmp管理端用户授权
                JSONObject data = new JSONObject();
                data.put("configure", ".*");
                data.put("write", ".*");
                data.put("read", ".*");
                Result resultData = RabbitMQHelper.put(RabbitMQHelper.buildPermissionsUrl(cms.getManageIp(),
                    cms.getUsername(), mqVhost.getName(), "user", false), data, cms.getUsername(),
                    MDSUtils.decrypt(cms.getPassword()));
                if (resultData.isFailed()) {
                    logger.error(resultData.getMessage());
                    return new BocloudResult(false, "为管理用户授权失败");
                }
                return new BocloudResult(true, cms.getId(), "创建vhost成功");
            }
            return new BocloudResult(false, "暂不支持其他消息队列");
        } catch (Exception e) {
            logger.error("create vhost failure:", e);
            return new BocloudResult(false, "创建消息队列失败");
        }
    }

    @Override
    public BocloudResult removeVhost(Long mqId, String vhostName) {
        try {
            Cms cms = cmsRepository.query(mqId);
            if (null == cms) {
                logger.warn("cms not exist");
                return new BocloudResult(false, "message queue不存在");
            }
            if (ResourceCategory.RabbitMQ.name().equalsIgnoreCase(cms.getType())) {
                String url = RabbitMQHelper.buildUrl(cms.getManageIp(), vhostName, "vhost", false);
                Result result =
                    RabbitMQHelper.delete(url, null, cms.getUsername(), MDSUtils.decrypt(cms.getPassword()));
                if (result.isFailed()) {
                    logger.error("delete vhost data is error{}", result.getMessage());
                    return new BocloudResult(false, cms.getId(), "删除vhost失败");
                }
                return new BocloudResult(true, cms.getId(), "删除vhost成功");
            }
            return new BocloudResult(false, "暂不支持其他消息队列");
        } catch (Exception e) {
            logger.error("delete vhost failure:", e);
            return new BocloudResult(false, "删除消息队列失败");
        }
    }

    @Override
    public BocloudResult<CmsVhost> detailOfVhost(Long mqId, String vhostName) {
        try {
            Cms cms = cmsRepository.query(mqId);
            if (null == cms) {
                logger.warn("cms not exist");
                return new BocloudResult<>(false, "message queue不存在");
            }
            if (ResourceCategory.RabbitMQ.name().equalsIgnoreCase(cms.getType())) {
                String url = RabbitMQHelper.buildUrl(cms.getManageIp(), vhostName, "vhost", false);
                Result result = RabbitMQHelper.get(url, cms.getUsername(), MDSUtils.decrypt(cms.getPassword()));
                if (result.isFailed()) {
                    logger.error("get vhost data is error{}", result.getMessage());
                    return new BocloudResult(false, "获取vhost失败");
                }
                CmsVhost vhost = JSONObject.parseObject(result.getMessage(), CmsVhost.class);
                return new BocloudResult<>(true, vhost, "查询vhost成功");
            }
            return new BocloudResult<>(false, "暂不支持其他消息队列");
        } catch (Exception e) {
            logger.error("get vhost failure:", e);
            return new BocloudResult<>(false, "获取vhost失败");
        }
    }

    @Override
    public BocloudResult<List<CmsVhost>> permissionOfVhostOfList(Long mqId, String vhostName) {
        try {
            Cms cms = cmsRepository.query(mqId);
            if (null == cms) {
                logger.warn("cms not exist");
                return new BocloudResult<>(false, "message queue不存在");
            }
            if (ResourceCategory.RabbitMQ.name().equalsIgnoreCase(cms.getType())) {
                String url = RabbitMQHelper.buildPermissionsUrl(cms.getManageIp(), null, vhostName, "vhost", true);
                Result result = RabbitMQHelper.get(url, cms.getUsername(), MDSUtils.decrypt(cms.getPassword()));
                if (result.isFailed()) {
                    logger.error("get vhost permissions is error{}", result.getMessage());
                    return new BocloudResult<>(false, "查询vhost权限列表失败");
                }
                List<CmsVhost> vhosts = JSONArray.parseArray(result.getMessage(), CmsVhost.class);
                return new BocloudResult<>(true, vhosts, "查询vhost权限列表成功");
            }
            return new BocloudResult<>(false, "暂不支持其他消息队列");
        } catch (Exception e) {
            logger.error("get vhost permissions is error:", e);
            return new BocloudResult<>(false, "查询vhost权限列表失败");
        }
    }

    @Override
    public BocloudResult createPermissionOfVhost(Long mqId, String vhostName, CmsVhost mqVhost) {
        try {
            Cms cms = cmsRepository.query(mqId);
            if (null == cms) {
                logger.warn("cms not exist");
                return new BocloudResult(false, "message queue不存在");
            }
            if (ResourceCategory.RabbitMQ.name().equalsIgnoreCase(cms.getType())) {
                String url =
                    RabbitMQHelper.buildPermissionsUrl(cms.getManageIp(), mqVhost.getUser(), vhostName, "vhost", false);
                mqVhost.setId(null);
                mqVhost.setUser(null);
                mqVhost.setName(null);
                Result result = RabbitMQHelper.put(url, JSONObject.parseObject(JSONObject.toJSONString(mqVhost)),
                    cms.getUsername(), MDSUtils.decrypt(cms.getPassword()));
                if (result.isFailed()) {
                    logger.error("create vhost permissions is error{}", result.getMessage());
                    return new BocloudResult(false, "设置vhost权限失败");
                }
                return new BocloudResult(true, cms.getId(), "设置vhost权限成功");
            }
            return new BocloudResult(false, "暂不支持其他消息队列");
        } catch (Exception e) {
            logger.error("create vhost permissions failure:", e);
            return new BocloudResult(false, "设置vhost权限失败");
        }
    }

    @Override
    public BocloudResult deletePermissionOfVhost(Long mqId, String vhostName, String userName) {
        try {
            Cms cms = cmsRepository.query(mqId);
            if (null == cms) {
                logger.warn("cms not exist");
                return new BocloudResult(false, "message queue不存在");
            }
            if (ResourceCategory.RabbitMQ.name().equalsIgnoreCase(cms.getType())) {
                String url = RabbitMQHelper.buildPermissionsUrl(cms.getManageIp(), userName, vhostName, "vhost", false);
                Result result =
                    RabbitMQHelper.delete(url, null, cms.getUsername(), MDSUtils.decrypt(cms.getPassword()));
                if (result.isFailed()) {
                    logger.error("delete vhost permissions is error{}", result.getMessage());
                    return new BocloudResult(false, cms.getId(), "vhost权限清除失败");
                }
                return new BocloudResult(true, cms.getId(), "vhost权限清除成功");
            }
            return new BocloudResult(false, "暂不支持其他消息队列");
        } catch (Exception e) {
            logger.error("delete vhost permissions failure:", e);
            return new BocloudResult(false, "vhost权限清除失败");
        }
    }

    @Override
    public BocloudResult<List<CmsUser>> permissionOfUserOfList(Long mqId, String userName) {
        try {
            Cms cms = cmsRepository.query(mqId);
            if (null == cms) {
                logger.warn("cms not exist");
                return new BocloudResult<>(false, "message queue不存在");
            }
            if (ResourceCategory.RabbitMQ.name().equalsIgnoreCase(cms.getType())) {
                String url = RabbitMQHelper.buildPermissionsUrl(cms.getManageIp(), userName, null, "user", true);
                Result result = RabbitMQHelper.get(url, cms.getUsername(), MDSUtils.decrypt(cms.getPassword()));
                if (result.isFailed()) {
                    logger.error("get user permissions is error{}", result.getMessage());
                    return new BocloudResult<>(false, "查询user权限列表失败");
                }
                List<CmsUser> users = JSONArray.parseArray(result.getMessage(), CmsUser.class);
                return new BocloudResult<>(true, users, "查询user权限列表成功");
            }
            return new BocloudResult<>(false, "暂不支持其他消息队列");
        } catch (Exception e) {
            logger.error("get user permissions is error:", e);
            return new BocloudResult<>(false, "查询user权限列表失败");
        }
    }

    @Override
    public BocloudResult createPermissionOfUser(Long mqId, String userName, CmsUser mqUser) {
        try {
            Cms cms = cmsRepository.query(mqId);
            if (null == cms) {
                logger.warn("cms not exist");
                return new BocloudResult(false, "message queue不存在");
            }
            if (ResourceCategory.RabbitMQ.name().equalsIgnoreCase(cms.getType())) {
                String url =
                    RabbitMQHelper.buildPermissionsUrl(cms.getManageIp(), userName, mqUser.getVhost(), "user", false);
                mqUser.setId(null);
                mqUser.setName(null);
                mqUser.setVhost(null);
                Result result = RabbitMQHelper.put(url, JSONObject.parseObject(JSONObject.toJSONString(mqUser)),
                    cms.getUsername(), MDSUtils.decrypt(cms.getPassword()));
                if (result.isFailed()) {
                    logger.error("create user permissions is error{}", result.getMessage());
                    return new BocloudResult(false, "设置user权限失败");
                }
                return new BocloudResult(true, cms.getId(), "设置user权限成功");
            }
            return new BocloudResult(false, "暂不支持其他消息队列");
        } catch (Exception e) {
            logger.error("create user permissions failure:", e);
            return new BocloudResult(false, "设置user权限失败");
        }
    }

    @Override
    public BocloudResult deletePermissionOfUser(Long mqId, String usertName, String vhostName) {
        try {
            Cms cms = cmsRepository.query(mqId);
            if (null == cms) {
                logger.warn("cms not exist");
                return new BocloudResult(false, "message queue不存在");
            }
            if (ResourceCategory.RabbitMQ.name().equalsIgnoreCase(cms.getType())) {
                String url = RabbitMQHelper.buildPermissionsUrl(cms.getManageIp(), usertName, vhostName, "user", false);
                Result result =
                    RabbitMQHelper.delete(url, null, cms.getUsername(), MDSUtils.decrypt(cms.getPassword()));
                if (result.isFailed()) {
                    logger.error("delete user permissions is error{}", result.getMessage());
                    return new BocloudResult(false, cms.getId(), "user权限清除失败");
                }
                return new BocloudResult(true, cms.getId(), "user权限清除成功");
            }
            return new BocloudResult(false, "暂不支持其他消息队列");
        } catch (Exception e) {
            logger.error("delete user permissions failure:", e);
            return new BocloudResult(false, "user权限清除失败");
        }
    }

    @Override
    public BocloudResult<GridBean<CmsUser>> listOfUser(Long mqId, Pager pager) {
        GridBean gridBean = null;
        try {
            Cms cms = cmsRepository.query(mqId);
            if (null == cms) {
                logger.warn("cms not exist");
                return new BocloudResult<>(false, "message queue不存在");
            }
            if (ResourceCategory.RabbitMQ.name().equalsIgnoreCase(cms.getType())) {
                String url = RabbitMQHelper.buildUrl(cms.getManageIp(), null, "user", true);
                Result result = RabbitMQHelper.get(url, cms.getUsername(), MDSUtils.decrypt(cms.getPassword()));
                if (result.isFailed()) {
                    logger.error("get user data is error{}", result.getMessage());
                    return new BocloudResult<>(false, "获取user失败");
                }
                // 查询权限
                Result permissionResult = RabbitMQHelper.get(RabbitMQHelper.buildPermissionsUrl(cms.getManageIp()),
                    cms.getUsername(), MDSUtils.decrypt(cms.getPassword()));
                if (permissionResult.isFailed()) {
                    logger.error("get user permissions is error{}", result.getMessage());
                    return new BocloudResult<>(false, "获取user权限失败");
                }
                Map<String, List<String>> userMap = Maps.newHashMap();
                List<JSONObject> permissionList =
                    JSONArray.parseArray(permissionResult.getMessage()).toJavaList(JSONObject.class);
                for (JSONObject jsonObject : permissionList) {
                    List<String> userList = userMap.get(jsonObject.getString("user"));
                    if (null == userList) {
                        userList = Lists.newArrayList();
                        userList.add(jsonObject.getString("vhost"));
                        userMap.put(jsonObject.getString("user"), userList);
                    } else {
                        userList.add(jsonObject.getString("vhost"));
                    }
                }
                List<CmsUser> users = JSONArray.parseArray(result.getMessage(), CmsUser.class);
                users.stream().map(user -> {
                    if (!userMap.isEmpty()) {
                        List<String> vhosts = userMap.get(user.getName());
                        user.setVhost(String.valueOf(vhosts));
                    }
                    return user;
                }).collect(Collectors.toList());
                gridBean = GridHelper.getBean(pager.getPage(), pager.getRows(), users.size(),
                    RabbitMQHelper.pageUtil(users, pager.getPage(), pager.getRows()));
                return new BocloudResult<>(true, gridBean, "查询user成功");
            }
            return new BocloudResult<>(false, "暂不支持其他消息队列");
        } catch (Exception e) {
            logger.error("get user failure:", e);
            return new BocloudResult<>(false, "获取user失败");
        }
    }

    @Override
    public BocloudResult create(Long mqId, CmsUser mqUser) {
        try {
            Cms cms = cmsRepository.query(mqId);
            if (null == cms) {
                logger.warn("cms not exist");
                return new BocloudResult(false, "message queue不存在");
            }
            if (ResourceCategory.RabbitMQ.name().equalsIgnoreCase(cms.getType())) {
                String url = RabbitMQHelper.buildUrl(cms.getManageIp(), mqUser.getName(), "user", false);
                Result result = RabbitMQHelper.put(url, RabbitMQHelper.buildUserModel(mqUser), cms.getUsername(),
                    MDSUtils.decrypt(cms.getPassword()));
                if (result.isFailed()) {
                    logger.error("create user is error{}", result.getMessage());
                    return new BocloudResult(false, "创建user失败");
                }
                return new BocloudResult(true, cms.getId(), "创建user成功");
            }
            return new BocloudResult(false, "暂不支持其他消息队列");
        } catch (Exception e) {
            logger.error("create user failure:", e);
            return new BocloudResult(false, "创建消息队列失败");
        }
    }

    @Override
    public BocloudResult removeUser(Long id, String userName) {
        try {
            Cms cms = cmsRepository.query(id);
            if (null == cms) {
                logger.warn("cms not exist");
                return new BocloudResult(false, "message queue不存在");
            }
            if (ResourceCategory.RabbitMQ.name().equalsIgnoreCase(cms.getType())) {
                if (userName.equals(cms.getUsername())) {
                    return new BocloudResult(false, "该管理用户不可删除");
                }
                String url = RabbitMQHelper.buildUrl(cms.getManageIp(), userName, "user", false);
                Result result =
                    RabbitMQHelper.delete(url, null, cms.getUsername(), MDSUtils.decrypt(cms.getPassword()));
                if (result.isFailed()) {
                    logger.error("delete user data is error{}", result.getMessage());
                    return new BocloudResult(false, cms.getId(), "删除user失败");
                }
                return new BocloudResult(true, cms.getId(), "删除user成功");
            }
            return new BocloudResult(false, "暂不支持其他消息队列");
        } catch (Exception e) {
            logger.error("delete user failure:", e);
            return new BocloudResult(false, "删除消息队列失败");
        }
    }

    @Override
    public BocloudResult updateUser(Long mqId, CmsUser mqUser) {
        try {
            Cms cms = cmsRepository.query(mqId);
            if (null == cms) {
                logger.warn("cms not exist");
                return new BocloudResult(false, "message queue不存在");
            }
            if (ResourceCategory.RabbitMQ.name().equalsIgnoreCase(cms.getType())) {
                String url = RabbitMQHelper.buildUrl(cms.getManageIp(), mqUser.getName(), "user", false);
                Result result = RabbitMQHelper.put(url, RabbitMQHelper.buildUserModel(mqUser), cms.getUsername(),
                    MDSUtils.decrypt(cms.getPassword()));
                if (result.isFailed()) {
                    logger.error("update user data is error{}", result.getMessage());
                    return new BocloudResult(false, cms.getId(), "更新user失败");
                }
                if (!StringUtils.isEmpty(mqUser.getPassword()) && "zeus".equals(mqUser.getUser())) {
                    cms.setPassword(mqUser.getPassword());
                    cmsRepository.update(cms);
                }
                return new BocloudResult(true, cms.getId(), "更新user成功");
            }
            return new BocloudResult(false, "暂不支持其他消息队列");
        } catch (Exception e) {
            logger.error("reset user failure:", e);
            return new BocloudResult(false, "更新user失败");
        }
    }

    @Override
    public BocloudResult<CmsUser> detailOfUser(Long mqId, String userName) {
        try {
            Cms cms = cmsRepository.query(mqId);
            if (null == cms) {
                logger.warn("cms not exist");
                return new BocloudResult<>(false, "message queue不存在");
            }
            if (ResourceCategory.RabbitMQ.name().equalsIgnoreCase(cms.getType())) {
                String url = RabbitMQHelper.buildUrl(cms.getManageIp(), userName, "user", false);
                Result result = RabbitMQHelper.get(url, cms.getUsername(), MDSUtils.decrypt(cms.getPassword()));
                if (result.isFailed()) {
                    logger.error("get user data is error{}", result.getMessage());
                    return new BocloudResult<>(false, "获取user失败");
                }
                CmsUser user = JSONObject.parseObject(result.getMessage(), CmsUser.class);
                return new BocloudResult<>(true, user, "查询user成功");
            }
            return new BocloudResult<>(false, "暂不支持其他消息队列");
        } catch (Exception e) {
            logger.error("get user failure:", e);
            return new BocloudResult<>(false, "获取user失败");
        }
    }

}
