package com.bocloud.pms.service;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.GridBean;
import com.bocloud.common.model.RequestContext;
import com.bocloud.common.model.Result;
import com.bocloud.common.utils.GridHelper;
import com.bocloud.pms.entity.Cms;
import com.bocloud.pms.entity.CmsConsumer;
import com.bocloud.pms.entity.CmsQueue;
import com.bocloud.pms.entity.resource.enums.ResourceCategory;
import com.bocloud.pms.interfaces.middleware.CmsQueueService;
import com.bocloud.pms.repository.CmsRepository;
import com.bocloud.pms.service.utils.MDSUtils;
import com.bocloud.pms.service.utils.RabbitMQHelper;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * Created by liuyuanyuan on 2020/3/11 17:03
 */
@Service("cmsQueueService")
public class CmsQueueServiceImpl implements CmsQueueService {

    private static Logger logger = LoggerFactory.getLogger(CmsQueueServiceImpl.class);
    @Autowired
    private CmsRepository cmsRepository;

    @Override
    public BocloudResult<GridBean<CmsQueue>> list(Long mqId, int page, int rows) {
        GridBean gridBean = null;
        try {
            Cms cms = cmsRepository.query(mqId);
            if (null == cms) {
                logger.warn("cms not exist");
                return new BocloudResult<>(false, "message queue不存在");
            }
            if (ResourceCategory.RabbitMQ.name().equalsIgnoreCase(cms.getType())) {
                String url = RabbitMQHelper.buildQueueUrl(cms.getManageIp(), null, null, "list");
                Result result = RabbitMQHelper.get(url, cms.getUsername(), MDSUtils.decrypt(cms.getPassword()));
                if (result.isFailed()) {
                    logger.error("get queue data is error{}", result.getMessage());
                    return new BocloudResult<>(false, "获取消息队列失败");
                }
                List<CmsQueue> queues = JSONArray.parseArray(result.getMessage(), CmsQueue.class);
                if (!queues.isEmpty()) {
                    for (CmsQueue mqQueue : queues) {
                        JSONObject messageStats = mqQueue.getMessageStats();
                        if (null != messageStats) {
                            mqQueue.setPublishInRate(null == messageStats.getJSONObject("publish_in_details") ? null : messageStats.getJSONObject("publish_in_details").getLong("rate"));
                            mqQueue.setDeliverGetRate(null == messageStats.getJSONObject("publish_out_details") ? null :
                                    messageStats.getJSONObject("publish_out_details").getLong("rate"));
                            mqQueue.setAckRate(null == messageStats.getJSONObject("ack_details") ? null : messageStats.getJSONObject("ack_details").getLong("rate"));
                        }
                    }
                }
                gridBean = GridHelper.getBean(page, rows, queues.size(),
                        RabbitMQHelper.pageUtil(queues, page, rows));
                return new BocloudResult<>(true, gridBean, "查询消息队列成功");
            }
            return new BocloudResult<>(false, "暂不支持其他消息队列");
        } catch (Exception e) {
            logger.error("get queue failure:", e);
            return new BocloudResult<>(false, "获取消息队列失败");
        }
    }

    @Override
    public BocloudResult create(Long mqId, CmsQueue mqQueue, RequestContext context) {
        try {
            Cms cms = cmsRepository.query(mqId);
            if (null == cms) {
                logger.warn("cms not exist");
                return new BocloudResult(false, "message queue不存在");
            }
            if (ResourceCategory.RabbitMQ.name().equalsIgnoreCase(cms.getType())) {
                String url = RabbitMQHelper.buildQueueUrl(cms.getManageIp(), mqQueue.getVhost(), mqQueue.getName(),
                        "create");
                Result result = RabbitMQHelper.put(url, RabbitMQHelper.buildQueueModel(mqQueue), cms.getUsername(),
                        MDSUtils.decrypt(cms.getPassword()));
                if (result.isFailed()) {
                    logger.error("create queue is error{}", result.getMessage());
                    return new BocloudResult(false, "创建queue失败");
                }
                return new BocloudResult(true, cms.getId(), "创建消息队列成功");
            }
            return new BocloudResult(false, "暂不支持其他消息队列");
        } catch (Exception e) {
            logger.error("create queue failure:", e);
            return new BocloudResult(false, "创建消息队列失败");
        }

    }

    @Override
    public BocloudResult remove(Long mqId, String vhost, String queueName) {
        Cms cms = null;
        try {
            cms = cmsRepository.query(mqId);
            if (null == cms) {
                logger.warn("cms not exist");
                return new BocloudResult(false, "message queue不存在");
            }
            if (ResourceCategory.RabbitMQ.name().equalsIgnoreCase(cms.getType())) {
                String url = RabbitMQHelper.buildQueueUrl(cms.getManageIp(), vhost, queueName, "delete");
                Result result = RabbitMQHelper.delete(url, null, cms.getUsername(),
                        MDSUtils.decrypt(cms.getPassword()));
                if (result.isFailed()) {
                    logger.error("delete queue data is error{}", result.getMessage());
                    return new BocloudResult(false, cms.getId(), "删除消息队列失败");
                }
                return new BocloudResult(true, cms.getId(), "删除消息队列成功");
            }
            return new BocloudResult(false, "暂不支持其他消息队列");
        } catch (Exception e) {
            logger.error("delete queue failure:", e);
            return new BocloudResult(false, null == cms ? null : cms.getId(), "删除消息队列失败");
        }
    }

    @Override
    public BocloudResult<CmsQueue> detail(Long mqId, String queueName, String vhost) {
        try {
            Cms cms = cmsRepository.query(mqId);
            if (null == cms) {
                logger.warn("cms not exist");
                return new BocloudResult<>(false, "message queue不存在");
            }
            if (ResourceCategory.RabbitMQ.name().equalsIgnoreCase(cms.getType())) {
                String url = RabbitMQHelper.buildQueueUrl(cms.getManageIp(), vhost, queueName, "detail");
                Result result = RabbitMQHelper.get(url, cms.getUsername(), MDSUtils.decrypt(cms.getPassword()));
                if (result.isFailed()) {
                    logger.error("get queue data is error{}", result.getMessage());
                    return new BocloudResult<>(false, "获取消息队列失败");
                }
                CmsQueue mqQueue = JSONObject.parseObject(result.getMessage(), CmsQueue.class);
                List<JSONObject> consumerDetails = mqQueue.getConsumerDetails();
                if (!consumerDetails.isEmpty()) {
                    for (JSONObject consumerDetail : consumerDetails) {
                        CmsConsumer consumer = consumerDetail.toJavaObject(CmsConsumer.class);
                        consumer.setChannelName(consumerDetail.getJSONObject("channel_details").getString("name"));
                    }
                    mqQueue.setConsumerDetails(consumerDetails);
                }
                return new BocloudResult<>(true, mqQueue, "查询消息队列成功");
            }
            return new BocloudResult<>(false, "暂不支持其他消息队列");
        } catch (Exception e) {
            logger.error("get queue failure:", e);
            return new BocloudResult<>(false, "获取消息队列失败");
        }
    }

}
