package com.bocloud.pms.service;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Calendar;
import java.util.Collection;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.Properties;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.kafka.clients.admin.AdminClient;
import org.apache.kafka.clients.admin.AdminClientConfig;
import org.apache.kafka.common.Node;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.amqp.core.AmqpTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.scheduling.concurrent.ThreadPoolTaskExecutor;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.GridBean;
import com.bocloud.common.model.OperateResult;
import com.bocloud.common.model.OperateResult.OperateCategory;
import com.bocloud.common.model.Pager;
import com.bocloud.common.model.Param;
import com.bocloud.common.model.RequestContext;
import com.bocloud.common.model.RequestContext.Catalog;
import com.bocloud.common.model.Result;
import com.bocloud.common.model.Sign;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.DateTools;
import com.bocloud.common.utils.GridHelper;
import com.bocloud.common.utils.IDFactory;
import com.bocloud.common.utils.JSONTools;
import com.bocloud.common.utils.ListTool;
import com.bocloud.common.utils.MapTools;
import com.bocloud.pms.entity.Cms;
import com.bocloud.pms.entity.CmsLog;
import com.bocloud.pms.entity.CmsNode;
import com.bocloud.pms.entity.resource.Business;
import com.bocloud.pms.entity.resource.CloudServer;
import com.bocloud.pms.entity.resource.CloudVendor;
import com.bocloud.pms.entity.resource.OrderItem;
import com.bocloud.pms.entity.resource.OrderMessage;
import com.bocloud.pms.entity.resource.Project;
import com.bocloud.pms.entity.resource.Recycled;
import com.bocloud.pms.entity.resource.Resource;
import com.bocloud.pms.entity.resource.Server;
import com.bocloud.pms.entity.resource.ServiceOrder;
import com.bocloud.pms.entity.resource.Snapshot;
import com.bocloud.pms.entity.resource.TaskInstance;
import com.bocloud.pms.entity.resource.TaskNode;
import com.bocloud.pms.entity.resource.Tenant;
import com.bocloud.pms.entity.resource.User;
import com.bocloud.pms.entity.resource.enums.BocloudStatus;
import com.bocloud.pms.entity.resource.enums.OrderItemStatus;
import com.bocloud.pms.entity.resource.enums.ResourceCategory;
import com.bocloud.pms.entity.resource.enums.Status;
import com.bocloud.pms.entity.resource.model.AlterationParam;
import com.bocloud.pms.entity.resource.model.CancelationParam;
import com.bocloud.pms.entity.resource.model.ExtensionParam;
import com.bocloud.pms.entity.resource.model.LocationModel;
import com.bocloud.pms.entity.resource.model.NotifyMessage;
import com.bocloud.pms.entity.resource.model.SnapshotModel;
import com.bocloud.pms.interfaces.middleware.CmsService;
import com.bocloud.pms.repository.CmsLogRepository;
import com.bocloud.pms.repository.CmsNodeRepository;
import com.bocloud.pms.repository.CmsRepository;
import com.bocloud.pms.repository.CmsTopicRepository;
import com.bocloud.pms.repository.resource.CloudServerRepository;
import com.bocloud.pms.repository.resource.CloudVendorRepository;
import com.bocloud.pms.repository.resource.ResourceRepository;
import com.bocloud.pms.repository.resource.ServerRepository;
import com.bocloud.pms.repository.resource.SnapshotRepository;
import com.bocloud.pms.service.internal.CosInternalService;
import com.bocloud.pms.service.internal.ImsCloudServerService;
import com.bocloud.pms.service.internal.SmsBusinessService;
import com.bocloud.pms.service.internal.SmsProjectService;
import com.bocloud.pms.service.internal.SmsTenantService;
import com.bocloud.pms.service.internal.SmsUserService;
import com.bocloud.pms.service.utils.MDSUtils;
import com.bocloud.pms.service.utils.RabbitMQHelper;
import com.bocloud.pms.service.utils.ResourceEventPublisher;
import com.bocloud.service.lock.BocloudACLock;
import com.bocloud.service.lock.LockFactory;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;

/**
 * 消息队列接口实现
 *
 * @author weiwei
 */
@Service("cmsService")
public class CmsServiceImpl implements CmsService {

    private static Logger logger = LoggerFactory.getLogger(CmsServiceImpl.class);
    @Autowired
    private LockFactory lockFactory;
    @Autowired
    private CmsRepository cmsRepository;
    @Autowired
    private CmsNodeRepository cmsNodeRepository;
    @Autowired
    private CmsLogRepository cmsLogRepository;
    @Autowired
    private CloudServerRepository cloudServerRepository;
    @Autowired
    private ResourceRepository resourceRepository;
    @Autowired
    private AmqpTemplate amqpTemplate;
    @Autowired
    private CloudVendorRepository cloudVendorRepository;
    @Autowired
    private SnapshotRepository snapshotRepository;
    @Autowired
    private ResourceEventPublisher resourceEventPublisher;
    @Autowired
    private ServerRepository serverRepository;
    @Autowired
    private CmsTopicRepository cmsTopicRepository;
    @Value("${middleware.server.remove:true}")
    private boolean removed;
    @Autowired
    private RedisTemplate<String, String> redisTemplate;
    @Autowired
    private CosInternalService cosInternalService;
    @Autowired
    private ImsCloudServerService imsCloudServerService;
    @Autowired
    private ThreadPoolTaskExecutor threadPoolTaskExecutor;
    @Autowired
    private SmsProjectService smsProjectService;
    @Autowired
    private SmsUserService smsUserService;
    @Autowired
    private SmsBusinessService smsBusinessService;
    @Autowired
    private SmsTenantService smsTenantService;

    @Override
    public BocloudResult<GridBean<Cms>> list(Pager pager, RequestContext context) {
        try {
            List<Param> params = pager.getParams();
            Integer page = pager.getPage();
            Integer rows = pager.getRows();
            Map<String, String> sorter = pager.getSorter();
            params = Optional.ofNullable(params).orElse(new ArrayList<>());
            sorter = Optional.ofNullable(sorter).orElse(Maps.newHashMap());
            sorter.put("gmtCreate", Common.ONE);
            List<Cms> list = null;
            int total = 0;
            if (context.getCatalog().equals(Catalog.Tenant)) {
                // 租户能看到 1.属于当前租户的资源
                list = cmsRepository.list(page, rows, params, sorter, context.getTenant());
                total = cmsRepository.count(params, context.getTenant());
            } else if (context.getCatalog().equals(Catalog.User)) {
                // 用户能看到 1.自己申请的资源 2.自己所在的资源可见项目（资源对项目下所有成员可见）下创建的资源 3.自己管理的项目下的资源
                List<Project> userManagerProjects = smsProjectService.listUserManagerProjects(context.getTarget());
                List<Project> visibleProjects = smsProjectService.listUserVisibleProjects(context.getTarget());
                List<Long> projectIds = new ArrayList<>();
                List<Long> userManagerProjectsIds =
                    userManagerProjects.stream().map(Project::getId).collect(Collectors.toList());
                if (!userManagerProjectsIds.isEmpty()) {
                    projectIds.addAll(userManagerProjectsIds);
                }
                List<Long> visibleProjectsIds =
                    visibleProjects.stream().map(Project::getId).collect(Collectors.toList());
                if (!visibleProjectsIds.isEmpty()) {
                    projectIds.addAll(visibleProjectsIds);
                }
                list = cmsRepository.list(page, rows, params, sorter, context.getTarget(), projectIds);
                total = cmsRepository.count(params, context.getTarget(), projectIds);
            } else {
                list = cmsRepository.list(page, rows, params, sorter, null);
                total = cmsRepository.count(params, null);
            }
            // 处理业务信息
            List<Long> tenantIds = new ArrayList<>();
            List<Long> userIds = new ArrayList<>();
            List<Long> businessIds = new ArrayList<>();
            List<Long> projectIds = new ArrayList<>();
            for (Cms cms : list) {
                if (cms.getTenantId() != null) {
                    tenantIds.add(cms.getTenantId());
                }
                if (cms.getBusinessId() != null) {
                    businessIds.add(cms.getBusinessId());
                }
                if (cms.getProjectId() != null) {
                    projectIds.add(cms.getProjectId());
                }
                if (cms.getCreatorId() != null) {
                    userIds.add(cms.getCreatorId());
                }
                if (cms.getOwnerId() != null) {
                    userIds.add(cms.getOwnerId());
                }
                if (cms.getMenderId() != null) {
                    userIds.add(cms.getMenderId());
                }
            }
            Map<Long, String> userMap =
                smsUserService.listUsersByIds(userIds).stream().collect(Collectors.toMap(User::getId, User::getName));
            Map<Long, String> projectMap = smsProjectService.listProjectsByIds(projectIds).stream()
                .collect(Collectors.toMap(Project::getId, Project::getName));
            Map<Long, String> businessMap = smsBusinessService.listBusinesssByIds(businessIds).stream()
                .collect(Collectors.toMap(Business::getId, Business::getName));
            Map<Long, String> tenantMap = smsTenantService.listTenantsByIds(tenantIds).stream()
                .collect(Collectors.toMap(Tenant::getId, Tenant::getName));
            for (Cms cms : list) {
                JSONObject specs = JSONObject.parseObject(cms.getSpec());
                if (null != specs) {
                    cms.setSpec(specs.get("cpu") + "核/" + specs.get("memory") + "GB/" + specs.get("disk") + "GB");
                    cms.setDataDisk(specs.getInteger("dataDisk"));
                }
                JSONObject inventory = JSONObject.parseObject(cms.getInventory());
                if (null != inventory) {
                    List<JSONObject> elements = JSONArray.parseArray(inventory.getString("elements"), JSONObject.class);
                    if (!ListTool.isEmpty(elements)) {
                        for (JSONObject object : elements) {
                            if (object.getString("serviceCode").equalsIgnoreCase("cms." + cms.getType())) {
                                continue;
                            }
                            cms.setDiskSkuId(object.getLong("skuId"));
                        }
                    }
                }
                cms.setTenantName(redisTemplate.opsForValue().get("Tenant_Name_" + cms.getTenantId()));
                if (ResourceCategory.RabbitMQ == ResourceCategory.getValue(cms.getType())) {
                    List<CmsNode> nodes = cmsNodeRepository.list(cms.getId());
                    String address = "";
                    for (CmsNode node : nodes) {
                        address += node.getManageIp() + ":" + node.getPort() + ",";
                    }
                    if (!StringUtils.isEmpty(address)) {
                        cms.setAddress(address.substring(0, address.length() - 1));
                    }
                }
                // 设置云主机归属信息
                if (cms.getTenantId() != null) {
                    cms.setTenantName(tenantMap.get(cms.getTenantId()));
                }
                if (cms.getBusinessId() != null) {
                    cms.setBusinessName(businessMap.get(cms.getBusinessId()));
                }
                if (cms.getProjectId() != null) {
                    cms.setProjectName(projectMap.get(cms.getProjectId()));
                }
                if (cms.getCreatorId() != null) {
                    cms.setCreatorName(userMap.get(cms.getCreatorId()));
                }
                if (cms.getOwnerId() != null) {
                    cms.setOwnerName(userMap.get(cms.getOwnerId()));
                }
                if (cms.getMenderId() != null) {
                    cms.setMenderName(userMap.get(cms.getMenderId()));
                }
            }
            GridBean gridBean = GridHelper.getBean(page, rows, total, list);
            return new BocloudResult<>(true, gridBean, "查询成功");
        } catch (Exception e) {
            logger.error("list cms failure:", e);
            return new BocloudResult<>(false, "查询失败", null, null);
        }
    }

    @Override
    public BocloudResult apply(Object object, ResourceCategory category, OrderMessage.Operation operation,
        RequestContext context) {
        BocloudResult result = null;
        switch (operation) {
            case Application:
                result = create(JSONObject.parseObject(JSONObject.toJSONString(object)), category, context);
                break;
            case Upgrade:
                result = modify((AlterationParam)object, context);
                break;
            case Cancelation:
                result = remove((CancelationParam)object, context);
                break;
            case Extension:
                result = postpone((ExtensionParam)object, context);
                break;
            default:
                result = new BocloudResult<>(false, "暂不支持该类操作");
                break;
        }
        return result;
    }

    @Override
    public BocloudResult recycle(Long id, RequestContext context) {
        try {
            Cms cms = cmsRepository.query(id);
            // List<CmsNode> nodes = cmsNodeRepository.list(id);
            // List<Long> serverIds = Lists.newArrayList();
            // for (CmsNode node : nodes) {
            // CloudServer server = cloudServerRepository.get(node.getInstanceId());
            // Optional.ofNullable(server).orElseThrow(() -> new Exception("云主机不存在"));
            // CloudVendor vendor = cloudVendorRepository.query(server.getVendorId());
            // Optional.ofNullable(vendor).orElseThrow(() -> new Exception("云平台不存在"));
            // serverIds.add(node.getServerId());
            // }
            // BocloudResult batchOperations = cloudServerService.batchOperations(String.valueOf(serverIds), "Stop",
            // context);
            // if (batchOperations.isFailed()) {
            // return new BocloudResult<>(false, "中间件关机失败！");
            // }
            // 更新实例状态
            cmsRepository.recycle(id, context.getTarget());
            // 回收站生成记录
            JSONObject specs = JSONObject.parseObject(cms.getSpec());
            String spec = null;
            if (null != specs) {
                spec = specs.get("cpu") + "核/" + specs.get("memory") + "GB/" + specs.get("disk") + "GB";
            }
            Recycled recycled =
                new Recycled(cms.getName(), cms.getUuid(), cms.getType(), cms.getId(), null, cms.getGmtCatalog(), spec);
            if (Catalog.Tenant == Catalog.valueOf(cms.getGmtCatalog())) {
                recycled.setTenantId(cms.getTenantId());
            } else if (Catalog.User == Catalog.valueOf(cms.getGmtCatalog())) {
                recycled.setUserId(cms.getOwnerId());
                recycled.setTenantId(cms.getTenantId());
                recycled.setOwnerId(cms.getOwnerId());
            }
            BocloudResult joinRecycle = cosInternalService.joinRecycle(recycled);
            logger.info("生成回收站记录结果：{}", JSONObject.toJSON(joinRecycle));
            // 处理订单
            cosInternalService.updateItemStatus(cms.getUuid(), OrderItemStatus.Termination.name());
            // 处理账单
            BocloudResult result = cosInternalService.recycleBill(cms.getUuid());
            logger.info("放入回收站生产账单结果：{}", JSONObject.toJSON(result));
            RequestContext requestContext = new RequestContext(Catalog.valueOf(cms.getGmtCatalog()), cms.getTenantId(),
                cms.getCreatorId(), null, null);
            // 发送站内信
            NotifyMessage notifyMessage = new NotifyMessage(requestContext, cms.getType() + "实例放入回收站通知",
                cms.getType() + "【" + cms.getName() + "】已删除放入回收站，30天后将被彻底删除", true, false);
            amqpTemplate.convertAndSend("event.notice.queue", notifyMessage);
            return new BocloudResult<>(true, "中间件【" + cms.getName() + "】删除成功，可在回收站查看");
        } catch (Exception e) {
            logger.error("remove cms fail:", e);
            return new BocloudResult<>(false, "移入回收车失败");
        }
    }

    @Override
    public BocloudResult restore(Long id, RequestContext context) {
        try {
            Cms cms = cmsRepository.query(id);
            // List<CmsNode> nodes = cmsNodeRepository.list(id);
            // List<Long> serverIds = Lists.newArrayList();
            // for (CmsNode node : nodes) {
            // CloudServer server = cloudServerRepository.get(node.getInstanceId());
            // Optional.ofNullable(server).orElseThrow(() -> new Exception("云主机不存在"));
            // CloudVendor vendor = cloudVendorRepository.query(server.getVendorId());
            // Optional.ofNullable(vendor).orElseThrow(() -> new Exception("云平台不存在"));
            // serverIds.add(node.getServerId());
            // }
            // BocloudResult batchOperations = imsCloudServerService.batchOperations(String.valueOf(serverIds),
            // "Start");
            // if (batchOperations.isFailed()) {
            // return new BocloudResult<>(false, "中间件开机失败！");
            // }
            // 更新实例状态
            cmsRepository.restore(id, context.getTarget());
            // 处理订单
            cosInternalService.updateItemStatus(cms.getUuid(), OrderItemStatus.OpenedSuccess.name());
            return new BocloudResult<>(true, cms.getType() + "【" + cms.getName() + "】还原成功");
        } catch (Exception e) {
            logger.error("restore cms fail:", e);
            return new BocloudResult<>(false, "中间件还原失败");
        }
    }

    private BocloudResult modify(AlterationParam param, RequestContext context) {
        String path = Cms.class.getSimpleName() + "_" + param.getInstance();
        try (BocloudACLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            Cms cms = cmsRepository.get(param.getInstance());
            if (null == cms) {
                logger.warn("cms does not exist!");
                return new BocloudResult<>(false, "数据库不存在!");
            }
            ResourceCategory category = ResourceCategory.getValue(cms.getType());
            List<CmsNode> nodes = cmsNodeRepository.list(cms.getId());
            if (ListTool.isEmpty(nodes)) {
                return new BocloudResult<>(false, "数据库节点不存在");
            }
            List<CloudServer> servers = new ArrayList<>();
            for (CmsNode node : nodes) {
                CloudServer server = cloudServerRepository.getByInstanceId(node.getInstanceId());
                if (null == server) {
                    server = cloudServerRepository.get(node.getServerId());
                }
                if (null != server) {
                    servers.add(server);
                }
            }
            if (ListTool.isEmpty(servers)) {
                return new BocloudResult<>(false, "数据库节点不存在");
            }
            Long vendorId = servers.get(0).getVendorId();
            CloudVendor vendor = cloudVendorRepository.query(vendorId);
            if (null == vendor) {
                logger.warn("Cloud vendor does not exist!");
                return new BocloudResult<>(false, "云平台不存在!");
            }
            JSONObject specs = JSONObject.parseObject(cms.getSpec());
            if (null == param.getCpu() && null != param.getDataDisk()) {
                if (param.getDataDisk() <= specs.getIntValue("dataDisk")) {
                    return new BocloudResult<>(false, "数据盘扩容大小不能等于小于当前大小");
                }
                for (CloudServer server : servers) {
                    JSONObject object = new JSONObject();
                    object.put("id", server.getId());
                    object.put("name", server.getName());
                    object.put("intanceId", server.getInstanceId());
                    object.put("dataDisk", param.getDataDisk());
                    logger.info("数据盘扩容参数：" + object.toJSONString());
                    BocloudResult result = imsCloudServerService.resize(server.getId(), object, "upgradeconfig");
                    if (result.isFailed()) {
                        return new BocloudResult<>(false, "数据盘扩容失败：" + result.getMessage());
                    }
                }
                specs.put("dataDisk", param.getDataDisk());
                cms.setSpec(specs.toJSONString());
                // cms.setUuid(param.getNewInstance());
                cms.setMenderId(context.getTarget());
                cms.setGmtModify(new Date());
                cmsRepository.update(cms);
                MDSUtils.send(amqpTemplate, cms, nodes.size(), OrderMessage.Operation.Upgrade,
                    OrderItemStatus.ChangeSuccess, MDSUtils.CMS, param, context, null);
                return new BocloudResult<>(true, category.name() + "数据盘扩容成功");
            }
            // CPU
            Integer cpu = param.getCpu();
            // 内存
            BigDecimal memory = param.getMemory();
            // 磁盘
            // Integer disk = param.getSystemDisk();
            if (cpu < specs.getIntValue("cpu") || memory.doubleValue() < specs.getBigDecimal("memory").doubleValue()) {
                return new BocloudResult<>(false, "升级配置不能小于当前配置");
            }
            if (cpu.equals(specs.getIntValue("cpu")) && memory.equals(specs.getBigDecimal("memory"))) {
                return new BocloudResult<>(false, "升级配置等于当前配置");
            }
            for (CloudServer server : servers) {
                JSONObject object = new JSONObject();
                object.put("id", server.getId());
                object.put("name", server.getName());
                object.put("intanceId", server.getInstanceId());
                object.put("cpu", param.getCpu());
                object.put("memory", param.getMemory());
                object.put("systemDisk", param.getSystemDisk());
                BocloudResult result = imsCloudServerService.resize(server.getId(), object, "upgradeconfig");
                if (result.isFailed()) {
                    return new BocloudResult<>(false, "配置升级失败：" + result.getMessage());
                }
            }
            specs.put("cpu", param.getCpu());
            specs.put("memory", param.getMemory());
            // specs.put("disk", param.getSystemDisk());
            cms.setSpec(specs.toJSONString());
            // cms.setUuid(param.getNewInstance());
            cms.setMenderId(context.getTarget());
            cms.setGmtModify(new Date());
            cms.setSkuId(param.getUpgradeSkuId());
            cmsRepository.update(cms);
            MDSUtils.send(amqpTemplate, cms, nodes.size(), OrderMessage.Operation.Upgrade,
                OrderItemStatus.ChangeSuccess, MDSUtils.CMS, param, context, null);
            return new BocloudResult<>(true, category.name() + "规格变更成功");
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return new BocloudResult<>(false, "数据库规格变更失败");
        }
    }

    private BocloudResult create(JSONObject object, ResourceCategory category, RequestContext context) {
        ServiceOrder order = JSON.parseObject(object.getString("order"), ServiceOrder.class);
        List<OrderItem> items = order.getItems();
        if (ListTool.isEmpty(items)) {
            return new BocloudResult<>(false, "订单数据错误！");
        }
        JSONObject emption = JSONObject.parseObject(object.getString("emption"));
        // 使用时长信息
        JSONObject duration = JSONObject.parseObject(emption.getString("duration"));
        // 使用时长单位Hour/Month
        String mode = duration.getString("mode");
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(new Date());
        if ("Month".equalsIgnoreCase(mode)) {
            calendar.add(Calendar.MONTH, duration.getInteger("amount"));
        } else if ("Year".equalsIgnoreCase(mode)) {
            calendar.add(Calendar.MONTH, duration.getInteger("amount"));
        } else {
            calendar.setTime(DateTools.stringToDateTime("2099-12-31 00:00:00"));
        }
        try {
            JSONObject params = MDSUtils.analyze(object);
            JSONObject software = params.getJSONObject("software");
            JSONObject resource = params.getJSONObject("resource");
            LocationModel location = JSONObject.parseObject(object.getString("location"), LocationModel.class);
            for (int i = 0; i < items.size(); i++) {
                if (!items.get(i).getServiceCode().startsWith("cms.")) {
                    continue;
                }
                Cms cms = new Cms();
                cms.setName(category.name() + IDFactory.instance().uuid());
                cms.setStatus(BocloudStatus.Server.BUILDING.name());
                cms.setUuid(items.get(i).getSn());
                cms.setInstanceId(IDFactory.instance().uuid());
                cms.setType(category.name());
                cms.setVersion(software.getString("version"));
                cms.setRegion(location.getRegion());
                cms.setResourceLabel(resource.getString("resourceLabel"));
                cms.setInventory(object.toJSONString());
                cms.setCategory(software.getString("categoryCode"));
                cms.setSkuId(software.getLong("skuId"));
                cms.setCategoryId(software.getLong("categoryId"));
                JSONObject props = software.getJSONObject("categoryProps");
                if (null != props) {
                    cms.setTopics(props.getInteger("topics"));
                }
                JSONObject specs = new JSONObject();
                specs.put("cpu", resource.getString("cpu"));
                specs.put("memory", resource.getString("memory"));
                specs.put("disk", resource.getString("disk"));
                List<Integer> disks = JSONObject.parseArray(resource.getString("disks"), Integer.class);
                if (!ListTool.isEmpty(disks)) {
                    specs.put("dataDisk", disks.get(0));
                }
                cms.setSpec(specs.toJSONString());
                if (ResourceCategory.RabbitMQ.equals(category)) {
                    cms.setUsername("zeus");
                    cms.setPassword(MDSUtils.encrypt("Zeus@v587"));
                }
                cms.setCreatorId(location.getCreatorId() == null ? 0 : location.getCreatorId());
                cms.setOwnerId(location.getOwnerId() == null ? 0 : location.getOwnerId());
                cms.setMenderId(location.getMenderId() == null ? 0 : location.getMenderId());
                cms.setGmtCatalog(context.getCatalog().name());
                cms.setTenantId(context.getTenant() == null ? 0 : context.getTenant());
                cms.setBusinessId(location.getBusinessId() == null ? 0 : location.getBusinessId());
                cms.setProjectId(location.getProjectId() == null ? 0 : location.getProjectId());
                cms.setPoolGroupId(location.getPoolGroupId());
                cms.setExpiredTime(calendar.getTime());
                cmsRepository.save(cms);
                JSONObject message = new JSONObject();
                message.put("context", context);
                message.put("queue", "service.task.result");
                message.put("params", params);
                JSONObject service = new JSONObject();
                service.put("uuid", cms.getUuid());
                service.put("category", category.name());
                message.put("service", service);
                this.amqpTemplate.convertAndSend("service.task.queue", message);
            }
            return new BocloudResult<>(true, category.name() + "订购成功");
        } catch (Exception e) {
            logger.error("save cms failure:", e);
            return new BocloudResult<>(false, "数据库订购失败");
        }
    }

    private BocloudResult remove(CancelationParam param, RequestContext context) {
        String path = Cms.class.getSimpleName() + "_" + param.getInstance();
        try (BocloudACLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            Cms cms = cmsRepository.get(param.getInstance());
            if (null == cms) {
                return new BocloudResult<>(false, "消息队列不存在");
            }
            List<CmsNode> nodes = cmsNodeRepository.list(cms.getId());
            for (CmsNode node : nodes) {
                CloudServer server = cloudServerRepository.getByInstanceId(node.getInstanceId());
                if (null == server) {
                    server = cloudServerRepository.get(node.getServerId());
                }
                if (null != server) {
                    imsCloudServerService.delete(server.getId());
                }
            }
            ResourceCategory category = ResourceCategory.getValue(cms.getType());
            cmsLogRepository.deleteByMq(cms.getId());
            if (ResourceCategory.Kafka.name().equalsIgnoreCase(cms.getType())) {
                cmsTopicRepository.deleteByMq(cms.getId());
            }
            cmsNodeRepository.deleteByCms(cms.getId());
            cmsRepository.remove(cms.getId(), context.getTarget());
            MDSUtils.send(amqpTemplate, cms, nodes.size(), OrderMessage.Operation.Cancelation,
                OrderItemStatus.UnsubscribeSuccess, MDSUtils.CMS, param, context, null);
            MDSUtils.send(amqpTemplate, cms.getId(), cms.getType(), "delete");
            // 清空回收站
            cosInternalService.recycle(cms.getUuid());
            return new BocloudResult<>(true, category.name() + "退订成功");
        } catch (Exception e) {
            logger.error("remove cms failure:", e);
            return new BocloudResult<>(false, "数据库退订失败");
        }
    }

    @Override
    public BocloudResult<Cms> detail(Long id) {
        try {
            Cms cms = cmsRepository.query(id);
            cms = Optional.ofNullable(cms).orElseThrow(() -> new Exception("数据不存在"));
            List<CmsNode> nodes = cmsNodeRepository.list(id);
            StringBuilder builder = new StringBuilder();
            List<Map<String, String>> connectionList = Lists.newArrayList();
            for (CmsNode node : nodes) {
                Map<String, String> collections = Maps.newHashMap();
                collections.put("role", node.getRole());
                collections.put("address", node.getAddress() + ":" + node.getPort());
                connectionList.add(collections);
                builder.append(node.getAddress() + ":" + node.getPort()).append(",");
            }
            String conn = StringUtils.substringBeforeLast(builder.toString(), ",");
            HashMap<String, String> connectionUrl = Maps.newHashMap();
            connectionUrl.put("role", "connectionUrl");
            connectionUrl.put("address", conn);
            connectionList.add(connectionUrl);
            cms.setConnections(connectionList);
            // 同步节点状态
            synchronousRole(nodes, cms);
            cms.setNodes(nodes);
            JSONObject specs = JSONObject.parseObject(cms.getSpec());
            if (null != specs) {
                cms.setSpec(specs.get("cpu") + "核/" + specs.get("memory") + "GB/" + specs.get("disk") + "GB");
                cms.setDataDisk(specs.getInteger("dataDisk"));
            }
            JSONObject inventory = JSONObject.parseObject(cms.getInventory());
            if (null != inventory) {
                JSONObject configs = inventory.getJSONObject("configs");
                cms.setDiskCategory(configs.getString("diskCategoryName"));
            }
            // 处理业务信息
            if (cms.getTenantId() != null && cms.getTenantId() != 0) {
                Map<Long, String> tenantMap = smsTenantService.listTenantsByIds(Arrays.asList(cms.getTenantId()))
                    .stream().collect(Collectors.toMap(Tenant::getId, Tenant::getName));
                cms.setTenantName(tenantMap.get(cms.getTenantId()));
            }
            if (cms.getBusinessId() != null && cms.getBusinessId() != 0) {
                Map<Long, String> businessMap =
                    smsBusinessService.listBusinesssByIds(Arrays.asList(cms.getBusinessId())).stream()
                        .collect(Collectors.toMap(Business::getId, Business::getName));
                cms.setBusinessName(businessMap.get(cms.getBusinessId()));
            }
            if (cms.getProjectId() != null && cms.getProjectId() != 0) {
                Map<Long, String> projectMap = smsProjectService.listProjectsByIds(Arrays.asList(cms.getProjectId()))
                    .stream().collect(Collectors.toMap(Project::getId, Project::getName));
                cms.setProjectName(projectMap.get(cms.getProjectId()));
            }
            List<Long> userIds = new ArrayList<>();
            if (cms.getCreatorId() != null && cms.getCreatorId() != 0) {
                userIds.add(cms.getCreatorId());
            }
            if (cms.getOwnerId() != null && cms.getOwnerId() != 0) {
                userIds.add(cms.getOwnerId());
            }
            if (cms.getMenderId() != null && cms.getMenderId() != 0) {
                userIds.add(cms.getMenderId());
            }
            if (!ListTool.isEmpty(userIds)) {
                Map<Long, String> userMap = smsUserService.listUsersByIds(userIds).stream()
                    .collect(Collectors.toMap(User::getId, User::getName));
                cms.setCreatorName(userMap.get(cms.getCreatorId()));
                cms.setOwnerName(userMap.get(cms.getOwnerId()));
                cms.setMenderName(userMap.get(cms.getMenderId()));
            }
            return new BocloudResult<>(true, cms, "查询详情成功");
        } catch (Exception e) {
            logger.error("query cms detail fail:", e);
            return new BocloudResult<>(false, "查询详情失败");
        }
    }

    @Override
    public BocloudResult<Cms> basics(Long id) {
        try {
            Cms cms = cmsRepository.query(id);
            cms = Optional.ofNullable(cms).orElseThrow(() -> new Exception("数据不存在"));
            List<CmsNode> nodes = cmsNodeRepository.list(id);
            cms.setNodes(nodes);
            return new BocloudResult<>(true, cms, "查询成功");
        } catch (Exception e) {
            logger.error("query cms detail fail:", e);
            return new BocloudResult<>(false, "查询详情失败");
        }
    }

    @Override
    public BocloudResult remove(Long id, RequestContext context) {
        String path = Cms.class.getSimpleName() + "_" + id;
        try (BocloudACLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            Cms cms = cmsRepository.query(id);
            cms = Optional.ofNullable(cms).orElseThrow(() -> new Exception("数据不存在"));
            if (null != cms.getTenantId() && cms.getTenantId() > 0) {
                return new BocloudResult<>(false, "租户数据不可删除");
            }
            cmsLogRepository.deleteByMq(cms.getId());
            if (ResourceCategory.Kafka.name().equalsIgnoreCase(cms.getType())) {
                cmsTopicRepository.deleteByMq(cms.getId());
            }
            cmsNodeRepository.deleteByCms(cms.getId());
            cmsRepository.remove(cms.getId(), context.getTarget());
            return new BocloudResult<>(true, cms.getName() + "删除成功");
        } catch (Exception e) {
            logger.error("remove ass failure:", e);
            return new BocloudResult<>(false, "删除失败");
        }
    }

    private void synchronousRole(List<CmsNode> nodes, Cms cms) throws Exception {
        if (ResourceCategory.Kafka.name().equalsIgnoreCase(cms.getType())) {
            BocloudResult result = executeKafka(nodes);
            if (result.isFailed()) {
                nodes = nodes.stream().map(node -> {
                    node.setStatus(BocloudStatus.Server.STOPPED.name());
                    node.setRole("(not reachable/healthy)");
                    return node;
                }).collect(Collectors.toList());
                cmsNodeRepository.batchUpdate(nodes, nodes.size());
            } else {
                Map<String, CmsNode> nodeMap = Maps.newHashMap();
                for (CmsNode node : nodes) {
                    nodeMap.put(node.getManageIp(), node);
                }
                List<String> ips = JSONArray.parseArray(JSON.toJSONString(result.getData()), String.class);
                for (String ip : ips) {
                    if (nodeMap.containsKey(ip)) {
                        CmsNode cmsNode = nodeMap.get(ip);
                        if (!BocloudStatus.Server.RUNNING.name().equals(cmsNode.getStatus())) {
                            cmsNode.setStatus(BocloudStatus.Server.RUNNING.name());
                            cmsNode.setRole("node");
                            cmsNodeRepository.update(cmsNode);
                        }
                        nodeMap.remove(ip);
                    }
                }
                for (String ip : nodeMap.keySet()) {
                    CmsNode cmsNode = nodeMap.get(ip);
                    cmsNode.setStatus(BocloudStatus.Server.STOPPED.name());
                    cmsNode.setRole("(not reachable/healthy)");
                    cmsNodeRepository.update(cmsNode);
                }
            }
        }
        // else if (ResourceCategory.ActiveMQ.name().equalsIgnoreCase(cms.getType())) {
        // for (CmsNode node : nodes) {
        // String status = BocloudStatus.Server.RUNNING.name();
        // ActiveMQConnectionFactory factory =
        // new ActiveMQConnectionFactory("tcp://" + node.getManageIp() + ":" + node.getPort());
        // Connection connection = null;
        // try {
        // connection = factory.createConnection(cms.getUsername(), MDSUtils.decrypt(cms.getPassword()));
        // connection.start();
        // } catch (Exception e) {
        // status = BocloudStatus.Server.STOPPED.name();
        // } finally {
        // if (null != connection) {
        // connection.close();
        // }
        // }
        // if (!status.equals(node.getStatus())) {
        // node.setStatus(status);
        // cmsNodeRepository.update(node);
        // }
        // }
        // }
        else if (ResourceCategory.RabbitMQ.name().equalsIgnoreCase(cms.getType())) {
            List<String> ips = new ArrayList<>();
            for (CmsNode node : nodes) {
                String url = "http://" + node.getManageIp() + ":" + node.getPort() + RabbitMQHelper.WHOAMI_API;
                Result result = RabbitMQHelper.get(url, cms.getUsername(), MDSUtils.decrypt(cms.getPassword()));
                if (result.isFailed()) {
                    if (!BocloudStatus.Server.STOPPED.name().equals(node.getStatus())) {
                        node.setStatus(BocloudStatus.Server.STOPPED.name());
                        node.setRole("(not reachable/healthy)");
                        cmsNodeRepository.update(node);
                        if (cms.getManageIp().equals(node.getManageIp())) {
                            cms.setManageIp(null);
                        }
                    }
                } else {
                    if (!BocloudStatus.Server.RUNNING.name().equals(node.getStatus())) {
                        node.setStatus(BocloudStatus.Server.RUNNING.name());
                        node.setRole("node");
                        cmsNodeRepository.update(node);
                    }
                    ips.add(node.getManageIp());
                }
            }
            if (!ips.isEmpty() && !ips.contains(cms.getManageIp())) {
                cms.setManageIp(ips.get(0));
            }
        }
        int statusCount = MDSUtils.statusCount(nodes, BocloudStatus.Server.RUNNING.name());
        if (0 == statusCount) {
            cms.setStatus(BocloudStatus.Server.STOPPED.name());
        } else if (statusCount == nodes.size()) {
            cms.setStatus(BocloudStatus.Server.RUNNING.name());
        } else {
            cms.setStatus(BocloudStatus.Server.EXCEPTION.name());
        }
        cmsRepository.update(cms);
    }

    @Override
    public void synchronousRoleState(JSONObject content) {
        try {
            Long id = content.getLong("id");
            Cms cms = cmsRepository.query(id);
            cms = Optional.ofNullable(cms).orElseThrow(() -> new Exception("数据不存在"));
            List<CmsNode> nodes = cmsNodeRepository.list(id);
            synchronousRole(nodes, cms);
            RequestContext context = new RequestContext();
            context.setTenant(cms.getTenantId());
            context.setTarget(cms.getTenantId());
            context.setCatalog(Catalog.valueOf(cms.getGmtCatalog()));
            resourceEventPublisher.send(new OperateResult(true, "节点角色状态变更通知",
                "node_change_" + cms.getType().toLowerCase(), null, OperateCategory.ServiceEvent, context));
        } catch (Exception e) {
            logger.error("集群角色同步失败：", e.getMessage());
        }
    }

    private BocloudResult executeKafka(List<CmsNode> nodes) {
        Properties admainProp = new Properties();
        StringBuilder ips = new StringBuilder();
        for (int i = 1; i <= nodes.size(); i++) {
            CmsNode cmsNode = nodes.get(i - 1);
            ips.append(cmsNode.getManageIp()).append(":" + cmsNode.getPort());
            if (i != nodes.size()) {
                ips.append(",");
            }
        }
        admainProp.put(AdminClientConfig.BOOTSTRAP_SERVERS_CONFIG, ips.toString());
        admainProp.put(AdminClientConfig.REQUEST_TIMEOUT_MS_CONFIG, 5000);
        try (AdminClient client = AdminClient.create(admainProp)) {
            Collection<Node> nodesList = client.describeCluster().nodes().get(6, TimeUnit.SECONDS);
            List<String> result = CollectionUtils.isNotEmpty(nodesList)
                ? nodesList.stream().map(Node::host).collect(Collectors.toList()) : Lists.newArrayList();
            return new BocloudResult<>(true, result, "获取节点信息成功");
            // DescribeClusterResult clusterResult = client.describeCluster();
            // while (true) {
            // if (clusterResult.nodes().isDone()) {
            // if (clusterResult.nodes().isCompletedExceptionally()) {
            // logger.error("获取节点信息失败:" + clusterResult.nodes());
            // return new BocloudResult<>(false, "获取节点信息失败");
            // }
            // List<String> result = Lists.newArrayList();
            // Iterator<Node> iterator = clusterResult.nodes().get().iterator();
            // while (iterator.hasNext()) {
            // result.add(iterator.next().host());
            // }
            // return new BocloudResult<>(true, result, "获取节点信息成功");
            // } else {
            // if (num > 2) {
            // logger.error("获取节点信息失败:" + clusterResult.nodes());
            // return new BocloudResult<>(false, "获取节点信息失败");
            // }
            // }
            // Thread.sleep(3000);
            // num++;
            // }
        } catch (Exception e) {
            logger.info("get kafka nodes error:", e);
            return new BocloudResult<>(false, "获取节点信息失败");
        }
    }

    @Override
    public void updateStatus(String content) {
        JSONObject object = JSONTools.isJSONObj(content);
        JSONObject service = object.getJSONObject("service");
        String uuid = service.getString("uuid");
        TaskInstance instance = JSONObject.parseObject(object.getString("instance"), TaskInstance.class);
        try {
            Cms cms = cmsRepository.get(uuid);
            if (null == cms) {
                return;
            }
            RequestContext context = JSONObject.parseObject(instance.getProps(), RequestContext.class);
            if (!"success".equalsIgnoreCase(instance.getStatus())) {
                List<TaskNode> nodes = instance.getNodes();
                String errorMsg = null;
                TaskNode softNode = null;
                for (TaskNode node : nodes) {
                    if (node.getCode().startsWith("task.software")) {
                        softNode = node;
                        continue;
                    }
                    JSONObject output = JSONObject.parseObject(node.getOutput());
                    if (null == output) {
                        continue;
                    }
                    if (Status.Node.FAILED.name().equalsIgnoreCase(node.getStatus()) && StringUtils.isEmpty(errorMsg)) {
                        errorMsg = output.getString("message");
                    }
                    JSONObject data = output.getJSONObject("data");
                    if (null == data) {
                        continue;
                    }
                    Long serverId = data.getLong("id");
                    if (null != serverId) {
                        logger.info("auto remove " + removed);
                        if (removed) {
                            imsCloudServerService.delete(serverId);
                        }
                    }
                }
                if (StringUtils.isEmpty(errorMsg)) {
                    JSONObject output = JSONObject.parseObject(softNode.getOutput());
                    List<JSONObject> datas = JSONObject.parseArray(output.getString("data"), JSONObject.class);
                    if (ListTool.isEmpty(datas)) {
                        errorMsg = output.getString("message");
                    } else {
                        for (JSONObject data : datas) {
                            if (!StringUtils.isEmpty(data.getString("message"))) {
                                errorMsg = data.getString("message");
                                break;
                            }
                        }
                    }
                }
                cmsRepository.remove(cms.getId(), 0L);
                MDSUtils.send(amqpTemplate, cms, nodes.size() - 1, OrderMessage.Operation.Application,
                    OrderItemStatus.OpenedFailure, MDSUtils.CMS, null,
                    JSONObject.parseObject(instance.getProps(), RequestContext.class), errorMsg);
                resourceEventPublisher.send(new OperateResult(false, errorMsg, "software.install", null,
                    OperateCategory.SoftwareInstall, context));
                return;
            }
            ResourceCategory category = ResourceCategory.getValue(cms.getType());
            Integer port = null;
            if (ResourceCategory.Kafka.equals(category)) {
                port = 9092;
            } else if (ResourceCategory.RabbitMQ.equals(category)) {
                port = 15672;
            }
            List<TaskNode> nodes = instance.getNodes();
            for (TaskNode node : nodes) {
                if (node.getCode().startsWith("task.resource.cvm")) {
                    JSONObject output = JSONObject.parseObject(node.getOutput());
                    if (null == output) {
                        continue;
                    }
                    JSONObject data = output.getJSONObject("data");
                    if (null == data) {
                        continue;
                    }
                    List<JSONObject> beans = JSONArray.parseArray(data.getString("privateIps"), JSONObject.class);
                    String address = null;
                    String managerIp = data.getString("managerIp");
                    for (JSONObject bean : beans) {
                        if (bean.getString("address").equals(managerIp)) {
                            continue;
                        }
                        address = bean.getString("address");
                    }
                    if (StringUtils.isEmpty(address)) {
                        address = managerIp;
                    }
                    cms.setManageIp(managerIp);
                    cms.setAddress(address + ":" + port);
                    CmsNode queueNode = new CmsNode(address, port, nodes.size() > 2 ? "node" : "standalone",
                        BocloudStatus.Server.RUNNING.name(), data.getString("instanceId"), cms.getId());
                    queueNode.setManageIp(managerIp);
                    cmsNodeRepository.save(queueNode);
                    cms.setVendorId(data.getLong("vendorId"));
                }
            }
            cms.setStatus(BocloudStatus.Server.RUNNING.name());
            cms.setTaskId(instance.getId());
            cmsRepository.update(cms);
            MDSUtils.send(amqpTemplate, cms, nodes.size() - 1, OrderMessage.Operation.Application,
                OrderItemStatus.OpenedSuccess, MDSUtils.CMS, null,
                JSONObject.parseObject(instance.getProps(), RequestContext.class), null);
            MDSUtils.send(amqpTemplate, cms.getId(), cms.getType(), "create");
            resourceEventPublisher.send(new OperateResult(true, "消息中间件申请安装成功", "software.install", null,
                OperateCategory.SoftwareInstall, context));
        } catch (Exception e) {
            logger.error("update cms status fail:", e);
        }
    }

    private BocloudResult postpone(ExtensionParam param, RequestContext context) {
        String path = Cms.class.getSimpleName() + "_" + param.getInstance();
        try (BocloudACLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            Cms cms = cmsRepository.get(param.getInstance());
            if (null == cms) {
                logger.warn("cms data not exist");
                return new BocloudResult<>(false, "数据不存在");
            }
            List<CmsNode> cmsNodes = cmsNodeRepository.list(cms.getId());
            for (CmsNode node : cmsNodes) {
                CloudServer server = cloudServerRepository.getByInstanceId(node.getInstanceId());
                if (null == server) {
                    server = cloudServerRepository.get(node.getServerId());
                }
                // 延期时扫描如果机器是关机状态则开机
                if (null != server && !BocloudStatus.Server.RUNNING.name().equals(server.getStatus())) {
                    imsCloudServerService.operation(server.getId(), "start");
                }
            }
            ResourceCategory category = ResourceCategory.getValue(cms.getType());
            Calendar calendar = Calendar.getInstance();
            calendar.setTime(cms.getExpiredTime());
            calendar.add(Calendar.MONTH, param.getExtension());
            cms.setExpiredTime(calendar.getTime());
            cmsRepository.update(cms);
            MDSUtils.send(amqpTemplate, cms, null, OrderMessage.Operation.Extension, OrderItemStatus.ExtensionSuccess,
                MDSUtils.CMS, param, context, null);
            return new BocloudResult<>(true, category.name() + "延期成功");
        } catch (Exception e) {
            logger.error("postpone cms fail:", e);
            return new BocloudResult<>(false, "消息队列延期失败");
        }
    }

    @Override
    public BocloudResult<GridBean<CmsLog>> logs(Pager pager, RequestContext context) {
        try {
            List<Param> params = pager.getParams();
            Integer page = pager.getPage();
            Integer rows = pager.getRows();
            Map<String, String> sorter = pager.getSorter();
            if (null == params) {
                params = new ArrayList<>();
            }
            if (null == sorter) {
                sorter = Maps.newHashMap();
            }
            sorter.put("gmtCreate", Common.ONE);
            List<CmsLog> logs = cmsLogRepository.list(page, rows, params, sorter);
            for (CmsLog log : logs) {
                log.setResult(JSONObject.parseObject(log.getResult()).getString("message"));
            }
            int total = cmsLogRepository.count(params);
            GridBean gridBean = GridHelper.getBean(page, rows, total, logs);
            return new BocloudResult<>(true, gridBean, "查询成功");
        } catch (Exception e) {
            logger.error("list cms logs failure:", e);
            return new BocloudResult<>(false, "查询失败");
        }
    }

    @Override
    public BocloudResult backups(Long id, RequestContext context) {
        try {
            List<CmsNode> nodes = cmsNodeRepository.list(id);
            if (ListTool.isEmpty(nodes)) {
                return new BocloudResult<>(false, "数据不存在");
            }
            String instanceId = null;
            Long serverId = null;
            for (CmsNode node : nodes) {
                if ("master".equalsIgnoreCase(node.getRole())) {
                    instanceId = node.getInstanceId();
                    serverId = node.getServerId();
                    break;
                }
            }
            CloudServer server = cloudServerRepository.getByInstanceId(instanceId);
            if (null == server) {
                server = cloudServerRepository.get(serverId);
            }
            if (null == server) {
                return new BocloudResult<>(false, "云主机不存在");
            }
            List<Snapshot> snapshots = snapshotRepository.listByVmid(server.getId());
            return new BocloudResult<>(true, snapshots, "查询成功");
        } catch (Exception e) {
            logger.error("list cms backups failure:", e);
            return new BocloudResult<>(false, "查询失败");
        }
    }

    private BocloudResult backup(CloudServer server, String type, RequestContext context) {
        try {
            CloudVendor vendor = cloudVendorRepository.query(server.getVendorId());
            if (null == vendor) {
                logger.warn("get Cloud vendor failure!");
                return new BocloudResult<>(false, "数据库备份失败");
            }
            String name = "MessageQueue-backup" + System.currentTimeMillis();
            // 开启一个线程更新状态
            threadPoolTaskExecutor.submit(() -> {
                String operate = "cms." + type + ".backup";
                try {
                    SnapshotModel model = new SnapshotModel(name, null, null, server.getInstanceId());
                    // BocloudResult result = serverDriver.snapshot(model, vendor.getUuid(), null);
                    // if (result.isSuccess()) {
                    // Snapshot snapshot = JSONObject.parseObject(JSONObject.toJSONString(result.getData()),
                    // Snapshot.class);
                    // snapshot.preSave(context.getTarget(), context.getTarget(), server.getVendorId(),
                    // server.getId(), server.getTenantId());
                    // snapshotRepository.save(snapshot);
                    // }
                    // resourceEventPublisher.send(
                    // new OperateResult(result.isSuccess(), "数据库备份" + (result.isSuccess() ? "成功" : "失败"),
                    // operate, null, OperateCategory.ResourceEvent, context));
                } catch (Exception e) {
                    resourceEventPublisher.send(
                        new OperateResult(false, "数据库备份失败", operate, null, OperateCategory.ResourceEvent, context));
                }
            });
            return new BocloudResult<>(true, "数据库备份任务已下发，正在执行...");
        } catch (Exception e) {
            logger.error("backup cms error:", e);
            return new BocloudResult<>(false, "数据库备份失败");
        }
    }

    @Override
    public BocloudResult backup(Long id, RequestContext context) {
        String path = Cms.class.getSimpleName() + "_" + id;
        try (BocloudACLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            List<CmsNode> nodes = cmsNodeRepository.list(id);
            if (ListTool.isEmpty(nodes)) {
                return new BocloudResult<>(false, "数据不存在");
            }
            Cms cms = cmsRepository.query(id);
            if (null == cms) {
                return new BocloudResult<>(false, "cms不存在");
            }
            String instanceId = null;
            Long serverId = null;
            for (CmsNode node : nodes) {
                if ("master".equalsIgnoreCase(node.getRole())) {
                    instanceId = node.getInstanceId();
                    serverId = node.getServerId();
                    break;
                }
            }
            CloudServer server = cloudServerRepository.getByInstanceId(instanceId);
            if (null == server) {
                server = cloudServerRepository.get(serverId);
            }
            if (null == server) {
                return new BocloudResult<>(false, "云主机不存在");
            }
            List<Param> params = new ArrayList<>();
            params.add(new Param(MapTools.simpleMap("vmId", server.getId()), Sign.EQ));
            int count = snapshotRepository.count(params);
            if (count >= 3) {
                return new BocloudResult<>(false, "数据库备份不能超过3次");
            }
            return backup(server, cms.getType(), context);
        } catch (Exception e) {
            logger.error("backup cms fail:", e);
            return new BocloudResult<>(false, "数据库备份失败");
        }
    }

    @Override
    public BocloudResult revert(Long id, Long snapshotId, RequestContext context) {
        String path = Cms.class.getSimpleName() + "_" + id;
        try (BocloudACLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            Cms cms = cmsRepository.query(id);
            if (null == cms) {
                return new BocloudResult<>(false, "cms不存在");
            }
            return revert(snapshotId, cms.getType(), context);
        } catch (Exception e) {
            logger.error("revert cms fail:", e);
            return new BocloudResult<>(false, "数据库备份失败");
        }
    }

    public BocloudResult revert(Long id, String type, RequestContext context) {
        String path = Snapshot.class.getSimpleName() + "_" + id;
        try (BocloudACLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            Snapshot snap = snapshotRepository.query(id);
            if (null == snap) {
                logger.warn("snapshot data not exist");
                return new BocloudResult<>(false, "恢复失败");
            }
            CloudServer server = cloudServerRepository.query(snap.getVmId());
            if (null == server) {
                logger.warn("cloud server data not exist");
                return new BocloudResult<>(false, "恢复失败");
            }
            // 云供应商
            CloudVendor vendor = cloudVendorRepository.query(snap.getVendorId());
            if (null == vendor) {
                logger.warn("cloud vendor data not exist");
                return new BocloudResult<>(false, "恢复失败");
            }
            // 启动一个线程从MQ上获取数据
            threadPoolTaskExecutor.submit(() -> {
                String subPath = CloudServer.class.getSimpleName() + "_" + server.getId();
                String operate = "cms." + type + ".reavert";
                BocloudResult result = null;
                String content = null;
                try (BocloudACLock bLock = lockFactory.getACLock(subPath)) {
                    if (!lock.acquire(10, TimeUnit.SECONDS)) {
                        logger.warn("Get BocloudLock time out!");
                        resourceEventPublisher.send(new OperateResult(false, "当前有恢复任务正在执行", operate, null,
                            OperateCategory.ResourceEvent, context));
                        return;
                    }
                    SnapshotModel model = new SnapshotModel();
                    model.setSnapshotUuid(snap.getSnapshotUuid());
                    model.setVmName(server.getName());
                    Server host = serverRepository.query(server.getHostId());
                    if (null != host) {
                        model.setHostName(host.getName());
                    }
                    logger.info("start to revert cloud server");
                    // result = snapshotDriver.revert(vendor.getUuid(), null, model);
                    content = result.isSuccess() ? "恢复成功" : "恢复失败";
                } catch (Exception e) {
                    logger.error("revert snapshot fail:", e);
                    resourceEventPublisher
                        .send(new OperateResult(false, "恢复失败", operate, null, OperateCategory.ResourceEvent, context));
                } finally {
                    if (result.isSuccess()) {
                        MDSUtils.powerOn(imsCloudServerService, server);
                    }
                    resourceEventPublisher.send(new OperateResult(result.isSuccess(), content, operate, null,
                        OperateCategory.ResourceEvent, context));
                }
            });
            return new BocloudResult<>(true, "恢复任务已经下发，正在执行……");
        } catch (Exception e) {
            logger.error("revert snapshot fail:", e);
            return new BocloudResult<>(false, "恢复失败");
        }
    }

    @Override
    public BocloudResult securities(Long id, RequestContext context) {
        try {
            List<CmsNode> nodes = cmsNodeRepository.list(id);
            if (ListTool.isEmpty(nodes)) {
                return new BocloudResult<>(false, "数据不存在");
            }
            for (CmsNode node : nodes) {
                if ("master".equalsIgnoreCase(node.getRole())) {
                    break;
                }
            }
            List<Object> groups = new ArrayList<>();
            return new BocloudResult<>(true, groups, "查询成功");
        } catch (Exception e) {
            logger.error("list cms securities failure:", e);
            return new BocloudResult<>(false, "查询失败");
        }
    }

    @Override
    public void saveLog(CmsLog log) {
        try {
            cmsLogRepository.save(log);
        } catch (Exception e) {
            logger.error("save cms log failure:", e);
        }
    }

    @Override
    public BocloudResult removeBackup(Long snapshotId, RequestContext context) {
        String path = Snapshot.class.getSimpleName() + "_" + snapshotId;
        try (BocloudACLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            Snapshot snap = snapshotRepository.query(snapshotId);
            if (null == snap) {
                logger.warn("snapshot data not exist");
                return new BocloudResult<>(false, "数据不存在");
            }
            CloudServer server = cloudServerRepository.query(snap.getVmId());
            if (null == server) {
                logger.warn("cloud server data not exist");
                return new BocloudResult<>(false, "删除数据库恢复失败");
            }
            // 云供应商
            CloudVendor vendor = cloudVendorRepository.query(snap.getVendorId());
            if (null == vendor) {
                return new BocloudResult<>(false, "数据库备份删除失败");
            }
            // CloudProvider type = CloudProvider.valueOf(vendor.getType());
            // 启动一个线程从MQ上获取数据
            threadPoolTaskExecutor.submit(() -> {
                String operate = "cms.backup";
                BocloudResult result = new BocloudResult<>(false, "数据库备份删除失败");
                try {
                    SnapshotModel model = new SnapshotModel();
                    model.setSnapshotUuid(snap.getSnapshotUuid());
                    model.setVmName(server.getName());
                    // switch (type) {
                    // case VMWARE:
                    // result = snapshotDriver.remove(vendor.getUuid(), null, model);
                    // break;
                    // default:
                    // break;
                    // }
                    String content = result.getMessage();
                    if (result.isSuccess()) {
                        snapshotRepository.remove(snapshotId, context.getTarget());
                        content = "数据库备份删除成功";
                    }
                    resourceEventPublisher.send(new OperateResult(result.isSuccess(), content, operate, null,
                        OperateCategory.ResourceEvent, context));
                } catch (Exception e) {
                    logger.error("Get Message Exception:", e);
                    resourceEventPublisher.send(new OperateResult(false, result.getMessage(), operate, null,
                        OperateCategory.ResourceEvent, context));
                }
            });
            return new BocloudResult<>(true, "删除任务已经下发，正在执行……");
        } catch (Exception e) {
            logger.error("remove snapshot fail:", e);
            return new BocloudResult<>(false, "删除失败");
        }
    }

    @Override
    public void install(String params, RequestContext context) {
        TaskNode node = JSONObject.parseObject(params, TaskNode.class);
        JSONObject output = JSONObject.parseObject(node.getOutput());
        if (null == output) {
            return;
        }
        List<JSONObject> hosts = JSONObject.parseArray(output.getString("data"), JSONObject.class);
        if (ListTool.isEmpty(hosts)) {
            return;
        }
        JSONObject input = JSONObject.parseObject(node.getParams());
        if (null == input) {
            return;
        }
        Integer port = null;
        ResourceCategory category = null;
        JSONObject fronts = JSONObject.parseObject(input.getString("fronts"));
        String version = fronts.getString("version");
        JSONObject config = fronts.getJSONObject("software_config");
        String username = null;
        String password = null;
        switch (node.getCode()) {
            case "task.software.rocketmq":
                category = ResourceCategory.RocketMQ;
                port = config.getInteger("rocketmq_nameserver_port");
                port = null == port ? 9876 : port;
                break;
            case "task.software.activemq":
                category = ResourceCategory.ActiveMQ;
                port = 61616;
                username = "zeus";
                try {
                    password = MDSUtils.encrypt("Zeus@v587");
                } catch (Exception e) {
                }
                break;
            case "task.software.rabbitmq":
                category = ResourceCategory.RabbitMQ;
                username = "zeus";
                try {
                    password = MDSUtils.encrypt("Zeus@v587");
                } catch (Exception e) {
                }
                port = 15672;
                break;
            case "task.software.kafka":
                category = ResourceCategory.Kafka;
                port = config.getInteger("port");
                port = null == port ? 9092 : port;
                break;
            default:
                return;
        }
        try {
            if (null != fronts.getBoolean("cluster_install") && fronts.getBoolean("cluster_install")) {
                if (Status.Node.FAILED.name().equals(node.getStatus())) {
                    return;
                }
                Cms cms = new Cms(category.name() + System.currentTimeMillis(), BocloudStatus.Server.RUNNING.name(),
                    context.getTarget(), IDFactory.instance().uuid(), category.name(),
                    hosts.size() > 1 ? "cluster" : "standalone", version, context.getTenant(), username, password, null,
                    node.getInstanceId());
                cms.setGmtCatalog(context.getCatalog().name());
                cms.setOwnerId(context.getTarget());
                cmsRepository.save(cms);
                String role = hosts.size() > 1 ? "node" : "standalone";
                String address = "";
                for (JSONObject host : hosts) {
                    if (ResourceCategory.RocketMQ.equals(category)) {
                        List<JSONObject> groups = JSONObject.parseArray(fronts.getString("groups"), JSONObject.class);
                        for (JSONObject group : groups) {
                            if (group.getString("hosts").contains(host.getString("host"))) {
                                if (group.getString("name").contains("slave")) {
                                    role = "slave";
                                } else if (group.getString("name").contains("master")) {
                                    role = "master";
                                } else if (group.getString("name").contains("namesrv")) {
                                    role = "namesrv";
                                }
                                break;
                            }
                        }
                        if ("master".equals(role)) {
                            cms.setManageIp(host.getString("host"));
                            cms.setAddress(host.getString("host") + ":" + port);
                        }
                    } else {
                        cms.setManageIp(host.getString("host"));
                    }
                    CmsNode cmsNode = new CmsNode(host.getString("host"), port, role,
                        BocloudStatus.Server.RUNNING.name(), null, cms.getId());
                    cmsNode.setManageIp(host.getString("host"));
                    if ("CloudServer".equals(host.getString("catalog"))) {
                        cmsNode.setServerId(host.getLong("id"));
                    } else {
                        Resource resource = resourceRepository.query(host.getLong("id"));
                        if (null != resource) {
                            cmsNode.setServerId(resource.getTarget());
                        }
                    }
                    cmsNodeRepository.save(cmsNode);
                    address += host.getString("host") + ":" + port + ",";
                }
                cms.setAddress(address.substring(0, address.length() - 1));
                cmsRepository.update(cms);
                MDSUtils.send(amqpTemplate, cms.getId(), cms.getType(), "create");
                return;
            }
            JSONObject target = JSONObject.parseObject(input.getString("target"));
            List<JSONObject> datas = JSONObject.parseArray(target.getString("data"), JSONObject.class);
            if (ListTool.isEmpty(datas)) {
                return;
            }
            Map<String, JSONObject> hostMap = new HashMap<>();
            for (JSONObject host : hosts) {
                if (host.getBoolean("success")) {
                    hostMap.put(host.getString("host"), host);
                }
            }
            for (JSONObject data : datas) {
                if (!hostMap.containsKey(data.getString("host"))) {
                    continue;
                }
                Cms cms = new Cms(category.name() + System.currentTimeMillis(), BocloudStatus.Server.RUNNING.name(),
                    context.getTarget(), IDFactory.instance().uuid(), category.name(), "standalone", version,
                    context.getTenant(), username, password, null, node.getInstanceId());
                cms.setManageIp(data.getString("host"));
                cms.setAddress(data.getString("host") + ":" + port);
                cms.setGmtCatalog(context.getCatalog().name());
                cms.setOwnerId(context.getTarget());
                cmsRepository.save(cms);
                CmsNode cmsNode = new CmsNode(data.getString("host"), port, "standalone",
                    BocloudStatus.Server.RUNNING.name(), null, cms.getId());
                cmsNode.setManageIp(data.getString("host"));
                if ("CloudServer".equals(data.getString("catalog"))) {
                    cmsNode.setServerId(data.getLong("id"));
                } else {
                    Resource resource = resourceRepository.query(data.getLong("id"));
                    if (null != resource) {
                        cmsNode.setServerId(resource.getTarget());
                    }
                }
                cmsNodeRepository.save(cmsNode);
                MDSUtils.send(amqpTemplate, cms.getId(), cms.getType(), "create");
            }
        } catch (Exception e) {
            logger.error("save Cms softwart install failure:", e);
        }
    }

}
