package com.bocloud.pms.service;

import java.io.ByteArrayInputStream;
import java.math.BigDecimal;
import java.net.InetSocketAddress;
import java.net.SocketAddress;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.Properties;
import java.util.Set;
import java.util.concurrent.Callable;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.Future;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

import org.apache.commons.lang3.StringUtils;
import org.bson.Document;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.amqp.core.AmqpTemplate;
import org.springframework.amqp.rabbit.core.RabbitAdmin;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.scheduling.concurrent.ThreadPoolTaskExecutor;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.GridBean;
import com.bocloud.common.model.OperateResult;
import com.bocloud.common.model.OperateResult.OperateCategory;
import com.bocloud.common.model.Pager;
import com.bocloud.common.model.Param;
import com.bocloud.common.model.RequestContext;
import com.bocloud.common.model.RequestContext.Catalog;
import com.bocloud.common.model.Sign;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.DateTools;
import com.bocloud.common.utils.GridHelper;
import com.bocloud.common.utils.IDFactory;
import com.bocloud.common.utils.JSONTools;
import com.bocloud.common.utils.ListTool;
import com.bocloud.common.utils.MapTools;
import com.bocloud.pms.entity.Kvs;
import com.bocloud.pms.entity.KvsLog;
import com.bocloud.pms.entity.KvsNode;
import com.bocloud.pms.entity.KvsParamLog;
import com.bocloud.pms.entity.KvsUser;
import com.bocloud.pms.entity.resource.Business;
import com.bocloud.pms.entity.resource.CloudServer;
import com.bocloud.pms.entity.resource.CloudVendor;
import com.bocloud.pms.entity.resource.OrderItem;
import com.bocloud.pms.entity.resource.OrderMessage;
import com.bocloud.pms.entity.resource.Project;
import com.bocloud.pms.entity.resource.Recycled;
import com.bocloud.pms.entity.resource.Resource;
import com.bocloud.pms.entity.resource.Server;
import com.bocloud.pms.entity.resource.ServiceOrder;
import com.bocloud.pms.entity.resource.Snapshot;
import com.bocloud.pms.entity.resource.TaskInstance;
import com.bocloud.pms.entity.resource.TaskNode;
import com.bocloud.pms.entity.resource.Tenant;
import com.bocloud.pms.entity.resource.User;
import com.bocloud.pms.entity.resource.enums.BocloudStatus;
import com.bocloud.pms.entity.resource.enums.CloudProvider;
import com.bocloud.pms.entity.resource.enums.OrderItemStatus;
import com.bocloud.pms.entity.resource.enums.ResourceCategory;
import com.bocloud.pms.entity.resource.enums.Status;
import com.bocloud.pms.entity.resource.model.AlterationParam;
import com.bocloud.pms.entity.resource.model.CancelationParam;
import com.bocloud.pms.entity.resource.model.ExtensionParam;
import com.bocloud.pms.entity.resource.model.LocationModel;
import com.bocloud.pms.entity.resource.model.NotifyMessage;
import com.bocloud.pms.entity.resource.model.SnapshotModel;
import com.bocloud.pms.interfaces.middleware.KvsService;
import com.bocloud.pms.repository.KvsLogRepository;
import com.bocloud.pms.repository.KvsNodeRepository;
import com.bocloud.pms.repository.KvsParamLogRepository;
import com.bocloud.pms.repository.KvsRepository;
import com.bocloud.pms.repository.KvsUserRepository;
import com.bocloud.pms.repository.resource.CloudServerRepository;
import com.bocloud.pms.repository.resource.CloudVendorRepository;
import com.bocloud.pms.repository.resource.ResourceRepository;
import com.bocloud.pms.repository.resource.ServerRepository;
import com.bocloud.pms.repository.resource.SnapshotRepository;
import com.bocloud.pms.service.internal.CosInternalService;
import com.bocloud.pms.service.internal.ImsCloudServerService;
import com.bocloud.pms.service.internal.SmsBusinessService;
import com.bocloud.pms.service.internal.SmsProjectService;
import com.bocloud.pms.service.internal.SmsTenantService;
import com.bocloud.pms.service.internal.SmsUserService;
import com.bocloud.pms.service.utils.MDSUtils;
import com.bocloud.pms.service.utils.MongodbconnectionRunable;
import com.bocloud.pms.service.utils.ResourceEventPublisher;
import com.bocloud.service.http.core.ServiceFactory;
import com.bocloud.service.lock.BocloudACLock;
import com.bocloud.service.lock.LockFactory;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import com.mongodb.MongoClient;
import com.mongodb.MongoClientOptions;
import com.mongodb.MongoCredential;
import com.mongodb.ReadPreference;
import com.mongodb.ServerAddress;
import com.mongodb.client.MongoDatabase;

import lombok.extern.slf4j.Slf4j;
import net.spy.memcached.MemcachedClient;
import redis.clients.jedis.Jedis;

/**
 * kvs Service接口实现
 *
 * @author weiwei
 */
@Slf4j
@Service("kvsService")
public class KvsServiceImpl implements KvsService {

    private static Logger logger = LoggerFactory.getLogger(KvsServiceImpl.class);
    @Autowired
    private LockFactory lockFactory;
    @Autowired
    private KvsRepository kvsRepository;
    @Autowired
    private KvsNodeRepository kvsNodeRepository;
    @Autowired
    private KvsUserRepository kvsUserRepository;
    @Autowired
    private KvsLogRepository kvsLogRepository;
    @Autowired
    private KvsParamLogRepository kvsParamLogRepository;
    @Autowired
    private SnapshotRepository snapshotRepository;
    @Autowired
    private CloudVendorRepository cloudVendorRepository;
    @Autowired
    private CloudServerRepository cloudServerRepository;
    @Autowired
    private ResourceRepository resourceRepository;
    @Autowired
    private AmqpTemplate amqpTemplate;
    @Autowired
    private ResourceEventPublisher resourceEventPublisher;
    @Autowired
    private ServerRepository serverRepository;
    @Autowired
    private ServiceFactory serviceFactory;
    @Autowired
    private RabbitAdmin rabbitAdmin;
    @Value("${middleware.server.remove:true}")
    private boolean removed;
    @Autowired
    private RedisTemplate<String, String> redisTemplate;
    @Autowired
    private CosInternalService cosInternalService;
    @Autowired
    private ImsCloudServerService imsCloudServerService;
    @Autowired
    private ThreadPoolTaskExecutor threadPoolTaskExecutor;
    @Autowired
    private SmsProjectService smsProjectService;
    @Autowired
    private SmsUserService smsUserService;
    @Autowired
    private SmsBusinessService smsBusinessService;
    @Autowired
    private SmsTenantService smsTenantService;

    @Override
    public BocloudResult<GridBean<Kvs>> list(Pager pager, RequestContext context) {
        try {
            List<Param> params = pager.getParams();
            Integer page = pager.getPage();
            Integer rows = pager.getRows();
            Map<String, String> sorter = pager.getSorter();
            params = Optional.ofNullable(params).orElse(new ArrayList<>());
            sorter = Optional.ofNullable(sorter).orElse(new HashMap<>());
            sorter.put("gmtCreate", Common.ONE);
            List<Kvs> list = null;
            int total = 0;
            if (context.getCatalog().equals(Catalog.Tenant)) {
                // 租户能看到 1.属于当前租户的资源
                list = kvsRepository.list(page, rows, params, sorter, context.getTenant());
                total = kvsRepository.count(params, context.getTenant());
            } else if (context.getCatalog().equals(Catalog.User)) {
                // 用户能看到 1.自己申请的资源 2.自己所在的资源可见项目（资源对项目下所有成员可见）下创建的资源 3.自己管理的项目下的资源
                List<Project> userManagerProjects = smsProjectService.listUserManagerProjects(context.getTarget());
                List<Project> visibleProjects = smsProjectService.listUserVisibleProjects(context.getTarget());
                List<Long> projectIds = new ArrayList<>();
                List<Long> userManagerProjectsIds =
                    userManagerProjects.stream().map(Project::getId).collect(Collectors.toList());
                if (!userManagerProjectsIds.isEmpty()) {
                    projectIds.addAll(userManagerProjectsIds);
                }
                List<Long> visibleProjectsIds =
                    visibleProjects.stream().map(Project::getId).collect(Collectors.toList());
                if (!visibleProjectsIds.isEmpty()) {
                    projectIds.addAll(visibleProjectsIds);
                }
                list = kvsRepository.list(page, rows, params, sorter, context.getTarget(), projectIds);
                total = kvsRepository.count(params, context.getTarget(), projectIds);
            } else {
                list = kvsRepository.list(page, rows, params, sorter, null);
                total = kvsRepository.count(params, null);
            }
            // 处理业务信息
            List<Long> tenantIds = new ArrayList<>();
            List<Long> userIds = new ArrayList<>();
            List<Long> businessIds = new ArrayList<>();
            List<Long> projectIds = new ArrayList<>();
            for (Kvs kvs : list) {
                if (kvs.getTenantId() != null) {
                    tenantIds.add(kvs.getTenantId());
                }
                if (kvs.getBusinessId() != null) {
                    businessIds.add(kvs.getBusinessId());
                }
                if (kvs.getProjectId() != null) {
                    projectIds.add(kvs.getProjectId());
                }
                if (kvs.getCreatorId() != null) {
                    userIds.add(kvs.getCreatorId());
                }
                if (kvs.getOwnerId() != null) {
                    userIds.add(kvs.getOwnerId());
                }
                if (kvs.getMenderId() != null) {
                    userIds.add(kvs.getMenderId());
                }
            }
            Map<Long, String> userMap =
                smsUserService.listUsersByIds(userIds).stream().collect(Collectors.toMap(User::getId, User::getName));
            Map<Long, String> projectMap = smsProjectService.listProjectsByIds(projectIds).stream()
                .collect(Collectors.toMap(Project::getId, Project::getName));
            Map<Long, String> businessMap = smsBusinessService.listBusinesssByIds(businessIds).stream()
                .collect(Collectors.toMap(Business::getId, Business::getName));
            Map<Long, String> tenantMap = smsTenantService.listTenantsByIds(tenantIds).stream()
                .collect(Collectors.toMap(Tenant::getId, Tenant::getName));
            for (Kvs kvs : list) {
                JSONObject specs = JSONObject.parseObject(kvs.getSpec());
                if (null != specs) {
                    kvs.setSpec(specs.get("cpu") + "核/" + specs.get("memory") + "GB/" + specs.get("disk") + "GB");
                    kvs.setDataDisk(specs.getInteger("dataDisk"));
                }
                JSONObject inventory = JSONObject.parseObject(kvs.getInventory());
                if (null != inventory) {
                    List<JSONObject> elements = JSONArray.parseArray(inventory.getString("elements"), JSONObject.class);
                    if (!ListTool.isEmpty(elements)) {
                        for (JSONObject object : elements) {
                            if (object.getString("serviceCode").equalsIgnoreCase("kvs." + kvs.getType())) {
                                continue;
                            }
                            kvs.setDiskSkuId(object.getLong("skuId"));
                        }
                    }
                }
                if (!"standalone".equals(kvs.getCategory())) {
                    List<KvsNode> nodes = kvsNodeRepository.list(kvs.getId());
                    String address = "";
                    for (KvsNode node : nodes) {
                        if ("sentinel".equalsIgnoreCase(node.getRole()) || "arbiter".equalsIgnoreCase(node.getRole())
                            || "magent".equalsIgnoreCase(node.getRole())) {
                            continue;
                        }
                        address += node.getManageIp() + ":" + node.getPort() + ",";
                    }
                    if (!StringUtils.isEmpty(address)) {
                        kvs.setAddress(address.substring(0, address.length() - 1));
                    }
                }
                // 设置云主机归属信息
                if (kvs.getTenantId() != null) {
                    kvs.setTenantName(tenantMap.get(kvs.getTenantId()));
                }
                if (kvs.getBusinessId() != null) {
                    kvs.setBusinessName(businessMap.get(kvs.getBusinessId()));
                }
                if (kvs.getProjectId() != null) {
                    kvs.setProjectName(projectMap.get(kvs.getProjectId()));
                }
                if (kvs.getCreatorId() != null) {
                    kvs.setCreatorName(userMap.get(kvs.getCreatorId()));
                }
                if (kvs.getOwnerId() != null) {
                    kvs.setOwnerName(userMap.get(kvs.getOwnerId()));
                }
                if (kvs.getMenderId() != null) {
                    kvs.setMenderName(userMap.get(kvs.getMenderId()));
                }
            }
            GridBean gridBean = GridHelper.getBean(page, rows, total, list);
            return new BocloudResult<>(true, gridBean, "查询成功");
        } catch (Exception e) {
            logger.error("list kvs failure:", e);
            return new BocloudResult<>(false, "查询失败", null, null);
        }
    }

    @Override
    public BocloudResult apply(Object object, ResourceCategory category, OrderMessage.Operation operation,
        RequestContext context) {
        BocloudResult result = null;
        switch (operation) {
            case Application:
                result = create(JSONObject.parseObject(JSONObject.toJSONString(object)), category, context);
                break;
            case Upgrade:
                result = modify((AlterationParam)object, context);
                break;
            case Cancelation:
                result = remove((CancelationParam)object, context);
                break;
            case Extension:
                result = postpone((ExtensionParam)object, context);
                break;
            default:
                result = new BocloudResult<>(false, "暂不支持该类操作");
                break;
        }
        return result;
    }

    private BocloudResult modify(AlterationParam param, RequestContext context) {
        String path = Kvs.class.getSimpleName() + "_" + param.getInstance();
        try (BocloudACLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            Kvs kvs = kvsRepository.get(param.getInstance());
            kvs = Optional.ofNullable(kvs).orElseThrow(() -> new Exception("数据不存在"));
            ResourceCategory category = ResourceCategory.getValue(kvs.getType());
            List<KvsNode> nodes = kvsNodeRepository.list(kvs.getId());
            if (ListTool.isEmpty(nodes)) {
                return new BocloudResult<>(false, "数据库节点不存在");
            }
            List<CloudServer> servers = new ArrayList<>();
            for (KvsNode node : nodes) {
                CloudServer server = cloudServerRepository.getByInstanceId(node.getInstanceId());
                if (null == server) {
                    server = cloudServerRepository.get(node.getServerId());
                }
                if (null != server) {
                    servers.add(server);
                }
            }
            if (ListTool.isEmpty(servers)) {
                return new BocloudResult<>(false, "数据库节点不存在");
            }
            CloudVendor vendor = cloudVendorRepository.query(servers.get(0).getVendorId());
            if (null == vendor) {
                logger.warn("Cloud vendor does not exist!");
                return new BocloudResult<>(false, "云平台不存在!");
            }
            JSONObject specs = JSONObject.parseObject(kvs.getSpec());
            if (null == param.getCpu() && null != param.getDataDisk()) {
                if (param.getDataDisk() <= specs.getIntValue("dataDisk")) {
                    return new BocloudResult<>(false, "数据盘扩容大小不能等于小于当前大小");
                }
                for (CloudServer server : servers) {
                    JSONObject object = new JSONObject();
                    object.put("id", server.getId());
                    object.put("name", server.getName());
                    object.put("intanceId", server.getInstanceId());
                    object.put("dataDisk", param.getDataDisk());
                    logger.info("数据盘扩容参数：" + object.toJSONString());
                    BocloudResult result = imsCloudServerService.resize(server.getId(), object, "upgradeconfig");
                    if (result.isFailed()) {
                        return new BocloudResult<>(false, "数据盘扩容失败：" + result.getMessage());
                    }
                }
                specs.put("dataDisk", param.getDataDisk());
                kvs.setSpec(specs.toJSONString());
                // kvs.setUuid(param.getNewInstance());
                kvs.setMenderId(context.getTarget());
                kvs.setGmtModify(new Date());
                kvsRepository.update(kvs);
                MDSUtils.send(amqpTemplate, kvs, nodes.size(), OrderMessage.Operation.Upgrade,
                    OrderItemStatus.ChangeSuccess, MDSUtils.KVS, param, context, null);
                return new BocloudResult<>(true, category.name() + "数据盘扩容成功");
            }
            // CPU
            Integer cpu = param.getCpu();
            // 内存
            BigDecimal memory = param.getMemory();
            // 磁盘
            // Integer disk = param.getSystemDisk();
            // if (cpu < specs.getIntValue("cpu") || memory.doubleValue() < specs.getBigDecimal("memory").doubleValue()
            // || disk < specs.getIntValue("disk")) {
            if (cpu < specs.getIntValue("cpu") || memory.doubleValue() < specs.getBigDecimal("memory").doubleValue()) {
                return new BocloudResult<>(false, "升级配置不能小于当前配置");
            }
            // if (cpu.equals(specs.getIntValue("cpu")) && memory.equals(specs.getBigDecimal("memory"))
            // && disk.equals(specs.getIntValue("disk"))) {
            if (cpu.equals(specs.getIntValue("cpu")) && memory.equals(specs.getBigDecimal("memory"))) {
                return new BocloudResult<>(false, "升级配置等于当前配置");
            }
            for (CloudServer server : servers) {
                JSONObject object = new JSONObject();
                object.put("id", server.getId());
                object.put("name", server.getName());
                object.put("intanceId", server.getInstanceId());
                object.put("cpu", param.getCpu());
                object.put("memory", param.getMemory());
                object.put("systemDisk", param.getSystemDisk());
                logger.info("配置升级参数：" + object.toJSONString());
                BocloudResult result = imsCloudServerService.resize(server.getId(), object, "upgradeconfig");
                if (result.isFailed()) {
                    return new BocloudResult<>(false, "配置升级失败：" + result.getMessage());
                }
            }
            specs.put("cpu", param.getCpu());
            specs.put("memory", param.getMemory());
            // specs.put("disk", param.getSystemDisk());
            kvs.setSpec(specs.toJSONString());
            // kvs.setUuid(param.getNewInstance());
            kvs.setMenderId(context.getTarget());
            kvs.setGmtModify(new Date());
            kvs.setSkuId(param.getUpgradeSkuId());
            kvsRepository.update(kvs);
            MDSUtils.send(amqpTemplate, kvs, nodes.size(), OrderMessage.Operation.Upgrade,
                OrderItemStatus.ChangeSuccess, MDSUtils.KVS, param, context, null);
            return new BocloudResult<>(true, category.name() + "规格变更成功");
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return new BocloudResult<>(false, "数据库规格变更失败");
        }
    }

    private BocloudResult create(JSONObject object, ResourceCategory category, RequestContext context) {
        ServiceOrder order = JSON.parseObject(object.getString("order"), ServiceOrder.class);
        List<OrderItem> items = order.getItems();
        if (ListTool.isEmpty(items)) {
            return new BocloudResult<>(false, "订单数据错误！");
        }
        JSONObject emption = JSONObject.parseObject(object.getString("emption"));
        // 使用时长信息
        JSONObject duration = JSONObject.parseObject(emption.getString("duration"));
        // 使用时长单位Hour/Month
        String mode = duration.getString("mode");
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(new Date());
        if ("Month".equalsIgnoreCase(mode)) {
            calendar.add(Calendar.MONTH, duration.getInteger("amount"));
        } else if ("Year".equalsIgnoreCase(mode)) {
            calendar.add(Calendar.MONTH, duration.getInteger("amount"));
        } else {
            calendar.setTime(DateTools.stringToDateTime("2099-12-31 00:00:00"));
        }
        try {
            JSONObject params = MDSUtils.analyze(object);
            JSONObject software = params.getJSONObject("software");
            JSONObject resource = params.getJSONObject("resource");
            LocationModel location = JSONObject.parseObject(object.getString("location"), LocationModel.class);
            for (int i = 0; i < items.size(); i++) {
                if (!items.get(i).getServiceCode().startsWith("kvs.")) {
                    continue;
                }
                Kvs kvs = new Kvs(category.name() + IDFactory.instance().uuid(), BocloudStatus.Server.BUILDING.name(),
                    null, IDFactory.instance().uuid(), category.name(), software.getString("categoryCode"),
                    software.getString("version"), null, null, null, null, null);
                kvs.setUuid(items.get(i).getSn());
                kvs.setRegion(location.getRegion());
                kvs.setResourceLabel(resource.getString("resourceLabel"));
                kvs.setInventory(object.toJSONString());
                kvs.setSkuId(software.getLong("skuId"));
                kvs.setCategoryId(software.getLong("categoryId"));
                JSONObject specs = new JSONObject();
                specs.put("cpu", resource.getString("cpu"));
                specs.put("memory", resource.getString("memory"));
                specs.put("disk", resource.getString("disk"));
                List<Integer> disks = JSONObject.parseArray(resource.getString("disks"), Integer.class);
                if (!ListTool.isEmpty(disks)) {
                    specs.put("dataDisk", disks.get(0));
                }
                kvs.setSpec(specs.toJSONString());
                if (ResourceCategory.Redis.equals(category)) {
                    JSONObject config = software.getJSONObject("manage_config");
                    kvs.setPassword(config.getString("redis_password"));
                } else if (ResourceCategory.MongoDB.equals(category)) {
                    kvs.setUsername("zeus");
                    kvs.setPassword(MDSUtils.encrypt("Zeus@v587"));
                }

                // 自服务需要用到
                kvs.setCreatorId(location.getCreatorId() == null ? 0 : location.getCreatorId());
                kvs.setOwnerId(location.getOwnerId() == null ? 0 : location.getOwnerId());
                kvs.setMenderId(location.getMenderId() == null ? 0 : location.getMenderId());
                kvs.setGmtCatalog(context.getCatalog().name());
                kvs.setTenantId(context.getTenant() == null ? 0 : context.getTenant());
                kvs.setBusinessId(location.getBusinessId() == null ? 0 : location.getBusinessId());
                kvs.setProjectId(location.getProjectId() == null ? 0 : location.getProjectId());
                kvs.setPoolGroupId(location.getPoolGroupId());
                kvs.setExpiredTime(calendar.getTime());
                kvsRepository.save(kvs);
                JSONObject message = new JSONObject();
                message.put("params", params);
                message.put("context", context);
                message.put("queue", "service.task.result");
                JSONObject service = new JSONObject();
                service.put("uuid", kvs.getUuid());
                service.put("category", category.name());
                message.put("service", service);
                this.amqpTemplate.convertAndSend("service.task.queue", message);
            }
            return new BocloudResult<>(true, category.name() + "订购成功");
        } catch (Exception e) {
            logger.error("save kvs failure:", e);
            return new BocloudResult<>(false, "数据库订购失败");
        }
    }

    private BocloudResult remove(CancelationParam param, RequestContext context) {
        String path = Kvs.class.getSimpleName() + "_" + param.getInstance();
        try (BocloudACLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            Kvs kvs = kvsRepository.get(param.getInstance());
            kvs = Optional.ofNullable(kvs).orElseThrow(() -> new Exception("数据不存在"));
            List<KvsNode> nodes = kvsNodeRepository.list(kvs.getId());
            for (KvsNode node : nodes) {
                CloudServer server = cloudServerRepository.getByInstanceId(node.getInstanceId());
                if (null == server) {
                    server = cloudServerRepository.get(node.getServerId());
                }
                if (null != server) {
                    imsCloudServerService.delete(server.getId());
                }
            }
            ResourceCategory category = ResourceCategory.getValue(kvs.getType());
            kvsLogRepository.deleteByKvs(kvs.getId());
            kvsParamLogRepository.deleteByKvs(kvs.getId());
            kvsNodeRepository.deleteByKvs(kvs.getId());
            kvsUserRepository.deleteByKvs(kvs.getId());
            kvsRepository.remove(kvs.getId(), context.getTarget());
            MDSUtils.send(amqpTemplate, kvs, nodes.size(), OrderMessage.Operation.Cancelation,
                OrderItemStatus.UnsubscribeSuccess, MDSUtils.KVS, param, context, null);
            MDSUtils.send(amqpTemplate, kvs.getId(), kvs.getType(), "delete");
            // 清空回收站
            cosInternalService.recycle(kvs.getUuid());
            return new BocloudResult<>(true, category.name() + "退订成功");
        } catch (Exception e) {
            logger.error("remove kvs failure:", e);
            return new BocloudResult<>(false, "数据库退订失败");
        }
    }

    @Override
    public BocloudResult<Kvs> detail(Long id) {
        try {
            Kvs kvs = kvsRepository.query(id);
            kvs = Optional.ofNullable(kvs).orElseThrow(() -> new Exception("数据不存在"));
            List<KvsNode> nodes = kvsNodeRepository.list(id);
            nodes = synchronousRole(nodes, kvs);
            StringBuilder builder = new StringBuilder();
            List<Map<String, String>> connectionList = Lists.newArrayList();
            if (ResourceCategory.MongoDB.name().equalsIgnoreCase(kvs.getType())) {
                kvs.setUsers(kvsUserRepository.list(id));
                for (KvsNode kvsNode : nodes) {
                    Map<String, String> collections = Maps.newHashMap();
                    if (!"arbiter".equalsIgnoreCase(kvsNode.getRole())) {
                        collections.put("role", kvsNode.getRole());
                        collections.put("address", kvsNode.getAddress() + ":" + kvsNode.getPort());
                        connectionList.add(collections);
                        builder.append(kvsNode.getAddress() + ":" + kvsNode.getPort()).append(",");
                    }
                }
            } else if (ResourceCategory.Redis.name().equalsIgnoreCase(kvs.getType())) {
                for (KvsNode kvsNode : nodes) {
                    Map<String, String> collections = Maps.newHashMap();
                    if (!"sentinel".equalsIgnoreCase(kvsNode.getRole())) {
                        String role = kvsNode.getRole();
                        if (role.contains("master")) {
                            role = "master";
                        }
                        collections.put("role", role);
                        collections.put("address", kvsNode.getAddress() + ":" + kvsNode.getPort());
                        connectionList.add(collections);
                        builder.append(kvsNode.getAddress() + ":" + kvsNode.getPort()).append(",");
                    }
                }
            } else if (ResourceCategory.MemCache.name().equalsIgnoreCase(kvs.getType())) {
                for (KvsNode kvsNode : nodes) {
                    Map<String, String> collections = Maps.newHashMap();
                    if (!"magent".equalsIgnoreCase(kvsNode.getRole())) {
                        collections.put("role", kvsNode.getRole());
                        collections.put("address", kvsNode.getAddress() + ":" + kvsNode.getPort());
                        connectionList.add(collections);
                        builder.append(kvsNode.getAddress() + ":" + kvsNode.getPort()).append(",");
                    }
                }
            }
            String conn = StringUtils.substringBeforeLast(builder.toString(), ",");
            HashMap<String, String> connectionUrl = Maps.newHashMap();
            connectionUrl.put("role", "connectionUrl");
            connectionUrl.put("address", conn);
            connectionList.add(connectionUrl);
            kvs.setConnections(connectionList);
            JSONObject specs = JSONObject.parseObject(kvs.getSpec());
            if (null != specs) {
                kvs.setSpec(specs.get("cpu") + "核/" + specs.get("memory") + "GB/" + specs.get("disk") + "GB");
                kvs.setDataDisk(specs.getInteger("dataDisk"));
            }
            JSONObject inventory = JSONObject.parseObject(kvs.getInventory());
            if (null != inventory) {
                JSONObject configs = inventory.getJSONObject("configs");
                kvs.setDiskCategory(configs.getString("diskCategoryName"));
            }
            kvs.setNodes(nodes);
            // 处理业务信息
            if (kvs.getTenantId() != null && kvs.getTenantId() != 0) {
                Map<Long, String> tenantMap = smsTenantService.listTenantsByIds(Arrays.asList(kvs.getTenantId()))
                    .stream().collect(Collectors.toMap(Tenant::getId, Tenant::getName));
                kvs.setTenantName(tenantMap.get(kvs.getTenantId()));
            }
            if (kvs.getBusinessId() != null && kvs.getBusinessId() != 0) {
                Map<Long, String> businessMap =
                    smsBusinessService.listBusinesssByIds(Arrays.asList(kvs.getBusinessId())).stream()
                        .collect(Collectors.toMap(Business::getId, Business::getName));
                kvs.setBusinessName(businessMap.get(kvs.getBusinessId()));
            }
            if (kvs.getProjectId() != null && kvs.getProjectId() != 0) {
                Map<Long, String> projectMap = smsProjectService.listProjectsByIds(Arrays.asList(kvs.getProjectId()))
                    .stream().collect(Collectors.toMap(Project::getId, Project::getName));
                kvs.setProjectName(projectMap.get(kvs.getProjectId()));
            }
            List<Long> userIds = new ArrayList<>();
            if (kvs.getCreatorId() != null && kvs.getCreatorId() != 0) {
                userIds.add(kvs.getCreatorId());
            }
            if (kvs.getOwnerId() != null && kvs.getOwnerId() != 0) {
                userIds.add(kvs.getOwnerId());
            }
            if (kvs.getMenderId() != null && kvs.getMenderId() != 0) {
                userIds.add(kvs.getMenderId());
            }
            if (!ListTool.isEmpty(userIds)) {
                Map<Long, String> userMap = smsUserService.listUsersByIds(userIds).stream()
                    .collect(Collectors.toMap(User::getId, User::getName));
                kvs.setCreatorName(userMap.get(kvs.getCreatorId()));
                kvs.setOwnerName(userMap.get(kvs.getOwnerId()));
                kvs.setMenderName(userMap.get(kvs.getMenderId()));
            }
            return new BocloudResult<>(true, kvs, "查询详情成功");
        } catch (Exception e) {
            logger.error("query kvs detail fail:", e);
            return new BocloudResult<>(false, "查询详情失败");
        }
    }

    @Override
    public BocloudResult<Kvs> basics(Long id) {
        try {
            Kvs kvs = kvsRepository.query(id);
            kvs = Optional.ofNullable(kvs).orElseThrow(() -> new Exception("数据不存在"));
            List<KvsNode> nodes = kvsNodeRepository.list(id);
            kvs.setNodes(nodes);
            return new BocloudResult<>(true, kvs, "查询成功");
        } catch (Exception e) {
            logger.error("query kvs detail fail:", e);
            return new BocloudResult<>(false, "查询详情失败");
        }
    }

    private List<KvsNode> synchronousRole(List<KvsNode> nodes, Kvs kvs) throws Exception {
        List<KvsNode> nodeList = new ArrayList<>();
        if ("standalone".equals(kvs.getCategory())) {
            KvsNode node = nodes.get(0);
            BocloudResult result = null;
            if (ResourceCategory.MongoDB.name().equalsIgnoreCase(kvs.getType())) {
                result = executeMongod(kvs, null, null, null, true, false);
            } else if (ResourceCategory.Redis.name().equalsIgnoreCase(kvs.getType())) {
                result = executeRedis(kvs, null, null, true);
            } else if (ResourceCategory.MemCache.name().equalsIgnoreCase(kvs.getType())) {
                result = executeMemCache(nodes);
            }
            String status;
            if (null == result) {
                status = BocloudStatus.Server.STOPPED.name();
            } else {
                status = result.isFailed() ? BocloudStatus.Server.STOPPED.name() : BocloudStatus.Server.RUNNING.name();
            }

            if (null != result && result.isSuccess()
                && ResourceCategory.MemCache.name().equalsIgnoreCase(kvs.getType())) {
                JSONObject object = JSONObject.parseObject(JSONObject.toJSONString(result.getData()));
                status = object.getBoolean(node.getManageIp()) ? BocloudStatus.Server.RUNNING.name()
                    : BocloudStatus.Server.STOPPED.name();
            }
            if (!status.equals(node.getStatus())) {
                node.setStatus(status);
                kvsNodeRepository.update(node);
                kvs.setStatus(status);
                kvsRepository.update(kvs);
            }
            nodeList.add(node);
        } else {
            Map<String, KvsNode> nodeMap = Maps.newHashMap();
            if (ResourceCategory.MongoDB.name().equalsIgnoreCase(kvs.getType())) {
                for (KvsNode node : nodes) {
                    if ("arbiter".equals(node.getRole())) {
                        nodeList.add(node);
                    } else {
                        nodeMap.put(node.getAddress(), node);
                    }
                }
                BocloudResult result = executeMongod(kvs, null, null, null, true, true);
                boolean flag = false;
                if (result.isFailed()) {
                    // 当前master节点无法连接，启动线程通过replicas节点去获取节点的角色信息
                    ExecutorService mainExecutor =
                        Executors.newFixedThreadPool(Runtime.getRuntime().availableProcessors() + 1);
                    Set<Callable<BocloudResult>> callables = new HashSet<Callable<BocloudResult>>();
                    for (KvsNode node : nodes) {
                        if (!"arbiter".equals(node.getRole())) {
                            callables.add(new MongodbconnectionRunable(kvs, node));
                        }
                    }
                    List<Future<BocloudResult>> futures = mainExecutor.invokeAll(callables);
                    List<JSONObject> members = Lists.newArrayList();
                    for (Future<BocloudResult> future : futures) {
                        @SuppressWarnings("unchecked")
                        List<JSONObject> data = (List<JSONObject>)future.get().getData();
                        if (!data.isEmpty()) {
                            members.addAll(data);
                            break;
                        }
                    }
                    mainExecutor.shutdown();
                    mainExecutor.awaitTermination(5, TimeUnit.MINUTES);
                    if (!members.isEmpty()) {
                        checkMongodbNodeRole(kvs, members, nodeMap);
                    }
                    flag = members.isEmpty();
                } else {
                    JSONObject object = JSONObject.parseObject(JSONObject.toJSONString(result.getData()));
                    List<JSONObject> members = object.getJSONArray("members").toJavaList(JSONObject.class);
                    checkMongodbNodeRole(kvs, members, nodeMap);
                }
                for (KvsNode node : nodeMap.values()) {
                    if (flag) {
                        node.setStatus(BocloudStatus.Server.STOPPED.name());
                        kvsNodeRepository.update(node);
                    }
                    nodeList.add(node);
                }
            } else if (ResourceCategory.Redis.name().equalsIgnoreCase(kvs.getType())) {
                for (KvsNode node : nodes) {
                    if ("sentinel".equals(node.getRole())) {
                        nodeList.add(node);
                    } else {
                        nodeMap.put(node.getManageIp(), node);
                    }
                }
                List<JSONObject> objects = executeRedis(kvs, nodes);
                for (JSONObject data : objects) {
                    String role = data.getString("role");
                    String address = StringUtils.substringBefore(data.getString("ip"), ":");
                    if (nodeMap.containsKey(address)) {
                        KvsNode node = nodeMap.get(address);
                        if (!role.equals(node.getRole())) {
                            node.setRole(role);
                            kvsNodeRepository.update(node);
                        }
                        if ("(not reachable/healthy)".equals(role)) {
                            node.setStatus(BocloudStatus.Server.STOPPED.name());
                        } else {
                            node.setStatus(BocloudStatus.Server.RUNNING.name());
                        }
                        nodeMap.remove(address);
                        if ("master".equals(role)) {
                            String slaveAddress = data.getString("slaveAddress");
                            if (StringUtils.isNotEmpty(slaveAddress)) {
                                node.setRole(role + "(slave:" + slaveAddress + ")");
                            }
                        }
                        nodeList.add(node);
                    }
                }
                if (!nodeMap.isEmpty()) {
                    for (String ip : nodeMap.keySet()) {
                        KvsNode node = nodeMap.get(ip);
                        node.setRole("(not reachable/healthy)");
                        node.setStatus(BocloudStatus.Server.STOPPED.name());
                        kvsNodeRepository.update(node);
                        nodeList.add(node);
                    }
                }
            } else if (ResourceCategory.MemCache.name().equalsIgnoreCase(kvs.getType())) {
                BocloudResult result = executeMemCache(nodes);
                JSONObject nodeState = JSONObject.parseObject(JSONObject.toJSONString(result.getData()));
                for (KvsNode node : nodes) {
                    if ("magent".equals(node.getRole())) {
                        nodeList.add(node);
                        continue;
                    }
                    String status = nodeState.getBoolean(node.getManageIp()) ? BocloudStatus.Server.RUNNING.name()
                        : BocloudStatus.Server.STOPPED.name();
                    if (!status.equals(node.getStatus())) {
                        node.setStatus(status);
                        kvsNodeRepository.update(node);
                    }
                    nodeList.add(node);
                }
            }
            int statusCount = MDSUtils.statusCount(nodeList, BocloudStatus.Server.RUNNING.name());
            if (0 == statusCount) {
                kvs.setStatus(BocloudStatus.Server.STOPPED.name());
            } else if (statusCount == nodeList.size()) {
                kvs.setStatus(BocloudStatus.Server.RUNNING.name());
            } else {
                kvs.setStatus(BocloudStatus.Server.EXCEPTION.name());
            }
            kvsRepository.update(kvs);
        }
        return nodeList;
    }

    private void checkMongodbNodeRole(Kvs kvs, List<JSONObject> members, Map<String, KvsNode> nodeMap)
        throws Exception {
        for (JSONObject member : members) {
            String name = StringUtils.substringBefore(member.getString("name"), ":");
            String role = member.getString("stateStr");
            String status = BocloudStatus.Server.RUNNING.name();
            switch (role) {
                case "SECONDARY":
                    role = "replicas";
                    break;
                case "PRIMARY":
                    role = "master";
                    kvs.setManageIp(nodeMap.get(name).getManageIp());
                    kvs.setAddress(name + ":" + nodeMap.get(name).getPort());
                    break;
                case "ARBITER":
                    role = "arbiter";
                    break;
                default:
                    role = "(not reachable/healthy)";
                    status = BocloudStatus.Server.STOPPED.name();
                    break;
            }
            if (!nodeMap.containsKey(name)) {
                continue;
            }
            KvsNode node = nodeMap.get(name);
            if (!role.equals(node.getRole())) {
                node.setStatus(status);
                node.setRole(role);
                kvsNodeRepository.update(node);
            }
        }
    }

    @Override
    public BocloudResult remove(Long id, RequestContext context) {
        String path = Kvs.class.getSimpleName() + "_" + id;
        try (BocloudACLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            Kvs kvs = kvsRepository.query(id);
            kvs = Optional.ofNullable(kvs).orElseThrow(() -> new Exception("数据不存在"));
            if (null != kvs.getTenantId() && kvs.getTenantId() > 0) {
                return new BocloudResult<>(false, "租户数据不可删除");
            }
            kvsLogRepository.deleteByKvs(kvs.getId());
            kvsParamLogRepository.deleteByKvs(kvs.getId());
            kvsNodeRepository.deleteByKvs(kvs.getId());
            kvsUserRepository.deleteByKvs(kvs.getId());
            kvsRepository.remove(kvs.getId(), context.getTarget());
            return new BocloudResult<>(true, kvs.getName() + "删除成功");
        } catch (Exception e) {
            logger.error("remove ass failure:", e);
            return new BocloudResult<>(false, "删除失败");
        }
    }

    @Override
    public void updateStatus(String content) {
        JSONObject object = JSONTools.isJSONObj(content);
        JSONObject service = object.getJSONObject("service");
        String uuid = service.getString("uuid");
        TaskInstance instance = JSONObject.parseObject(object.getString("instance"), TaskInstance.class);
        try {
            Kvs kvs = kvsRepository.get(uuid);
            if (null == kvs) {
                return;
            }
            RequestContext context = JSONObject.parseObject(instance.getProps(), RequestContext.class);
            if (!"success".equalsIgnoreCase(instance.getStatus())) {
                List<TaskNode> nodes = instance.getNodes();
                String errorMsg = null;
                TaskNode softNode = null;
                for (TaskNode node : nodes) {
                    if (node.getCode().startsWith("task.software")) {
                        softNode = node;
                        continue;
                    }
                    JSONObject output = JSONObject.parseObject(node.getOutput());
                    if (null == output) {
                        continue;
                    }
                    if (Status.Node.FAILED.name().equalsIgnoreCase(node.getStatus()) && StringUtils.isEmpty(errorMsg)) {
                        errorMsg = output.getString("message");
                    }
                    JSONObject data = output.getJSONObject("data");
                    if (null == data) {
                        continue;
                    }
                    Long serverId = data.getLong("id");
                    if (null != serverId) {
                        if (removed) {
                            imsCloudServerService.delete(serverId);
                        }
                    }
                }
                if (StringUtils.isEmpty(errorMsg)) {
                    JSONObject output = JSONObject.parseObject(softNode.getOutput());
                    List<JSONObject> datas = JSONObject.parseArray(output.getString("data"), JSONObject.class);
                    if (ListTool.isEmpty(datas)) {
                        errorMsg = output.getString("message");
                    } else {
                        for (JSONObject data : datas) {
                            if (!StringUtils.isEmpty(data.getString("message"))) {
                                errorMsg = data.getString("message");
                                break;
                            }
                        }
                    }
                }
                kvsRepository.remove(kvs.getId(), 0L);
                MDSUtils.send(amqpTemplate, kvs, nodes.size() - 1, OrderMessage.Operation.Application,
                    OrderItemStatus.OpenedFailure, MDSUtils.KVS, null,
                    JSONObject.parseObject(instance.getProps(), RequestContext.class), errorMsg);
                resourceEventPublisher.send(new OperateResult(false, errorMsg, "software.install", null,
                    OperateCategory.SoftwareInstall, context));
                return;
            }
            List<TaskNode> nodes = instance.getNodes();
            List<JSONObject> groups = Lists.newArrayList();
            for (TaskNode node : nodes) {
                if ("task.software.redis".equals(node.getCode()) || "task.software.mongodb".equals(node.getCode())) {
                    JSONObject input = JSONObject.parseObject(node.getInput());
                    if (null != input) {
                        JSONObject fronts = input.getJSONObject("fronts");
                        groups = JSONObject.parseArray(fronts.getString("groups"), JSONObject.class);
                    }
                    break;
                }
            }
            ResourceCategory category = ResourceCategory.getValue(kvs.getType());
            Integer port = null;
            if (ResourceCategory.Redis.equals(category)) {
                port = 6379;
            } else if (ResourceCategory.MongoDB.equals(category)) {
                port = 27017;
            }
            for (TaskNode node : nodes) {
                if (node.getCode().startsWith("task.resource.cvm")) {
                    JSONObject output = JSONObject.parseObject(node.getOutput());
                    if (null == output) {
                        continue;
                    }
                    JSONObject data = output.getJSONObject("data");
                    if (null == data) {
                        continue;
                    }
                    String role = "standalone";
                    if (!ListTool.isEmpty(groups)) {
                        for (JSONObject group : groups) {
                            if (group.getString("hosts").contains(node.getNodeId())) {
                                if (group.getString("name").contains("master")) {
                                    role += " master";
                                } else if (group.getString("name").contains("slave")) {
                                    role += " slave";
                                } else if (group.getString("name").contains("sentinel")) {
                                    role += " sentinel";
                                } else if (group.getString("name").contains("replicas")) {
                                    role += " replicas";
                                } else if (group.getString("name").contains("arbiter")) {
                                    role += " arbiter";
                                }
                                break;
                            }
                        }
                        role = role.trim();
                    }
                    if (StringUtils.isEmpty(role)) {
                        role = "standalone";
                    }
                    List<JSONObject> beans = JSONArray.parseArray(data.getString("privateIps"), JSONObject.class);
                    String address = null;
                    String managerIp = data.getString("managerIp");
                    for (JSONObject bean : beans) {
                        if (bean.getString("address").equals(managerIp)) {
                            continue;
                        }
                        address = bean.getString("address");
                    }
                    if (StringUtils.isEmpty(address)) {
                        address = managerIp;
                    }
                    if (role.contains("master") || "standalone".equals(role)) {
                        kvs.setManageIp(managerIp);
                        kvs.setAddress(address + ":" + port);
                    }
                    KvsNode kvsNode = new KvsNode(address, port, role, BocloudStatus.Server.RUNNING.name(),
                        data.getString("instanceId"), kvs.getId());
                    if (ResourceCategory.Redis.equals(category) && role.equals("sentinel")) {
                        kvsNode.setPort(26379);
                    }
                    kvsNode.setManageIp(managerIp);
                    kvsNodeRepository.save(kvsNode);
                    kvs.setVendorId(data.getLong("vendorId"));
                }
            }
            if (ResourceCategory.MongoDB.equals(category)) {
                JSONObject inventory = JSONObject.parseObject(kvs.getInventory());
                if (null != inventory) {
                    JSONObject configs = inventory.getJSONObject("configs");
                    JSONObject config = configs.getJSONObject("manage_config");
                    KvsUser user = new KvsUser(config.getString("mongodb_root_admin_name"),
                        config.getString("mongodb_root_admin_password"), null, kvs.getId(), null);
                    kvsUserRepository.save(user);
                }
            }
            kvs.setStatus(BocloudStatus.Server.RUNNING.name());
            kvs.setTaskId(instance.getId());
            kvsRepository.update(kvs);
            MDSUtils.send(amqpTemplate, kvs, nodes.size() - 1, OrderMessage.Operation.Application,
                OrderItemStatus.OpenedSuccess, MDSUtils.KVS, null,
                JSONObject.parseObject(instance.getProps(), RequestContext.class), null);
            MDSUtils.send(amqpTemplate, kvs.getId(), kvs.getType(), "create");
            resourceEventPublisher.send(new OperateResult(true, "数据库申请安装成功", "software.install", null,
                OperateCategory.SoftwareInstall, context));
        } catch (Exception e) {
            logger.error("update kvs status fail:", e);
        }
    }

    private BocloudResult postpone(ExtensionParam param, RequestContext context) {
        String path = Kvs.class.getSimpleName() + "_" + param.getInstance();
        try (BocloudACLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            Kvs kvs = kvsRepository.get(param.getInstance());
            kvs = Optional.ofNullable(kvs).orElseThrow(() -> new Exception("数据不存在"));
            List<KvsNode> kvsNodes = kvsNodeRepository.list(kvs.getId());
            for (KvsNode node : kvsNodes) {
                CloudServer server = cloudServerRepository.getByInstanceId(node.getInstanceId());
                if (null == server) {
                    server = cloudServerRepository.get(node.getServerId());
                }
                // 延期时扫描如果机器是关机状态则开机
                if (null != server && !BocloudStatus.Server.RUNNING.name().equals(server.getStatus())) {
                    imsCloudServerService.operation(server.getId(), "start");
                }
            }
            ResourceCategory category = ResourceCategory.getValue(kvs.getType());
            Calendar calendar = Calendar.getInstance();
            calendar.setTime(kvs.getExpiredTime());
            calendar.add(Calendar.MONTH, param.getExtension());
            kvs.setExpiredTime(calendar.getTime());
            kvsRepository.update(kvs);
            MDSUtils.send(amqpTemplate, kvs, null, OrderMessage.Operation.Extension, OrderItemStatus.ExtensionSuccess,
                MDSUtils.KVS, param, context, null);
            return new BocloudResult<>(true, category.name() + "延期成功");
        } catch (Exception e) {
            logger.error("postpone kvs fail:", e);
            return new BocloudResult<>(false, "数据库延期失败");
        }
    }

    @Override
    public BocloudResult<GridBean<KvsLog>> logs(Pager pager, RequestContext context) {
        try {
            List<Param> params = pager.getParams();
            Integer page = pager.getPage();
            Integer rows = pager.getRows();
            Map<String, String> sorter = pager.getSorter();
            params = Optional.ofNullable(params).orElse(new ArrayList<>());
            sorter = Optional.ofNullable(sorter).orElse(new HashMap<>());
            sorter.put("gmtCreate", Common.ONE);
            List<KvsLog> logs = kvsLogRepository.list(page, rows, params, sorter);
            for (KvsLog log : logs) {
                log.setResult(JSONObject.parseObject(log.getResult()).getString("message"));
            }
            int total = kvsLogRepository.count(params);
            GridBean gridBean = GridHelper.getBean(page, rows, total, logs);
            return new BocloudResult<>(true, gridBean, "查询成功");
        } catch (Exception e) {
            logger.error("list kvs logs failure:", e);
            return new BocloudResult<>(false, "查询失败");
        }
    }

    @Override
    public BocloudResult backups(Long id, RequestContext context) {
        try {
            List<KvsNode> nodes = kvsNodeRepository.list(id);
            if (ListTool.isEmpty(nodes)) {
                return new BocloudResult<>(false, "数据不存在");
            }
            String instanceId = null;
            Long serverId = null;
            for (KvsNode node : nodes) {
                if ("master".equalsIgnoreCase(node.getRole())) {
                    instanceId = node.getInstanceId();
                    serverId = node.getServerId();
                    break;
                }
            }
            CloudServer server = cloudServerRepository.getByInstanceId(instanceId);
            if (null == server) {
                server = cloudServerRepository.get(serverId);
            }
            if (null == server) {
                return new BocloudResult<>(false, "云主机不存在");
            }
            List<Snapshot> snapshots = snapshotRepository.listByVmid(server.getId());
            return new BocloudResult<>(true, snapshots, "查询成功");
        } catch (Exception e) {
            logger.error("list kvs backups failure:", e);
            return new BocloudResult<>(false, "查询失败");
        }
    }

    private BocloudResult backup(CloudServer server, String type, RequestContext context) {
        try {
            CloudVendor vendor = cloudVendorRepository.query(server.getVendorId());
            if (null == vendor) {
                logger.warn("get Cloud vendor failure!");
                return new BocloudResult<>(false, "数据库备份失败");
            }
            String name = "Kvs-backup" + System.currentTimeMillis();
            // 开启一个线程更新状态
            threadPoolTaskExecutor.submit(() -> {
                String operate = "kvs." + type + ".backup";
                try {
                    SnapshotModel model = new SnapshotModel(name, null, null, server.getInstanceId());
                    // BocloudResult result = serverDriver.snapshot(model, vendor.getUuid(), null);
                    // if (result.isSuccess()) {
                    // Snapshot snapshot = JSONObject.parseObject(JSONObject.toJSONString(result.getData()),
                    // Snapshot.class);
                    // snapshot.preSave(context.getTarget(), context.getTarget(), server.getVendorId(),
                    // server.getId(), server.getTenantId());
                    // snapshotRepository.save(snapshot);
                    // }
                    // resourceEventPublisher.send(
                    // new OperateResult(result.isSuccess(), "数据库备份" + (result.isSuccess() ? "成功" : "失败"),
                    // operate, null, OperateCategory.ResourceEvent, context));
                } catch (Exception e) {
                    resourceEventPublisher.send(
                        new OperateResult(false, "数据库备份失败", operate, null, OperateCategory.ResourceEvent, context));
                }
            });
            return new BocloudResult<>(true, "数据库备份任务已下发，正在执行...");
        } catch (Exception e) {
            logger.error("backup kvs error:", e);
            return new BocloudResult<>(false, "数据库备份失败");
        }
    }

    @Override
    public BocloudResult backup(Long id, RequestContext context) {
        String path = Kvs.class.getSimpleName() + "_" + id;
        try (BocloudACLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            List<KvsNode> nodes = kvsNodeRepository.list(id);
            if (ListTool.isEmpty(nodes)) {
                return new BocloudResult<>(false, "数据不存在");
            }
            Kvs kvs = kvsRepository.query(id);
            kvs = Optional.ofNullable(kvs).orElseThrow(() -> new Exception("数据不存在"));
            String instanceId = null;
            Long serverId = null;
            for (KvsNode node : nodes) {
                if ("master".equalsIgnoreCase(node.getRole())) {
                    instanceId = node.getInstanceId();
                    serverId = node.getServerId();
                    break;
                }
            }
            CloudServer server = cloudServerRepository.getByInstanceId(instanceId);
            if (null == server) {
                server = cloudServerRepository.get(serverId);
            }
            if (null == server) {
                return new BocloudResult<>(false, "云主机不存在");
            }
            List<Param> params = new ArrayList<>();
            params.add(new Param(MapTools.simpleMap("vmId", server.getId()), Sign.EQ));
            int count = snapshotRepository.count(params);
            if (count >= 3) {
                return new BocloudResult<>(false, "数据库备份不能超过3次");
            }
            return backup(server, kvs.getType(), context);
        } catch (Exception e) {
            logger.error("backup kvs fail:", e);
            return new BocloudResult<>(false, "数据库备份失败");
        }
    }

    @Override
    public BocloudResult revert(Long id, Long snapshotId, RequestContext context) {
        String path = Kvs.class.getSimpleName() + "_" + id;
        try (BocloudACLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            Kvs kvs = kvsRepository.query(id);
            kvs = Optional.ofNullable(kvs).orElseThrow(() -> new Exception("数据不存在"));
            return revert(snapshotId, kvs.getType(), context);
        } catch (Exception e) {
            logger.error("revert kvs fail:", e);
            return new BocloudResult<>(false, "数据库备份失败");
        }
    }

    public BocloudResult revert(Long id, String type, RequestContext context) {
        String path = Snapshot.class.getSimpleName() + "_" + id;
        try (BocloudACLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            Snapshot snap = snapshotRepository.query(id);
            snap = Optional.ofNullable(snap).orElseThrow(() -> new Exception("数据不存在"));
            CloudServer server = cloudServerRepository.query(snap.getVmId());
            server = Optional.ofNullable(server).orElseThrow(() -> new Exception("云主机不存在"));
            // 云供应商
            CloudVendor vendor = cloudVendorRepository.query(snap.getVendorId());
            vendor = Optional.ofNullable(vendor).orElseThrow(() -> new Exception("云平台不存在"));
            // 启动一个线程从MQ上获取数据
            CloudServer finalServer = server;
            Snapshot finalSnap = snap;
            threadPoolTaskExecutor.submit(() -> {
                String subPath = CloudServer.class.getSimpleName() + "_" + finalServer.getId();
                String operate = "kvs." + type + ".revert";
                BocloudResult result = null;
                String content = null;
                try (BocloudACLock bLock = lockFactory.getACLock(subPath)) {
                    if (!lock.acquire(10, TimeUnit.SECONDS)) {
                        logger.warn("Get BocloudLock time out!");
                        resourceEventPublisher.send(new OperateResult(false, "当前有恢复任务正在执行", operate, null,
                            OperateCategory.ResourceEvent, context));
                        return;
                    }
                    SnapshotModel model = new SnapshotModel();
                    model.setSnapshotUuid(finalSnap.getSnapshotUuid());
                    model.setVmName(finalServer.getName());
                    Server host = serverRepository.query(finalServer.getHostId());
                    if (null != host) {
                        model.setHostName(host.getName());
                    }
                    logger.info("start to revert cloud server");
                    // result = snapshotDriver.revert(vendor.getUuid(), null, model);
                    content = result.isSuccess() ? "恢复成功" : "恢复失败";
                } catch (Exception e) {
                    logger.error("revert snapshot fail:", e);
                    resourceEventPublisher
                        .send(new OperateResult(false, "恢复失败", operate, null, OperateCategory.ResourceEvent, context));
                } finally {
                    if (result.isSuccess()) {
                        MDSUtils.powerOn(imsCloudServerService, finalServer);
                    }
                    resourceEventPublisher.send(new OperateResult(result.isSuccess(), content, operate, null,
                        OperateCategory.ResourceEvent, context));
                }
            });
            return new BocloudResult<>(true, "恢复任务已经下发，正在执行……");
        } catch (Exception e) {
            logger.error("revert snapshot fail:", e);
            return new BocloudResult<>(false, "恢复失败");
        }
    }

    @Override
    public BocloudResult securities(Long id, RequestContext context) {
        try {
            List<KvsNode> nodes = kvsNodeRepository.list(id);
            if (ListTool.isEmpty(nodes)) {
                return new BocloudResult<>(false, "数据不存在");
            }
            // String groupId = null;
            for (KvsNode node : nodes) {
                if ("master".equalsIgnoreCase(node.getRole())) {
                    // groupId = node.getGroupId();
                    break;
                }
            }
            List<Object> groups = new ArrayList<>();
            return new BocloudResult<>(true, groups, "查询成功");
        } catch (Exception e) {
            logger.error("list kvs securities failure:", e);
            return new BocloudResult<>(false, "查询失败");
        }
    }

    @Override
    public BocloudResult<List<JSONObject>> parameters(Long id, RequestContext context) {
        try {
            Kvs kvs = kvsRepository.query(id);
            kvs = Optional.ofNullable(kvs).orElseThrow(() -> new Exception("数据不存在"));
            if (kvs.getStatus().equals(BocloudStatus.Server.STOPPED.name())) {
                return new BocloudResult<>(false, "数据库运行状态异常");
            }
            if (ResourceCategory.MongoDB.name().equalsIgnoreCase(kvs.getType())) {
                BocloudResult result = executeMongod(kvs, null, null, null, true, false);
                if (result.isFailed()) {
                    return result;
                }
                JSONObject object = JSONObject.parseObject(JSONObject.toJSONString(result.getData()));
                List<JSONObject> list = new ArrayList<>();
                if (object.containsKey("cursorTimeoutMillis")) {
                    JSONObject param = new JSONObject();
                    param.put("name", "cursorTimeoutMillis");
                    param.put("value", object.get("cursorTimeoutMillis"));
                    param.put("type", "number");
                    param.put("default", 600000);
                    param.put("reference", "1-2147483647");
                    list.add(param);
                }
                if (object.containsKey("failIndexKeyTooLong")) {
                    JSONObject param = new JSONObject();
                    param.put("name", "failIndexKeyTooLong");
                    param.put("value", object.get("failIndexKeyTooLong"));
                    param.put("type", "boolean");
                    param.put("default", true);
                    param.put("reference", "true|false");
                    list.add(param);
                }
                if (object.containsKey("internalQueryExecMaxBlockingSortBytes")) {
                    JSONObject param = new JSONObject();
                    param.put("name", "internalQueryExecMaxBlockingSortBytes");
                    param.put("value", object.get("internalQueryExecMaxBlockingSortBytes"));
                    param.put("type", "number");
                    param.put("default", 33554432);
                    param.put("reference", "33554432-268435456");
                    list.add(param);
                }
                if (object.containsKey("scramIterationCount")) {
                    JSONObject param = new JSONObject();
                    param.put("name", "scramIterationCount");
                    param.put("value", object.get("scramIterationCount"));
                    param.put("type", "number");
                    param.put("default", 10000);
                    param.put("reference", ">=5000");
                    list.add(param);
                }
                // if (object.containsKey("sslWithholdClientCertificate")) {
                // JSONObject param = new JSONObject();
                // param.put("name", "sslWithholdClientCertificate");
                // param.put("value", object.get("sslWithholdClientCertificate"));
                // param.put("type", "boolean");
                // param.put("default", false);
                // param.put("reference", "true|false");
                // list.add(param);
                // }
                if (object.containsKey("connPoolMaxShardedConnsPerHost")) {
                    JSONObject param = new JSONObject();
                    param.put("name", "connPoolMaxShardedConnsPerHost");
                    param.put("value", object.get("connPoolMaxShardedConnsPerHost"));
                    param.put("type", "number");
                    param.put("default", 200);
                    param.put("reference", null);
                    list.add(param);
                }
                if (object.containsKey("ttlMonitorEnabled")) {
                    JSONObject param = new JSONObject();
                    param.put("name", "ttlMonitorEnabled");
                    param.put("value", object.get("ttlMonitorEnabled"));
                    param.put("type", "boolean");
                    param.put("default", true);
                    param.put("reference", "true|false");
                    list.add(param);
                }
                if (object.containsKey("logLevel")) {
                    JSONObject param = new JSONObject();
                    param.put("name", "logLevel");
                    param.put("value", object.get("logLevel"));
                    param.put("type", "number");
                    param.put("default", 0);
                    param.put("reference", "0-5");
                    list.add(param);
                }
                if (object.containsKey("traceExceptions")) {
                    JSONObject param = new JSONObject();
                    param.put("name", "traceExceptions");
                    param.put("value", object.get("traceExceptions"));
                    param.put("type", "boolean");
                    param.put("default", false);
                    param.put("reference", "true|false");
                    list.add(param);
                }
                if (object.containsKey("diagnosticDataCollectionEnabled")) {
                    JSONObject param = new JSONObject();
                    param.put("name", "diagnosticDataCollectionEnabled");
                    param.put("value", object.get("diagnosticDataCollectionEnabled"));
                    param.put("type", "boolean");
                    param.put("default", true);
                    param.put("reference", "true|false");
                    list.add(param);
                }
                if (object.containsKey("diagnosticDataCollectionDirectorySizeMB")) {
                    JSONObject param = new JSONObject();
                    param.put("name", "diagnosticDataCollectionDirectorySizeMB");
                    param.put("value", object.get("diagnosticDataCollectionDirectorySizeMB"));
                    param.put("type", "number");
                    param.put("default", 200);
                    param.put("reference", null);
                    list.add(param);
                }
                if (object.containsKey("diagnosticDataCollectionFileSizeMB")) {
                    JSONObject param = new JSONObject();
                    param.put("name", "diagnosticDataCollectionFileSizeMB");
                    param.put("value", object.get("diagnosticDataCollectionFileSizeMB"));
                    param.put("type", "number");
                    param.put("default", 10);
                    param.put("reference", null);
                    list.add(param);
                }
                if (object.containsKey("diagnosticDataCollectionPeriodMillis")) {
                    JSONObject param = new JSONObject();
                    param.put("name", "diagnosticDataCollectionPeriodMillis");
                    param.put("value", object.get("diagnosticDataCollectionPeriodMillis"));
                    param.put("type", "number");
                    param.put("default", 1000);
                    param.put("reference", null);
                    list.add(param);
                }
                return new BocloudResult<>(true, list, "查询数据库参数成功");
            }
            BocloudResult result = executeRedis(kvs, null, null, true);
            if (result.isFailed()) {
                return result;
            }
            List<String> params = JSONArray.parseArray(JSONObject.toJSONString(result.getData()), String.class);
            Map<String, String> map = new HashMap<>();
            if (!ListTool.isEmpty(params)) {
                int size = params.size();
                for (int i = 0; i < size / 2; i++) {
                    map.put(params.get(i), params.get(i + 1));
                }
            }
            List<JSONObject> list = new ArrayList<>();
            for (String key : map.keySet()) {
                JSONObject param = new JSONObject();
                param.put("name", key);
                param.put("value", map.get(key));
                if ("hash-max-ziplist-entries".equals(key)) {
                    param.put("default", 512);
                    param.put("reference", "0-999999999999999");
                } else if ("hash-max-ziplist-value".equals(key)) {
                    param.put("default", 64);
                    param.put("reference", "0-999999999999999");
                } else if ("hz".equals(key)) {
                    param.put("default", 10);
                    param.put("reference", "1-500");
                } else if ("list-compress-depth".equals(key)) {
                    param.put("default", 0);
                    param.put("reference", "0-65535");
                } else if ("list-max-ziplist-size".equals(key)) {
                    param.put("default", -2);
                    param.put("reference", "-5-99999999");
                } else if ("maxmemory-policy".equals(key)) {
                    param.put("default", "noeviction");
                    param.put("reference",
                        "noeviction|allkeys-lru|volatile-lru|allkeys-random|volatile-random|volatile-ttl");
                } else if ("notify-keyspace-events".equals(key)) {
                    param.put("default", "");
                    param.put("reference", ".*");
                } else if ("set-max-intset-entries".equals(key)) {
                    param.put("default", 512);
                    param.put("reference", "0-999999999999999");
                } else if ("slowlog-log-slower-than".equals(key)) {
                    param.put("default", 10000);
                    param.put("reference", "0-10000000");
                } else if ("slowlog-max-len".equals(key)) {
                    param.put("default", 128);
                    param.put("reference", "100-10000");
                } else if ("timeout".equals(key)) {
                    param.put("default", 0);
                    param.put("reference", "0-100000");
                } else if ("zset-max-ziplist-entries".equals(key)) {
                    param.put("default", 128);
                    param.put("reference", "0-999999999999999");
                } else if ("zset-max-ziplist-value".equals(key)) {
                    param.put("default", 64);
                    param.put("reference", "0-999999999999999");
                }
                if (param.containsKey("default")) {
                    list.add(param);
                }
            }
            return new BocloudResult<>(true, list, "查询数据库参数成功");
        } catch (Exception e) {
            return new BocloudResult<>(false, "查询数据库参数失败");
        }
    }

    @Override
    public void synchronousRoleState(JSONObject content) {
        try {
            Long id = content.getLong("id");
            Kvs kvs = kvsRepository.query(id);
            kvs = Optional.ofNullable(kvs).orElseThrow(() -> new Exception("数据不存在"));
            List<KvsNode> nodes = kvsNodeRepository.list(id);
            synchronousRole(nodes, kvs);
            RequestContext context = new RequestContext();
            context.setTenant(kvs.getTenantId());
            context.setTarget(kvs.getTenantId());
            context.setCatalog(Catalog.valueOf(kvs.getGmtCatalog()));
            resourceEventPublisher.send(new OperateResult(true, "节点角色状态变更通知！",
                "node_change_" + kvs.getType().toLowerCase(), null, OperateCategory.ServiceEvent, context));
        } catch (Exception e) {
            logger.error("集群角色同步失败：", e.getMessage());
        }
    }

    private BocloudResult executeRedis(Kvs kvs, String key, String value, boolean isGet) {
        String type = kvs.getCategory();
        try {
            if ("cluster".equalsIgnoreCase(type)) {
                List<KvsNode> kvsNodes = kvsNodeRepository.list(kvs.getId());
                if (isGet) {
                    List<String> list;
                    for (KvsNode kvsNode : kvsNodes) {
                        if ("sentinel".equals(kvsNode.getRole())) {
                            continue;
                        }
                        logger.info("connect redis node info:" + kvsNode.getManageIp() + ":" + 6379);
                        try (Jedis redisNode = new Jedis(kvsNode.getManageIp(), 6379)) {

                            redisNode.auth(MDSUtils.decrypt(kvs.getPassword()));
                            list = redisNode.configGet("*");
                            if (ListTool.isEmpty(list)) {
                                continue;
                            }
                            return new BocloudResult<>(true, list, "数据库操作成功");
                        } catch (Exception exception) {
                            logger.error("节点：" + kvsNode.getManageIp() + "数据库查询异常", exception);
                        }
                    }
                    return new BocloudResult<>(true, "数据库操作成功");
                }
                for (KvsNode kvsNode : kvsNodes) {
                    if ("sentinel".equals(kvsNode.getRole())) {
                        continue;
                    }
                    try (Jedis redis = new Jedis(kvsNode.getManageIp(), 6379)) {
                        redis.auth(MDSUtils.decrypt(kvs.getPassword()));
                        String result = redis.configSet(key, value);
                        if (!"OK".equalsIgnoreCase(result)) {
                            logger.error("节点:" + kvsNode.getManageIp() + ",数据库操作异常");
                        }
                    } catch (Exception exception) {
                        logger.error("节点:" + kvsNode.getManageIp() + "数据库操作异常", exception);
                    }
                }
                return new BocloudResult<>(true, "数据库操作成功");
            }
            logger.info("connect redis node info:" + kvs.getManageIp() + ":" + 6379);
            try (Jedis jedis = new Jedis(kvs.getManageIp(), 6379)) {
                jedis.auth(MDSUtils.decrypt(kvs.getPassword()));
                if (isGet) {
                    List<String> list = jedis.configGet("*");
                    return new BocloudResult<>(true, list, "数据库操作成功");
                }
                String result = jedis.configSet(key, value);
                if (!"OK".equalsIgnoreCase(result)) {
                    return new BocloudResult<>(false, "数据库操作失败");
                }
                return new BocloudResult<>(true, "数据库操作成功");
            } catch (Exception e) {
                logger.info("connect kvs error:", e);
                return new BocloudResult<>(false, "数据库操作失败");
            }
        } catch (Exception e) {
            logger.info("connect kvs error:", e);
            return new BocloudResult<>(false, "数据库操作失败");
        }
    }

    private BocloudResult executeMemCache(List<KvsNode> nodes) {
        JSONObject resultData = new JSONObject();
        for (KvsNode node : nodes) {
            if ("magent".equals(node.getRole())) {
                continue;
            }
            MemcachedClient client = null;
            try {
                client = new MemcachedClient(new InetSocketAddress(node.getManageIp(), node.getPort()));
                Map<SocketAddress, String> version = client.getVersions();
                resultData.put(node.getManageIp(), !version.isEmpty());
            } catch (Exception e) {
                logger.error("节点：" + node.getManageIp() + "连接异常", e);
                resultData.put(node.getManageIp(), false);
            } finally {
                if (null != client) {
                    client.shutdown();
                }
            }
        }
        return new BocloudResult<>(true, resultData, "数据库操作成功");
    }

    private BocloudResult setMongoParams(Kvs kvs, String key, Object value, String type) {
        try {
            List<String> ips = new ArrayList<String>();
            List<KvsNode> nodes = kvsNodeRepository.list(kvs.getId());
            for (KvsNode node : nodes) {
                if ("arbiter".equals(node.getRole())) {
                    continue;
                }
                BocloudResult result = setMongoParam(kvs, node, key, value, type);
                if (result.isFailed()) {
                    return new BocloudResult<>(false, "数据库操作失败");
                }
            }
            return new BocloudResult<>(true, "数据库操作成功");
        } catch (Exception e) {
            logger.info("connect database error:", e);
            return new BocloudResult<>(false, "数据库操作失败");
        }
    }

    private BocloudResult setMongoParam(Kvs kvs, KvsNode node, String key, Object value, String type) {
        MongoClient client = null;
        try {
            // int port = "cluster".equals(kvs.getCategory()) ? 27017 : 27017;
            int port = 27017;
            MongoCredential credential = MongoCredential.createScramSha1Credential(kvs.getUsername(), "admin",
                MDSUtils.decrypt(kvs.getPassword()).toCharArray());
            MongoClientOptions.Builder builder = MongoClientOptions.builder();
            builder.readPreference(ReadPreference.secondaryPreferred()).socketTimeout(5000).maxWaitTime(5000)
                .connectTimeout(5000).serverSelectionTimeout(5000);
            MongoClientOptions options = builder.build();
            client = new MongoClient(new ServerAddress(node.getManageIp(), port), credential, options);
            MongoDatabase database = client.getDatabase("admin");
            Document doc = new Document();
            doc.append("setParameter", 1);
            doc.append(key, "number".equalsIgnoreCase(type) ? Long.valueOf(value.toString()) : value);
            Document document = database.runCommand(doc);
            if (null != document && document.get("ok").equals(1.0)) {
                return new BocloudResult<>(true, "数据库操作成功");
            }
            return new BocloudResult<>(false, "数据库操作失败");
        } catch (Exception e) {
            logger.info("connect mongodb error:", e);
            return new BocloudResult<>(false, "数据库操作失败");
        } finally {
            if (null != client) {
                client.close();
            }
        }
    }

    private BocloudResult executeMongod(Kvs kvs, String key, Object value, String type, boolean isGet,
        boolean acquireStatus) {
        MongoClient client = null;
        try {
            // int port = "cluster".equals(kvs.getCategory()) ? 27017 : 27017;
            int port = 27017;
            MongoCredential credential = MongoCredential.createScramSha1Credential(kvs.getUsername(), "admin",
                MDSUtils.decrypt(kvs.getPassword()).toCharArray());
            MongoClientOptions.Builder builder = MongoClientOptions.builder();
            builder.readPreference(ReadPreference.secondaryPreferred()).socketTimeout(5000).maxWaitTime(5000)
                .connectTimeout(5000).serverSelectionTimeout(5000);
            MongoClientOptions options = builder.build();
            client = new MongoClient(new ServerAddress(kvs.getManageIp(), port), credential, options);
            MongoDatabase database = client.getDatabase("admin");
            if (isGet) {
                Document doc = new Document();
                if (acquireStatus) {
                    doc.append("replSetGetStatus", 1);
                    doc.append("initialSync", 1);
                } else {
                    doc.append("getParameter", "*");
                }
                return new BocloudResult<>(true, database.runCommand(doc), "数据库操作成功");
            }
            Document doc = new Document();
            doc.append("setParameter", 1);
            doc.append(key, "number".equalsIgnoreCase(type) ? Long.valueOf(value.toString()) : value);
            Document document = database.runCommand(doc);
            if (null != document && document.get("ok").equals(1.0)) {
                return new BocloudResult<>(true, "数据库操作成功");
            }
            return new BocloudResult<>(false, "数据库操作失败");
        } catch (Exception e) {
            logger.info("connect mongodb error:", e);
            return new BocloudResult<>(false, "数据库操作失败");
        } finally {
            if (null != client) {
                client.close();
            }
        }
    }

    private List<JSONObject> executeRedis(Kvs kvs, List<KvsNode> nodes) {
        // int port = "cluster".equals(kvs.getCategory()) ? 6379 : 6379;
        int port = 6379;
        List<JSONObject> nodesData = new ArrayList<>();
        boolean isThree = nodes.size() == 3;
        for (KvsNode kvsNode : nodes) {
            if ("sentinel".equals(kvsNode.getRole())) {
                continue;
            }
            JSONObject map = new JSONObject();
            Jedis jedis = null;
            try {
                map.put("ip", kvsNode.getManageIp() + ":" + kvsNode.getPort());
                jedis = new Jedis(kvsNode.getManageIp(), port);
                jedis.auth(MDSUtils.decrypt(kvs.getPassword()));
                String info = jedis.info();
                Properties properties = new Properties();
                ByteArrayInputStream stream = new ByteArrayInputStream(info.getBytes());
                properties.load(stream);
                String role = properties.getProperty("role");
                map.put("role", role);
                map.put("exception", false);
                if ("master".equalsIgnoreCase(role)) {
                    String slave0 = properties.getProperty("slave0");
                    String slavePort =
                        StringUtils.substringAfter(StringUtils.substringBefore(slave0, ",state"), "port=");
                    String slaveIp = StringUtils.substringAfter(StringUtils.substringBefore(slave0, ",port"), "=");
                    String address = "";
                    if (StringUtils.isNotEmpty(slaveIp)) {
                        address = slaveIp + ":" + slavePort;
                    }
                    if (isThree) {
                        String slave1 = properties.getProperty("slave1");
                        String slave1Port =
                            StringUtils.substringAfter(StringUtils.substringBefore(slave1, ",state"), "port=");
                        String slave1Ip = StringUtils.substringAfter(StringUtils.substringBefore(slave1, ",port"), "=");
                        if (StringUtils.isNotEmpty(slave1Ip)) {
                            address = (StringUtils.isEmpty(address) ? "" : address + ",") + slave1Ip + ":" + slave1Port;
                        }
                    }
                    map.put("slaveAddress", address);
                }
                nodesData.add(map);
            } catch (Exception exception) {
                logger.error("节点：" + kvsNode.getManageIp() + "数据库查询异常", exception);
            } finally {
                if (null != jedis) {
                    jedis.close();
                }
            }
        }
        return nodesData;
    }

    @Override
    public BocloudResult setting(String name, String oldValue, String value, Long id, String type,
        RequestContext context) {
        KvsParamLog log = new KvsParamLog(name, oldValue, value, new Date(), id, context.getTarget(), false);
        try {
            Kvs kvs = kvsRepository.query(id);
            kvs = Optional.ofNullable(kvs).orElseThrow(() -> new Exception("数据不存在"));
            if (kvs.getStatus().equals(BocloudStatus.Server.STOPPED.name())) {
                return new BocloudResult<>(false, "数据库运行状态异常");
            }
            BocloudResult result = null;
            if (ResourceCategory.MongoDB.name().equalsIgnoreCase(kvs.getType())) {
                result = setMongoParams(kvs, name, value, type);
            } else {
                result = executeRedis(kvs, name, value, false);
            }
            log.setEffective(result.isSuccess());
            save(log);
            if (result.isFailed()) {
                return new BocloudResult<>(false, "设置数据库参数失败");
            }
            return new BocloudResult<>(true, "设置数据库参数成功");
        } catch (Exception e) {
            save(log);
            return new BocloudResult<>(false, "设置数据库参数失败");
        }
    }

    @Override
    public BocloudResult<GridBean> paramLogs(Pager pager, RequestContext context) {
        try {
            List<Param> params = pager.getParams();
            Integer page = pager.getPage();
            Integer rows = pager.getRows();
            Map<String, String> sorter = pager.getSorter();
            params = Optional.ofNullable(params).orElse(new ArrayList<>());
            sorter = Optional.ofNullable(sorter).orElse(new HashMap<>());
            sorter.put("gmtCreate", Common.ONE);
            List<KvsParamLog> logs = kvsParamLogRepository.list(page, rows, params, sorter);
            int total = kvsParamLogRepository.count(params);
            GridBean gridBean = GridHelper.getBean(page, rows, total, logs);
            return new BocloudResult<>(true, gridBean, "查询成功");
        } catch (Exception e) {
            logger.error("list kvs param logs failure:", e);
            return new BocloudResult<>(false, "查询失败");
        }
    }

    private void save(KvsParamLog log) {
        try {
            kvsParamLogRepository.save(log);
        } catch (Exception e) {
            logger.error("save kvs param log failure:", e);
        }
    }

    @Override
    public void saveLog(KvsLog log) {
        try {
            kvsLogRepository.save(log);
        } catch (Exception e) {
            logger.error("save kvs log failure:", e);
        }
    }

    @Override
    public BocloudResult removeBackup(Long snapshotId, RequestContext context) {
        String path = Snapshot.class.getSimpleName() + "_" + snapshotId;
        try (BocloudACLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            Snapshot snap = snapshotRepository.query(snapshotId);
            snap = Optional.ofNullable(snap).orElseThrow(() -> new Exception("数据不存在"));
            CloudServer server = cloudServerRepository.query(snap.getVmId());
            server = Optional.ofNullable(server).orElseThrow(() -> new Exception("云主机不存在"));
            // 云供应商
            CloudVendor vendor = cloudVendorRepository.query(snap.getVendorId());
            vendor = Optional.ofNullable(vendor).orElseThrow(() -> new Exception("云平台不存在"));
            CloudProvider type = CloudProvider.valueOf(vendor.getType());
            // 启动一个线程从MQ上获取数据
            Snapshot finalSnap = snap;
            CloudServer finalServer = server;
            CloudVendor finalVendor = vendor;
            threadPoolTaskExecutor.submit(() -> {
                String operate = "kvs.backup";
                BocloudResult result = new BocloudResult<>(false, "数据库备份删除失败");
                try {
                    SnapshotModel model = new SnapshotModel();
                    model.setSnapshotUuid(finalSnap.getSnapshotUuid());
                    model.setVmName(finalServer.getName());
                    // switch (type) {
                    // case VMWARE:
                    // result = snapshotDriver.remove(vendor.getUuid(), null, model);
                    // break;
                    // default:
                    // break;
                    // }
                    String content = result.getMessage();
                    if (result.isSuccess()) {
                        snapshotRepository.remove(snapshotId, context.getTarget());
                        content = "数据库备份删除成功";
                    }
                    resourceEventPublisher.send(new OperateResult(result.isSuccess(), content, operate,
                        MapTools.simpleMap("vendorId", finalVendor.getId()), OperateCategory.ResourceEvent, context));
                } catch (Exception e) {
                    logger.error("Get Message Exception:", e);
                    resourceEventPublisher.send(new OperateResult(false, result.getMessage(), operate, null,
                        OperateCategory.ResourceEvent, context));
                }
            });
            return new BocloudResult<>(true, "删除任务已经下发，正在执行……");
        } catch (Exception e) {
            logger.error("remove snapshot fail:", e);
            return new BocloudResult<>(false, "删除失败");
        }
    }

    @Override
    public BocloudResult reset(Long id, String password, String newPassword, String requestIp, RequestContext context) {
        String path = Kvs.class.getSimpleName() + "_" + id;
        try (BocloudACLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            Kvs kvs = kvsRepository.query(id);
            kvs = Optional.ofNullable(kvs).orElseThrow(() -> new Exception("数据不存在"));
            if (!kvs.getPassword().equals(password)) {
                return new BocloudResult<>(false, id, "原密码输入错误");
            }
            if ("STOPPED".equals(kvs.getStatus())) {
                return new BocloudResult<>(false, id, "不是运行状态");
            }
            List<KvsNode> nodes = kvsNodeRepository.list(id);
            List<KvsNode> list = new ArrayList<>();
            for (KvsNode node : nodes) {
                CloudServer server = cloudServerRepository.getByInstanceId(node.getInstanceId());
                if (null == server) {
                    server = cloudServerRepository.get(node.getServerId());
                }
                if (null == server || BocloudStatus.Server.STOPPED.name().equals(server.getStatus())) {
                    continue;
                }
                node.setAccount(server.getAccount());
                node.setPassword(server.getPswd());
                list.add(node);
            }
            if (ListTool.isEmpty(list)) {
                return new BocloudResult<>(false, id, "密码重置失败：请检查数据库当前状态");
            }
            Kvs finalKvs = kvs;
            threadPoolTaskExecutor.submit(() -> {
                long start = System.currentTimeMillis();
                BocloudResult result = new BocloudResult<>(false, "");
                try {
                    result = executeRedis(finalKvs, "requirepass", MDSUtils.decrypt(newPassword), false);
                    if (result.isSuccess()) {
                        finalKvs.setPassword(newPassword);
                        logger.info("调用worker操作，执行修改配置文件操作。。。");
                        result = MDSUtils.invoke(serviceFactory,
                            MDSUtils.buildParams(nodes.size() > 1, list, password, newPassword), rabbitAdmin);
                        if (result.isFailed()) {
                            logger.error("worker执行修改密码任务失败：" + result.getMessage());
                            executeRedis(finalKvs, "requirepass", MDSUtils.decrypt(password), false);
                        } else {
                            finalKvs.setMenderId(context.getTarget());
                            finalKvs.setGmtModify(new Date());
                            kvsRepository.update(finalKvs);
                        }
                    }
                } catch (Exception e) {
                    logger.error("reset kvs user failure:", e);
                }
                resourceEventPublisher.send(new OperateResult(result.isSuccess(),
                    "【" + finalKvs.getName() + "】密码重置" + (result.isSuccess() ? "成功" : "失败"), "resource.redis", null,
                    OperateCategory.ResourceEvent, context));
                KvsLog log = new KvsLog("数据库", "数据库密码重置",
                    JSONObject.toJSONString(
                        new BocloudResult<>(result.isSuccess(), "密码重置" + (result.isSuccess() ? "成功" : "失败"))),
                    null, requestIp, new Date(), System.currentTimeMillis() - start, id, context.getTarget());
                saveLog(log);
            });
            return new BocloudResult<>(true, "密码重置任务已下发");
        } catch (Exception e) {
            logger.error("reset kvs user failure:", e);
            return new BocloudResult<>(false, "密码重置失败");
        }
    }

    @Override
    public BocloudResult recycle(Long id, RequestContext context) {
        try {
            Kvs kvs = kvsRepository.query(id);
            // List<KvsNode> nodes = kvsNodeRepository.list(id);
            // List<Long> serverIds = Lists.newArrayList();
            // for (KvsNode node : nodes) {
            // CloudServer server = cloudServerRepository.get(node.getInstanceId());
            // Optional.ofNullable(server).orElseThrow(() -> new Exception("云主机不存在"));
            // CloudVendor vendor = cloudVendorRepository.query(server.getVendorId());
            // Optional.ofNullable(vendor).orElseThrow(() -> new Exception("云平台不存在"));
            // serverIds.add(node.getServerId());
            // }
            // BocloudResult batchOperations = cloudServerService.batchOperations(String.valueOf(serverIds), "Stop",
            // context);
            // if (batchOperations.isFailed()) {
            // return new BocloudResult<>(false, "数据库关机失败！");
            // }
            // 更新实例状态
            kvsRepository.recycle(id, context.getTarget());
            // 回收站生成记录
            JSONObject specs = JSONObject.parseObject(kvs.getSpec());
            String spec = null;
            if (null != specs) {
                spec = specs.get("cpu") + "核/" + specs.get("memory") + "GB/" + specs.get("disk") + "GB";
            }
            Recycled recycled =
                new Recycled(kvs.getName(), kvs.getUuid(), kvs.getType(), kvs.getId(), null, kvs.getGmtCatalog(), spec);
            if (Catalog.Tenant == Catalog.valueOf(kvs.getGmtCatalog())) {
                recycled.setTenantId(kvs.getTenantId());
            } else if (Catalog.User == Catalog.valueOf(kvs.getGmtCatalog())) {
                recycled.setTenantId(kvs.getTenantId());
                recycled.setUserId(kvs.getOwnerId());
                recycled.setOwnerId(kvs.getOwnerId());
            }
            BocloudResult joinRecycle = cosInternalService.joinRecycle(recycled);
            logger.info("生成回收站记录结果：{}", JSONObject.toJSON(joinRecycle));
            // 处理订单
            cosInternalService.updateItemStatus(kvs.getUuid(), OrderItemStatus.Termination.name());
            // 处理账单
            BocloudResult result = cosInternalService.recycleBill(kvs.getUuid());
            logger.info("放入回收站生产账单结果：{}", JSONObject.toJSON(result));
            RequestContext requestContext = new RequestContext(Catalog.valueOf(kvs.getGmtCatalog()), kvs.getTenantId(),
                kvs.getCreatorId(), null, null);
            // 发送站内信
            NotifyMessage notifyMessage = new NotifyMessage(requestContext, kvs.getType() + "实例放入回收站通知",
                kvs.getType() + "【" + kvs.getName() + "】已删除放入回收站，30天后将被彻底删除", true, false);
            amqpTemplate.convertAndSend("event.notice.queue", notifyMessage);
            return new BocloudResult<>(true, "云主机【" + kvs.getName() + "】删除成功，可在回收站查看");
        } catch (Exception e) {
            logger.error("remove kvs fail:", e);
            return new BocloudResult<>(false, "移入回收车失败");
        }
    }

    @Override
    public BocloudResult restore(Long id, RequestContext context) {
        try {
            Kvs kvs = kvsRepository.query(id);
            // List<KvsNode> nodes = kvsNodeRepository.list(id);
            // List<Long> serverIds = Lists.newArrayList();
            // for (KvsNode node : nodes) {
            // CloudServer server = cloudServerRepository.get(node.getInstanceId());
            // Optional.ofNullable(server).orElseThrow(() -> new Exception("云主机不存在"));
            // CloudVendor vendor = cloudVendorRepository.query(server.getVendorId());
            // Optional.ofNullable(vendor).orElseThrow(() -> new Exception("云平台不存在"));
            // serverIds.add(node.getServerId());
            // }
            // BocloudResult batchOperations = cloudServerService.batchOperations(String.valueOf(serverIds), "Start",
            // context);
            // if (batchOperations.isFailed()) {
            // return new BocloudResult<>(false, "数据库开机失败！");
            // }
            // 更新实例状态
            kvsRepository.restore(id, context.getTarget());
            // 处理订单
            cosInternalService.updateItemStatus(kvs.getUuid(), OrderItemStatus.OpenedSuccess.name());
            return new BocloudResult<>(true, kvs.getType() + "【" + kvs.getName() + "】还原成功");
        } catch (Exception e) {
            logger.error("restore kvs fail:", e);
            return new BocloudResult<>(false, "数据库还原失败");
        }
    }

    @Override
    public void install(String params, RequestContext context) {
        TaskNode node = JSONObject.parseObject(params, TaskNode.class);
        JSONObject output = JSONObject.parseObject(node.getOutput());
        if (null == output) {
            return;
        }
        List<JSONObject> hosts = JSONObject.parseArray(output.getString("data"), JSONObject.class);
        if (ListTool.isEmpty(hosts)) {
            return;
        }
        JSONObject input = JSONObject.parseObject(node.getParams());
        if (null == input) {
            return;
        }
        Integer port = null;
        ResourceCategory category = null;
        JSONObject fronts = JSONObject.parseObject(input.getString("fronts"));
        String version = fronts.getString("version");
        JSONObject config = fronts.getJSONObject("software_config");
        String password = null;
        String username = null;
        switch (node.getCode()) {
            case "task.software.redis":
                category = ResourceCategory.Redis;
                port = config.getInteger("redis_port");
                port = null == port ? 6379 : port;
                JSONObject manageConfig = fronts.getJSONObject("manage_config");
                if (null != manageConfig) {
                    password = manageConfig.getString("redis_password");
                }
                break;
            case "task.software.mongodb":
                category = ResourceCategory.MongoDB;
                port = config.getInteger("mongodb_net_port");
                port = null == port ? 27017 : port;
                username = "zeus";
                try {
                    password = MDSUtils.encrypt("Zeus@v587");
                } catch (Exception e) {
                }
                break;
            case "task.software.memcache":
                category = ResourceCategory.MemCache;
                port = config.getInteger("memcached_port");
                port = null == port ? 11211 : port;
                break;
            default:
                return;
        }
        try {
            if (null != fronts.getBoolean("cluster_install") && fronts.getBoolean("cluster_install")) {
                if (Status.Node.FAILED.name().equals(node.getStatus())) {
                    return;
                }
                Kvs kvs = new Kvs(category.name() + System.currentTimeMillis(), BocloudStatus.Server.RUNNING.name(),
                    context.getTarget(), IDFactory.instance().uuid(), category.name(),
                    hosts.size() > 1 ? "cluster" : "standalone", version, context.getTenant(), username, password, null,
                    node.getInstanceId());
                kvs.setOwnerId(context.getTarget());
                kvs.setGmtCatalog(context.getCatalog().name());
                kvsRepository.save(kvs);
                List<JSONObject> groups = JSONObject.parseArray(fronts.getString("groups"), JSONObject.class);
                for (JSONObject host : hosts) {
                    String role = "";
                    if (!ListTool.isEmpty(groups)) {
                        for (JSONObject group : groups) {
                            if (group.getString("hosts").contains(host.getString("host"))) {
                                if (group.getString("name").contains("master")) {
                                    role += " master";
                                } else if (group.getString("name").contains("slave")) {
                                    role += " slave";
                                } else if (group.getString("name").contains("sentinel")) {
                                    role += " sentinel";
                                } else if (group.getString("name").contains("replicas")) {
                                    role += " replicas";
                                } else if (group.getString("name").contains("arbiter")) {
                                    role += " arbiter";
                                } else if (group.getString("name").contains("backup")) {
                                    role += " backup";
                                } else if (group.getString("name").contains("magent")) {
                                    role += " magent";
                                }
                            }
                        }
                        role = role.trim();
                    }
                    if (StringUtils.isEmpty(role)) {
                        role = "standalone";
                    }
                    KvsNode kvsNode = new KvsNode(host.getString("host"), port, role,
                        BocloudStatus.Server.RUNNING.name(), null, kvs.getId());
                    if (ResourceCategory.Redis.equals(category) && role.equals("sentinel")) {
                        kvsNode.setPort(26379);
                    }
                    kvsNode.setManageIp(host.getString("host"));
                    if ("CloudServer".equals(host.getString("catalog"))) {
                        kvsNode.setServerId(host.getLong("id"));
                    } else {
                        Resource resource = resourceRepository.query(host.getLong("id"));
                        if (null != resource) {
                            kvsNode.setServerId(resource.getTarget());
                        }
                    }
                    if (role.contains("master") || "standalone".equals(role)) {
                        kvs.setManageIp(host.getString("host"));
                        kvs.setAddress(host.getString("host") + ":" + port);
                    }
                    kvsNodeRepository.save(kvsNode);
                }
                kvsRepository.update(kvs);
                if (ResourceCategory.MongoDB.equals(category)) {
                    List<JSONObject> users = fronts.parseArray(fronts.getString("mongodb_users"), JSONObject.class);
                    if (!ListTool.isEmpty(users)) {
                        for (JSONObject user : users) {
                            KvsUser kvsUser = new KvsUser(user.getString("name"), user.getString("password"), null,
                                kvs.getId(), null);
                            kvsUserRepository.save(kvsUser);
                        }
                    }
                    JSONObject manageConfig = fronts.getJSONObject("manage_config");
                    KvsUser user = new KvsUser(manageConfig.getString("mongodb_root_admin_name"),
                        manageConfig.getString("mongodb_root_admin_password"), null, kvs.getId(), null);
                    kvsUserRepository.save(user);
                }
                MDSUtils.send(amqpTemplate, kvs.getId(), kvs.getType(), "create");
                return;
            }
            JSONObject target = JSONObject.parseObject(input.getString("target"));
            List<JSONObject> datas = JSONObject.parseArray(target.getString("data"), JSONObject.class);
            if (ListTool.isEmpty(datas)) {
                return;
            }
            Map<String, JSONObject> hostMap = new HashMap<>();
            for (JSONObject host : hosts) {
                if (host.getBoolean("success")) {
                    hostMap.put(host.getString("host"), host);
                }
            }
            for (JSONObject data : datas) {
                if (!hostMap.containsKey(data.getString("host"))) {
                    continue;
                }
                Kvs kvs = new Kvs(category.name() + System.currentTimeMillis(), BocloudStatus.Server.RUNNING.name(),
                    context.getTarget(), IDFactory.instance().uuid(), category.name(), "standalone", version,
                    context.getTenant(), username, password, null, node.getInstanceId());
                kvs.setGmtCatalog(context.getCatalog().name());
                kvs.setOwnerId(context.getTarget());
                kvs.setManageIp(data.getString("host"));
                kvs.setAddress(data.getString("host") + ":" + port);
                kvsRepository.save(kvs);
                KvsNode kvsNode = new KvsNode(data.getString("host"), port, "standalone",
                    BocloudStatus.Server.RUNNING.name(), null, kvs.getId());
                kvsNode.setManageIp(data.getString("host"));
                if ("CloudServer".equals(data.getString("catalog"))) {
                    kvsNode.setServerId(data.getLong("id"));
                } else {
                    Resource resource = resourceRepository.query(data.getLong("id"));
                    if (null != resource) {
                        kvsNode.setServerId(resource.getTarget());
                    }
                }
                kvsNodeRepository.save(kvsNode);
                if (ResourceCategory.MongoDB.equals(category)) {
                    List<JSONObject> users = fronts.parseArray(fronts.getString("mongodb_users"), JSONObject.class);
                    if (!ListTool.isEmpty(users)) {
                        for (JSONObject user : users) {
                            KvsUser kvsUser = new KvsUser(user.getString("name"), user.getString("password"), null,
                                kvs.getId(), null);
                            kvsUserRepository.save(kvsUser);
                        }
                    }
                    JSONObject manageConfig = fronts.getJSONObject("manage_config");
                    KvsUser user = new KvsUser(manageConfig.getString("mongodb_root_admin_name"),
                        manageConfig.getString("mongodb_root_admin_password"), null, kvs.getId(), null);
                    kvsUserRepository.save(user);
                }
                MDSUtils.send(amqpTemplate, kvs.getId(), kvs.getType(), "create");
            }
        } catch (Exception e) {
            logger.error("save kvs softwart install failure:", e);
        }
    }

}
