package com.bocloud.pms.service;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.concurrent.TimeUnit;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.amqp.core.AmqpTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.http.HttpClient;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.GridBean;
import com.bocloud.common.model.OperateResult;
import com.bocloud.common.model.Pager;
import com.bocloud.common.model.Param;
import com.bocloud.common.model.RequestContext;
import com.bocloud.common.model.RequestContext.Catalog;
import com.bocloud.common.model.Result;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.GridHelper;
import com.bocloud.common.utils.IDFactory;
import com.bocloud.common.utils.ListTool;
import com.bocloud.pms.entity.Lbs;
import com.bocloud.pms.entity.LbsNode;
import com.bocloud.pms.entity.resource.Recycled;
import com.bocloud.pms.entity.resource.Resource;
import com.bocloud.pms.entity.resource.TaskNode;
import com.bocloud.pms.entity.resource.enums.BocloudStatus;
import com.bocloud.pms.entity.resource.enums.OrderItemStatus;
import com.bocloud.pms.entity.resource.enums.ResourceCategory;
import com.bocloud.pms.entity.resource.model.NotifyMessage;
import com.bocloud.pms.interfaces.middleware.LbsService;
import com.bocloud.pms.repository.LbsNodeRepository;
import com.bocloud.pms.repository.LbsRepository;
import com.bocloud.pms.repository.resource.CloudServerRepository;
import com.bocloud.pms.repository.resource.CloudVendorRepository;
import com.bocloud.pms.repository.resource.ResourceRepository;
import com.bocloud.pms.service.internal.CosInternalService;
import com.bocloud.pms.service.internal.ImsCloudServerService;
import com.bocloud.pms.service.utils.MDSUtils;
import com.bocloud.pms.service.utils.ResourceEventPublisher;
import com.bocloud.service.lock.BocloudACLock;
import com.bocloud.service.lock.LockFactory;
import com.google.common.collect.Maps;

/**
 * lbs Service接口实现
 *
 * @author weiwei
 */
@Service("lbsService")
public class LbsServiceImpl implements LbsService {

    private static Logger logger = LoggerFactory.getLogger(LbsServiceImpl.class);
    @Autowired
    private LockFactory lockFactory;
    @Autowired
    private LbsRepository lbsRepository;
    @Autowired
    private LbsNodeRepository lbsNodeRepository;
    @Autowired
    private ResourceRepository resourceRepository;
    @Autowired
    private RedisTemplate<String, String> redisTemplate;
    @Autowired
    private ResourceEventPublisher resourceEventPublisher;
    @Autowired
    private CloudServerRepository cloudServerRepository;
    @Autowired
    private CloudVendorRepository cloudVendorRepository;
    @Autowired
    private CosInternalService cosInternalService;
    @Autowired
    private AmqpTemplate amqpTemplate;
    @Autowired
    private ImsCloudServerService imsCloudServerService;

    @Override
    public BocloudResult<GridBean<Lbs>> list(Pager pager, RequestContext context) {
        try {
            List<Param> params = pager.getParams();
            Integer page = pager.getPage();
            Integer rows = pager.getRows();
            Map<String, String> sorter = pager.getSorter();
            params = Optional.ofNullable(params).orElse(new ArrayList<>());
            sorter = Optional.ofNullable(sorter).orElse(Maps.newHashMap());
            sorter.put("gmtCreate", Common.ONE);
            List<Lbs> list = lbsRepository.list(page, rows, params, sorter);
            for (Lbs lbs : list) {
                JSONObject specs = JSONObject.parseObject(lbs.getSpec());
                if (null != specs) {
                    lbs.setSpec(specs.get("cpu") + "核/" + specs.get("memory") + "GB/" + specs.get("disk") + "GB");
                    lbs.setDataDisk(specs.getInteger("dataDisk"));
                }
                JSONObject inventory = JSONObject.parseObject(lbs.getInventory());
                if (null != inventory) {
                    List<JSONObject> elements = JSONArray.parseArray(inventory.getString("elements"), JSONObject.class);
                    if (!ListTool.isEmpty(elements)) {
                        for (JSONObject object : elements) {
                            if (object.getString("serviceCode").contains("lbs." + lbs.getType())) {
                                continue;
                            }
                            lbs.setDiskSkuId(object.getLong("skuId"));
                        }
                    }
                }
                lbs.setTenantName(redisTemplate.opsForValue().get("Tenant_Name_" + lbs.getTenantId()));
            }
            int total = lbsRepository.count(params);
            GridBean gridBean = GridHelper.getBean(page, rows, total, list);
            return new BocloudResult<>(true, gridBean, "查询成功");
        } catch (Exception e) {
            logger.error("list lbs failure:", e);
            return new BocloudResult<>(false, "查询失败", null, null);
        }
    }

    @Override
    public BocloudResult create(String params, RequestContext context) {
        return null;
    }

    private void syncNodeStatus(LbsNode node, Lbs lbs, HttpClient client, Result result) {
        switch (ResourceCategory.valueOf(lbs.getType())) {
            case Nginx:
                result = client.get(null, null, "http://" + node.getManageIp() + ":" + node.getPort());
                break;
            default:
                break;
        }
        if (result.isSuccess()) {
            if (!"RUNNING".equals(node.getStatus())) {
                node.setStatus("RUNNING");
                lbsNodeRepository.update(node);
                lbs.setStatus("RUNNING");
                lbsRepository.update(lbs);
            }
        } else {
            if (!"STOPPED".equals(node.getStatus())) {
                node.setStatus("STOPPED");
                lbsNodeRepository.update(node);
                lbs.setStatus("STOPPED");
                lbsRepository.update(lbs);
            }
        }
    }

    @Override
    public BocloudResult<Lbs> detail(Long id) {
        try {
            Lbs lbs = lbsRepository.query(id);
            lbs = Optional.ofNullable(lbs).orElseThrow(() -> new Exception("数据不存在"));
            List<LbsNode> nodes = lbsNodeRepository.list(id);
            HttpClient client = new HttpClient(30000);
            Result result = null;
            for (LbsNode node : nodes) {
                syncNodeStatus(node, lbs, client, result);
            }
            lbs.setNodes(nodes);
            JSONObject specs = JSONObject.parseObject(lbs.getSpec());
            if (null != specs) {
                lbs.setSpec(specs.get("cpu") + "核/" + specs.get("memory") + "GB/" + specs.get("disk") + "GB");
                lbs.setDataDisk(specs.getInteger("dataDisk"));
            }
            JSONObject inventory = JSONObject.parseObject(lbs.getInventory());
            if (null != inventory) {
                JSONObject configs = inventory.getJSONObject("configs");
                lbs.setDiskCategory(configs.getString("diskCategoryName"));
            }
            return new BocloudResult<>(true, lbs, "查询详情成功");
        } catch (Exception e) {
            logger.error("query lbs detail fail:", e);
            return new BocloudResult<>(false, "查询详情失败");
        }
    }

    @Override
    public BocloudResult recycle(Long id, RequestContext context) {
        try {
            Lbs lbs = lbsRepository.query(id);
            // List<LbsNode> nodes = lbsNodeRepository.list(id);
            // List<Long> serverIds = Lists.newArrayList();
            // for (LbsNode node : nodes) {
            // CloudServer server = cloudServerRepository.get(node.getInstanceId());
            // Optional.ofNullable(server).orElseThrow(() -> new Exception("云主机不存在"));
            // CloudVendor vendor = cloudVendorRepository.query(server.getVendorId());
            // Optional.ofNullable(vendor).orElseThrow(() -> new Exception("云平台不存在"));
            // serverIds.add(node.getServerId());
            // }
            // BocloudResult batchOperations = imsCloudServerService.batchOperations(String.valueOf(serverIds), "Stop");
            // if (batchOperations.isFailed()) {
            // return new BocloudResult(false, "中间件关机失败！");
            // }
            // 更新实例状态
            lbsRepository.recycle(id, context.getTarget());
            // 回收站生成记录
            JSONObject specs = JSONObject.parseObject(lbs.getSpec());
            String spec = null;
            if (null != specs) {
                spec = specs.get("cpu") + "核/" + specs.get("memory") + "GB/" + specs.get("disk") + "GB";
            }
            Recycled recycled =
                new Recycled(lbs.getName(), lbs.getUuid(), lbs.getType(), lbs.getId(), null, lbs.getGmtCatalog(), spec);
            if (Catalog.Tenant.equals(context.getCatalog())) {
                recycled.setTenantId(lbs.getTenantId());
            } else if (Catalog.User.equals(context.getCatalog())) {
                recycled.setUserId(lbs.getOwnerId());
            }
            BocloudResult joinRecycle = cosInternalService.joinRecycle(recycled);
            logger.info("生成回收站记录结果：{}", JSONObject.toJSON(joinRecycle));
            // 处理订单
            cosInternalService.updateItemStatus(lbs.getUuid(), OrderItemStatus.Termination.name());
            // 处理账单
            BocloudResult result = cosInternalService.recycleBill(lbs.getUuid());
            logger.info("放入回收站生产账单结果：{}", JSONObject.toJSON(result));
            // 发送站内信
            NotifyMessage notifyMessage = new NotifyMessage(context, lbs.getType() + "实例放入回收站通知",
                lbs.getType() + "【" + lbs.getName() + "】已删除放入回收站，30天后将被彻底删除", true, false);
            amqpTemplate.convertAndSend("event.notice.queue", notifyMessage);
            return new BocloudResult(true, "中间件【" + lbs.getName() + "】删除成功，可在回收站查看");
        } catch (Exception e) {
            logger.error("remove lbs fail:", e);
            return new BocloudResult(false, "移入回收车失败");
        }
    }

    @Override
    public BocloudResult restore(Long id, RequestContext context) {
        try {
            Lbs lbs = lbsRepository.query(id);
            // List<LbsNode> nodes = lbsNodeRepository.list(id);
            // List<Long> serverIds = Lists.newArrayList();
            // for (LbsNode node : nodes) {
            // CloudServer server = cloudServerRepository.get(node.getInstanceId());
            // Optional.ofNullable(server).orElseThrow(() -> new Exception("云主机不存在"));
            // CloudVendor vendor = cloudVendorRepository.query(server.getVendorId());
            // Optional.ofNullable(vendor).orElseThrow(() -> new Exception("云平台不存在"));
            // serverIds.add(node.getServerId());
            // }
            // BocloudResult batchOperations = imsCloudServerService.batchOperations(String.valueOf(serverIds),
            // "Start");
            // if (batchOperations.isFailed()) {
            // return new BocloudResult(false, "中间件开机失败！");
            // }
            // 更新实例状态
            lbsRepository.restore(id, context.getTarget());
            // 处理订单
            cosInternalService.updateItemStatus(lbs.getUuid(), OrderItemStatus.OpenedSuccess.name());
            return new BocloudResult(true, lbs.getType() + "【" + lbs.getName() + "】还原成功");
        } catch (Exception e) {
            logger.error("restore lbs fail:", e);
            return new BocloudResult(false, "中间件还原失败");
        }
    }

    @Override
    public void synchronousRoleState(JSONObject content) {
        try {
            Long id = content.getLong("id");
            Lbs lbs = lbsRepository.query(id);
            lbs = Optional.ofNullable(lbs).orElseThrow(() -> new Exception("数据不存在"));
            List<LbsNode> nodes = lbsNodeRepository.list(id);
            HttpClient client = new HttpClient(30000);
            Result result = null;
            for (LbsNode node : nodes) {
                syncNodeStatus(node, lbs, client, result);
            }
            int statusCount = MDSUtils.statusCount(nodes, BocloudStatus.Server.RUNNING.name());
            if (0 == statusCount) {
                if (!BocloudStatus.Server.STOPPED.name().equals(lbs.getStatus())) {
                    lbs.setStatus(BocloudStatus.Server.STOPPED.name());
                    lbsRepository.update(lbs);
                }
            } else if (statusCount == nodes.size()) {
                if (!BocloudStatus.Server.RUNNING.name().equals(lbs.getStatus())) {
                    lbs.setStatus(BocloudStatus.Server.RUNNING.name());
                    lbsRepository.update(lbs);
                }
            } else {
                if (!BocloudStatus.Server.EXCEPTION.name().equals(lbs.getStatus())) {
                    lbs.setStatus(BocloudStatus.Server.EXCEPTION.name());
                    lbsRepository.update(lbs);
                }
            }
            RequestContext context = new RequestContext();
            context.setTenant(lbs.getTenantId());
            context.setTarget(lbs.getTenantId());
            context.setCatalog(Catalog.valueOf(lbs.getGmtCatalog()));
            resourceEventPublisher
                .send(new OperateResult(true, "节点角色状态变更通知", "node_change_" + lbs.getType().toLowerCase(), null,
                    OperateResult.OperateCategory.ServiceEvent, context));
        } catch (Exception e) {
            logger.error("集群角色同步失败：", e.getMessage());
        }
    }

    @Override
    public BocloudResult remove(Long id, RequestContext context) {
        String path = Lbs.class.getSimpleName() + "_" + id;
        try (BocloudACLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            Lbs lbs = lbsRepository.query(id);
            lbs = Optional.ofNullable(lbs).orElseThrow(() -> new Exception("数据不存在"));
            if (null != lbs.getTenantId() && lbs.getTenantId() > 0) {
                return new BocloudResult(false, "租户数据不可删除");
            }
            lbsNodeRepository.deleteByLbs(lbs.getId());
            lbsRepository.remove(lbs.getId(), context.getTarget());
            MDSUtils.send(amqpTemplate, lbs.getId(), lbs.getType(), "delete");
            return new BocloudResult(true, lbs.getName() + "删除成功");
        } catch (Exception e) {
            logger.error("remove lbs failure:", e);
            return new BocloudResult(false, "删除失败");
        }
    }

    @Override
    public void install(String content, RequestContext context) {
        TaskNode node = JSONObject.parseObject(content, TaskNode.class);
        JSONObject output = JSONObject.parseObject(node.getOutput());
        if (null == output) {
            return;
        }
        List<JSONObject> hosts = JSONObject.parseArray(output.getString("data"), JSONObject.class);
        if (ListTool.isEmpty(hosts)) {
            return;
        }
        JSONObject input = JSONObject.parseObject(node.getParams());
        if (null == input) {
            return;
        }
        JSONObject target = JSONObject.parseObject(input.getString("target"));
        List<JSONObject> datas = JSONObject.parseArray(target.getString("data"), JSONObject.class);
        if (ListTool.isEmpty(datas)) {
            return;
        }
        Map<String, JSONObject> hostMap = new HashMap<>();
        for (JSONObject host : hosts) {
            if (host.getBoolean("success")) {
                hostMap.put(host.getString("host"), host);
            }
        }
        try {
            if ("task.software.nginx".equals(node.getCode())) {
                for (JSONObject data : datas) {
                    if (!hostMap.containsKey(data.getString("host"))) {
                        continue;
                    }
                    Lbs lbs = new Lbs();
                    lbs.setName("Nginx" + System.currentTimeMillis());
                    lbs.setStatus(BocloudStatus.Server.RUNNING.name());
                    lbs.setCreatorId(context.getTarget());
                    lbs.setTenantId(context.getTenant());
                    lbs.setInstanceId(IDFactory.instance().uuid());
                    lbs.setType(ResourceCategory.Nginx.name());
                    lbs.setGmtCatalog(context.getCatalog().name());
                    lbs.setManageIp(data.getString("host"));
                    lbs.setAddress(data.getString("host") + ":" + 80);
                    lbs.setVersion("1.12.2");
                    lbs.setCategory("standalone");
                    lbs.setTaskId(node.getInstanceId());
                    lbsRepository.save(lbs);
                    LbsNode lbsNode = new LbsNode(data.getString("host"), 80, "standalone",
                        BocloudStatus.Server.RUNNING.name(), null, lbs.getId());
                    lbsNode.setManageIp(data.getString("host"));
                    if ("CloudServer".equals(data.getString("catalog"))) {
                        lbsNode.setServerId(data.getLong("id"));
                    } else {
                        Resource resource = resourceRepository.query(data.getLong("id"));
                        if (null != resource) {
                            lbsNode.setServerId(resource.getTarget());
                        }
                    }
                    lbsNodeRepository.save(lbsNode);
                    MDSUtils.send(amqpTemplate, lbs.getId(), lbs.getType(), "create");
                }
            }
        } catch (Exception e) {
            logger.error("save lbs softwart install failure:", e);
        }
    }

}
