package com.bocloud.pms.service;

import java.math.BigDecimal;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.Set;
import java.util.concurrent.Callable;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.Future;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.amqp.core.AmqpTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.scheduling.concurrent.ThreadPoolTaskExecutor;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.GridBean;
import com.bocloud.common.model.OperateResult;
import com.bocloud.common.model.OperateResult.OperateCategory;
import com.bocloud.common.model.Pager;
import com.bocloud.common.model.Param;
import com.bocloud.common.model.RequestContext;
import com.bocloud.common.model.RequestContext.Catalog;
import com.bocloud.common.model.Sign;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.DateTools;
import com.bocloud.common.utils.GridHelper;
import com.bocloud.common.utils.IDFactory;
import com.bocloud.common.utils.JSONTools;
import com.bocloud.common.utils.ListTool;
import com.bocloud.common.utils.MapTools;
import com.bocloud.pms.entity.Rds;
import com.bocloud.pms.entity.RdsDb;
import com.bocloud.pms.entity.RdsLog;
import com.bocloud.pms.entity.RdsNode;
import com.bocloud.pms.entity.RdsParamLog;
import com.bocloud.pms.entity.RdsUser;
import com.bocloud.pms.entity.resource.Business;
import com.bocloud.pms.entity.resource.CloudServer;
import com.bocloud.pms.entity.resource.CloudVendor;
import com.bocloud.pms.entity.resource.OrderItem;
import com.bocloud.pms.entity.resource.OrderMessage;
import com.bocloud.pms.entity.resource.Project;
import com.bocloud.pms.entity.resource.Recycled;
import com.bocloud.pms.entity.resource.Resource;
import com.bocloud.pms.entity.resource.Server;
import com.bocloud.pms.entity.resource.ServiceOrder;
import com.bocloud.pms.entity.resource.Snapshot;
import com.bocloud.pms.entity.resource.TaskInstance;
import com.bocloud.pms.entity.resource.TaskNode;
import com.bocloud.pms.entity.resource.Tenant;
import com.bocloud.pms.entity.resource.User;
import com.bocloud.pms.entity.resource.enums.BocloudStatus;
import com.bocloud.pms.entity.resource.enums.CloudProvider;
import com.bocloud.pms.entity.resource.enums.OrderItemStatus;
import com.bocloud.pms.entity.resource.enums.ResourceCategory;
import com.bocloud.pms.entity.resource.enums.Status;
import com.bocloud.pms.entity.resource.model.AlterationParam;
import com.bocloud.pms.entity.resource.model.CancelationParam;
import com.bocloud.pms.entity.resource.model.ExtensionParam;
import com.bocloud.pms.entity.resource.model.LocationModel;
import com.bocloud.pms.entity.resource.model.NotifyMessage;
import com.bocloud.pms.entity.resource.model.SnapshotModel;
import com.bocloud.pms.interfaces.middleware.RdsService;
import com.bocloud.pms.repository.RdsDbRepository;
import com.bocloud.pms.repository.RdsLogRepository;
import com.bocloud.pms.repository.RdsNodeRepository;
import com.bocloud.pms.repository.RdsParamLogRepository;
import com.bocloud.pms.repository.RdsRepository;
import com.bocloud.pms.repository.RdsUserRepository;
import com.bocloud.pms.repository.resource.CloudServerRepository;
import com.bocloud.pms.repository.resource.CloudVendorRepository;
import com.bocloud.pms.repository.resource.IpRepository;
import com.bocloud.pms.repository.resource.ResourceRepository;
import com.bocloud.pms.repository.resource.ServerRepository;
import com.bocloud.pms.repository.resource.SnapshotRepository;
import com.bocloud.pms.service.internal.CosInternalService;
import com.bocloud.pms.service.internal.ImsCloudServerService;
import com.bocloud.pms.service.internal.ImsIpService;
import com.bocloud.pms.service.internal.SmsBusinessService;
import com.bocloud.pms.service.internal.SmsProjectService;
import com.bocloud.pms.service.internal.SmsTenantService;
import com.bocloud.pms.service.internal.SmsUserService;
import com.bocloud.pms.service.utils.MDSUtils;
import com.bocloud.pms.service.utils.MysqlconnectionRunable;
import com.bocloud.pms.service.utils.ResourceEventPublisher;
import com.bocloud.service.lock.BocloudACLock;
import com.bocloud.service.lock.LockFactory;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;

/**
 * rds Service接口实现
 *
 * @author weiwei
 */
@Service("rdsService")
public class RdsServiceImpl implements RdsService {

    private static Logger logger = LoggerFactory.getLogger(RdsServiceImpl.class);
    @Autowired
    private LockFactory lockFactory;
    @Autowired
    private RdsRepository rdsRepository;
    @Autowired
    private RdsNodeRepository rdsNodeRepository;
    @Autowired
    private RdsDbRepository rdsDbRepository;
    @Autowired
    private CloudServerRepository cloudServerRepository;
    @Autowired
    private ResourceRepository resourceRepository;
    @Autowired
    private AmqpTemplate amqpTemplate;
    @Autowired
    private CloudVendorRepository cloudVendorRepository;
    @Autowired
    private RdsUserRepository rdsUserRepository;
    @Autowired
    private RdsLogRepository rdsLogRepository;
    @Autowired
    private RdsParamLogRepository rdsParamLogRepository;
    @Autowired
    private SnapshotRepository snapshotRepository;
    @Autowired
    private ResourceEventPublisher resourceEventPublisher;
    @Autowired
    private ServerRepository serverRepository;
    @Autowired
    private IpRepository ipRepository;
    @Value("${middleware.server.remove:true}")
    private boolean removed;
    @Autowired
    private RedisTemplate<String, String> redisTemplate;
    @Autowired
    private CosInternalService cosInternalService;
    @Autowired
    private ImsCloudServerService imsCloudServerService;
    @Autowired
    private ImsIpService imsIpService;
    @Autowired
    private ThreadPoolTaskExecutor threadPoolTaskExecutor;
    @Autowired
    private SmsProjectService smsProjectService;
    @Autowired
    private SmsUserService smsUserService;
    @Autowired
    private SmsBusinessService smsBusinessService;
    @Autowired
    private SmsTenantService smsTenantService;

    @Override
    public BocloudResult<GridBean<Rds>> list(Pager pager, RequestContext context) {
        try {
            List<Param> params = pager.getParams();
            Integer page = pager.getPage();
            Integer rows = pager.getRows();
            Map<String, String> sorter = pager.getSorter();
            params = Optional.ofNullable(params).orElse(new ArrayList<>());
            sorter = Optional.ofNullable(sorter).orElse(Maps.newHashMap());
            sorter.put("gmtCreate", Common.ONE);
            List<Rds> list = null;
            int total = 0;
            if (context.getCatalog().equals(Catalog.Tenant)) {
                // 租户能看到 1.属于当前租户的资源
                list = rdsRepository.list(page, rows, params, sorter, context.getTenant());
                total = rdsRepository.count(params, context.getTenant());
            } else if (context.getCatalog().equals(Catalog.User)) {
                // 用户能看到 1.自己申请的资源 2.自己所在的资源可见项目（资源对项目下所有成员可见）下创建的资源 3.自己管理的项目下的资源
                List<Project> userManagerProjects = smsProjectService.listUserManagerProjects(context.getTarget());
                List<Project> visibleProjects = smsProjectService.listUserVisibleProjects(context.getTarget());
                List<Long> projectIds = new ArrayList<>();
                List<Long> userManagerProjectsIds =
                    userManagerProjects.stream().map(Project::getId).collect(Collectors.toList());
                if (!userManagerProjectsIds.isEmpty()) {
                    projectIds.addAll(userManagerProjectsIds);
                }
                List<Long> visibleProjectsIds =
                    visibleProjects.stream().map(Project::getId).collect(Collectors.toList());
                if (!visibleProjectsIds.isEmpty()) {
                    projectIds.addAll(visibleProjectsIds);
                }
                list = rdsRepository.list(page, rows, params, sorter, context.getTarget(), projectIds);
                total = rdsRepository.count(params, context.getTarget(), projectIds);
            } else {
                list = rdsRepository.list(page, rows, params, sorter, null);
                total = rdsRepository.count(params, null);
            }

            // 处理业务信息
            List<Long> tenantIds = new ArrayList<>();
            List<Long> userIds = new ArrayList<>();
            List<Long> businessIds = new ArrayList<>();
            List<Long> projectIds = new ArrayList<>();
            for (Rds rds : list) {
                if (rds.getTenantId() != null) {
                    tenantIds.add(rds.getTenantId());
                }
                if (rds.getBusinessId() != null) {
                    businessIds.add(rds.getBusinessId());
                }
                if (rds.getProjectId() != null) {
                    projectIds.add(rds.getProjectId());
                }
                if (rds.getCreatorId() != null) {
                    userIds.add(rds.getCreatorId());
                }
                if (rds.getOwnerId() != null) {
                    userIds.add(rds.getOwnerId());
                }
                if (rds.getMenderId() != null) {
                    userIds.add(rds.getMenderId());
                }
            }
            Map<Long, String> userMap =
                smsUserService.listUsersByIds(userIds).stream().collect(Collectors.toMap(User::getId, User::getName));
            Map<Long, String> projectMap = smsProjectService.listProjectsByIds(projectIds).stream()
                .collect(Collectors.toMap(Project::getId, Project::getName));
            Map<Long, String> businessMap = smsBusinessService.listBusinesssByIds(businessIds).stream()
                .collect(Collectors.toMap(Business::getId, Business::getName));
            Map<Long, String> tenantMap = smsTenantService.listTenantsByIds(tenantIds).stream()
                .collect(Collectors.toMap(Tenant::getId, Tenant::getName));
            for (Rds rds : list) {
                JSONObject specs = JSONObject.parseObject(rds.getSpec());
                if (null != specs) {
                    rds.setSpec(specs.get("cpu") + "核/" + specs.get("memory") + "GB/" + specs.get("disk") + "GB");
                    rds.setDataDisk(specs.getInteger("dataDisk"));
                }
                JSONObject inventory = JSONObject.parseObject(rds.getInventory());
                if (null != inventory) {
                    List<JSONObject> elements = JSONArray.parseArray(inventory.getString("elements"), JSONObject.class);
                    if (!ListTool.isEmpty(elements)) {
                        for (JSONObject object : elements) {
                            if (object.getString("serviceCode").equalsIgnoreCase("rds." + rds.getType())) {
                                continue;
                            }
                            rds.setDiskSkuId(object.getLong("skuId"));
                        }
                    }
                }

                if ("MySQL".equalsIgnoreCase(rds.getType())) {
                    if ("mgr.single".equalsIgnoreCase(rds.getProps())) {
                        rds.setCategoryName("MGR单主版");
                    } else if ("mgr.multi".equalsIgnoreCase(rds.getProps())) {
                        rds.setCategoryName("MGR多主版");
                    } else if ("mha".equalsIgnoreCase(rds.getProps())) {
                        rds.setCategoryName("MHA版");
                    } else if ("msv".equalsIgnoreCase(rds.getProps())) {
                        rds.setCategoryName("主从版");
                    } else if ("msb".equalsIgnoreCase(rds.getProps())) {
                        rds.setCategoryName("主备版");
                    } else {
                        rds.setCategoryName("单机版");
                    }
                    if ("mgr.multi".equalsIgnoreCase(rds.getProps()) || "msv".equalsIgnoreCase(rds.getProps())) {
                        List<RdsNode> nodes = rdsNodeRepository.list(rds.getId());
                        String address = "";
                        for (RdsNode node : nodes) {
                            address += node.getManageIp() + ":" + node.getPort() + ",";
                        }
                        if (!StringUtils.isEmpty(address)) {
                            rds.setAddress(address.substring(0, address.length() - 1));
                        }
                    }
                }
                // 设置云主机归属信息
                if (rds.getTenantId() != null) {
                    rds.setTenantName(tenantMap.get(rds.getTenantId()));
                }
                if (rds.getBusinessId() != null) {
                    rds.setBusinessName(businessMap.get(rds.getBusinessId()));
                }
                if (rds.getProjectId() != null) {
                    rds.setProjectName(projectMap.get(rds.getProjectId()));
                }
                if (rds.getCreatorId() != null) {
                    rds.setCreatorName(userMap.get(rds.getCreatorId()));
                }
                if (rds.getOwnerId() != null) {
                    rds.setOwnerName(userMap.get(rds.getOwnerId()));
                }
                if (rds.getMenderId() != null) {
                    rds.setMenderName(userMap.get(rds.getMenderId()));
                }
            }
            GridBean gridBean = GridHelper.getBean(page, rows, total, list);
            return new BocloudResult<>(true, gridBean, "查询成功");
        } catch (Exception e) {
            logger.error("list rds failure:", e);
            return new BocloudResult<>(false, "查询失败", null, null);
        }
    }

    @Override
    public BocloudResult apply(Object object, ResourceCategory category, OrderMessage.Operation operation,
        RequestContext context) {
        BocloudResult result = null;
        switch (operation) {
            case Application:
                result = create(JSONObject.parseObject(JSONObject.toJSONString(object)), category, context);
                break;
            case Upgrade:
                result = modify((AlterationParam)object, context);
                break;
            case Cancelation:
                result = remove((CancelationParam)object, context);
                break;
            case Extension:
                result = postpone((ExtensionParam)object, context);
                break;
            default:
                result = new BocloudResult<>(false, "暂不支持该类操作");
                break;
        }
        return result;
    }

    private BocloudResult create(JSONObject object, ResourceCategory category, RequestContext context) {
        ServiceOrder order = JSON.parseObject(object.getString("order"), ServiceOrder.class);
        List<OrderItem> items = order.getItems();
        if (ListTool.isEmpty(items)) {
            return new BocloudResult<>(false, "订单数据错误！");
        }
        JSONObject emption = JSONObject.parseObject(object.getString("emption"));
        // 使用时长信息
        JSONObject duration = JSONObject.parseObject(emption.getString("duration"));
        // 使用时长单位Hour/Month
        String mode = duration.getString("mode");
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(new Date());
        if ("Month".equalsIgnoreCase(mode)) {
            calendar.add(Calendar.MONTH, duration.getInteger("amount"));
        } else if ("Year".equalsIgnoreCase(mode)) {
            calendar.add(Calendar.MONTH, duration.getInteger("amount"));
        } else {
            calendar.setTime(DateTools.stringToDateTime("2099-12-31 00:00:00"));
        }
        try {
            JSONObject params = MDSUtils.analyze(object);
            JSONObject software = params.getJSONObject("software");
            JSONObject resource = params.getJSONObject("resource");
            LocationModel location = JSONObject.parseObject(object.getString("location"), LocationModel.class);
            for (int i = 0; i < items.size(); i++) {
                if (!items.get(i).getServiceCode().startsWith("rds.")) {
                    continue;
                }
                Rds rds = new Rds();
                rds.setName(category.name() + IDFactory.instance().uuid());
                rds.setStatus(BocloudStatus.Server.BUILDING.name());
                rds.setUuid(items.get(i).getSn());
                rds.setInstanceId(IDFactory.instance().uuid());
                rds.setType(category.name());
                rds.setVersion(software.getString("version"));
                rds.setRegion(location.getRegion());
                rds.setResourceLabel(resource.getString("resourceLabel"));
                rds.setInventory(object.toJSONString());
                String categoryCode = software.getString("categoryCode");
                if (software.getBooleanValue("clusterInstall")) {
                    rds.setCategory("cluster");
                    if (null != categoryCode && categoryCode.contains("cluster_")) {
                        rds.setProps(categoryCode.substring(categoryCode.indexOf("_") + 1));
                    }
                } else {
                    rds.setCategory(categoryCode);
                }
                rds.setSkuId(software.getLong("skuId"));
                rds.setCategoryId(software.getLong("categoryId"));
                JSONObject specs = new JSONObject();
                specs.put("cpu", resource.getString("cpu"));
                specs.put("memory", resource.getString("memory"));
                specs.put("disk", resource.getString("disk"));
                List<Integer> disks = JSONObject.parseArray(resource.getString("disks"), Integer.class);
                if (!ListTool.isEmpty(disks)) {
                    specs.put("dataDisk", disks.get(0));
                }
                rds.setSpec(specs.toJSONString());
                if (ResourceCategory.SqlServer.name().equalsIgnoreCase(rds.getType())) {
                    rds.setUsername("sa");
                } else {
                    rds.setUsername("zeus");
                }
                rds.setPassword(MDSUtils.encrypt("Zeus@v587"));
                // 自服务需要用到
                rds.setCreatorId(location.getCreatorId() == null ? 0 : location.getCreatorId());
                rds.setOwnerId(location.getOwnerId() == null ? 0 : location.getOwnerId());
                rds.setMenderId(location.getMenderId() == null ? 0 : location.getMenderId());
                rds.setGmtCatalog(context.getCatalog().name());
                rds.setTenantId(context.getTenant() == null ? 0 : context.getTenant());
                rds.setBusinessId(location.getBusinessId() == null ? 0 : location.getBusinessId());
                rds.setProjectId(location.getProjectId() == null ? 0 : location.getProjectId());
                rds.setPoolGroupId(location.getPoolGroupId());
                rds.setExpiredTime(calendar.getTime());
                rdsRepository.save(rds);
                JSONObject message = new JSONObject();
                message.put("context", context);
                message.put("queue", "service.task.result");
                message.put("params", params);
                JSONObject service = new JSONObject();
                service.put("uuid", rds.getUuid());
                service.put("category", category.name());
                message.put("service", service);
                this.amqpTemplate.convertAndSend("service.task.queue", message);
            }
            return new BocloudResult<>(true, category.name() + "订购成功");
        } catch (Exception e) {
            logger.error("save rds failure:", e);
            return new BocloudResult<>(false, "数据库订购失败");
        }
    }

    private BocloudResult modify(AlterationParam param, RequestContext context) {
        String path = Rds.class.getSimpleName() + "_" + param.getInstance();
        try (BocloudACLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            Rds rds = rdsRepository.get(param.getInstance());
            if (null == rds) {
                logger.warn("rds does not exist!");
                return new BocloudResult<>(false, "数据库不存在!");
            }
            ResourceCategory category = ResourceCategory.getValue(rds.getType());
            List<RdsNode> nodes = rdsNodeRepository.list(rds.getId());
            if (ListTool.isEmpty(nodes)) {
                return new BocloudResult<>(false, "数据库节点不存在");
            }
            List<CloudServer> servers = new ArrayList<>();
            for (RdsNode node : nodes) {
                CloudServer server = cloudServerRepository.getByInstanceId(node.getInstanceId());
                if (null == server) {
                    server = cloudServerRepository.get(node.getServerId());
                }
                if (null != server) {
                    servers.add(server);
                }
            }
            if (ListTool.isEmpty(servers)) {
                return new BocloudResult<>(false, "数据库节点不存在");
            }
            Long vendorId = servers.get(0).getVendorId();
            CloudVendor vendor = cloudVendorRepository.query(vendorId);
            if (null == vendor) {
                logger.warn("Cloud vendor does not exist!");
                return new BocloudResult<>(false, "云平台不存在!");
            }
            JSONObject specs = JSONObject.parseObject(rds.getSpec());
            if (null == param.getCpu() && null != param.getDataDisk()) {
                if (param.getDataDisk() <= specs.getIntValue("dataDisk")) {
                    return new BocloudResult<>(false, "数据盘扩容大小不能等于小于当前大小");
                }
                for (CloudServer server : servers) {
                    JSONObject object = new JSONObject();
                    object.put("id", server.getId());
                    object.put("name", server.getName());
                    object.put("intanceId", server.getInstanceId());
                    object.put("dataDisk", param.getDataDisk());
                    logger.info("数据盘扩容参数：" + object.toJSONString());
                    BocloudResult result = imsCloudServerService.resize(server.getId(), object, "upgradeconfig");
                    if (result.isFailed()) {
                        return new BocloudResult<>(false, "数据盘扩容失败：" + result.getMessage());
                    }
                }
                specs.put("dataDisk", param.getDataDisk());
                rds.setSpec(specs.toJSONString());
                // rds.setUuid(param.getNewInstance());
                rds.setMenderId(context.getTarget());
                rds.setGmtModify(new Date());
                rdsRepository.update(rds);
                MDSUtils.send(amqpTemplate, rds, nodes.size(), OrderMessage.Operation.Upgrade,
                    OrderItemStatus.ChangeSuccess, MDSUtils.RDS, param, context, null);
                return new BocloudResult<>(true, category.name() + "数据盘扩容成功");
            }
            // CPU
            Integer cpu = param.getCpu();
            // 内存
            BigDecimal memory = param.getMemory();
            // 磁盘
            // Integer disk = param.getSystemDisk();
            if (cpu < specs.getIntValue("cpu") || memory.doubleValue() < specs.getBigDecimal("memory").doubleValue()) {
                return new BocloudResult<>(false, "升级配置不能小于当前配置");
            }
            if (cpu.equals(specs.getIntValue("cpu")) && memory.equals(specs.getBigDecimal("memory"))) {
                return new BocloudResult<>(false, "升级配置等于当前配置");
            }
            for (CloudServer server : servers) {
                JSONObject object = new JSONObject();
                object.put("id", server.getId());
                object.put("name", server.getName());
                object.put("intanceId", server.getInstanceId());
                object.put("cpu", param.getCpu());
                object.put("memory", param.getMemory());
                object.put("systemDisk", param.getSystemDisk());
                logger.info("配置升级参数：" + object.toJSONString());
                BocloudResult result = imsCloudServerService.resize(server.getId(), object, "upgradeconfig");
                if (result.isFailed()) {
                    return new BocloudResult<>(false, "配置升级失败：" + result.getMessage());
                }
            }
            specs.put("cpu", param.getCpu());
            specs.put("memory", param.getMemory());
            // specs.put("disk", param.getSystemDisk());
            rds.setSpec(specs.toJSONString());
            // rds.setUuid(param.getNewInstance());
            rds.setMenderId(context.getTarget());
            rds.setGmtModify(new Date());
            rds.setSkuId(param.getUpgradeSkuId());
            rdsRepository.update(rds);
            MDSUtils.send(amqpTemplate, rds, nodes.size(), OrderMessage.Operation.Upgrade,
                OrderItemStatus.ChangeSuccess, MDSUtils.RDS, param, context, null);
            return new BocloudResult<>(true, category.name() + "规格变更成功");
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return new BocloudResult<>(false, "数据库规格变更失败");
        }
    }

    private BocloudResult remove(CancelationParam param, RequestContext context) {
        String path = Rds.class.getSimpleName() + "_" + param.getInstance();
        try (BocloudACLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            Rds rds = rdsRepository.get(param.getInstance());
            if (null == rds) {
                return new BocloudResult<>(false, "关系数据库不存在");
            }
            List<RdsNode> nodes = rdsNodeRepository.list(rds.getId());
            for (RdsNode node : nodes) {
                CloudServer server = cloudServerRepository.getByInstanceId(node.getInstanceId());
                if (null == server) {
                    server = cloudServerRepository.get(node.getServerId());
                }
                if (null != server) {
                    imsCloudServerService.delete(server.getId());
                }
            }
            ResourceCategory category = ResourceCategory.getValue(rds.getType());
            rdsLogRepository.deleteByRds(rds.getId());
            rdsParamLogRepository.deleteByRds(rds.getId());
            rdsNodeRepository.deleteByRds(rds.getId());
            rdsDbRepository.deleteByRds(rds.getId());
            rdsUserRepository.deleteByRds(rds.getId());
            if ((ResourceCategory.MySQL == category || ResourceCategory.PostgreSQL == category)
                && "cluster".equals(rds.getCategory())) {
                JSONObject inventory = JSONObject.parseObject(rds.getInventory());
                if (null != inventory) {
                    String vip = rds.getAddress().split(":")[0];
                    JSONObject configs = inventory.getJSONObject("configs");
                    imsIpService.deleteBySubnetId(vip, configs.getLong("subnetId"));
                }
            }
            rdsRepository.remove(rds.getId(), context.getTarget());
            MDSUtils.send(amqpTemplate, rds, nodes.size(), OrderMessage.Operation.Cancelation,
                OrderItemStatus.UnsubscribeSuccess, MDSUtils.RDS, param, context, null);
            MDSUtils.send(amqpTemplate, rds.getId(), rds.getType(), "delete");
            // 清空回收站
            cosInternalService.recycle(rds.getUuid());
            return new BocloudResult<>(true, category.name() + "退订成功");
        } catch (Exception e) {
            logger.error("remove rds failure:", e);
            return new BocloudResult<>(false, "数据库退订失败");
        }
    }

    @Override
    public BocloudResult<Rds> detail(Long id) {
        try {
            Rds rds = rdsRepository.query(id);
            if (null == rds) {
                logger.warn("rds data not exist");
                return new BocloudResult<>(false, "数据不存在");
            }
            List<RdsNode> nodes = rdsNodeRepository.list(id);
            if (ResourceCategory.MySQL.name().equalsIgnoreCase(rds.getType())) {
                rds.setStatus(syncMysqlRole(nodes, rds));
            } else if (ResourceCategory.SqlServer.name().equalsIgnoreCase(rds.getType())) {
                rds.setStatus(synMssqlRole(nodes, rds));
            } else if (ResourceCategory.PostgreSQL.name().equalsIgnoreCase(rds.getType())) {
                rds.setStatus(synPostgreSqlRole(nodes, rds));
                List<Map<String, String>> connections = Lists.newArrayList();
                if ("cluster".equals(rds.getCategory())) {
                    Map<String, String> write = Maps.newHashMap();
                    write.put("role", "write+read");
                    write.put("address", rds.getAddress());
                    connections.add(write);
                    Map<String, String> read = Maps.newHashMap();
                    read.put("role", "read only");
                    read.put("address", rds.getAddress().replace("5000", "5001"));
                    connections.add(read);
                } else {
                    Map<String, String> connection = Maps.newHashMap();
                    connection.put("role", "standalone");
                    connection.put("address", rds.getAddress());
                    connections.add(connection);
                }
                rds.setConnections(connections);
            }
            if ("MySQL".equalsIgnoreCase(rds.getType())) {
                if ("mgr.single".equalsIgnoreCase(rds.getProps())) {
                    rds.setCategoryName("MGR单主版");
                } else if ("mgr.multi".equalsIgnoreCase(rds.getProps())) {
                    rds.setCategoryName("MGR多主版");
                } else if ("mha".equalsIgnoreCase(rds.getProps())) {
                    rds.setCategoryName("MHA版");
                } else if ("msv".equalsIgnoreCase(rds.getProps())) {
                    rds.setCategoryName("主从版");
                } else if ("msb".equalsIgnoreCase(rds.getProps())) {
                    rds.setCategoryName("主备版");
                } else {
                    rds.setCategoryName("单机版");
                }
                if ("mgr.multi".equalsIgnoreCase(rds.getProps()) || "msv".equalsIgnoreCase(rds.getProps())) {
                    String address = "";
                    for (RdsNode node : nodes) {
                        address += node.getManageIp() + ":" + node.getPort() + ",";
                    }
                    if (!StringUtils.isEmpty(address)) {
                        rds.setAddress(address.substring(0, address.length() - 1));
                    }
                }
            }
            rds.setNodes(rdsNodeRepository.list(rds.getId()));
            JSONObject specs = JSONObject.parseObject(rds.getSpec());
            if (null != specs) {
                rds.setSpec(specs.get("cpu") + "核/" + specs.get("memory") + "GB/" + specs.get("disk") + "GB");
                rds.setDataDisk(specs.getInteger("dataDisk"));
            }
            JSONObject inventory = JSONObject.parseObject(rds.getInventory());
            if (null != inventory) {
                JSONObject configs = inventory.getJSONObject("configs");
                rds.setDiskCategory(configs.getString("diskCategoryName"));
            }

            // 处理业务信息
            if (rds.getTenantId() != null && rds.getTenantId() != 0) {
                Map<Long, String> tenantMap = smsTenantService.listTenantsByIds(Arrays.asList(rds.getTenantId()))
                    .stream().collect(Collectors.toMap(Tenant::getId, Tenant::getName));
                rds.setTenantName(tenantMap.get(rds.getTenantId()));
            }
            if (rds.getBusinessId() != null && rds.getBusinessId() != 0) {
                Map<Long, String> businessMap =
                    smsBusinessService.listBusinesssByIds(Arrays.asList(rds.getBusinessId())).stream()
                        .collect(Collectors.toMap(Business::getId, Business::getName));
                rds.setBusinessName(businessMap.get(rds.getBusinessId()));
            }
            if (rds.getProjectId() != null && rds.getProjectId() != 0) {
                Map<Long, String> projectMap = smsProjectService.listProjectsByIds(Arrays.asList(rds.getProjectId()))
                    .stream().collect(Collectors.toMap(Project::getId, Project::getName));
                rds.setProjectName(projectMap.get(rds.getProjectId()));
            }
            List<Long> userIds = new ArrayList<>();
            if (rds.getCreatorId() != null && rds.getCreatorId() != 0) {
                userIds.add(rds.getCreatorId());
            }
            if (rds.getOwnerId() != null && rds.getOwnerId() != 0) {
                userIds.add(rds.getOwnerId());
            }
            if (rds.getMenderId() != null && rds.getMenderId() != 0) {
                userIds.add(rds.getMenderId());
            }
            if (!ListTool.isEmpty(userIds)) {
                Map<Long, String> userMap = smsUserService.listUsersByIds(userIds).stream()
                    .collect(Collectors.toMap(User::getId, User::getName));
                rds.setCreatorName(userMap.get(rds.getCreatorId()));
                rds.setOwnerName(userMap.get(rds.getOwnerId()));
                rds.setMenderName(userMap.get(rds.getMenderId()));
            }
            return new BocloudResult<>(true, rds, "查询详情成功");
        } catch (

        Exception e) {
            logger.error("query rds detail fail:", e);
            return new BocloudResult<>(false, "查询详情失败");
        }
    }

    @Override
    public BocloudResult<Rds> basics(Long id) {
        try {
            Rds rds = rdsRepository.query(id);
            rds = Optional.ofNullable(rds).orElseThrow(() -> new Exception("数据不存在"));
            if ("MySQL".equalsIgnoreCase(rds.getType())) {
                if ("mgr.single".equalsIgnoreCase(rds.getProps())) {
                    rds.setCategoryName("MGR单主版");
                } else if ("mgr.multi".equalsIgnoreCase(rds.getProps())) {
                    rds.setCategoryName("MGR多主版");
                } else if ("mha".equalsIgnoreCase(rds.getProps())) {
                    rds.setCategoryName("MHA版");
                } else if ("msv".equalsIgnoreCase(rds.getProps())) {
                    rds.setCategoryName("主从版");
                } else if ("msb".equalsIgnoreCase(rds.getProps())) {
                    rds.setCategoryName("主备版");
                } else {
                    rds.setCategoryName("单机版");
                }
            }
            List<RdsNode> nodes = rdsNodeRepository.list(id);
            rds.setNodes(nodes);
            return new BocloudResult<>(true, rds, "查询成功");
        } catch (Exception e) {
            logger.error("query rds detail fail:", e);
            return new BocloudResult<>(false, "查询详情失败");
        }
    }

    @Override
    public BocloudResult remove(Long id, RequestContext context) {
        String path = Rds.class.getSimpleName() + "_" + id;
        try (BocloudACLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            Rds rds = rdsRepository.query(id);
            rds = Optional.ofNullable(rds).orElseThrow(() -> new Exception("数据不存在"));
            if (null != rds.getTenantId() && rds.getTenantId() > 0) {
                return new BocloudResult<>(false, "租户数据不可删除");
            }
            rdsLogRepository.deleteByRds(rds.getId());
            rdsParamLogRepository.deleteByRds(rds.getId());
            rdsNodeRepository.deleteByRds(rds.getId());
            rdsDbRepository.deleteByRds(rds.getId());
            rdsUserRepository.deleteByRds(rds.getId());
            rdsRepository.remove(rds.getId(), context.getTarget());
            return new BocloudResult<>(true, rds.getName() + "删除成功");
        } catch (Exception e) {
            logger.error("remove ass failure:", e);
            return new BocloudResult<>(false, "删除失败");
        }
    }

    private String synMssqlRole(List<RdsNode> nodes, Rds rds) throws Exception {
        BocloudResult result = executeSqlServer("sp_configure", rds, true);
        RdsNode node = nodes.get(0);
        String status = rds.getStatus();
        if (result.isFailed()) {
            if (!BocloudStatus.Server.STOPPED.name().equals(node.getStatus())) {
                node.setStatus(BocloudStatus.Server.STOPPED.name());
                node.setRole("(not reachable/healthy)");
                rdsNodeRepository.update(node);
                rds.setStatus(BocloudStatus.Server.STOPPED.name());
                rdsRepository.update(rds);
                status = BocloudStatus.Server.STOPPED.name();
            }
        } else {
            if (!BocloudStatus.Server.RUNNING.name().equals(node.getStatus())) {
                node.setStatus(BocloudStatus.Server.RUNNING.name());
                node.setRole("standalone");
                rdsNodeRepository.update(node);
                rds.setStatus(BocloudStatus.Server.RUNNING.name());
                rdsRepository.update(rds);
                status = BocloudStatus.Server.RUNNING.name();
            }
        }
        return status;
    }

    private String synPostgreSqlRole(List<RdsNode> nodes, Rds rds) throws Exception {
        if ("standalone".equals(rds.getCategory())) {
            BocloudResult result = executePostgresql("show all;", rds, true, false, null);
            RdsNode node = nodes.get(0);
            String status = rds.getStatus();
            if (result.isFailed()) {
                if (!BocloudStatus.Server.STOPPED.name().equals(node.getStatus())) {
                    node.setStatus(BocloudStatus.Server.STOPPED.name());
                    node.setRole("(not reachable/healthy)");
                    rdsNodeRepository.update(node);
                    rds.setStatus(BocloudStatus.Server.STOPPED.name());
                    rdsRepository.update(rds);
                    status = BocloudStatus.Server.STOPPED.name();
                }
            } else {
                if (!BocloudStatus.Server.RUNNING.name().equals(node.getStatus())) {
                    node.setStatus(BocloudStatus.Server.RUNNING.name());
                    node.setRole("standalone");
                    rdsNodeRepository.update(node);
                    rds.setStatus(BocloudStatus.Server.RUNNING.name());
                    rdsRepository.update(rds);
                    status = BocloudStatus.Server.RUNNING.name();
                }
            }
            return status;
        }
        // 集群的状态的处理
        List<RdsNode> rdsNodes = rdsNodeRepository.list(rds.getId());
        String sql = "SELECT pid,usename,application_name,client_addr,state,sync_state  FROM pg_stat_replication;";
        for (RdsNode rdsNode : rdsNodes) {
            BocloudResult result = executePostgresql(sql, rds, true, true, rdsNode.getManageIp());
            if (result.isFailed()) {
                if (!BocloudStatus.Server.STOPPED.name().equals(rdsNode.getStatus())) {
                    rdsNode.setStatus(BocloudStatus.Server.STOPPED.name());
                    rdsNode.setRole("(not reachable/healthy)");
                    rdsNodeRepository.update(rdsNode);
                }
            } else {
                List<JSONObject> params =
                    JSONArray.parseArray(JSONObject.toJSONString(result.getData()), JSONObject.class);
                if (null == params || 0 == params.size()) {
                    if (!"slave".equals(rdsNode.getRole())) {
                        rdsNode.setStatus(BocloudStatus.Server.RUNNING.name());
                        rdsNode.setRole("slave");
                        rdsNodeRepository.update(rdsNode);
                    }
                } else {
                    if (!"master".equals(rdsNode.getRole())) {
                        rdsNode.setStatus(BocloudStatus.Server.RUNNING.name());
                        rdsNode.setRole("master");
                        rdsNodeRepository.update(rdsNode);
                        rds.setManageIp(rdsNode.getManageIp());
                        // rds.setAddress(rdsNode.getManageIp() + ":5432");
                        rdsRepository.update(rds);
                    }
                }
            }
        }
        // 复查去除判断是否存在所有slave节点全部挂掉的情况
        List<RdsNode> nodeList = rdsNodeRepository.list(rds.getId());
        int statusCount = MDSUtils.statusCount(nodeList, BocloudStatus.Server.RUNNING.name());
        if (0 == statusCount) {
            rds.setStatus(BocloudStatus.Server.STOPPED.name());
        } else if (statusCount == nodes.size()) {
            rds.setStatus(BocloudStatus.Server.RUNNING.name());
        } else {
            rds.setStatus(BocloudStatus.Server.EXCEPTION.name());
        }
        rdsRepository.update(rds);
        return rds.getStatus();
    }

    @SuppressWarnings("unchecked")
    private String syncMysqlRole(List<RdsNode> nodes, Rds rds) throws Exception {
        String sql = "select * from information_schema.processlist as p where p.command = 'Binlog Dump';";
        if (StringUtils.isNotEmpty(rds.getProps()) && rds.getProps().toLowerCase().contains("mgr")) {
            sql = "select * from performance_schema.replication_group_members;";
        }
        BocloudResult bocloudResult = execute(sql, rds, true, true);
        if (bocloudResult.isFailed()) {
            logger.error(bocloudResult.getMessage());
            if ("cluster".equals(rds.getCategory())) {
                return BocloudStatus.Server.UNKNOWN.name();
            }
            RdsNode node = nodes.get(0);
            if (!"(not reachable/healthy)".equals(node.getRole())) {
                node.setStatus(BocloudStatus.Server.STOPPED.name());
                node.setRole("(not reachable/healthy)");
                rdsNodeRepository.update(node);
                rds.setStatus(BocloudStatus.Server.STOPPED.name());
                rdsRepository.update(rds);
            }
            return BocloudStatus.Server.STOPPED.name();
        }
        if (!"cluster".equals(rds.getCategory())) {
            if (!BocloudStatus.Server.RUNNING.name().equals(rds.getStatus())) {
                RdsNode node = nodes.get(0);
                node.setStatus(BocloudStatus.Server.RUNNING.name());
                node.setRole("standalone");
                rdsNodeRepository.update(node);
                rds.setStatus(BocloudStatus.Server.RUNNING.name());
                rdsRepository.update(rds);
            }
            return BocloudStatus.Server.RUNNING.name();
        }
        List<JSONObject> params =
            JSONArray.parseArray(JSONObject.toJSONString(bocloudResult.getData()), JSONObject.class);
        StringBuilder ips = new StringBuilder();
        for (JSONObject object : params) {
            ips.append(object.getString("host")).append(";");
        }
        ExecutorService mainExecutor = Executors.newFixedThreadPool(Runtime.getRuntime().availableProcessors() + 1);
        Set<Callable<BocloudResult>> callables = new HashSet<Callable<BocloudResult>>();
        for (RdsNode node : nodes) {
            callables.add(new MysqlconnectionRunable(sql, node, rds));
        }
        List<Future<BocloudResult>> futures = mainExecutor.invokeAll(callables);
        for (Future<BocloudResult> future : futures) {
            BocloudResult result = future.get();
            Map<String, Object> resultData = (Map<String, Object>)result.getData();
            RdsNode node = (RdsNode)resultData.get("node");
            List<JSONObject> slaves = new ArrayList<>();
            if (StringUtils.isNotEmpty(rds.getProps()) && rds.getProps().toLowerCase().contains("mgr")) {
                List<JSONObject> hosts =
                    JSONArray.parseArray(JSONObject.toJSONString(resultData.get("params")), JSONObject.class);
                boolean isMaster = false;
                for (JSONObject host : hosts) {
                    if ("PRIMARY".equalsIgnoreCase(host.getString("role"))
                        && node.getManageIp().equalsIgnoreCase(host.getString("host"))) {
                        isMaster = true;
                        break;
                    }
                }
                if (isMaster) {
                    for (JSONObject host : hosts) {
                        if (!node.getManageIp().equalsIgnoreCase(host.getString("host"))) {
                            slaves.add(host);
                        }
                    }
                }
            } else {
                slaves
                    .addAll(JSONArray.parseArray(JSONObject.toJSONString(resultData.get("params")), JSONObject.class));
            }
            if (result.isFailed()) {
                if (!"(not reachable/healthy)".equals(node.getRole())) {
                    // 节点无法连接
                    node.setStatus(BocloudStatus.Server.STOPPED.name());
                    node.setRole("(not reachable/healthy)");
                    rdsNodeRepository.update(node);
                }
                continue;
            }
            if ("master".equals(node.getRole())) {
                if (slaves.isEmpty()) {
                    node.setRole("slave");
                    rdsNodeRepository.update(node);
                }
            } else if ("slave".equals(node.getRole())) {
                if (!slaves.isEmpty()) {
                    node.setRole("master");
                    rdsNodeRepository.update(node);
                    rds.setManageIp(node.getManageIp());
                } else {
                    if (ips.length() == 0) {
                        node.setRole("standalone");
                        rdsNodeRepository.update(node);
                    }
                }
            } else if ("standalone".equals(node.getRole())) {
                if (ips.toString().contains(node.getManageIp())) {
                    node.setRole("slave");
                    rdsNodeRepository.update(node);
                }
                if (!slaves.isEmpty()) {
                    node.setRole("master");
                    rdsNodeRepository.update(node);
                    rds.setManageIp(node.getManageIp());
                }
            } else {
                if (result.isSuccess()) {
                    node.setStatus(BocloudStatus.Server.RUNNING.name());
                    if (!slaves.isEmpty()) {
                        node.setRole("master");
                        rdsNodeRepository.update(node);
                        rds.setManageIp(node.getManageIp());
                    } else {
                        node.setRole(ips.toString().contains(node.getManageIp()) ? "slave" : "standalone");
                        rdsNodeRepository.update(node);
                    }
                }
            }
        }
        mainExecutor.shutdown();
        mainExecutor.awaitTermination(5, TimeUnit.MINUTES);
        // 复查去除判断是否存在所有slave节点全部挂掉的情况
        List<RdsNode> rdsNodes = rdsNodeRepository.list(rds.getId());
        int count = 0;
        String manageIp = null;
        List<Long> nodeIps = Lists.newArrayList();
        for (RdsNode rdsNode : rdsNodes) {
            if (BocloudStatus.Server.RUNNING.name().equals(rdsNode.getStatus())) {
                nodeIps.add(rdsNode.getId());
                if ("master".equals(rdsNode.getRole())) {
                    manageIp = rdsNode.getManageIp();
                }
                count++;
            }
        }
        if (1 == count) {
            RdsNode node = rdsNodeRepository.query(nodeIps.get(0));
            if (null != node) {
                node.setRole("standalone");
                rdsNodeRepository.update(node);
            }
        }

        int statusCount = MDSUtils.statusCount(rdsNodes, BocloudStatus.Server.RUNNING.name());
        if (0 == statusCount) {
            rds.setStatus(BocloudStatus.Server.STOPPED.name());
        } else if (statusCount == nodes.size()) {
            rds.setStatus(BocloudStatus.Server.RUNNING.name());
        } else {
            rds.setStatus(BocloudStatus.Server.EXCEPTION.name());
        }
        // 复查master节点是否已变更，值为空当前集群不存在
        if (null != manageIp) {
            rds.setManageIp(manageIp);
        }
        rdsRepository.update(rds);
        return rds.getStatus();
    }

    @Override
    public void updateStatus(String content) {
        JSONObject object = JSONTools.isJSONObj(content);
        JSONObject service = object.getJSONObject("service");
        String uuid = service.getString("uuid");
        TaskInstance instance = JSONObject.parseObject(object.getString("instance"), TaskInstance.class);
        try {
            Rds rds = rdsRepository.get(uuid);
            if (null == rds) {
                return;
            }
            RequestContext context = JSONObject.parseObject(instance.getProps(), RequestContext.class);
            if (!"success".equalsIgnoreCase(instance.getStatus())) {
                List<TaskNode> nodes = instance.getNodes();
                String errorMsg = null;
                TaskNode softNode = null;
                for (TaskNode node : nodes) {
                    if (node.getCode().startsWith("task.software")) {
                        softNode = node;
                        continue;
                    }
                    JSONObject output = JSONObject.parseObject(node.getOutput());
                    if (null == output) {
                        continue;
                    }
                    if (Status.Node.FAILED.name().equalsIgnoreCase(node.getStatus()) && StringUtils.isEmpty(errorMsg)) {
                        errorMsg = output.getString("message");
                    }
                    JSONObject data = output.getJSONObject("data");
                    if (null == data) {
                        continue;
                    }
                    Long resId = data.getLong("id");
                    if (null != resId && removed) {
                        if ("task.resource.cvm.vsp".equals(node.getCode())) {
                            imsCloudServerService.delete(resId);
                        } else if ("task.resource.vpc.vip".equals(node.getCode())) {
                            ipRepository.delete(resId);
                        }
                    }
                }
                if (StringUtils.isEmpty(errorMsg)) {
                    JSONObject output = JSONObject.parseObject(softNode.getOutput());
                    List<JSONObject> datas = JSONObject.parseArray(output.getString("data"), JSONObject.class);
                    if (ListTool.isEmpty(datas)) {
                        errorMsg = output.getString("message");
                    } else {
                        for (JSONObject data : datas) {
                            if (!StringUtils.isEmpty(data.getString("message"))) {
                                errorMsg = data.getString("message");
                                break;
                            }
                        }
                    }
                }
                rdsRepository.remove(rds.getId(), 1L);
                MDSUtils.send(amqpTemplate, rds, nodes.size() - 1, OrderMessage.Operation.Application,
                    OrderItemStatus.OpenedFailure, MDSUtils.RDS, null, context, errorMsg);
                resourceEventPublisher.send(new OperateResult(false, errorMsg, "software.install", null,
                    OperateCategory.SoftwareInstall, context));
                return;
            }
            ResourceCategory category = ResourceCategory.getValue(rds.getType());
            Integer port = null;
            if (ResourceCategory.MySQL.equals(category)) {
                port = 3306;
            } else if (ResourceCategory.PostgreSQL.equals(category)) {
                port = 5432;
            } else if (ResourceCategory.SqlServer.equals(category)) {
                port = 1433;
            }
            List<TaskNode> nodes = instance.getNodes();
            List<JSONObject> groups = Lists.newArrayList();
            String publicAddress = null;
            for (TaskNode node : nodes) {
                if ("task.software.mysql".equals(node.getCode()) || "task.software.postgresql".equals(node.getCode())) {
                    JSONObject input = JSONObject.parseObject(node.getInput());
                    if (null != input) {
                        JSONObject fronts = input.getJSONObject("fronts");
                        groups = JSONObject.parseArray(fronts.getString("groups"), JSONObject.class);
                    }
                } else if ("task.resource.vpc.vip".equals(node.getCode())) {
                    JSONObject output = JSONObject.parseObject(node.getOutput());
                    if (null == output || null == output.getJSONObject("data")) {
                        continue;
                    }
                    publicAddress = output.getJSONObject("data").getString("ip");
                }
            }
            String manageIp = null;
            for (TaskNode node : nodes) {
                if (node.getCode().startsWith("task.resource.cvm")) {
                    JSONObject output = JSONObject.parseObject(node.getOutput());
                    if (null == output) {
                        continue;
                    }
                    JSONObject data = output.getJSONObject("data");
                    if (null == data) {
                        continue;
                    }
                    String role = "standalone";
                    if (!ListTool.isEmpty(groups)) {
                        for (JSONObject group : groups) {
                            if (!group.getString("hosts").contains(node.getNodeId())) {
                                continue;
                            }
                            if (group.getString("name").contains("master")) {
                                role = "master";
                            } else if (group.getString("name").contains("slave")) {
                                role = "slave";
                            } else {
                                if (ResourceCategory.PostgreSQL == category) {
                                    List<JSONObject> hosts =
                                        JSONObject.parseArray(group.getString("hosts"), JSONObject.class);
                                    for (int i = 0; i < hosts.size(); i++) {
                                        if (!hosts.get(i).getString("host").equals(node.getNodeId())) {
                                            continue;
                                        }
                                        role = i == 0 ? "master" : "slave";
                                        break;
                                    }
                                }
                                continue;
                            }
                            break;
                        }
                    }
                    List<JSONObject> beans = JSONArray.parseArray(data.getString("privateIps"), JSONObject.class);
                    String address = null;
                    for (JSONObject bean : beans) {
                        if (bean.getString("address").equals(data.getString("managerIp"))) {
                            continue;
                        }
                        address = bean.getString("address");
                    }
                    if (StringUtils.isEmpty(address)) {
                        address = data.getString("managerIp");
                    }
                    if ("master".equals(role) || "standalone".equals(role)) {
                        manageIp = data.getString("managerIp");
                        if (StringUtils.isEmpty(publicAddress)) {
                            publicAddress = address;
                        }
                    }
                    RdsNode rdsNode = new RdsNode(address, port, role, BocloudStatus.Server.RUNNING.name(),
                        data.getString("instanceId"), rds.getId());
                    rdsNode.setManageIp(data.getString("managerIp"));
                    rdsNodeRepository.save(rdsNode);
                    rds.setVendorId(data.getLong("vendorId"));
                }
            }
            rds.setManageIp(manageIp);
            if (ResourceCategory.PostgreSQL == category && "cluster".equals(rds.getCategory())) {
                rds.setAddress(publicAddress + ":" + 5000);
            } else {
                rds.setAddress(publicAddress + ":" + port);
            }
            JSONObject param = JSONObject.parseObject(rds.getInventory());
            if (null != param) {
                saveUserAndDb(param, category, rds);
            }
            rds.setStatus(BocloudStatus.Server.RUNNING.name());
            rds.setTaskId(instance.getId());
            rdsRepository.update(rds);
            MDSUtils.send(amqpTemplate, rds, nodes.size() - 1, OrderMessage.Operation.Application,
                OrderItemStatus.OpenedSuccess, MDSUtils.RDS, null, context, null);
            MDSUtils.send(amqpTemplate, rds.getId(), rds.getType(), "create");
            resourceEventPublisher.send(new OperateResult(true, "数据库申请安装成功", "software.install", null,
                OperateCategory.SoftwareInstall, context));
        } catch (Exception e) {
            logger.error("update rds status fail:", e);
        }
    }

    private BocloudResult postpone(ExtensionParam param, RequestContext context) {
        String path = Rds.class.getSimpleName() + "_" + param.getInstance();
        try (BocloudACLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            Rds rds = rdsRepository.get(param.getInstance());
            if (null == rds) {
                logger.warn("rds data not exist");
                return new BocloudResult<>(false, "数据不存在");
            }
            List<RdsNode> rdsNodes = rdsNodeRepository.list(rds.getId());
            for (RdsNode node : rdsNodes) {
                CloudServer server = cloudServerRepository.getByInstanceId(node.getInstanceId());
                if (null == server) {
                    server = cloudServerRepository.get(node.getServerId());
                }
                // 延期时扫描如果机器是关机状态则开机
                if (null != server && !BocloudStatus.Server.RUNNING.name().equals(server.getStatus())) {
                    imsCloudServerService.operation(server.getId(), "start");
                }
            }
            ResourceCategory category = ResourceCategory.getValue(rds.getType());
            Calendar calendar = Calendar.getInstance();
            calendar.setTime(rds.getExpiredTime());
            calendar.add(Calendar.MONTH, param.getExtension());
            rds.setExpiredTime(calendar.getTime());
            rdsRepository.update(rds);
            MDSUtils.send(amqpTemplate, rds, null, OrderMessage.Operation.Extension, OrderItemStatus.ExtensionSuccess,
                MDSUtils.RDS, param, context, null);
            return new BocloudResult<>(true, category.name() + "延期成功");
        } catch (Exception e) {
            logger.error("postpone rds fail:", e);
            return new BocloudResult<>(false, "数据库延期失败");
        }
    }

    @Override
    public BocloudResult<GridBean<RdsDb>> connections(Pager pager, RequestContext context) {
        try {
            List<Param> params = pager.getParams();
            Integer page = pager.getPage();
            Integer rows = pager.getRows();
            Map<String, String> sorter = pager.getSorter();
            GridBean gridBean = null;
            if (null == params) {
                params = new ArrayList<>();
            }
            if (null == sorter) {
                sorter = Maps.newHashMap();
            }
            sorter.put("gmtCreate", Common.ONE);
            List<RdsDb> rdsDbs = rdsDbRepository.list(page, rows, params, sorter);
            if (ListTool.isEmpty(rdsDbs)) {
                return new BocloudResult<>(true, GridHelper.getBean(page, rows, 0, rdsDbs), "查询成功");
            }
            Rds rds = rdsRepository.query(rdsDbs.get(0).getRdsId());
            if (null == rds) {
                return new BocloudResult<>(false, "数据库不存在");
            }
            List<RdsUser> users = rdsUserRepository.list(rds.getId());
            List<String> userList = new ArrayList<>();
            Map<Long, List<String>> userMap = new HashMap<>();
            for (RdsUser rdsUser : users) {
                if ("all".equalsIgnoreCase(rdsUser.getPermission())) {
                    userList.add(rdsUser.getUsername());
                    continue;
                }
                if (StringUtils.isEmpty(rdsUser.getDbs())) {
                    continue;
                }
                String[] dbIds = rdsUser.getDbs().split(",");
                for (String dbId : dbIds) {
                    Long key = Long.valueOf(dbId);
                    if (userMap.containsKey(key)) {
                        userMap.get(key).add(rdsUser.getUsername());
                    } else {
                        List<String> list = new ArrayList<>();
                        list.add(rdsUser.getUsername());
                        userMap.put(key, list);
                    }
                }
            }
            for (RdsDb db : rdsDbs) {
                db.setAddress(rds.getAddress());
                if (userMap.containsKey(db.getId())) {
                    List<String> list = userMap.get(db.getId());
                    list.addAll(userList);
                    db.setUsers(JSONObject.toJSONString(list));
                } else {
                    db.setUsers(JSONObject.toJSONString(userList));
                }
            }
            int total = rdsDbRepository.count(params);
            gridBean = GridHelper.getBean(page, rows, total, rdsDbs);
            return new BocloudResult<>(true, gridBean, "查询成功");
        } catch (Exception e) {
            logger.error("list rds connections failure:", e);
            return new BocloudResult<>(false, "查询失败");
        }
    }

    @Override
    public BocloudResult<GridBean<RdsLog>> logs(Pager pager, RequestContext context) {
        try {
            List<Param> params = pager.getParams();
            Integer page = pager.getPage();
            Integer rows = pager.getRows();
            Map<String, String> sorter = pager.getSorter();
            if (null == params) {
                params = new ArrayList<>();
            }
            if (null == sorter) {
                sorter = Maps.newHashMap();
            }
            sorter.put("gmtCreate", Common.ONE);
            List<RdsLog> logs = rdsLogRepository.list(page, rows, params, sorter);
            for (RdsLog log : logs) {
                log.setResult(JSONObject.parseObject(log.getResult()).getString("message"));
            }
            int total = rdsLogRepository.count(params);
            GridBean gridBean = GridHelper.getBean(page, rows, total, logs);
            return new BocloudResult<>(true, gridBean, "查询成功");
        } catch (Exception e) {
            logger.error("list rds logs failure:", e);
            return new BocloudResult<>(false, "查询失败");
        }
    }

    @Override
    public BocloudResult<List<Snapshot>> backups(Long id, RequestContext context) {
        try {
            List<RdsNode> nodes = rdsNodeRepository.list(id);
            if (ListTool.isEmpty(nodes)) {
                return new BocloudResult<>(false, "数据不存在");
            }
            String instanceId = null;
            Long serverId = null;
            for (RdsNode node : nodes) {
                if ("master".equalsIgnoreCase(node.getRole())) {
                    instanceId = node.getInstanceId();
                    serverId = node.getServerId();
                    break;
                }
            }
            CloudServer server = cloudServerRepository.getByInstanceId(instanceId);
            if (null == server) {
                server = cloudServerRepository.get(serverId);
            }
            if (null == server) {
                return new BocloudResult<>(false, "云主机不存在");
            }
            List<Snapshot> snapshots = snapshotRepository.listByVmid(server.getId());
            return new BocloudResult<>(true, snapshots, "查询成功");
        } catch (Exception e) {
            logger.error("list rds backups failure:", e);
            return new BocloudResult<>(false, "查询失败");
        }
    }

    private BocloudResult backup(CloudServer server, String type, RequestContext context) {
        try {
            CloudVendor vendor = cloudVendorRepository.query(server.getVendorId());
            if (null == vendor) {
                logger.warn("get Cloud vendor failure!");
                return new BocloudResult<>(false, "数据库备份失败");
            }
            String name = "Rds-backup" + System.currentTimeMillis();
            // 开启一个线程更新状态
            threadPoolTaskExecutor.submit(() -> {
                String operate = "rds." + type + ".backup";
                try {
                    SnapshotModel model = new SnapshotModel(name, null, null, server.getInstanceId());
                    // BocloudResult result = serverDriver.snapshot(model, vendor.getUuid(), null);
                    // if (result.isSuccess()) {
                    // Snapshot snapshot = JSONObject.parseObject(JSONObject.toJSONString(result.getData()),
                    // Snapshot.class);
                    // snapshot.preSave(context.getTarget(), context.getTarget(), server.getVendorId(),
                    // server.getId(), server.getTenantId());
                    // snapshotRepository.save(snapshot);
                    // }
                    // resourceEventPublisher.send(
                    // new OperateResult(result.isSuccess(), "数据库备份" + (result.isSuccess() ? "成功" : "失败"),
                    // operate, null, OperateCategory.ResourceEvent, context));
                } catch (Exception e) {
                    resourceEventPublisher.send(
                        new OperateResult(false, "数据库备份失败", operate, null, OperateCategory.ResourceEvent, context));
                }
            });
            return new BocloudResult<>(true, "数据库备份任务已下发，正在执行...");
        } catch (Exception e) {
            logger.error("backup rds error:", e);
            return new BocloudResult<>(false, "数据库备份失败");
        }
    }

    @Override
    public BocloudResult backup(Long id, RequestContext context) {
        String path = Rds.class.getSimpleName() + "_" + id;
        try (BocloudACLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            List<RdsNode> nodes = rdsNodeRepository.list(id);
            if (ListTool.isEmpty(nodes)) {
                return new BocloudResult<>(false, "数据不存在");
            }
            Rds rds = rdsRepository.query(id);
            if (null == rds) {
                return new BocloudResult<>(false, "rds数据不存在");
            }
            String instanceId = null;
            Long serverId = null;
            for (RdsNode node : nodes) {
                if ("master".equalsIgnoreCase(node.getRole())) {
                    instanceId = node.getInstanceId();
                    serverId = node.getServerId();
                    break;
                }
            }
            CloudServer server = cloudServerRepository.getByInstanceId(instanceId);
            if (null == server) {
                server = cloudServerRepository.get(serverId);
            }
            if (null == server) {
                return new BocloudResult<>(false, "云主机不存在");
            }
            List<Param> params = new ArrayList<>();
            params.add(new Param(MapTools.simpleMap("vmId", server.getId()), Sign.EQ));
            int count = snapshotRepository.count(params);
            if (count >= 3) {
                return new BocloudResult<>(false, "数据库备份不能超过3次");
            }
            return backup(server, rds.getType(), context);
        } catch (Exception e) {
            logger.error("backup rds fail:", e);
            return new BocloudResult<>(false, "数据库备份失败");
        }
    }

    @Override
    public BocloudResult revert(Long id, Long snapshotId, RequestContext context) {
        String path = Rds.class.getSimpleName() + "_" + id;
        try (BocloudACLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            Rds rds = rdsRepository.query(id);
            if (null == rds) {
                return new BocloudResult<>(false, "rds数据不存在");
            }
            return revert(snapshotId, rds.getType(), context);
        } catch (Exception e) {
            logger.error("revert rds fail:", e);
            return new BocloudResult<>(false, "数据库备份失败");
        }
    }

    public BocloudResult revert(Long id, String type, RequestContext context) {
        String path = Snapshot.class.getSimpleName() + "_" + id;
        try (BocloudACLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            Snapshot snap = snapshotRepository.query(id);
            if (null == snap) {
                logger.warn("snapshot data not exist");
                return new BocloudResult<>(false, "恢复失败");
            }
            CloudServer server = cloudServerRepository.query(snap.getVmId());
            if (null == server) {
                logger.warn("cloud server data not exist");
                return new BocloudResult<>(false, "恢复失败");
            }
            // 云供应商
            CloudVendor vendor = cloudVendorRepository.query(snap.getVendorId());
            if (null == vendor) {
                logger.warn("cloud vendor data not exist");
                return new BocloudResult<>(false, "恢复失败");
            }
            // 启动一个线程从MQ上获取数据
            threadPoolTaskExecutor.submit(() -> {
                String subPath = CloudServer.class.getSimpleName() + "_" + server.getId();
                String operate = "rds." + type + ".revert";
                BocloudResult result = null;
                String content = null;
                try (BocloudACLock bLock = lockFactory.getACLock(subPath)) {
                    if (!lock.acquire(10, TimeUnit.SECONDS)) {
                        logger.warn("Get BocloudLock time out!");
                        resourceEventPublisher.send(new OperateResult(false, "当前有恢复任务正在执行", operate, null,
                            OperateCategory.ResourceEvent, context));
                        return;
                    }
                    SnapshotModel model = new SnapshotModel();
                    model.setSnapshotUuid(snap.getSnapshotUuid());
                    model.setVmName(server.getName());
                    Server host = serverRepository.query(server.getHostId());
                    if (null != host) {
                        model.setHostName(host.getName());
                    }
                    logger.info("start to revert cloud server");
                    // result = snapshotDriver.revert(vendor.getUuid(), null, model);
                    content = result.isSuccess() ? "恢复成功" : "恢复失败";

                } catch (Exception e) {
                    logger.error("revert snapshot fail:", e);
                    resourceEventPublisher
                        .send(new OperateResult(false, "恢复失败", operate, null, OperateCategory.ResourceEvent, context));
                } finally {
                    if (result.isSuccess()) {
                        MDSUtils.powerOn(imsCloudServerService, server);
                    }
                    resourceEventPublisher.send(new OperateResult(result.isSuccess(), content, operate, null,
                        OperateCategory.ResourceEvent, context));
                }
            });
            return new BocloudResult<>(true, "恢复任务已经下发，正在执行……");
        } catch (Exception e) {
            logger.error("revert snapshot fail:", e);
            return new BocloudResult<>(false, "恢复失败");
        }
    }

    @Override
    public BocloudResult securities(Long id, RequestContext context) {
        try {
            List<RdsNode> nodes = rdsNodeRepository.list(id);
            if (ListTool.isEmpty(nodes)) {
                return new BocloudResult<>(false, "数据不存在");
            }
            // String groupId = null;
            for (RdsNode node : nodes) {
                if ("master".equalsIgnoreCase(node.getRole())) {
                    // groupId = node.getGroupId();
                    break;
                }
            }
            List<Object> groups = new ArrayList<>();
            return new BocloudResult<>(true, groups, "查询成功");
        } catch (Exception e) {
            logger.error("list rds securities failure:", e);
            return new BocloudResult<>(false, "查询失败");
        }
    }

    @Override
    public BocloudResult<List<JSONObject>> parameters(Long id, RequestContext context) {
        try {
            Rds rds = rdsRepository.query(id);
            if (null == rds) {
                logger.warn("rds not exist");
                return new BocloudResult<>(false, "数据库不存在");
            }
            if ("STOPPED".equals(rds.getStatus())) {
                return new BocloudResult<>(false, "数据库不是运行状态");
            }
            BocloudResult result = null;
            if (ResourceCategory.PostgreSQL.name().equalsIgnoreCase(rds.getType())) {
                result = executePostgresql("show all;", rds, true, false, null);
            } else if (ResourceCategory.SqlServer.name().equalsIgnoreCase(rds.getType())) {
                result = executeSqlServer("sp_configure", rds, true);
            } else {
                if ("cluster".equals(rds.getCategory())) {
                    List<RdsNode> rdsNodes = rdsNodeRepository.list(id);
                    boolean checkCluster = MDSUtils.checkCluster(rdsNodes);
                    if (!checkCluster) {
                        return new BocloudResult<>(false, "集群结构不存在！请检查！");
                    }
                }
                String sql =
                    "show variables where variable_name LIKE 'automatic_%' or variable_name LIKE 'long_query_time%' or variable_name LIKE 'slow_query_log' or variable_name LIKE 'max_connections%' or variable_name LIKE 'auto_increment_%' or variable_name LIKE 'avoid_temporal_%' or variable_name LIKE 'binlog_cache_size%'"
                        + " or variable_name LIKE 'binlog_order_commits%' or variable_name LIKE 'binlog_row_image%' or variable_name LIKE 'binlog_rows_query_log_events%' or variable_name LIKE 'binlog_stmt_cache_size%'"
                        + " or variable_name LIKE 'bulk_insert_buffer_size%' or variable_name LIKE 'character_set_filesystem%' or variable_name LIKE 'concurrent_insert%';";
                result = execute(sql, rds, true, false);
            }
            if (result.isFailed()) {
                return result;
            }
            List<JSONObject> list = new ArrayList<>();
            List<JSONObject> params = JSONArray.parseArray(JSONObject.toJSONString(result.getData()), JSONObject.class);
            if (ResourceCategory.PostgreSQL.name().equalsIgnoreCase(rds.getType())) {
                for (JSONObject param : params) {
                    String name = param.getString("name");
                    if ("autovacuum_vacuum_cost_delay".equalsIgnoreCase(name)) {
                        param.put("default", "2ms");
                        param.put("reference", "-1|0-50ms");
                    } else if ("autovacuum_vacuum_cost_limit".equalsIgnoreCase(name)) {
                        param.put("default", "-1");
                        param.put("reference", "-1|1000-10000");
                    } else if ("autovacuum_vacuum_scale_factor".equalsIgnoreCase(name)) {
                        param.put("default", "0.2");
                        param.put("reference", "0.1-1");
                    } else if ("default_transaction_deferrable".equalsIgnoreCase(name)) {
                        param.put("default", "off");
                        param.put("reference", "off|on");
                    } else if ("enable_partitionwise_aggregate".equalsIgnoreCase(name)) {
                        param.put("default", "off");
                        param.put("reference", "off|on");
                    } else if ("enable_partitionwise_join".equalsIgnoreCase(name)) {
                        param.put("default", "off");
                        param.put("reference", "off|on");
                    } else if ("extra_float_digits".equalsIgnoreCase(name)) {
                        param.put("default", "3");
                        param.put("reference", "-15-3");
                    } else if ("idle_in_transaction_session_timeout".equalsIgnoreCase(name)) {
                        param.put("default", "0");
                        param.put("reference", "0|3000-36000000");
                    } else if ("jit".equalsIgnoreCase(name)) {
                        param.put("default", "on");
                        param.put("reference", "off|on");
                    } else if ("lock_timeout".equalsIgnoreCase(name)) {
                        param.put("default", "0");
                        param.put("reference", "0-36000000");
                    } else if ("log_connections".equalsIgnoreCase(name)) {
                        param.put("default", "off");
                        param.put("reference", "off|on");
                    } else if ("log_disconnections".equalsIgnoreCase(name)) {
                        param.put("default", "off");
                        param.put("reference", "off|on");
                    } else if ("log_duration".equalsIgnoreCase(name)) {
                        param.put("default", "off");
                        param.put("reference", "off|on");
                    } else if ("log_min_duration_statement".equalsIgnoreCase(name)) {
                        param.put("default", "-1");
                        param.put("reference", "-1|0|1000-600000");
                    } else if ("log_statement".equalsIgnoreCase(name)) {
                        param.put("default", "none");
                        param.put("reference", "none|ddl|mod|all");
                    } else if ("log_temp_files".equalsIgnoreCase(name)) {
                        param.put("default", "-1");
                        param.put("reference", "-1|0|4096-1048576");
                    } else if ("ssl".equalsIgnoreCase(name)) {
                        param.put("default", "off");
                        param.put("reference", "off|on");
                    } else if ("statement_timeout".equalsIgnoreCase(name)) {
                        param.put("default", "0");
                        param.put("reference", "0-86400000");
                    } else if ("synchronous_commit".equalsIgnoreCase(name)) {
                        param.put("default", "on");
                        param.put("reference", "off|on");
                    } else if ("vacuum_defer_cleanup_age".equalsIgnoreCase(name)) {
                        param.put("default", "0");
                        param.put("reference", "0-5000000");
                    } else if ("wal_keep_segments".equalsIgnoreCase(name)) {
                        param.put("default", "0");
                        param.put("reference", "0-1024000");
                    } else {
                        continue;
                    }
                    list.add(param);
                }
            } else if (ResourceCategory.SqlServer.name().equalsIgnoreCase(rds.getType())) {
                for (JSONObject param : params) {
                    String name = param.getString("name");
                    if (name.contains("allow polybase export")) {
                        param.put("default", "0");
                        param.put("reference", "0-1");
                    } else if (name.contains("allow updates")) {
                        param.put("default", "0");
                        param.put("reference", "0-1");
                    } else if (name.contains("backup checksum default")) {
                        param.put("default", "0");
                        param.put("reference", "0-1");
                    } else if (name.contains("backup compression default")) {
                        param.put("default", "0");
                        param.put("reference", "0-1");
                    } else if (name.contains("clr enabled")) {
                        param.put("default", "0");
                        param.put("reference", "0-1");
                    } else if (name.contains("column encryption enclave type")) {
                        param.put("default", "0");
                        param.put("reference", "0-2");
                    } else if (name.contains("contained database authentication")) {
                        param.put("default", "0");
                        param.put("reference", "0-1");
                    } else if (name.contains("cross db ownership chaining")) {
                        param.put("default", "0");
                        param.put("reference", "0-1");
                    } else if (name.contains("default language")) {
                        param.put("default", "30");
                        param.put("reference", "0-9999");
                    } else if (name.contains("hadoop connectivity")) {
                        param.put("default", "0");
                        param.put("reference", "0-7");
                    } else if (name.contains("max text repl size (B)")) {
                        param.put("default", "65536");
                        param.put("reference", "-1-2147483647");
                    } else if (name.contains("nested triggers")) {
                        param.put("default", "1");
                        param.put("reference", "0-1");
                    } else if (name.contains("polybase enabled")) {
                        param.put("default", "0");
                        param.put("reference", "0-1");
                    } else if (name.contains("polybase network encryption")) {
                        param.put("default", "1");
                        param.put("reference", "0-1");
                    } else if (name.contains("remote access")) {
                        param.put("default", "1");
                        param.put("reference", "0-1");
                    } else if (name.contains("remote admin connections")) {
                        param.put("default", "0");
                        param.put("reference", "0-1");
                    } else if (name.contains("remote data archive")) {
                        param.put("default", "0");
                        param.put("reference", "0-1");
                    } else if (name.contains("remote login timeout (s)")) {
                        param.put("default", "10");
                        param.put("reference", "0-2147483647");
                    } else if (name.contains("remote proc trans")) {
                        param.put("default", "0");
                        param.put("reference", "0-1");
                    } else if (name.contains("remote query timeout (s)")) {
                        param.put("default", "600");
                        param.put("reference", "0-2147483647");
                    } else if (name.contains("server trigger recursion")) {
                        param.put("default", "1");
                        param.put("reference", "0-1");
                    } else if (name.contains("show advanced options")) {
                        param.put("default", "0");
                        param.put("reference", "0-1");
                    } else if (name.contains("user options")) {
                        param.put("default", "0");
                        param.put("reference", "0-32767");
                    } else {
                        continue;
                    }
                    list.add(param);
                }
            } else {
                for (JSONObject param : params) {
                    String name = param.getString("name");
                    if (name.contains("auto_increment_")) {
                        param.put("default", "1");
                        param.put("reference", "1-65535");
                    } else if (name.contains("max_connections")) {
                        param.put("default", "100");
                        param.put("reference", "1-16384");
                    } else if (name.contains("long_query_time")) {
                        param.put("default", "10.000000");
                        param.put("reference", "1-16384");
                    } else if (name.contains("slow_query_log")) {
                        param.put("default", "OFF");
                        param.put("reference", "ON|OFF");
                    } else if (name.contains("automatic_sp_privileges") || name.contains("binlog_order_commits")) {
                        param.put("default", "ON");
                        param.put("reference", "ON|OFF");
                    } else if (name.contains("avoid_temporal_upgrade")
                        || name.contains("binlog_rows_query_log_events")) {
                        param.put("default", "OFF");
                        param.put("reference", "ON|OFF");
                    } else if (name.contains("binlog_cache_size")) {
                        param.put("default", "32768");
                        param.put("reference", "4096-16777216");
                    } else if (name.contains("binlog_row_image")) {
                        param.put("default", "FULL");
                        param.put("reference", "FULL|MINIMAL");
                    } else if (name.contains("binlog_stmt_cache_size")) {
                        param.put("default", "32768");
                        param.put("reference", "4096-16777216");
                    } else if (name.contains("bulk_insert_buffer_size")) {
                        param.put("default", "8388608");
                        param.put("reference", "0-4294967295");
                    } else if (name.contains("character_set_filesystem")) {
                        param.put("default", "binary");
                        param.put("reference", "utf8|gbk|binary");
                    } else if (name.contains("concurrent_insert")) {
                        param.put("default", "AUTO");
                        param.put("reference", "0|1|2");
                    } else {
                        continue;
                    }
                    list.add(param);
                }
            }
            return new BocloudResult<>(true, list, "查询数据库参数成功");
        } catch (Exception e) {
            return new BocloudResult<>(false, "查询数据库参数失败");
        }
    }

    private BocloudResult executeSqlServer(String sql, Rds rds, boolean isQuery) throws ClassNotFoundException {
        Class.forName("com.microsoft.sqlserver.jdbc.SQLServerDriver");
        try (Connection conn =
            DriverManager.getConnection("jdbc:sqlserver://" + rds.getManageIp() + ":1433;database=master",
                rds.getUsername(), MDSUtils.decrypt(rds.getPassword()))) {
            // 设置自动提交关闭
            conn.setAutoCommit(false);
            try (PreparedStatement pst = conn.prepareStatement(sql)) {
                if (isQuery) {
                    try (ResultSet rs = pst.executeQuery()) {
                        List<JSONObject> params = new ArrayList<JSONObject>();
                        while (rs.next()) {
                            JSONObject param = new JSONObject();
                            param.put("name", rs.getString("name"));
                            param.put("value", rs.getString("config_value"));
                            params.add(param);
                        }
                        conn.commit();
                        return new BocloudResult<>(true, params, "数据库操作成功");
                    } catch (SQLException e) {
                        logger.info("operate database error:", e);
                        return new BocloudResult<>(false, e.getMessage(), "操作数据库失败");
                    }
                }
                pst.execute();
                conn.commit();
                return new BocloudResult<>(true, "数据库操作成功");
            } catch (SQLException e) {
                logger.info("operate database error:", e);
                return new BocloudResult<>(false, e.getMessage(), "操作数据库失败");
            }
        } catch (Exception e) {
            logger.info("operate database error:", e);
            return new BocloudResult<>(false, e.getMessage(), "操作数据库失败");
        }
    }

    private BocloudResult executePostgresql(String sql, Rds rds, boolean isQuery, boolean isQuerySlave, String ip)
        throws ClassNotFoundException {
        Class.forName("org.postgresql.Driver");
        String address;
        if (null != ip) {
            address = ip + ":5432/postgres";
        } else {
            address = rds.getManageIp() + ":5432/postgres";
        }
        try (Connection conn = DriverManager.getConnection("jdbc:postgresql://" + address, rds.getUsername(),
            MDSUtils.decrypt(rds.getPassword())); PreparedStatement pst = conn.prepareStatement(sql)) {
            if (isQuery) {
                try (ResultSet rs = pst.executeQuery()) {
                    List<JSONObject> params = new ArrayList<JSONObject>();
                    while (rs.next()) {
                        JSONObject param = new JSONObject();
                        if (isQuerySlave) {
                            param.put("host", rs.getString("client_addr"));
                        } else {
                            param.put("name", rs.getString("name"));
                            param.put("value", rs.getString("setting"));
                        }
                        params.add(param);
                    }
                    return new BocloudResult<>(true, params, "数据库操作成功");
                } catch (SQLException e) {
                    logger.info("connect database error:", e);
                    return new BocloudResult<>(false, "数据库操作失败");
                }
            }
            pst.execute();
            try (PreparedStatement reloadPst = conn.prepareStatement("SELECT pg_reload_conf();")) {
                reloadPst.execute();
                return new BocloudResult<>(true, "数据库操作成功");
            } catch (SQLException e) {
                logger.info("connect database error:", e);
                return new BocloudResult<>(false, "数据库操作失败");
            }
        } catch (Exception e) {
            logger.info("connect database error:", e);
            return new BocloudResult<>(false, "数据库操作失败");
        }
    }

    private BocloudResult setParams(String sql, Rds rds) {
        try {
            List<String> ips = new ArrayList<String>();
            List<RdsNode> rdsNodes = rdsNodeRepository.list(rds.getId());
            for (RdsNode rdsNode : rdsNodes) {
                ips.add(rdsNode.getManageIp() + ":3306");
            }
            for (String address : ips) {
                BocloudResult result = setParam(sql, address, rds);
                if (result.isFailed()) {
                    return new BocloudResult<>(false, "数据库操作失败");
                }
            }
            return new BocloudResult<>(true, "数据库操作成功");
        } catch (Exception e) {
            logger.info("connect database error:", e);
            return new BocloudResult<>(false, "数据库操作失败");
        }
    }

    private BocloudResult setParam(String sql, String address, Rds rds) throws ClassNotFoundException {
        Class.forName("com.mysql.cj.jdbc.Driver");
        try (Connection conn = DriverManager.getConnection(
            "jdbc:mysql://" + address + "/mysql?characterEncoding=utf-8&serverTimezone=UTC&useSSL=false",
            rds.getUsername(), MDSUtils.decrypt(rds.getPassword()))) {
            // 设置自动提交关闭
            conn.setAutoCommit(false);
            try (PreparedStatement pst = conn.prepareStatement(sql)) {
                pst.execute();
                conn.commit();
                return new BocloudResult<>(true, "数据库操作成功");
            } catch (SQLException e) {
                logger.info("connect database error:", e);
                return new BocloudResult<>(false, "数据库操作失败");
            }
        } catch (Exception e) {
            logger.info("connect database error:", e);
            return new BocloudResult<>(false, "数据库操作失败");
        }
    }

    private BocloudResult execute(String sql, Rds rds, boolean isQuery, boolean isQuerySlave)
        throws ClassNotFoundException {
        Class.forName("com.mysql.cj.jdbc.Driver");
        try {
            StringBuilder ips = new StringBuilder();
            if ("standalone".equals(rds.getCategory())) {
                ips.append(rds.getManageIp()).append(":3306");
            } else {
                List<RdsNode> rdsNodes = rdsNodeRepository.list(rds.getId());
                List<RdsNode> masterNodes = rdsNodes.stream().filter(rdsNode -> "master".equals(rdsNode.getRole()))
                    .collect(Collectors.toList());
                if (0 == masterNodes.size()) {
                    return new BocloudResult<>(true, new ArrayList<JSONObject>(), "集群结构不存在！");
                }
                for (int i = 1; i <= rdsNodes.size(); i++) {
                    RdsNode rdsNode = rdsNodes.get(i - 1);
                    if (!"standalone".equals(rdsNode.getRole())) {
                        ips.append(rdsNode.getManageIp()).append(":3306");
                        if (i != rdsNodes.size()) {
                            ips.append(",");
                        }
                    }
                }
            }
            try (Connection conn = DriverManager.getConnection(
                "jdbc:mysql://" + ips + "/mysql?characterEncoding=utf-8&serverTimezone=UTC&useSSL=false",
                rds.getUsername(), MDSUtils.decrypt(rds.getPassword()))) {
                // 设置自动提交关闭
                conn.setAutoCommit(false);
                try (PreparedStatement pst = conn.prepareStatement(sql)) {
                    if (isQuery) {
                        try (ResultSet rs = pst.executeQuery()) {
                            List<JSONObject> params = new ArrayList<JSONObject>();
                            while (rs.next()) {
                                JSONObject param = new JSONObject();
                                if (isQuerySlave) {
                                    if (StringUtils.isNotEmpty(rds.getProps())
                                        && rds.getProps().toLowerCase().contains("mgr")) {
                                        param.put("host", rs.getString("MEMBER_HOST"));
                                    } else {
                                        param.put("host", rs.getString("HOST"));
                                    }
                                } else {
                                    param.put("name", rs.getString("Variable_name"));
                                    param.put("value", rs.getString("Value"));
                                }
                                params.add(param);
                            }
                            conn.commit();
                            return new BocloudResult<>(true, params, "数据库操作成功");
                        } catch (SQLException e) {
                            logger.info("connect database error:", e);
                            return new BocloudResult<>(false, "数据库操作失败");
                        }
                    }
                    pst.execute();
                    conn.commit();
                    return new BocloudResult<>(true, "数据库操作成功");
                } catch (SQLException e) {
                    logger.info("connect database error:", e);
                    return new BocloudResult<>(false, "数据库操作失败");
                }
            } catch (Exception e) {
                logger.info("connect database error:", e);
                return new BocloudResult<>(false, "数据库操作失败");
            }
        } catch (Exception e) {
            logger.info("connect database error:", e);
            return new BocloudResult<>(false, "数据库操作失败");
        }
    }

    @Override
    public BocloudResult setting(String name, String oldValue, String value, Long id, Object number,
        RequestContext context) {
        RdsParamLog log = new RdsParamLog(name, oldValue, value, new Date(), id, context.getTarget(), false);
        try {
            Rds rds = rdsRepository.query(id);
            if (null == rds) {
                logger.warn("rds not exist");
                return new BocloudResult<>(false, "数据库不存在");
            }
            if ("STOPPED".equals(rds.getStatus())) {
                return new BocloudResult<>(false, "数据库不是运行状态");
            }
            BocloudResult result = null;
            if (ResourceCategory.PostgreSQL.name().equalsIgnoreCase(rds.getType())) {
                String sql = "alter system set " + name + " = '" + value + "'";
                result = executePostgresql(sql, rds, false, false, null);
            } else if (ResourceCategory.SqlServer.name().equalsIgnoreCase(rds.getType())) {
                String sql = "sp_configure '" + name + "'," + Long.valueOf(value) + ";";
                result = executeSqlServer(sql, rds, false);
            } else {
                if ("cluster".equals(rds.getCategory())) {
                    List<RdsNode> rdsNodes = rdsNodeRepository.list(id);
                    boolean checkCluster = MDSUtils.checkCluster(rdsNodes);
                    if (!checkCluster) {
                        return new BocloudResult<>(false, "集群结构不存在！请检查！");
                    }
                }
                String sql = "set global " + name + " = " + value + ";";
                result = setParams(sql, rds);
            }
            log.setEffective(result.isSuccess());
            save(log);
            if (result.isFailed()) {
                return new BocloudResult<>(false, "设置数据库参数失败");
            }
            return new BocloudResult<>(true, "设置数据库参数成功");
        } catch (Exception e) {
            save(log);
            return new BocloudResult<>(false, "设置数据库参数失败");
        }
    }

    @Override
    public BocloudResult<GridBean<RdsParamLog>> paramLogs(Pager pager, RequestContext context) {
        try {
            List<Param> params = pager.getParams();
            Integer page = pager.getPage();
            Integer rows = pager.getRows();
            Map<String, String> sorter = pager.getSorter();
            if (null == params) {
                params = new ArrayList<>();
            }
            if (null == sorter) {
                sorter = Maps.newHashMap();
            }
            sorter.put("gmtCreate", Common.ONE);
            List<RdsParamLog> logs = rdsParamLogRepository.list(page, rows, params, sorter);
            int total = rdsParamLogRepository.count(params);
            GridBean gridBean = GridHelper.getBean(page, rows, total, logs);
            return new BocloudResult<>(true, gridBean, "查询成功");
        } catch (Exception e) {
            logger.error("list rds parameter logs failure:", e);
            return new BocloudResult<>(false, "查询失败");
        }
    }

    private void save(RdsParamLog log) {
        try {
            rdsParamLogRepository.save(log);
        } catch (Exception e) {
            logger.error("save rds param log failure:", e);
        }
    }

    @Override
    public void saveLog(RdsLog log) {
        try {
            rdsLogRepository.save(log);
        } catch (Exception e) {
            logger.error("save rds log failure:", e);
        }
    }

    @Override
    public void synchronousRoleState(JSONObject content) {
        try {
            Long id = content.getLong("id");
            Rds rds = rdsRepository.query(id);
            List<RdsNode> nodes = rdsNodeRepository.list(id);
            if (ResourceCategory.MySQL.name().equalsIgnoreCase(rds.getType())) {
                syncMysqlRole(nodes, rds);
            } else if (ResourceCategory.SqlServer.name().equalsIgnoreCase(rds.getType())) {
                synMssqlRole(nodes, rds);
            } else if (ResourceCategory.PostgreSQL.name().equalsIgnoreCase(rds.getType())) {
                synPostgreSqlRole(nodes, rds);
            }
            RequestContext context = new RequestContext();
            context.setTenant(rds.getTenantId());
            context.setTarget(rds.getTenantId());
            context.setCatalog(Catalog.valueOf(rds.getGmtCatalog()));
            resourceEventPublisher
                .send(new OperateResult(true, "节点角色状态变更通知！", "node_change_" + rds.getType().toLowerCase(),
                    Maps.newHashMap(), OperateCategory.ServiceEvent, context));
        } catch (Exception e) {
            logger.error("集群角色同步失败：", e.getMessage());
        }
    }

    @Override
    public BocloudResult removeBackup(Long snapshotId, RequestContext context) {
        String path = Snapshot.class.getSimpleName() + "_" + snapshotId;
        try (BocloudACLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            Snapshot snap = snapshotRepository.query(snapshotId);
            if (null == snap) {
                logger.warn("snapshot data not exist");
                return new BocloudResult<>(false, "数据不存在");
            }
            CloudServer server = cloudServerRepository.query(snap.getVmId());
            if (null == server) {
                logger.warn("cloud server data not exist");
                return new BocloudResult<>(false, "删除数据库恢复失败");
            }
            // 云供应商
            CloudVendor vendor = cloudVendorRepository.query(snap.getVendorId());
            if (null == vendor) {
                return new BocloudResult<>(false, "数据库备份删除失败");
            }
            CloudProvider type = CloudProvider.valueOf(vendor.getType());
            // 启动一个线程从MQ上获取数据
            threadPoolTaskExecutor.submit(() -> {
                String operate = "rds.backup";
                BocloudResult result = new BocloudResult<>(false, "数据库备份删除失败");
                try {
                    SnapshotModel model = new SnapshotModel();
                    model.setSnapshotUuid(snap.getSnapshotUuid());
                    model.setVmName(server.getName());
                    // switch (type) {
                    // case VMWARE:
                    // result = snapshotDriver.remove(vendor.getUuid(), null, model);
                    // break;
                    // default:
                    // break;
                    // }
                    String content = result.getMessage();
                    if (result.isSuccess()) {
                        snapshotRepository.remove(snapshotId, context.getTarget());
                        content = "数据库备份删除成功";
                    }
                    resourceEventPublisher.send(new OperateResult(result.isSuccess(), content, operate, null,
                        OperateCategory.ResourceEvent, context));
                } catch (Exception e) {
                    logger.error("Get Message Exception:", e);
                    resourceEventPublisher.send(new OperateResult(false, result.getMessage(), operate, null,
                        OperateCategory.ResourceEvent, context));
                }
            });
            return new BocloudResult<>(true, "删除任务已经下发，正在执行……");
        } catch (Exception e) {
            logger.error("remove snapshot fail:", e);
            return new BocloudResult<>(false, "删除失败");
        }
    }

    @Override
    public void install(String params, RequestContext context) {
        TaskNode node = JSONObject.parseObject(params, TaskNode.class);
        JSONObject output = JSONObject.parseObject(node.getOutput());
        if (null == output) {
            return;
        }
        List<JSONObject> hosts = JSONObject.parseArray(output.getString("data"), JSONObject.class);
        if (ListTool.isEmpty(hosts)) {
            return;
        }
        JSONObject input = JSONObject.parseObject(node.getParams());
        if (null == input) {
            return;
        }
        Integer port = null;
        ResourceCategory category = null;
        JSONObject fronts = JSONObject.parseObject(input.getString("fronts"));
        String version = fronts.getString("version");
        JSONObject config = fronts.getJSONObject("software_config");
        String username = "zeus";
        String password = "Zeus@v587".trim();
        switch (node.getCode()) {
            case "task.software.mysql":
                category = ResourceCategory.MySQL;
                port = config.getInteger("mysql_port");
                port = null == port ? 3306 : port;
                break;
            case "task.software.oracle":
                category = ResourceCategory.Oracle;
                port = config.getInteger("listener_port");
                port = null == port ? 1521 : port;
                break;
            case "task.software.sqlserver":
                category = ResourceCategory.SqlServer;
                port = config.getInteger("mssql_port");
                port = null == port ? 1433 : port;
                username = "sa";
                try {
                    password = MDSUtils.decrypt(config.getString("mssql_sa_pass"));
                } catch (Exception e) {
                }
                break;
            case "task.software.postgresql":
                category = ResourceCategory.PostgreSQL;
                port = config.getInteger("postgresql_port");
                port = null == port ? 5432 : port;
                break;
            default:
                return;
        }
        try {
            password = MDSUtils.encrypt(password);
            if (null != fronts.getBoolean("cluster_install") && fronts.getBoolean("cluster_install")) {
                if (Status.Node.FAILED.name().equals(node.getStatus())) {
                    return;
                }
                Rds rds = new Rds();
                rds.setName(category.name() + System.currentTimeMillis());
                rds.setStatus(BocloudStatus.Server.RUNNING.name());
                rds.setCreatorId(context.getTarget());
                rds.setOwnerId(context.getTarget());
                rds.setTenantId(context.getTenant());
                rds.setInstanceId(IDFactory.instance().uuid());
                rds.setType(category.name());
                rds.setGmtCatalog(context.getCatalog().name());
                rds.setVersion(version);
                rds.setCategory(hosts.size() > 1 ? "cluster" : "standalone");
                rds.setUsername(username);
                rds.setPassword(password);
                rds.setTaskId(node.getInstanceId());
                rds.setProps(fronts.getString("cluster_type"));
                rdsRepository.save(rds);
                List<JSONObject> groups = JSONObject.parseArray(fronts.getString("groups"), JSONObject.class);
                for (JSONObject host : hosts) {
                    String role = "standalone";
                    if (!ListTool.isEmpty(groups)) {
                        for (JSONObject group : groups) {
                            if (group.getString("hosts").contains(host.getString("host"))) {
                                if (group.getString("name").contains("slave")) {
                                    role = "slave";
                                } else if (group.getString("name").contains("master")) {
                                    role = "master";
                                } else {
                                    if (ResourceCategory.PostgreSQL == category) {
                                        role = hosts.indexOf(host) == 0 ? "master" : "slave";
                                    }
                                }
                                break;
                            }
                        }
                    }
                    RdsNode rdsNode = new RdsNode(host.getString("host"), port, role,
                        BocloudStatus.Server.RUNNING.name(), null, rds.getId());
                    rdsNode.setManageIp(host.getString("host"));
                    if ("CloudServer".equals(host.getString("catalog"))) {
                        rdsNode.setServerId(host.getLong("id"));
                    } else {
                        Resource resource = resourceRepository.query(host.getLong("id"));
                        if (null != resource) {
                            rdsNode.setServerId(resource.getTarget());
                        }
                    }
                    if ("master".equals(role) || "standalone".equals(role)) {
                        rds.setManageIp(host.getString("host"));
                        rds.setAddress(host.getString("host") + ":" + port);
                    }
                    rdsNodeRepository.save(rdsNode);
                }
                if (ResourceCategory.PostgreSQL == category) {
                    JSONObject clusterConfig = fronts.getJSONObject("cluster_config");
                    if (null == clusterConfig) {
                        return;
                    }
                    String vip = clusterConfig.getString("cluster_vip");
                    if (!StringUtils.isEmpty(vip)) {
                        rds.setAddress(vip + ":" + 5000);
                    }
                } else if (ResourceCategory.MySQL == category) {
                    String vip = fronts.getString("mysql_vip");
                    if (!StringUtils.isEmpty(vip)) {
                        rds.setAddress(vip + ":" + port);
                    }
                }
                rdsRepository.update(rds);
                saveUserAndDb(fronts, category, rds);
                MDSUtils.send(amqpTemplate, rds.getId(), rds.getType(), "create");
                return;
            }
            JSONObject target = JSONObject.parseObject(input.getString("target"));
            List<JSONObject> datas = JSONObject.parseArray(target.getString("data"), JSONObject.class);
            if (ListTool.isEmpty(datas)) {
                return;
            }
            Map<String, JSONObject> hostMap = new HashMap<>();
            for (JSONObject host : hosts) {
                if (host.getBoolean("success")) {
                    hostMap.put(host.getString("host"), host);
                }
            }
            for (JSONObject data : datas) {
                if (!hostMap.containsKey(data.getString("host"))) {
                    continue;
                }
                Rds rds = new Rds();
                rds.setName(category.name() + System.currentTimeMillis());
                rds.setStatus(BocloudStatus.Server.RUNNING.name());
                rds.setCreatorId(context.getTarget());
                rds.setOwnerId(context.getTarget());
                rds.setTenantId(context.getTenant());
                rds.setInstanceId(IDFactory.instance().uuid());
                rds.setType(category.name());
                rds.setGmtCatalog(context.getCatalog().name());
                rds.setVersion(version);
                rds.setCategory("standalone");
                rds.setManageIp(data.getString("host"));
                rds.setAddress(data.getString("host") + ":" + port);
                rds.setUsername(username);
                rds.setPassword(password);
                rds.setTaskId(node.getInstanceId());
                rdsRepository.save(rds);
                RdsNode rdsNode = new RdsNode(data.getString("host"), port, "standalone",
                    BocloudStatus.Server.RUNNING.name(), null, rds.getId());
                rdsNode.setManageIp(data.getString("host"));
                if ("CloudServer".equals(data.getString("catalog"))) {
                    rdsNode.setServerId(data.getLong("id"));
                } else {
                    Resource resource = resourceRepository.query(data.getLong("id"));
                    if (null != resource) {
                        rdsNode.setServerId(resource.getTarget());
                    }
                }
                rdsNodeRepository.save(rdsNode);
                saveUserAndDb(fronts, category, rds);
                MDSUtils.send(amqpTemplate, rds.getId(), rds.getType(), "create");
            }
        } catch (Exception e) {
            logger.error("save rds softwart install failure:", e);
        }
    }

    @Override
    public BocloudResult recycle(Long id, RequestContext context) {
        try {
            Rds rds = rdsRepository.query(id);
            // List<RdsNode> nodes = rdsNodeRepository.list(id);
            // List<Long> serverIds = Lists.newArrayList();
            // for (RdsNode node : nodes) {
            // CloudServer server = cloudServerRepository.get(node.getInstanceId());
            // Optional.ofNullable(server).orElseThrow(() -> new Exception("云主机不存在"));
            // CloudVendor vendor = cloudVendorRepository.query(server.getVendorId());
            // Optional.ofNullable(vendor).orElseThrow(() -> new Exception("云平台不存在"));
            // serverIds.add(node.getServerId());
            // }
            // BocloudResult batchOperations = cloudServerService.batchOperations(String.valueOf(serverIds), "Stop",
            // context);
            // if (batchOperations.isFailed()) {
            // return new BocloudResult<>(false, "数据库关机失败！");
            // }
            // 更新实例状态
            rdsRepository.recycle(id, context.getTarget());
            // 回收站生成记录
            JSONObject specs = JSONObject.parseObject(rds.getSpec());
            String spec = null;
            if (null != specs) {
                spec = specs.get("cpu") + "核/" + specs.get("memory") + "GB/" + specs.get("disk") + "GB";
            }
            Recycled recycled =
                new Recycled(rds.getName(), rds.getUuid(), rds.getType(), rds.getId(), null, rds.getGmtCatalog(), spec);
            if (Catalog.Tenant == Catalog.valueOf(rds.getGmtCatalog())) {
                recycled.setTenantId(rds.getTenantId());
            } else if (Catalog.User == Catalog.valueOf(rds.getGmtCatalog())) {
                recycled.setTenantId(rds.getTenantId());
                recycled.setUserId(rds.getOwnerId());
                recycled.setOwnerId(rds.getOwnerId());
            }
            BocloudResult joinRecycle = cosInternalService.joinRecycle(recycled);
            logger.info("生成回收站记录结果：{}", JSONObject.toJSON(joinRecycle));
            // 处理订单
            cosInternalService.updateItemStatus(rds.getUuid(), OrderItemStatus.Termination.name());
            // 处理账单
            BocloudResult result = cosInternalService.recycleBill(rds.getUuid());
            logger.info("放入回收站生产账单结果：{}", JSONObject.toJSON(result));
            RequestContext requestContext = new RequestContext(Catalog.valueOf(rds.getGmtCatalog()), rds.getTenantId(),
                rds.getCreatorId(), null, null);
            // 发送站内信
            NotifyMessage notifyMessage = new NotifyMessage(requestContext, rds.getType() + "实例放入回收站通知",
                rds.getType() + "【" + rds.getName() + "】已删除放入回收站，30天后将被彻底删除", true, false);
            amqpTemplate.convertAndSend("event.notice.queue", notifyMessage);
            return new BocloudResult<>(true, "数据库【" + rds.getName() + "】删除成功，可在回收站查看");
        } catch (Exception e) {
            logger.error("remove rds fail:", e);
            return new BocloudResult<>(false, "移入回收车失败");
        }
    }

    @Override
    public BocloudResult restore(Long id, RequestContext context) {
        try {
            Rds rds = rdsRepository.query(id);
            // List<RdsNode> nodes = rdsNodeRepository.list(id);
            // List<Long> serverIds = Lists.newArrayList();
            // for (RdsNode node : nodes) {
            // CloudServer server = cloudServerRepository.get(node.getInstanceId());
            // Optional.ofNullable(server).orElseThrow(() -> new Exception("云主机不存在"));
            // CloudVendor vendor = cloudVendorRepository.query(server.getVendorId());
            // Optional.ofNullable(vendor).orElseThrow(() -> new Exception("云平台不存在"));
            // serverIds.add(node.getServerId());
            // }
            // BocloudResult batchOperations = cloudServerService.batchOperations(String.valueOf(serverIds), "Start",
            // context);
            // if (batchOperations.isFailed()) {
            // return new BocloudResult<>(false, "数据库开机失败！");
            // }
            // 更新实例状态
            rdsRepository.restore(id, context.getTarget());
            // 处理订单
            cosInternalService.updateItemStatus(rds.getUuid(), OrderItemStatus.OpenedSuccess.name());
            return new BocloudResult<>(true, rds.getType() + "【" + rds.getName() + "】还原成功");
        } catch (Exception e) {
            logger.error("restore rds fail:", e);
            return new BocloudResult<>(false, "数据库还原失败");
        }
    }

    private void saveUserAndDb(JSONObject fronts, ResourceCategory category, Rds rds) throws Exception {
        if (ResourceCategory.MySQL.equals(category)) {
            if (fronts.containsKey("mysql_databases")) {
                List<JSONObject> databases =
                    JSONArray.parseArray(fronts.getString("mysql_databases"), JSONObject.class);
                if (!ListTool.isEmpty(databases)) {
                    for (JSONObject database : databases) {
                        RdsDb db =
                            new RdsDb(database.getString("encoding"), database.getString("collation"), rds.getId());
                        db.setName(database.getString("name"));
                        db.setCreatorId(rds.getCreatorId());
                        db.setGmtCreate(new Date());
                        db.setStatus(BocloudStatus.Basic.NORMAL.name());
                        rdsDbRepository.save(db);
                    }
                }
            }
            if (fronts.containsKey("mysql_users")) {
                List<JSONObject> users = JSONArray.parseArray(fronts.getString("mysql_users"), JSONObject.class);
                if (!ListTool.isEmpty(users)) {
                    for (JSONObject user : users) {
                        RdsUser dbUser = new RdsUser(user.getString("name"), user.getString("password"),
                            user.getString("host"), rds.getId());
                        rdsUserRepository.save(dbUser);
                    }
                }
            }
        } else if (ResourceCategory.PostgreSQL.equals(category)) {
            if (fronts.containsKey("postgresql_databases")) {
                List<JSONObject> databases =
                    JSONArray.parseArray(fronts.getString("postgresql_databases"), JSONObject.class);
                if (!ListTool.isEmpty(databases)) {
                    for (JSONObject database : databases) {
                        RdsDb db =
                            new RdsDb(database.getString("encoding"), database.getString("lc_collate"), rds.getId());
                        db.setCollctype(database.getString("lc_ctype"));
                        db.setName(database.getString("name"));
                        db.setCreatorId(rds.getCreatorId());
                        db.setGmtCreate(new Date());
                        db.setStatus(BocloudStatus.Basic.NORMAL.name());
                        rdsDbRepository.save(db);
                    }
                }
            }

            if (fronts.containsKey("postgresql_users")) {
                List<JSONObject> users = JSONArray.parseArray(fronts.getString("postgresql_users"), JSONObject.class);
                if (!ListTool.isEmpty(users)) {
                    List<JSONObject> privileges =
                        JSONArray.parseArray(fronts.getString("postgresql_privileges"), JSONObject.class);
                    for (JSONObject user : users) {
                        RdsUser dbUser = new RdsUser(user.getString("name"), user.getString("pass"), null, rds.getId());
                        for (JSONObject privilege : privileges) {
                            if (privilege.getString("name").equals(user.getString("name"))
                                && "superuser".equalsIgnoreCase(privilege.getString("role_attr_flags"))) {
                                dbUser.setPermission("all");
                                break;
                            }
                        }
                        rdsUserRepository.save(dbUser);
                    }
                }
            }
        }
    }

}
