package com.bocloud.pms.service;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.concurrent.TimeUnit;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.encrypt.AESEncrptor;
import com.bocloud.common.encrypt.DESEncrptor;
import com.bocloud.common.encrypt.Encryptor;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.GridBean;
import com.bocloud.common.model.Pager;
import com.bocloud.common.model.RequestContext;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.GridHelper;
import com.bocloud.common.utils.ListTool;
import com.bocloud.pms.entity.Rds;
import com.bocloud.pms.entity.RdsDb;
import com.bocloud.pms.entity.RdsNode;
import com.bocloud.pms.entity.RdsUser;
import com.bocloud.pms.entity.resource.CloudServer;
import com.bocloud.pms.entity.resource.enums.BocloudStatus;
import com.bocloud.pms.entity.resource.enums.ResourceCategory;
import com.bocloud.pms.interfaces.middleware.RdsUserService;
import com.bocloud.pms.repository.RdsDbRepository;
import com.bocloud.pms.repository.RdsNodeRepository;
import com.bocloud.pms.repository.RdsRepository;
import com.bocloud.pms.repository.RdsUserRepository;
import com.bocloud.pms.repository.resource.CloudServerRepository;
import com.bocloud.pms.service.internal.CopInternalService;
import com.bocloud.pms.service.utils.MDSUtils;
import com.bocloud.service.lock.BocloudACLock;
import com.bocloud.service.lock.LockFactory;

@Service("rdsUserService")
public class RdsUserServiceImpl implements RdsUserService {

    private static Logger logger = LoggerFactory.getLogger(RdsUserServiceImpl.class);
    @Autowired
    private RdsUserRepository rdsUserRepository;
    @Autowired
    private RdsRepository rdsRepository;
    @Autowired
    private RdsDbRepository rdsDbRepository;
    @Autowired
    private LockFactory lockFactory;
    @Autowired
    private RdsNodeRepository rdsNodeRepository;
    @Autowired
    private CloudServerRepository cloudServerRepository;
    @Autowired
    private CopInternalService copInternalService;

    @Override
    public BocloudResult<GridBean<RdsUser>> list(Pager pager, RequestContext context) {
        try {
            pager.setParams(Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>()));
            pager.setSorter(Optional.ofNullable(pager.getSorter()).orElse(new HashMap<>()));
            pager.getSorter().put("gmtCreate", Common.ONE);
            List<RdsUser> list = rdsUserRepository.list(pager);
            int total = rdsUserRepository.count(pager.getParams());
            GridBean bean = GridHelper.getBean(pager.getPage(), pager.getRows(), total, list);
            return new BocloudResult<>(true, bean, "查询成功");
        } catch (Exception e) {
            logger.error("list rds failure:", e);
            return new BocloudResult<>(false, "查询失败");
        }
    }

    @Override
    public BocloudResult create(RdsUser rdsUser, RequestContext context) {
        Rds rds = null;
        try {
            rds = rdsRepository.query(rdsUser.getRdsId());
            if (null == rds) {
                logger.warn("rds not exist");
                return new BocloudResult<>(false, "数据库不存在");
            }
            String password = MDSUtils.decrypt(rdsUser.getPassword());
            BocloudResult result = null;
            if (ResourceCategory.PostgreSQL.name().equalsIgnoreCase(rds.getType())) {
                List<JSONObject> targets = null;
                if ("cluster".equals(rds.getCategory())) {
                    targets = getTargets(rds.getId());
                }
                String sql = "create user " + rdsUser.getUsername() + " with password '" + password + "';";
                if (ListTool.isEmpty(targets)) {
                    result = executePostgresql(sql, rds, null);
                } else {
                    result = copInternalService.execute(getParams(targets, rdsUser.getUsername(), password, false));
                    logger.info("PostgreSQL新增用户调用worker结果：" + JSONObject.toJSONString(result));
                    if (result.isSuccess()) {
                        result = executePostgresql(sql, rds, null);
                    }
                }
            } else if (ResourceCategory.SqlServer.name().equalsIgnoreCase(rds.getType())) {
                String sql = "create login " + rdsUser.getUsername() + " with password = '" + password + "';";
                result = executeSqlServer(sql, rds, null);
            } else {
                if ("cluster".equals(rds.getCategory())) {
                    List<RdsNode> rdsNodes = rdsNodeRepository.list(rds.getId());
                    boolean checkCluster = MDSUtils.checkCluster(rdsNodes);
                    if (!checkCluster) {
                        return new BocloudResult<>(false, "集群结构不存在！请检查！");
                    }
                }
                String sql = "create user '" + rdsUser.getUsername() + "'@'" + rdsUser.getHost() + "' identified by '"
                    + password + "';";
                result = execute(sql, rds);
            }
            if (result.isFailed()) {
                return new BocloudResult<>(false, rds.getId(), "创建数据库用户失败");
            }

            rdsUser.setStatus(BocloudStatus.Basic.NORMAL.name());
            rdsUser.setCreatorId(context.getTarget());
            rdsUser.setGmtCreate(new Date());
            rdsUser.setPassword(rdsUser.getPassword());
            rdsUserRepository.save(rdsUser);
            return new BocloudResult<>(true, rds.getId(), "创建数据库用户成功");
        } catch (Exception e) {
            logger.error("create rds user failure:", e);
            return new BocloudResult<>(false, null == rds ? null : rds.getId(), "创建数据库用户失败");
        }
    }

    private String getPswd(String password) {
        Encryptor encryptor = new DESEncrptor();
        if (!StringUtils.isEmpty(password)) {
            try {
                return new AESEncrptor().encrypt(encryptor.decrypt(password, "BocloudCMP"), null);
            } catch (Exception e) {
                logger.error("Get error message：{}", e.getMessage());
            }
        }
        return password;
    }

    private List<JSONObject> getTargets(Long rdsId) throws Exception {
        List<JSONObject> targets = new ArrayList<>();
        List<RdsNode> nodes = rdsNodeRepository.list(rdsId);
        for (RdsNode node : nodes) {
            CloudServer server = cloudServerRepository.getByInstanceId(node.getInstanceId());
            if (null == server || !"RUNNING".equalsIgnoreCase(server.getStatus())) {
                continue;
            }
            JSONObject target = new JSONObject();
            target.put("category", "Linux");
            target.put("host", node.getAddress());
            target.put("user", "root");
            target.put("pasd", getPswd(server.getPassword()));
            target.put("sudo", true);
            target.put("port", 22);
            targets.add(target);
        }
        return targets;
    }

    private JSONObject getParams(List<JSONObject> targets, String username, String password, boolean remove) {
        JSONObject module = new JSONObject();
        module.put("name", "lineinfile");
        JSONObject args = new JSONObject();
        args.put("path", "/etc/pgbouncer/userlist.txt");
        args.put("regexp", "^\"" + username + "\"");
        args.put("line", "\"" + username + "\" \"" + password + "\"");
        args.put("state", remove ? "absent" : "present");
        module.put("args", args);

        JSONObject params = new JSONObject();
        params.put("module", module);
        params.put("targets", targets);
        params.put("sync", true);
        return params;
    }

    @Override
    public BocloudResult remove(Long id, RequestContext context) {
        String path = RdsUser.class.getSimpleName() + "_" + id;
        Rds rds = null;
        try (BocloudACLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            RdsUser rdsUser = rdsUserRepository.query(id);
            if (null == rdsUser) {
                logger.warn("rds user not exist");
                return new BocloudResult<>(false, "数据库用户不存在");
            }
            rds = rdsRepository.query(rdsUser.getRdsId());
            if (null == rds) {
                logger.warn("rds not exist");
                return new BocloudResult<>(false, "数据库不存在");
            }
            BocloudResult result = null;
            if (ResourceCategory.PostgreSQL.name().equalsIgnoreCase(rds.getType())) {
                List<JSONObject> targets = null;
                if ("cluster".equals(rds.getCategory())) {
                    targets = getTargets(rds.getId());
                }
                if (!StringUtils.isEmpty(rdsUser.getDbs())) {
                    String[] dbIds = rdsUser.getDbs().split(",");
                    List<RdsDb> dbs = new ArrayList<>();
                    for (String dbId : dbIds) {
                        RdsDb rdsDb = rdsDbRepository.query(Long.valueOf(dbId));
                        if (null == rdsDb) {
                            continue;
                        }
                        dbs.add(rdsDb);
                    }
                    for (RdsDb db : dbs) {
                        String sql = "revoke all on all tables in schema public from " + rdsUser.getUsername() + ";";
                        result = executePostgresql(sql, rds, db.getName());
                        if (result.isFailed()) {
                            return new BocloudResult<>(false, rds.getId(), "用户授权失败");
                        }
                    }
                }
                BocloudResult executePostgresql =
                    executePostgresql("REASSIGN OWNED BY " + rdsUser.getUsername() + " TO postgres;", rds, null);
                if (executePostgresql.isFailed()) {
                    return new BocloudResult<>(false, rds.getId(), "权限转让失败");
                }
                List<RdsDb> rdsDbs = rdsDbRepository.list(rds.getId());
                if (!rdsDbs.isEmpty()) {
                    for (RdsDb db : rdsDbs) {
                        BocloudResult bocloudResult =
                            executePostgresql("DROP OWNED BY " + rdsUser.getUsername() + ";", rds, db.getName());
                        if (bocloudResult.isFailed()) {
                            return new BocloudResult<>(false, rds.getId(), "回收权限失败");
                        }
                    }
                }
                String sql = "drop user if exists " + rdsUser.getUsername() + ";";
                result = executePostgresql(sql, rds, null);
                if (result.isSuccess() && !ListTool.isEmpty(targets)) {
                    copInternalService.execute(
                        getParams(targets, rdsUser.getUsername(), MDSUtils.decrypt(rdsUser.getPassword()), false));
                }
            } else if (ResourceCategory.SqlServer.name().equalsIgnoreCase(rds.getType())) {
                if (!StringUtils.isEmpty(rdsUser.getDbs())) {
                    String[] dbIds = rdsUser.getDbs().split(",");
                    List<RdsDb> dbs = new ArrayList<>();
                    for (String dbId : dbIds) {
                        RdsDb rdsDb = rdsDbRepository.query(Long.valueOf(dbId));
                        if (null == rdsDb) {
                            continue;
                        }
                        dbs.add(rdsDb);
                    }
                    for (RdsDb db : dbs) {
                        String sql = "drop user if exists " + rdsUser.getUsername() + ";";
                        result = executeSqlServer(sql, rds, db.getName());
                        if (result.isFailed()) {
                            return new BocloudResult<>(false, rds.getId(), "用户授权失败");
                        }
                    }
                }
                String sql = "drop login " + rdsUser.getUsername() + ";";
                result = executeSqlServer(sql, rds, null);
            } else {
                if ("cluster".equals(rds.getCategory())) {
                    List<RdsNode> rdsNodes = rdsNodeRepository.list(rds.getId());
                    boolean checkCluster = MDSUtils.checkCluster(rdsNodes);
                    if (!checkCluster) {
                        return new BocloudResult<>(false, "集群结构不存在！请检查！");
                    }
                }
                String sql = "drop user '" + rdsUser.getUsername() + "'@'" + rdsUser.getHost() + "';";
                result = execute(sql, rds);
            }
            if (result.isFailed()) {
                return new BocloudResult<>(false, rds.getId(), "删除数据库用户失败");
            }
            rdsUserRepository.remove(id, context.getTarget());
            return new BocloudResult<>(true, rds.getId(), "删除数据库用户成功");
        } catch (Exception e) {
            logger.error("create rds user failure:", e);
            return new BocloudResult<>(false, null == rds ? null : rds.getId(), "删除数据库用户失败");
        }
    }

    @Override
    public BocloudResult<RdsUser> detail(Long id) {
        try {
            RdsUser user = rdsUserRepository.query(id);
            if (null == user) {
                logger.warn("rds user not exist");
                return new BocloudResult<>(false, "数据不存在");
            }
            return new BocloudResult<>(true, user, "查询详情成功");
        } catch (Exception e) {
            logger.error("query rds user detail fail:", e);
            return new BocloudResult<>(false, "查询详情失败");
        }
    }

    @Override
    public BocloudResult reset(RdsUser rdsUser, RequestContext context) {
        String path = RdsUser.class.getSimpleName() + "_" + rdsUser.getId();
        Rds rds = null;
        try (BocloudACLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            RdsUser object = rdsUserRepository.query(rdsUser.getId());
            if (null == object) {
                logger.warn("rds user not exist");
                return new BocloudResult<>(false, "数据库用户不存在");
            }
            rds = rdsRepository.query(object.getRdsId());
            if (null == rds) {
                logger.warn("rds not exist");
                return new BocloudResult<>(false, "数据库不存在");
            }
            if (!object.getPassword().equals(rdsUser.getPassword())) {
                return new BocloudResult<>(false, rds.getId(), "原密码输入错误");
            }
            if ("STOPPED".equals(rds.getStatus())) {
                return new BocloudResult<>(false, rds.getId(), "数据库不是运行状态");
            }
            String newPassword = MDSUtils.decrypt(rdsUser.getNewPassword());
            BocloudResult result = null;
            if (ResourceCategory.PostgreSQL.name().equalsIgnoreCase(rds.getType())) {
                List<JSONObject> targets = null;
                if ("cluster".equals(rds.getCategory())) {
                    targets = getTargets(rds.getId());
                }
                String sql = "alter user " + object.getUsername() + " with password '" + newPassword + "';";
                if (ListTool.isEmpty(targets)) {
                    result = executePostgresql(sql, rds, null);
                } else {
                    result = copInternalService.execute(getParams(targets, rdsUser.getUsername(), newPassword, false));
                    logger.info("PostgreSQL重置用户调用worker结果：" + JSONObject.toJSONString(result));
                    if (result.isSuccess()) {
                        result = executePostgresql(sql, rds, null);
                    }
                }
            } else if (ResourceCategory.SqlServer.name().equalsIgnoreCase(rds.getType())) {
                String sql = "alter login " + object.getUsername() + " with password = '" + newPassword + "';";
                result = executeSqlServer(sql, rds, null);
            } else {
                if ("cluster".equals(rds.getCategory())) {
                    List<RdsNode> rdsNodes = rdsNodeRepository.list(rds.getId());
                    boolean checkCluster = MDSUtils.checkCluster(rdsNodes);
                    if (!checkCluster) {
                        return new BocloudResult<>(false, "集群结构不存在！请检查！");
                    }
                }
                String sql = null;
                if ("8.0".equals(rds.getVersion())) {
                    sql = "alter user '" + object.getUsername() + "'@'" + object.getHost() + "' identified by '"
                        + newPassword + "';";
                } else {
                    sql = "set password for '" + object.getUsername() + "'@'" + object.getHost() + "' = password('"
                        + newPassword + "');";
                }
                result = execute(sql, rds);
            }
            if (result.isFailed()) {
                return new BocloudResult<>(false, rds.getId(), "用户密码重置失败");
            }
            object.setPassword(rdsUser.getNewPassword());
            object.setMenderId(context.getTarget());
            object.setGmtModify(new Date());
            rdsUserRepository.update(object);
            return new BocloudResult<>(true, rds.getId(), "用户密码重置成功");
        } catch (Exception e) {
            logger.error("reset rds user failure:", e);
            return new BocloudResult<>(false, null == rds ? null : rds.getId(), "用户密码重置失败");
        }
    }

    @Override
    public BocloudResult grant(RdsUser rdsUser, RequestContext context) {
        String path = RdsUser.class.getSimpleName() + "_" + rdsUser.getId();
        Rds rds = null;
        try (BocloudACLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            RdsUser object = rdsUserRepository.query(rdsUser.getId());
            if (null == object) {
                logger.warn("rds user not exist");
                return new BocloudResult<>(false, "数据库用户不存在");
            }
            rds = rdsRepository.query(object.getRdsId());
            if (null == rds) {
                logger.warn("rds not exist");
                return new BocloudResult<>(false, "数据库不存在");
            }
            if ("STOPPED".equals(rds.getStatus())) {
                return new BocloudResult<>(false, rds.getId(), "数据库不是运行状态");
            }
            if (ResourceCategory.MySQL.name().equalsIgnoreCase(rds.getType()) && "cluster".equals(rds.getCategory())) {
                List<RdsNode> rdsNodes = rdsNodeRepository.list(rds.getId());
                boolean checkCluster = MDSUtils.checkCluster(rdsNodes);
                if (!checkCluster) {
                    return new BocloudResult<>(false, "集群结构不存在！请检查！");
                }
            }
            if ("premier".equalsIgnoreCase(rdsUser.getPermission())) {
                BocloudResult result = null;
                if (ResourceCategory.PostgreSQL.name().equalsIgnoreCase(rds.getType())) {
                    if (!StringUtils.isEmpty(object.getDbs())) {
                        String[] dbIds = object.getDbs().split(",");
                        List<RdsDb> dbs = new ArrayList<>();
                        for (String dbId : dbIds) {
                            RdsDb rdsDb = rdsDbRepository.query(Long.valueOf(dbId));
                            if (null != rdsDb) {
                                dbs.add(rdsDb);
                            }
                        }
                        for (RdsDb db : dbs) {
                            String sql = "revoke all on all tables in schema public from " + object.getUsername() + ";";
                            result = executePostgresql(sql, rds, db.getName());
                            if (result.isFailed()) {
                                return new BocloudResult<>(false, rds.getId(), "用户授权失败");
                            }
                        }
                    }
                    String sql = "alter user " + object.getUsername() + " superuser";
                    result = executePostgresql(sql, rds, null);
                } else if (ResourceCategory.SqlServer.name().equalsIgnoreCase(rds.getType())) {
                    if (!StringUtils.isEmpty(object.getDbs())) {
                        String[] dbIds = object.getDbs().split(",");
                        List<RdsDb> list = new ArrayList<>();
                        for (String dbId : dbIds) {
                            RdsDb rdsDb = rdsDbRepository.query(Long.valueOf(dbId));
                            if (null != rdsDb) {
                                list.add(rdsDb);
                            }
                        }
                        for (RdsDb db : list) {
                            String sql = "drop user if exists " + object.getUsername() + ";";
                            result = executeSqlServer(sql, rds, db.getName());
                            if (result.isFailed()) {
                                return new BocloudResult<>(false, rds.getId(), "用户授权失败");
                            }
                        }
                    }
                    String sql = "exec sp_addsrvrolemember " + object.getUsername() + ",sysadmin;";
                    result = executeSqlServer(sql, rds, null);
                } else {
                    String sql = "grant all privileges on *.* to '" + object.getUsername() + "'@'" + object.getHost()
                        + "' with grant option;";
                    result = execute(sql, rds);
                }
                if (result.isFailed()) {
                    return new BocloudResult<>(false, rds.getId(), "用户授权失败");
                }
                object.setDbs(null);
                object.setMenderId(context.getTarget());
                object.setGmtModify(new Date());
                object.setPermission(rdsUser.getPermission());
                rdsUserRepository.update(object);
                return new BocloudResult<>(true, rds.getId(), "用户授权成功");
            }
            if ("premier".equalsIgnoreCase(object.getPermission())) {
                BocloudResult result = null;
                if (ResourceCategory.PostgreSQL.name().equalsIgnoreCase(rds.getType())) {
                    if (!StringUtils.isEmpty(object.getDbs())) {
                        String[] dbIds = object.getDbs().split(",");
                        List<RdsDb> dbs = new ArrayList<>();
                        for (String dbId : dbIds) {
                            RdsDb rdsDb = rdsDbRepository.query(Long.valueOf(dbId));
                            if (null != rdsDb) {
                                dbs.add(rdsDb);
                            }
                        }
                        for (RdsDb db : dbs) {
                            String sql = "revoke all on all tables in schema public from " + object.getUsername() + ";";
                            result = executePostgresql(sql, rds, db.getName());
                            if (result.isFailed()) {
                                return new BocloudResult<>(false, rds.getId(), "回收用户权限失败");
                            }
                        }
                    }
                    String sql = "alter user " + object.getUsername() + " nosuperuser";
                    result = executePostgresql(sql, rds, null);
                } else if (ResourceCategory.SqlServer.name().equalsIgnoreCase(rds.getType())) {
                    if (!StringUtils.isEmpty(object.getDbs())) {
                        String[] dbIds = object.getDbs().split(",");
                        List<RdsDb> list = new ArrayList<>();
                        for (String dbId : dbIds) {
                            RdsDb rdsDb = rdsDbRepository.query(Long.valueOf(dbId));
                            if (null != rdsDb) {
                                list.add(rdsDb);
                            }
                        }
                        for (RdsDb db : list) {
                            String sql = "drop user if exists " + object.getUsername() + ";";
                            result = executeSqlServer(sql, rds, db.getName());
                            if (result.isFailed()) {
                                return new BocloudResult<>(false, rds.getId(), "用户授权失败");
                            }
                        }
                    }
                    String sql = "exec sp_dropsrvrolemember " + object.getUsername() + ",sysadmin;";
                    result = executeSqlServer(sql, rds, null);
                } else if (ResourceCategory.MySQL.name().equalsIgnoreCase(rds.getType())) {
                    String sql = "revoke all privileges,grant option from '" + object.getUsername() + "'@'"
                        + object.getHost() + "';";
                    result = execute(sql, rds);
                }
                if (result.isFailed()) {
                    return new BocloudResult<>(false, rds.getId(), "用户授权失败");
                }
            }
            String dbIds = "";
            List<RdsDb> dbs = rdsUser.getGrantDbs();
            if (StringUtils.isEmpty(object.getDbs())) {
                if (!ListTool.isEmpty(dbs)) {
                    if (ResourceCategory.PostgreSQL.name().equalsIgnoreCase(rds.getType())) {
                        for (RdsDb db : dbs) {
                            String sql = "grant all on all tables in schema public to " + object.getUsername() + ";";
                            BocloudResult result = executePostgresql(sql, rds, db.getName());
                            if (result.isSuccess()) {
                                dbIds += db.getId() + ",";
                            }
                        }
                    } else if (ResourceCategory.SqlServer.name().equalsIgnoreCase(rds.getType())) {
                        for (RdsDb db : dbs) {
                            String sql = "create user " + object.getUsername() + " for login " + object.getUsername()
                                + " with default_schema=dbo";
                            BocloudResult result = executeSqlServer(sql, rds, db.getName());
                            if (result.isSuccess()) {
                                sql = "exec sp_addrolemember \"db_owner\", \"" + object.getUsername() + "\";";
                                result = executeSqlServer(sql, rds, db.getName());
                                if (result.isSuccess()) {
                                    dbIds += db.getId() + ",";
                                }
                            }
                        }
                    } else {
                        for (RdsDb db : dbs) {
                            String sql = "grant all privileges on " + db.getName() + ".* to '" + object.getUsername()
                                + "'@'" + object.getHost() + "';";
                            BocloudResult result = execute(sql, rds);
                            if (result.isSuccess()) {
                                dbIds += db.getId() + ",";
                            }
                        }
                    }
                }
            } else {
                if (ResourceCategory.PostgreSQL.name().equalsIgnoreCase(rds.getType())) {
                    if (ListTool.isEmpty(dbs)) {
                        String[] values = object.getDbs().split(",");
                        List<RdsDb> list = new ArrayList<>();
                        for (String dbId : values) {
                            RdsDb rdsDb = rdsDbRepository.query(Long.valueOf(dbId));
                            if (null != rdsDb) {
                                list.add(rdsDb);
                            }
                        }
                        for (RdsDb db : list) {
                            String sql = "revoke all on all tables in schema public from " + object.getUsername() + ";";
                            BocloudResult result = executePostgresql(sql, rds, db.getName());
                            if (result.isFailed()) {
                                return new BocloudResult<>(false, rds.getId(), "用户授权失败");
                            }
                        }
                    } else {
                        Map<String, RdsDb> dbMap = new HashMap<>();
                        for (RdsDb db : dbs) {
                            dbMap.put(String.valueOf(db.getId()), db);
                        }
                        String[] values = object.getDbs().split(",");
                        for (String dbId : values) {
                            if (dbMap.containsKey(dbId)) {
                                dbIds += dbId + ",";
                                dbMap.remove(dbId);
                                continue;
                            }
                            RdsDb rdsDb = rdsDbRepository.query(Long.valueOf(dbId));
                            if (null == rdsDb) {
                                continue;
                            }
                            String sql = "revoke all on all tables in schema public from " + object.getUsername() + ";";
                            BocloudResult result = executePostgresql(sql, rds, rdsDb.getName());
                            if (result.isFailed()) {
                                return new BocloudResult<>(false, rds.getId(), "用户授权失败");
                            }
                        }
                        for (RdsDb db : dbMap.values()) {
                            String sql = "grant all on all tables in schema public to " + object.getUsername() + ";";
                            BocloudResult result = executePostgresql(sql, rds, db.getName());
                            if (result.isSuccess()) {
                                dbIds += db.getId() + ",";
                            }
                        }
                    }
                } else if (ResourceCategory.SqlServer.name().equalsIgnoreCase(rds.getType())) {
                    if (ListTool.isEmpty(dbs)) {
                        String[] values = object.getDbs().split(",");
                        List<RdsDb> list = new ArrayList<>();
                        for (String dbId : values) {
                            RdsDb rdsDb = rdsDbRepository.query(Long.valueOf(dbId));
                            if (null != rdsDb) {
                                list.add(rdsDb);
                            }
                        }
                        for (RdsDb db : list) {
                            String sql = "drop user if exists " + object.getUsername() + ";";
                            BocloudResult result = executeSqlServer(sql, rds, db.getName());
                            if (result.isFailed()) {
                                return new BocloudResult<>(false, rds.getId(), "用户授权失败");
                            }
                        }
                    } else {
                        Map<String, RdsDb> dbMap = new HashMap<>();
                        for (RdsDb db : dbs) {
                            dbMap.put(String.valueOf(db.getId()), db);
                        }
                        String[] values = object.getDbs().split(",");
                        for (String dbId : values) {
                            if (dbMap.containsKey(dbId)) {
                                dbIds += dbId + ",";
                                dbMap.remove(dbId);
                                continue;
                            }
                            RdsDb rdsDb = rdsDbRepository.query(Long.valueOf(dbId));
                            if (null == rdsDb) {
                                continue;
                            }
                            String sql = "drop user if exists " + object.getUsername() + ";";
                            BocloudResult result = executeSqlServer(sql, rds, rdsDb.getName());
                            if (result.isFailed()) {
                                return new BocloudResult<>(false, rds.getId(), "用户授权失败");
                            }
                        }
                        for (RdsDb db : dbMap.values()) {
                            String sql = "create user " + object.getUsername() + " for login " + object.getUsername()
                                + " with default_schema=dbo";
                            BocloudResult result = executeSqlServer(sql, rds, db.getName());
                            if (result.isSuccess()) {
                                sql = "exec sp_addrolemember \"db_owner\", \"" + object.getUsername() + "\";";
                                result = executeSqlServer(sql, rds, db.getName());
                                if (result.isSuccess()) {
                                    dbIds += db.getId() + ",";
                                }
                            }
                        }
                    }
                } else {
                    String sql = "revoke all privileges,grant option from '" + object.getUsername() + "'@'"
                        + object.getHost() + "';";
                    BocloudResult result = execute(sql, rds);
                    if (result.isFailed()) {
                        return new BocloudResult<>(false, rds.getId(), "用户授权失败");
                    }
                    if (!ListTool.isEmpty(dbs)) {
                        for (RdsDb db : dbs) {
                            String grantSql = "grant all privileges on " + db.getName() + ".* to '"
                                + object.getUsername() + "'@'" + object.getHost() + "';";
                            result = execute(grantSql, rds);
                            if (result.isSuccess()) {
                                dbIds += db.getId() + ",";
                            }
                        }
                    }
                }
            }
            if (dbIds.contains(",")) {
                dbIds = dbIds.substring(0, dbIds.length() - 1);
            }
            object.setPermission(rdsUser.getPermission());
            object.setDbs(dbIds);
            object.setMenderId(context.getTarget());
            object.setGmtModify(new Date());
            rdsUserRepository.update(object);
            return new BocloudResult<>(true, rds.getId(), "用户授权成功");
        } catch (Exception e) {
            logger.error("create rds user failure:", e);
            return new BocloudResult<>(false, null == rds ? null : rds.getId(), "用户授权失败");
        }
    }

    @Override
    public BocloudResult revoke(Long id, RequestContext context) {
        String path = RdsUser.class.getSimpleName() + "_" + id;
        Rds rds = null;
        try (BocloudACLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            RdsUser object = rdsUserRepository.query(id);
            if (null == object) {
                logger.warn("rds user not exist");
                return new BocloudResult<>(false, "数据库用户不存在");
            }
            rds = rdsRepository.query(object.getRdsId());
            if (null == rds) {
                logger.warn("rds not exist");
                return new BocloudResult<>(false, "数据库不存在");
            }
            if ("STOPPED".equals(rds.getStatus())) {
                return new BocloudResult<>(false, rds.getId(), "数据库不是运行状态");
            }
            BocloudResult result = null;
            if (ResourceCategory.PostgreSQL.name().equalsIgnoreCase(rds.getType())) {
                if (!StringUtils.isEmpty(object.getDbs())) {
                    String[] dbIds = object.getDbs().split(",");
                    List<RdsDb> dbs = new ArrayList<>();
                    for (String dbId : dbIds) {
                        RdsDb rdsDb = rdsDbRepository.query(Long.valueOf(dbId));
                        if (null == rdsDb) {
                            continue;
                        }
                        dbs.add(rdsDb);
                    }
                    for (RdsDb db : dbs) {
                        String sql = "revoke all on all tables in schema public from " + object.getUsername() + ";";
                        result = executePostgresql(sql, rds, db.getName());
                        if (result.isFailed()) {
                            return new BocloudResult<>(false, rds.getId(), "回收用户权限失败");
                        }
                    }
                }
                String sql = "alter user " + object.getUsername() + " nosuperuser";
                result = executePostgresql(sql, rds, null);
            } else if (ResourceCategory.SqlServer.name().equalsIgnoreCase(rds.getType())) {
                if (!StringUtils.isEmpty(object.getDbs())) {
                    String[] dbIds = object.getDbs().split(",");
                    List<RdsDb> list = new ArrayList<>();
                    for (String dbId : dbIds) {
                        RdsDb rdsDb = rdsDbRepository.query(Long.valueOf(dbId));
                        if (null != rdsDb) {
                            list.add(rdsDb);
                        }
                    }
                    for (RdsDb db : list) {
                        String sql = "drop user if exists " + object.getUsername() + ";";
                        result = executeSqlServer(sql, rds, db.getName());
                        if (result.isFailed()) {
                            return new BocloudResult<>(false, rds.getId(), "用户授权失败");
                        }
                    }
                }
                String sql = "exec sp_dropsrvrolemember " + object.getUsername() + ",sysadmin;";
                result = executeSqlServer(sql, rds, null);
            } else {
                if ("cluster".equals(rds.getCategory())) {
                    List<RdsNode> rdsNodes = rdsNodeRepository.list(rds.getId());
                    boolean checkCluster = MDSUtils.checkCluster(rdsNodes);
                    if (!checkCluster) {
                        return new BocloudResult<>(false, "集群结构不存在！请检查！");
                    }
                }
                String sql = "revoke all privileges,grant option from '" + object.getUsername() + "'@'"
                    + object.getHost() + "';";
                result = execute(sql, rds);
            }
            if (result.isFailed()) {
                return new BocloudResult<>(false, rds.getId(), "回收用户权限失败");
            }
            object.setDbs(null);
            object.setMenderId(context.getTarget());
            object.setGmtModify(new Date());
            object.setPermission(null);
            rdsUserRepository.update(object);
            return new BocloudResult<>(true, rds.getId(), "回收用户权限成功");
        } catch (Exception e) {
            logger.error("revoke rds user failure:", e);
            return new BocloudResult<>(false, null == rds ? null : rds.getId(), "回收用户权限失败");
        }
    }

    private BocloudResult execute(String sql, Rds rds) throws ClassNotFoundException {
        Class.forName("com.mysql.cj.jdbc.Driver");
        try (Connection conn = DriverManager.getConnection(
            "jdbc:mysql://" + rds.getManageIp() + "/mysql?characterEncoding=utf-8&serverTimezone=UTC&useSSL=false",
            rds.getUsername(), MDSUtils.decrypt(rds.getPassword()))) {
            // 设置自动提交关闭
            conn.setAutoCommit(false);
            try (PreparedStatement pst = conn.prepareStatement(sql);) {
                pst.execute();
                conn.commit();
                return new BocloudResult<>(true, "数据库操作成功");
            } catch (SQLException e) {
                logger.info("connect database error:", e);
                return new BocloudResult<>(false, "数据库操作失败");
            }
        } catch (Exception e) {
            logger.info("connect database error:", e);
            return new BocloudResult<>(false, "数据库操作失败");
        }
    }

    private BocloudResult executePostgresql(String sql, Rds rds, String database) throws ClassNotFoundException {
        Class.forName("org.postgresql.Driver");
        database = StringUtils.isEmpty(database) ? "postgres" : database;
        try (
            Connection conn =
                DriverManager.getConnection("jdbc:postgresql://" + rds.getManageIp() + ":5432/" + database,
                    rds.getUsername(), MDSUtils.decrypt(rds.getPassword()));
            PreparedStatement pst = conn.prepareStatement(sql)) {
            pst.execute();
            return new BocloudResult<>(true, "数据库操作成功");
        } catch (Exception e) {
            logger.info("connect database error:", e);
            return new BocloudResult<>(false, "数据库操作失败");
        }
    }

    private BocloudResult executeSqlServer(String sql, Rds rds, String database) throws ClassNotFoundException {
        Class.forName("com.microsoft.sqlserver.jdbc.SQLServerDriver");
        database = StringUtils.isEmpty(database) ? "master" : database;
        try (
            Connection conn =
                DriverManager.getConnection("jdbc:sqlserver://" + rds.getManageIp() + ":1433;database=" + database,
                    rds.getUsername(), MDSUtils.decrypt(rds.getPassword()));
            PreparedStatement pst = conn.prepareStatement(sql)) {
            pst.execute();
            return new BocloudResult<>(true, "数据库操作成功");
        } catch (Exception e) {
            logger.info("connect database error:", e);
            return new BocloudResult<>(false, "数据库操作失败");
        }
    }

}
