package com.bocloud.pms.service.internal;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.enums.BoCloudMethod;
import com.bocloud.common.enums.BoCloudService;
import com.bocloud.common.enums.PostDataFormat;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.BocloudToken;
import com.bocloud.common.model.Param;
import com.bocloud.common.model.Sign;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.MapTools;
import com.bocloud.pms.entity.resource.*;
import com.bocloud.pms.entity.resource.model.ElementModel;
import com.bocloud.service.core.CurrentService;
import com.bocloud.service.core.Service;
import com.bocloud.service.http.core.ServiceFactory;
import com.bocloud.service.http.model.RemoteService;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * cos服务内部访问
 *
 * @author wangyu
 * @version 1.0
 * @since 2018年1月11日
 */
@Component
public class CosInternalService implements InitializingBean {

    private static final Logger logger = LoggerFactory.getLogger(CosInternalService.class);
    @Autowired
    private ServiceFactory serviceFactory;
    @Autowired
    private CurrentService currentService;
    private static Service service = Service.create(BoCloudService.COS);
    private Map<String, Object> headers = new HashMap<>(4);

    public SkuPrice queryPrice(Long skuId) {
        List<Param> params = Lists.newArrayList();
        Map<String, Object> simpleMap = MapTools.simpleMap("skuId", skuId);
        simpleMap.put("period", "Hour");
        params.add(new Param(simpleMap, Sign.EQ));
        Map<String, Object> paramMap = MapTools.simpleMap("simple", false);
        paramMap.put("page", 1);
        paramMap.put("rows", 1000);
        paramMap.put(Common.PARAMS, JSONArray.toJSONString(params));
        RemoteService remote = serviceFactory.build(service, "/v1/skus/prices", BoCloudMethod.GET, headers, paramMap);
        BocloudResult result = remote.invoke();
        if (result.isFailed()) {
            return null;
        }
        JSONObject jsonObject = JSONObject.parseObject(JSONObject.toJSONString(result.getData()), JSONObject.class);
        List<SkuPrice> prices = JSONArray.parseArray(jsonObject.getString("rows"), SkuPrice.class);
        return prices.get(0);
    }

    public ServiceOrder createOrder(ServiceOrder order) {
        RemoteService remote = serviceFactory.build(service, "/v1/orders", BoCloudMethod.POST, headers,
                MapTools.simpleMap(Common.PARAMS, JSONObject.toJSONString(order)));
        BocloudResult result = remote.invoke();
        if (result.isFailed()) {
            return null;
        }
        return JSONObject.parseObject(JSONObject.toJSONString(result.getData()), ServiceOrder.class);
    }

    public BocloudResult recycle(String sn) {
        /*RemoteService remote = serviceFactory.build(service, "/v1/recycles/sn", BoCloudMethod.DELETE, headers,
                sn);
        return remote.invoke();*/
        return null;
    }

    public OrderItem createOrderItem(OrderItem item) {
        RemoteService remote = serviceFactory.build(service, "/v1/orders/items", BoCloudMethod.POST, headers,
                MapTools.simpleMap(Common.PARAMS, JSONObject.toJSONString(item)));
        BocloudResult result = remote.invoke();
        if (result.isFailed()) {
            return null;
        }
        return JSONObject.parseObject(JSONObject.toJSONString(result.getData()), OrderItem.class);
    }

    public BocloudResult updateStatus(Long id, String status) {
        JSONObject object = new JSONObject();
        object.put("resourceStatus", status);
        RemoteService remote = serviceFactory.build(service, "/v1/orders/" + id, BoCloudMethod.PATCH, headers,
                MapTools.simpleMap(Common.PARAMS, object.toJSONString()));
        return remote.invoke();
    }

    public BocloudResult updateElements(String sn, List<ElementModel> elements) {
        JSONObject object = new JSONObject();
        object.put("sn", sn);
        object.put("elements", JSONObject.toJSONString(elements));
        RemoteService remote = serviceFactory.build(service, "/v1/orders/items/elements", BoCloudMethod.PATCH, headers,
                MapTools.simpleMap(Common.PARAMS, object.toJSONString()));
        return remote.invoke();
    }

    public BocloudResult recycleBill(String orderItemSn) {
        JSONObject object = new JSONObject();
        object.put("sn", orderItemSn);
        RemoteService remote = serviceFactory.build(service, "/v1/bills", BoCloudMethod.PUT, headers,
                object);
        remote.setDataFormat(PostDataFormat.RAW);
        return remote.invoke();
    }

    public BocloudResult updateItemStatus(String sn, String status) {
        JSONObject object = new JSONObject();
        object.put("sn", sn);
        object.put("status", status);
        RemoteService remote = serviceFactory.build(service, "/v1/orders/items", BoCloudMethod.PATCH, headers,
                object);
        remote.setDataFormat(PostDataFormat.RAW);
        return remote.invoke();
    }

    public BocloudResult queryByResourceUuid(String resourceUuid, String resourceCategory) {
        JSONObject object = new JSONObject();
        object.put(Common.CONDITION, "queryByResourceUuid");
        object.put("resourceUuid", resourceUuid);
        object.put("resourceCategory", resourceCategory);
        RemoteService remote = serviceFactory.build(service, "/v1/orders", BoCloudMethod.GET, headers,
                MapTools.simpleMap(Common.CONDITION, object.toJSONString()));
        return remote.invoke();
    }

    public BocloudResult detail(Long id) {
        RemoteService remote = serviceFactory.build(service, "/v1/orders/" + id, BoCloudMethod.GET, headers, null);
        return remote.invoke();
    }

    public CloudService detailService(String serviceCode) {
        JSONObject object = new JSONObject();
        object.put(Common.CONDITION, "queryByCode");
        object.put("serviceCode", serviceCode);
        RemoteService remote = serviceFactory.build(service, "/v1/cloud/services", BoCloudMethod.GET, headers,
                MapTools.simpleMap(Common.CONDITION, object.toJSONString()));
        BocloudResult result = remote.invoke();
        if (result.isFailed()) {
            return null;
        }
        return JSONObject.parseObject(JSONObject.toJSONString(result.getData()), CloudService.class);
    }

    public BocloudResult getByOrderSn(String orderSn) {
        JSONObject object = new JSONObject();
        object.put(Common.CONDITION, "getByOrderSn");
        object.put("orderSn", orderSn);
        RemoteService remote = serviceFactory.build(service, "/v1/orders", BoCloudMethod.GET, headers,
                MapTools.simpleMap(Common.CONDITION, object.toJSONString()));
        return remote.invoke();
    }

    public ServiceOrder getServiceOrder(String itemSn) {
        JSONObject object = new JSONObject();
        object.put(Common.CONDITION, "getServiceOrder");
        object.put("itemSn", itemSn);
        RemoteService remote = serviceFactory.build(service, "/v1/orders", BoCloudMethod.GET, headers,
                MapTools.simpleMap(Common.CONDITION, object.toJSONString()));
        BocloudResult result = remote.invoke();
        if (result.isFailed()) {
            return null;
        }
        String content = JSONObject.toJSONString(result.getData());
        if (StringUtils.isEmpty(content)) {
            return null;
        }
        return JSONObject.parseObject(content, ServiceOrder.class);
    }

    public OrderItem queryByItemSn(String itemSn) {
        JSONObject object = new JSONObject();
        object.put(Common.CONDITION, "queryByItemSn");
        object.put("sn", itemSn);
        RemoteService remote = serviceFactory.build(service, "/v1/orders", BoCloudMethod.GET, headers,
                MapTools.simpleMap(Common.CONDITION, object.toJSONString()));
        BocloudResult result = remote.invoke();
        if (result.isFailed()) {
            return null;
        }
        String content = JSONObject.toJSONString(result.getData());
        if (StringUtils.isEmpty(content)) {
            return null;
        }
        return JSONObject.parseObject(content, OrderItem.class);
    }

    public BocloudResult removeByVendorId(Long vendorId) {
        Map<String, Object> params = Maps.newHashMap();
        JSONObject object = new JSONObject();
        object.put("vendorId", vendorId);
        params.put(Common.PARAMS, object.toJSONString());
        RemoteService remote = serviceFactory.build(service, "/price/removebyvendorid", BoCloudMethod.POST, headers,
                params);
        return remote.invoke();
    }

    public BocloudResult deleteByVendor(Long vendorId) {
        RemoteService remote = serviceFactory.build(service, "/v1/vendors/records/vendors/" + vendorId,
                BoCloudMethod.DELETE, headers, null);
        BocloudResult result = remote.invoke();
        return result;
    }

    public Integer list(Long userId, String apiKey) {
        List<Param> params = Lists.newArrayList();
        params.add(new Param(MapTools.simpleMap("status", "UNREAD"), Sign.EQ));
        params.add(new Param(MapTools.simpleMap("ownerId", userId), Sign.EQ));
        Map<String, Object> paramMap = MapTools.simpleMap("simple", true);
        paramMap.put("page", 1);
        paramMap.put("rows", Integer.MAX_VALUE);
        paramMap.put(Common.PARAMS, JSONArray.toJSONString(params));
        this.headers.put(Common.API_KEY, apiKey);
        RemoteService remote = serviceFactory.build(service, "/v1/messages", BoCloudMethod.GET, headers, paramMap);
        BocloudResult result = remote.invoke();
        if (result.isFailed()) {
            return 0;
        }
        JSONObject jsonObject = JSONObject.parseObject(JSONObject.toJSONString(result.getData()), JSONObject.class);
        return jsonObject.getInteger("total");
    }

    public BocloudResult updateQuotas(List<ResourceQuota> quotas, ResourceQuota.Action action, Long projectId) {
        Map<String, Object> paramMap = new HashMap<>();
        paramMap.put(Common.PARAMS, JSONObject.toJSONString(quotas));
        paramMap.put(Common.ACTION, action.name());
        paramMap.put("projectId", projectId);
        RemoteService remote = serviceFactory.build(service, "/v1/quotas", BoCloudMethod.PATCH, headers, paramMap);
        return remote.invoke();
    }

    public BocloudResult joinRecycle(Recycled recycled) {
        Map<String, Object> paramMap = new HashMap<>();
        paramMap.put(Common.PARAMS, JSONObject.toJSONString(recycled));
        RemoteService remote = serviceFactory.build(service, "/v1/recycles", BoCloudMethod.POST, headers, paramMap);
        return remote.invoke();
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        this.headers.put(BocloudToken.INTERNAL_TOKEN, currentService.getToken());
        this.headers.put("source", currentService.getService().getName());
    }
}
