package com.bocloud.pms.service.internal;

import java.util.HashMap;
import java.util.Map;

import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.enums.BoCloudMethod;
import com.bocloud.common.enums.BoCloudService;
import com.bocloud.common.enums.PostDataFormat;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.BocloudToken;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.MapTools;
import com.bocloud.service.core.CurrentService;
import com.bocloud.service.core.Service;
import com.bocloud.service.http.core.ServiceFactory;
import com.bocloud.service.http.model.RemoteService;

/**
 * 虚机ims服务调用
 *
 * @author lyy
 * @version 1.0
 * @date 2020/12/17 9:57
 */
@Component
public class ImsCloudServerService implements InitializingBean {
    @Autowired
    private ServiceFactory serviceFactory;
    @Autowired
    private CurrentService currentService;
    private static Service service = Service.create(BoCloudService.IMS);
    private Map<String, Object> headers = new HashMap<>(4);

    public BocloudResult resize(Long id, JSONObject param, String action) {
        RemoteService remoteService =
            serviceFactory.build(service, "/v1/vms/" + id + "/resize", BoCloudMethod.PATCH, headers, param);
        remoteService.setDataFormat(PostDataFormat.RAW);
        return remoteService.invoke();
    }

    public BocloudResult operation(Long id, String action) {
        JSONObject object = new JSONObject();
        object.put("id", id);
        Map<String, Object> paramMap = MapTools.simpleMap(Common.PARAMS, object);
        paramMap.put(Common.ACTION, action);
        RemoteService remoteService =
            serviceFactory.build(service, "/v1/vms/" + id, BoCloudMethod.PATCH, headers, paramMap);
        return remoteService.invoke();
    }

    public BocloudResult batchOperations(String params, String action) {
        Map<String, Object> paramMap = MapTools.simpleMap(Common.PARAMS, params);
        paramMap.put(Common.ACTION, action);
        RemoteService remoteService = serviceFactory.build(service, "/v1/vms/", BoCloudMethod.PATCH, headers, paramMap);
        return remoteService.invoke();
    }

    public BocloudResult delete(Long id) {
        RemoteService remoteService =
            serviceFactory.build(service, "/v1/vms/" + id, BoCloudMethod.DELETE, headers, null);
        return remoteService.invoke();
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        this.headers.put(BocloudToken.INTERNAL_TOKEN, currentService.getToken());
        this.headers.put("source", currentService.getService().getName());
    }
}
