package com.bocloud.pms.service.utils;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.amqp.core.AmqpTemplate;
import org.springframework.amqp.core.Binding;
import org.springframework.amqp.core.Binding.DestinationType;
import org.springframework.amqp.core.Message;
import org.springframework.amqp.core.Queue;
import org.springframework.amqp.rabbit.core.RabbitAdmin;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.encrypt.AESEncrptor;
import com.bocloud.common.encrypt.Encryptor;
import com.bocloud.common.enums.BoCloudExchange;
import com.bocloud.common.enums.BoCloudMethod;
import com.bocloud.common.enums.BoCloudService;
import com.bocloud.common.enums.OSCategory;
import com.bocloud.common.enums.PostDataFormat;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.RequestContext;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.IDFactory;
import com.bocloud.common.utils.ListTool;
import com.bocloud.common.utils.MapTools;
import com.bocloud.pms.entity.KvsNode;
import com.bocloud.pms.entity.RdsNode;
import com.bocloud.pms.entity.resource.CloudServer;
import com.bocloud.pms.entity.resource.OrderMessage;
import com.bocloud.pms.entity.resource.enums.Charset;
import com.bocloud.pms.entity.resource.enums.OrderItemStatus;
import com.bocloud.pms.entity.resource.enums.ResourceCategory;
import com.bocloud.pms.entity.resource.model.AlterationParam;
import com.bocloud.pms.entity.resource.model.CancelationParam;
import com.bocloud.pms.entity.resource.model.ExtensionParam;
import com.bocloud.pms.entity.resource.model.QuotaBean;
import com.bocloud.pms.entity.resource.model.ResourceModel;
import com.bocloud.pms.entity.resource.model.WorkerHost;
import com.bocloud.pms.service.internal.ImsCloudServerService;
import com.bocloud.service.http.core.ServiceFactory;
import com.bocloud.service.http.model.RemoteService;

public class MDSUtils {

    private static Logger logger = LoggerFactory.getLogger(MDSUtils.class);
    public static final String RDS = "RDS";
    public static final String KVS = "KVS";
    public static final String CMS = "CMS";
    public static final String ASS = "ASS";
    public static final String CCS = "CCS";
    public static final String LBS = "LBS";
    public static final String START = "started";
    public static final String STOP = "stopped";
    public static final String RESTART = "restarted";
    private static final String URL = "/api/worker/v1/job/submit";
    private static final BoCloudService SERVICE = BoCloudService.WORKER;
    private static Map<String, Object> headers = MapTools.simpleMap("Content-Type", "application/json");

    /**
     * 密码解密
     *
     * @param password
     * @return
     * @throws Exception
     */
    public static String decrypt(String password) throws Exception {
        if (StringUtils.isEmpty(password)) {
            return null;
        }
        return new AESEncrptor().decrypt(password, null);
    }

    /**
     * 密码加密
     *
     * @param password
     * @return
     * @throws Exception
     */
    public static String encrypt(String password) throws Exception {
        if (StringUtils.isEmpty(password)) {
            return null;
        }
        return new AESEncrptor().encrypt(password, null);
    }

    /**
     * 统计状态的数量
     *
     * @param objects
     * @param status
     * @return
     * @throws Exception
     */
    public static int statusCount(List<?> objects, String status) throws Exception {
        int runningCount = 0;
        for (Object object : objects) {
            JSONObject node = JSONObject.parseObject(JSONObject.toJSONString(object));
            if (status.equalsIgnoreCase(node.getString("status"))) {
                runningCount++;
            }
        }
        return runningCount;
    }

    /**
     * 解析软件服务的申请参数
     *
     * @param object
     * @return
     */
    public static JSONObject analyze(JSONObject object) {
        JSONObject params = new JSONObject();
        JSONObject resource = new JSONObject();
        JSONObject configs = object.getJSONObject("configs");
        resource.put("resourceLabel", configs.get("resourceLabel"));
        resource.put("templateName", configs.get("templateName"));
        resource.put("vpcId", configs.get("vpcId"));
        resource.put("subnetId", configs.get("subnetId"));
        resource.put("diskCategory", configs.get("diskCategory"));
        // resource.put("ipPoolId", configs.get("ipPoolId"));
        // resource.put("ipPolicy", "Auto");
        resource.put("osCategory", configs.get("osCategory"));
        resource.put("osVersion", configs.get("osVersion"));
        resource.put("imageId", configs.get("imageId"));
        resource.put("networkCardConfigs", configs.get("networkCardConfigs"));
        List<JSONObject> cardConfigs = JSONObject.parseArray(configs.getString("networkCardConfigs"), JSONObject.class);
        if (!ListTool.isEmpty(cardConfigs)) {
            resource.put("ipPoolId", cardConfigs.get(0).getLong("ipPoolId"));
        }
        List<JSONObject> skus = JSONArray.parseArray(object.getString("elements"), JSONObject.class);
        List<Integer> disks = new ArrayList<>();
        String serviceCode = object.getString("service");
        for (JSONObject sku : skus) {
            List<JSONObject> specs = JSONObject.parseArray(sku.getString("specs"), JSONObject.class);
            if (ListTool.isEmpty(specs)) {
                continue;
            }
            if (serviceCode.equals(sku.getString("serviceCode"))) {
                for (JSONObject spec : specs) {
                    if (spec.containsKey("cpu")) {
                        resource.put("cpu", spec.get("cpu"));
                    } else if (spec.containsKey("memory")) {
                        resource.put("memory", spec.get("memory"));
                    } else if (spec.containsKey("disk")) {
                        resource.put("disk", spec.get("disk"));
                    }
                }
                continue;
            }
            for (JSONObject spec : specs) {
                if (spec.containsKey("disk")) {
                    disks.add(spec.getIntValue("disk"));
                }
            }
        }
        resource.put("disks", disks);
        boolean clusterInstall = false;
        String categoryCode = configs.getString("categoryCode");
        if (!StringUtils.isEmpty(categoryCode)
            && ("cluster".equalsIgnoreCase(categoryCode) || categoryCode.toLowerCase().contains("cluster"))) {
            clusterInstall = true;
        }
        configs.put("clusterInstall", clusterInstall);
        configs.put("category", configs.getString("category"));
        params.put("software", configs);
        params.put("resource", resource);
        params.put("location", object.get("location"));
        return params;
    }

    public static Map<String, Object> getParam(CloudServer server, String service, String state, OSCategory category)
        throws Exception {
        // worker服务需要的参数module
        Map<String, Object> argsObj = new HashMap<>(4);
        if (service.contains("MSSQL")) {
            argsObj.put("force_dependent_services", "yes");
        }
        argsObj.put(Common.NAME, service);
        argsObj.put(Constant.STATE, state);
        Map<String, Object> module = new HashMap<>(4);
        if (OSCategory.WINDOWS == category) {
            module.put(Common.NAME, "win_service");
        } else {
            module.put(Common.NAME, Constant.SERVICE);
        }
        module.put("args", argsObj);
        List<WorkerHost> hosts = new ArrayList<>();
        Encryptor encryptor = new AESEncrptor();
        Integer port = server.getManagerPort();
        if (OSCategory.WINDOWS == category) {
            port = null == port ? 5986 : port;
        } else {
            port = null == port ? 22 : port;
        }
        WorkerHost host = new WorkerHost(server.getManagerIp(), server.getAccount(),
            encryptor.encrypt(server.getPswd(), null), port, category);
        hosts.add(host);
        Map<String, Object> paramMap = new HashMap<>(4);
        paramMap.put("targets", hosts);
        paramMap.put("module", module);
        return paramMap;
    }

    public static BocloudResult invoke(ServiceFactory serviceFactory, Map<String, Object> params,
        RabbitAdmin rabbitAdmin) throws Exception {
        if (serviceFactory.exist(BoCloudService.WORKER).isFailed()) {
            logger.error("remote worker does not exist!");
            return BocloudResult.FAILED("远程Worker不存在!");
        }
        String callback = IDFactory.instance().uuid();
        params.put("queue", callback);
        BocloudResult result = null;
        RemoteService service = serviceFactory.build(SERVICE, URL, BoCloudMethod.OPERATE, headers, params);
        service.setDataFormat(PostDataFormat.RAW);
        try {
            // 声明worker结果的存放队列并将其绑定到指定的exchange:bocloud.direct.exchange
            rabbitAdmin.declareQueue(new Queue(callback, false, false, true, MapTools.simpleMap("x-ha-policy", "all")));
            rabbitAdmin.declareBinding(new Binding(callback, DestinationType.QUEUE,
                BoCloudExchange.Default.DIRECT.getExchange(), callback, MapTools.simpleMap("x-ha-policy", "all")));
            BocloudResult invoke = service.invoke();
            if (invoke.isFailed()) {
                logger.error("invoke remote worker [{}] failed for : {}", service.getUrl(), invoke.getMessage());
                return new BocloudResult(false, invoke.getMessage());
            }
            // 这里说明请求成功发送到worker上，接下来可以等待结果消息了
            Message message = rabbitAdmin.getRabbitTemplate().receive(callback, (long)1800 * 1000 + 60 * 1000);
            if (null == message) {
                logger.error("get callback from remote worker by queue [{}] timeout", callback);
                result = new BocloudResult(false, "获取任务结果消息超时！");
            } else {
                logger.info("get callback from remote worker by queue [{}]", callback);
                String content = new String(message.getBody(), Charset.UTF8.name());
                logger.info("worker任务执行结果：" + content);
                try {
                    result = JSONObject.parseObject(content, BocloudResult.class);
                } catch (Exception e) {
                    logger.error("callback data transfer exception!", e);
                    result = new BocloudResult(false, "返回数据格式异常!");
                }
            }
        } catch (Exception e) {
            logger.error("等待Worker 【{}】返回任务结果消息异常:", service.getUrl(), e);
            result = new BocloudResult(false, e.getMessage());
        } finally {
            rabbitAdmin.deleteQueue(callback);
        }
        return result;
    }

    /**
     * 快照恢复后对机器进行状态检验是否为running状态
     */
    public static void powerOn(ImsCloudServerService imsCloudServerService, CloudServer server) {
        RequestContext requestContext = new RequestContext();
        requestContext.setTarget(1L);
        JSONObject object = new JSONObject();
        object.put("id", server.getId());
        BocloudResult result = imsCloudServerService.operation(server.getId(), "start");
        if (result.isFailed()) {
            logger.error("恢复完成开机失败", result);
        }
    }

    public static List<QuotaBean> getQuotas(String value, String code, Integer nodes) {
        nodes = (null == nodes ? 1 : nodes);
        List<QuotaBean> quotas = new ArrayList<>();
        JSONObject spec = JSONObject.parseObject(value);
        quotas.add(new QuotaBean("meta.cpu", spec.getInteger("cpu") * nodes));
        quotas.add(new QuotaBean("meta.mem", spec.getInteger("memory") * nodes));
        Integer disk = spec.getInteger("disk");
        if (spec.containsKey("dataDisk")) {
            disk += spec.getInteger("dataDisk");
        }
        quotas.add(new QuotaBean("meta.disk", disk * nodes));
        quotas.add(new QuotaBean(code, 1));
        return quotas;
    }

    public static void send(AmqpTemplate amqpTemplate, Object object, Integer nodes, OrderMessage.Operation operation,
        OrderItemStatus status, String type, Object param, RequestContext context, String errorMsg) {
        JSONObject json = JSONObject.parseObject(JSONObject.toJSONString(object));
        ResourceCategory category = ResourceCategory.getValue(json.getString("type"));
        OrderMessage message = null;
        String content = null;
        switch (operation) {
            case Application:
                List<ResourceModel> list = new ArrayList<>();
                content = OrderItemStatus.OpenedSuccess == status ? category + "订购成功" : category + "订购失败:" + errorMsg;
                list.add(new ResourceModel(json.getString("name"), json.getString("uuid"),
                    type.toLowerCase() + "." + json.getString("type").toLowerCase()));
                message = new OrderMessage(json.getString("uuid"), operation, status, null, content,
                    json.getDate("gmtCreate"), json.getDate("expiredTime"), list, category.name(), context);
                if (OrderItemStatus.OpenedFailure == status) {
                    message.setQuotas(getQuotas(json.getString("spec"),
                        type.toLowerCase() + "." + json.getString("type").toLowerCase(), nodes));
                }
                break;
            case Cancelation:
                CancelationParam cancelation = (CancelationParam)param;
                content = category + "【" + json.getString("name") + "】退订成功";
                message = new OrderMessage(cancelation.getInstance(), operation, status, null, content, null, null,
                    new ArrayList<>(), category.name(), context);
                message.setQuotas(getQuotas(json.getString("spec"),
                    type.toLowerCase() + "." + json.getString("type").toLowerCase(), nodes));
                break;
            case Extension:
                ExtensionParam extension = (ExtensionParam)param;
                content = category + "【" + json.getString("name") + "】延期成功";
                message = new OrderMessage(extension.getInstance(), operation, status, extension.getExtension(),
                    content, null, json.getDate("expiredTime"), null, category.name(), context);
                break;
            case Upgrade:
                AlterationParam alteration = (AlterationParam)param;
                content = category + "【" + json.getString("name") + "】变更成功";
                List<ResourceModel> models = new ArrayList<>();
                models.add(new ResourceModel(json.getString("name"), alteration.getNewInstance(),
                    type.toLowerCase() + "." + json.getString("type").toLowerCase()));
                message = new OrderMessage(alteration.getInstance(), operation, status, null, content,
                    json.getDate("gmtModify"), json.getDate("expiredTime"), models, category.name(), context);
                message.setNewSn(alteration.getNewInstance());
                break;
            default:
                break;
        }
        if (null != message) {
            amqpTemplate.convertAndSend("resource.result.queue", message);
        }
    }

    public static String getService(String type, String version) {
        ResourceCategory category = ResourceCategory.getValue(type);
        String service = null;
        switch (category) {
            case MongoDB:
                service = "mongod";
                break;
            case Redis:
                service = "redis_6379";
                break;
            case MySQL:
                service = "mysqld";
                break;
            case SqlServer:
                service = "MSSQL$SQLINSTANCE";
                break;
            case PostgreSQL:
                service = "postgresql-" + version;
                break;
            case Kafka:
                service = "kafka";
                break;
            case RabbitMQ:
                service = "rabbitmq";
                break;
            default:
                break;
        }
        return service;
    }

    public static void send(AmqpTemplate amqpTemplate, Long id, String type, String action) {
        JSONObject message = new JSONObject();
        message.put("action", action);
        JSONObject data = new JSONObject();
        data.put("id", id);
        data.put("type", type);
        message.put("data", data);
        logger.info("中间件【" + type + "】，发送消息至software.event.queue队列：" + message.toJSONString());
        amqpTemplate.convertAndSend("software.event.queue", message);
    }

    /**
     * 检查集群状态
     *
     * @param nodes
     * @return
     */
    public static boolean checkCluster(List<RdsNode> nodes) {
        List<RdsNode> masterNodes =
            nodes.stream().filter(rdsNode -> "master".equals(rdsNode.getRole())).collect(Collectors.toList());
        if (0 == masterNodes.size()) {
            return false;
        } else {
            return true;
        }
    }

    public static Map<String, Object> buildParams(Boolean cluster, List<KvsNode> nodes, String password,
        String newPassword) throws Exception {
        Map<String, Object> params = new HashMap<>();
        Map<String, Object> module = new HashMap<>();
        module.put("name", "redis_update_password");
        Map<String, Object> args = new HashMap<>();
        args.put("cluster_install", cluster);
        args.put("redis_port", 6379);
        if (cluster) {
            args.put("redis_sentinel_port", 26379);
        }
        Encryptor encryptor = new AESEncrptor();
        args.put("redis_old_password", password);
        args.put("redis_new_password", newPassword);
        args.put("update_telegraf", true);
        module.put("args", args);
        params.put("module", module);
        params.put("timeout", 1800);
        List<Map<String, Object>> targets = new ArrayList<>();
        if (!cluster) {
            Map<String, Object> target = new HashMap<>();
            target.put("sudo", true);
            target.put("host", nodes.get(0).getManageIp());
            target.put("user", "root");
            target.put("pasd", encryptor.encrypt(nodes.get(0).getPassword(), null));
            target.put("port", 22);
            target.put("category", "Linux");
            targets.add(target);
            params.put("targets", targets);
            return params;
        }
        List<Map<String, Object>> masters = new ArrayList<>();
        List<Map<String, Object>> slaves = new ArrayList<>();
        List<Map<String, Object>> sentinels = new ArrayList<>();
        for (KvsNode node : nodes) {
            Map<String, Object> target = new HashMap<>();
            target.put("sudo", true);
            target.put("host", node.getManageIp());
            target.put("user", "root");
            target.put("pasd", encryptor.encrypt(node.getPassword(), null));
            target.put("port", 22);
            target.put("category", "Linux");
            targets.add(target);
            Map<String, Object> host = new HashMap<>();
            host.put("host", node.getManageIp());
            if ("master".equals(node.getRole())) {
                masters.add(host);
            } else if ("slave".equals(node.getRole())) {
                slaves.add(host);
            } else {
                sentinels.add(host);
            }
        }
        params.put("targets", targets);
        List<Map<String, Object>> groups = new ArrayList<>();
        Map<String, Object> mgroup = new HashMap<>();
        mgroup.put("name", "redis_master");
        mgroup.put("hosts", masters);
        groups.add(mgroup);
        Map<String, Object> sgroup = new HashMap<>();
        sgroup.put("name", "redis_slave");
        sgroup.put("hosts", slaves);
        groups.add(sgroup);
        Map<String, Object> agroup = new HashMap<>();
        agroup.put("name", "redis_sentinel");
        agroup.put("hosts", sentinels);
        groups.add(agroup);
        params.put("groups", groups);
        return params;
    }

}
