package com.bocloud.pms.service.utils;

import java.util.concurrent.Callable;

import org.bson.Document;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.pms.entity.Kvs;
import com.bocloud.pms.entity.KvsNode;
import com.google.common.collect.Lists;
import com.mongodb.MongoClient;
import com.mongodb.MongoClientOptions;
import com.mongodb.MongoCredential;
import com.mongodb.ReadPreference;
import com.mongodb.ServerAddress;
import com.mongodb.client.MongoDatabase;

/**
 * Created by liuyuanyuan on 2020/5/13 16:25
 */
public class MongodbconnectionRunable implements Callable<BocloudResult> {
    private Logger logger = LoggerFactory.getLogger(MongodbconnectionRunable.class);
    private Kvs kvs;
    private KvsNode kvsNode;

    public MongodbconnectionRunable(Kvs kvs, KvsNode kvsNode) {
        this.kvs = kvs;
        this.kvsNode = kvsNode;
    }

    @Override
    public boolean equals(Object obj) {
        return super.equals(obj);
    }

    @Override
    public int hashCode() {
        return super.hashCode();
    }

    @Override
    public BocloudResult call() throws Exception {
        MongoClient client = null;
        try {
            int port = 27017;
            MongoCredential credential = MongoCredential.createScramSha1Credential(kvs.getUsername(), "admin",
                MDSUtils.decrypt(kvs.getPassword()).toCharArray());
            MongoClientOptions.Builder builder = MongoClientOptions.builder();
            builder.readPreference(ReadPreference.secondaryPreferred()).socketTimeout(5000).maxWaitTime(5000)
                .connectTimeout(5000).serverSelectionTimeout(5000);
            MongoClientOptions options = builder.build();
            client = new MongoClient(new ServerAddress(kvsNode.getManageIp(), port), credential, options);
            MongoDatabase database = client.getDatabase("admin");
            Document doc = new Document();
            doc.append("replSetGetStatus", 1);
            doc.append("initialSync", 1);
            JSONObject object = JSONObject.parseObject(JSONObject.toJSONString(database.runCommand(doc)));
            return new BocloudResult(true,
                null != object ? object.getJSONArray("members").toJavaList(JSONObject.class) : Lists.newArrayList(),
                "数据库操作成功");
        } catch (Exception e) {
            logger.info("connect mongodb error:", e);
            return new BocloudResult(false, Lists.newArrayList(), "数据库操作失败");
        } finally {
            if (null != client) {
                client.close();
            }
        }
    }
}
