package com.bocloud.pms.service.utils;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.concurrent.Callable;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.pms.entity.Rds;
import com.bocloud.pms.entity.RdsNode;
import com.google.common.collect.Maps;

/**
 * Created by liuyuanyuan on 2020/4/29 19:11
 */
public class MysqlconnectionRunable implements Callable {
    private Logger logger = LoggerFactory.getLogger(MysqlconnectionRunable.class);

    private String sql;
    private RdsNode node;
    private Rds rds;

    @Override
    public boolean equals(Object obj) {
        return super.equals(obj);
    }

    @Override
    public int hashCode() {
        return super.hashCode();
    }

    public MysqlconnectionRunable(String sql, RdsNode node, Rds rds) {
        this.sql = sql;
        this.node = node;
        this.rds = rds;
    }

    @Override
    public BocloudResult call() throws Exception {
        Class.forName("com.mysql.cj.jdbc.Driver");
        Map<String, Object> result = Maps.newHashMap();
        result.put("node", node);
        result.put("params", new ArrayList<JSONObject>());
        try (Connection conn = DriverManager.getConnection(
            "jdbc:mysql://" + node.getManageIp()
                + ":3306/mysql?characterEncoding=utf-8&serverTimezone=UTC&useSSL=false",
            rds.getUsername(), MDSUtils.decrypt(rds.getPassword()))) {
            // 设置自动提交关闭
            conn.setAutoCommit(false);
            try (PreparedStatement pst = conn.prepareStatement(sql); ResultSet rs = pst.executeQuery()) {
                List<JSONObject> params = new ArrayList<>();
                if (StringUtils.isNotEmpty(rds.getProps()) && rds.getProps().toLowerCase().contains("mgr")) {
                    while (rs.next()) {
                        JSONObject param = new JSONObject();

                        param.put("host", rs.getString("MEMBER_HOST"));
                        // 加入版本获取mgr集群角色逻辑
                        if ("5.7".equals(rds.getVersion())) {
                            try (
                                PreparedStatement statement =
                                    conn.prepareStatement("show VARIABLES like\"super_read_only\";");
                                ResultSet resultSet = statement.executeQuery()) {
                                while (resultSet.next()) {
                                    final String superReadOnly = resultSet.getString("Value");
                                    if ("OFF".equalsIgnoreCase(superReadOnly)) {
                                        param.put("role", "PRIMARY");
                                    } else if ("ON".equalsIgnoreCase(superReadOnly)) {
                                        param.put("role", "SECONDARY");
                                    } else {
                                        logger.info("节点：" + node.getManageIp() + ",获取节点角色异常");
                                        return new BocloudResult(false, result, "获取节点角色异常");
                                    }
                                }
                            } catch (SQLException e) {
                                logger.info("节点：" + node.getManageIp() + ",数据库服务异常");
                                return new BocloudResult(false, result, e.getMessage());
                            }
                        } else {
                            param.put("role", rs.getString("MEMBER_ROLE"));
                        }
                        params.add(param);
                    }
                } else {
                    while (rs.next()) {
                        JSONObject param = new JSONObject();
                        param.put("host", rs.getString("HOST"));
                        params.add(param);
                    }
                }
                conn.commit();
                result.put("params", params);
                return new BocloudResult(true, result, "数据库操作成功");
            } catch (SQLException e) {
                logger.info("节点：" + node.getManageIp() + ",数据库服务异常");
                return new BocloudResult(false, result, e.getMessage());
            }
        } catch (Exception e) {
            logger.info("节点：" + node.getManageIp() + ",数据库服务异常");
            return new BocloudResult(false, result, e.getMessage());
        }
    }
}
