package com.bocloud.pms.service.utils;

import java.util.ArrayList;
import java.util.Base64;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.lang3.StringUtils;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.encrypt.AESEncrptor;
import com.bocloud.common.enums.PostDataFormat;
import com.bocloud.common.http.HttpClient;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Result;
import com.bocloud.common.utils.ListTool;
import com.bocloud.pms.entity.CmsBinding;
import com.bocloud.pms.entity.CmsConnection;
import com.bocloud.pms.entity.CmsExchange;
import com.bocloud.pms.entity.CmsQueue;
import com.bocloud.pms.entity.CmsUser;
import com.bocloud.pms.entity.CmsVhost;

public class RabbitMQHelper {

    public static final String PORT = "15672";
    public static final String API = "/api/";
    public static final String EXCHANGE_API = "/api/exchanges/";
    public static final String QUEUE_API = "/api/queues/";
    public static final String BINDING_API = "/api/bindings/";
    public static final String CONNECTION_API = "/api/connections/";
    public static final String CHANNEL_API = "/api/channels/";
    public static final String WHOAMI_API = "/api/whoami";

    public static Result post(String url, JSONObject body, String username, String password) throws Exception {
        HttpClient httpClient = new HttpClient(30000);
        Map<String, Object> header = RabbitMQHelper.buildHeader(username, password);
        httpClient.setDataFormat(PostDataFormat.RAW);
        Result result = httpClient.post(header, body, url);
        if (result.isFailed()) {
            return new Result(false, "请求{POST}失败:" + result.getMessage());
        }
        return result;
    }

    public static Result put(String url, JSONObject body, String username, String password) throws Exception {
        HttpClient httpClient = new HttpClient(30000);
        Map<String, Object> header = RabbitMQHelper.buildHeader(username, password);
        httpClient.setDataFormat(PostDataFormat.RAW);
        Result result = httpClient.put(header, body, url);
        if (result.isFailed()) {
            return new Result(false, "请求{PUT}失败:" + result.getMessage());
        }
        return result;
    }

    public static Result delete(String url, JSONObject body, String username, String password) throws Exception {
        HttpClient httpClient = new HttpClient(30000);
        Map<String, Object> header = RabbitMQHelper.buildHeader(username, password);
        httpClient.setDataFormat(PostDataFormat.RAW);
        Result result = httpClient.delete(header, body, url);
        if (result.isFailed()) {
            return new Result(false, "请求{DELETE}失败:" + result.getMessage());
        }
        return result;
    }

    public static Result get(String url, String username, String password) {
        HttpClient httpClient = new HttpClient(5000);
        Map<String, Object> header = RabbitMQHelper.buildHeader(username, password);
        Result result = httpClient.get(header, null, url);
        if (result.isFailed()) {
            return new BocloudResult(false, "请求{GET}失败:" + result.getMessage());
        }
        return result;
    }

    /**
     * 构建header
     *
     * @return
     */
    protected static Map<String, Object> buildHeader(String username, String password) {
        Map<String, Object> header = new HashMap<>();
        header.put("accept", "application/json");
        header.put("content-type", "application/json");
        header.put("Authorization",
            "Basic " + Base64.getUrlEncoder().encodeToString((username + ":" + password).getBytes()));
        return header;
    }

    public static void main(String[] args) {
        List<String> values = new ArrayList<String>();
        values.add("aa");
        values.add("bb");
        values.add("cc");
        values.add("dd");
        List<String> subList = values.subList(0, values.size() + 1);
        System.out.println(subList);
    }

    /**
     * 数据分页工具
     *
     * @return
     */
    public static List<?> pageUtil(List<?> objects, Integer pageNum, Integer pageSize) {
        if (ListTool.isEmpty(objects)) {
            return objects;
        }
        Integer count = objects.size(); // 记录总数
        int fromIndex = (pageNum - 1) * pageSize; // 开始索引
        int toIndex = fromIndex + pageSize; // 结束索引
        if (toIndex > count) {
            toIndex = count;
        }
        return objects.subList(fromIndex, toIndex);
    }

    /**
     * 构建url
     *
     * @param manageIp
     * @param vhost
     * @param name
     * @param action
     * @return
     */
    public static String buildExchangeUrl(String manageIp, String vhost, String name, String action) {
        if ("virgule".equals(vhost)) {
            vhost = "%2f";
        }
        switch (action) {
            case "create":
            case "delete":
            case "detail":
                return "http://" + manageIp + ":" + PORT + EXCHANGE_API + vhost + "/" + name;
            case "list":
                return "http://" + manageIp + ":" + PORT + EXCHANGE_API;
            default:
                return null;
        }
    }

    /**
     * 构建url
     *
     * @param manageIp
     * @param name
     * @param action
     * @return
     */
    public static String buildConnectionUrl(String manageIp, String name, String action) {
        switch (action) {
            case "close":
            case "detail":
                return "http://" + manageIp + ":" + PORT + CONNECTION_API + name;
            case "getChannels":
                return "http://" + manageIp + ":" + PORT + CONNECTION_API + name + "/channels";
            case "list":
                return "http://" + manageIp + ":" + PORT + CONNECTION_API;
            default:
                return null;
        }
    }

    /**
     * 构建url
     *
     * @param manageIp
     * @param name
     * @param action
     * @return
     */
    public static String buildChannelnUrl(String manageIp, String name, String action) {
        switch (action) {
            case "detail":
                return "http://" + manageIp + ":" + PORT + CHANNEL_API + name;
            case "list":
                return "http://" + manageIp + ":" + PORT + CHANNEL_API;
            default:
                return null;
        }
    }

    /**
     * 构建url
     *
     * @param manageIp
     * @param vhost
     * @param name
     * @return
     */
    public static String buildQueueUrl(String manageIp, String vhost, String name, String action) {
        if ("virgule".equals(vhost) || "/".equals(vhost)) {
            vhost = "%2f";
        }
        switch (action) {
            case "create":
            case "delete":
            case "detail":
                return "http://" + manageIp + ":" + PORT + QUEUE_API + vhost + "/" + name;
            case "list":
                return "http://" + manageIp + ":" + PORT + QUEUE_API;
            default:
                return null;
        }
    }

    /**
     * 构建url
     *
     * @param manageIp
     * @param name
     * @return
     */
    public static String buildUrl(String manageIp, String name, String type, Boolean isQuery) {

        switch (type) {
            case "vhost":
                if ("/".equals(name)) {
                    name = "%2f";
                }
                if (isQuery) {
                    return "http://" + manageIp + ":" + PORT + API + "vhosts";
                }
                return "http://" + manageIp + ":" + PORT + API + "vhosts/" + name;
            case "user":
                if (isQuery) {
                    return "http://" + manageIp + ":" + PORT + API + "users";
                }
                return "http://" + manageIp + ":" + PORT + API + "users/" + name;
            default:
                return null;
        }
    }

    /**
     * 构建权限url
     *
     * @param manageIp
     * @param userName
     * @param vhostName
     * @param type
     * @param isQuery
     * @return
     */
    public static String buildPermissionsUrl(String manageIp, String userName, String vhostName, String type,
        Boolean isQuery) {
        if ("virgule".equals(vhostName) || "/".equals(vhostName)) {
            vhostName = "%2f";
        }
        if (isQuery) {
            switch (type) {
                case "vhost":
                    return "http://" + manageIp + ":" + PORT + API + "vhosts/" + vhostName + "/permissions";
                case "user":
                    return "http://" + manageIp + ":" + PORT + API + "users/" + userName + "/permissions";
                default:
                    return null;
            }
        } else {
            return "http://" + manageIp + ":" + PORT + API + "permissions/" + vhostName + "/" + userName;
        }
    }

    public static String buildPermissionsUrl(String manageIp) {
        return "http://" + manageIp + ":" + PORT + API + "permissions";
    }

    /**
     * 重置参数
     *
     * @return
     */
    public static JSONObject buildExchangeModel(CmsExchange exchange) throws Exception {
        CmsExchange object = new CmsExchange();
        object.setType(exchange.getType());
        object.setAutoDelete(exchange.getAutoDelete());
        object.setDurable(exchange.getDurable());
        object.setInternal(exchange.getInternal());
        object.setArguments(exchange.getArguments());
        return JSONObject.parseObject(JSONObject.toJSONString(object));
    }

    /**
     * 重置参数
     *
     * @return
     */
    public static JSONObject buildQueueModel(CmsQueue queue) throws Exception {
        CmsQueue object = new CmsQueue();
        object.setAutoDelete(queue.getAutoDelete());
        object.setDurable(queue.getDurable());
        object.setArguments(queue.getArguments());
        object.setNode(queue.getNode());
        return JSONObject.parseObject(JSONObject.toJSONString(object));
    }

    /**
     * 构建url
     *
     * @param manageIp
     * @param mqBinding
     * @param action
     * @return
     */
    public static String buildBindingUrl(String manageIp, CmsBinding mqBinding, String action) {
        String vhost = mqBinding.getVhost();
        if ("/".equals(vhost)) {
            vhost = "%2f";
        }
        String variate = null;
        if ("exchange".equals(mqBinding.getDestinationType())) {
            variate = "/e/";
        }
        if ("queue".equals(mqBinding.getDestinationType())) {
            variate = "/q/";
        }
        switch (action) {
            case "detail":
            case "delete":
                return "http://" + manageIp + ":" + PORT + BINDING_API + vhost + "/e/" + mqBinding.getSource() + variate
                    + mqBinding.getDestination() + "/" + mqBinding.getPropertiesKey();
            case "list":
            case "create":
                return "http://" + manageIp + ":" + PORT + BINDING_API + vhost + "/e/" + mqBinding.getSource() + variate
                    + mqBinding.getDestination();
            default:
                return null;
        }
    }

    /**
     * 重置参数
     *
     * @return
     */
    public static JSONObject buildBindingsModel(CmsBinding binding) throws Exception {
        CmsBinding object = new CmsBinding();
        object.setRoutingKey(binding.getRoutingKey());
        object.setArguments(binding.getArguments());
        return JSONObject.parseObject(JSONObject.toJSONString(object));
    }

    /**
     * 重置参数
     *
     * @return
     */
    public static JSONObject buildUserModel(CmsUser mqUser) throws Exception {
        CmsUser object = new CmsUser();
        if (!StringUtils.isEmpty(mqUser.getPassword())) {
            object.setPassword(new AESEncrptor().decrypt(mqUser.getPassword(), null));
        } else {
            object.setPasswordHash(null);
        }
        object.setTags(mqUser.getTags());
        return JSONObject.parseObject(JSONObject.toJSONString(object));
    }

    /**
     * 重置参数
     *
     * @return
     */
    public static JSONObject buildVhostModel() {
        return JSONObject.parseObject(JSONObject.toJSONString(new CmsVhost()));
    }

    /**
     * 重置参数
     *
     * @return
     */
    public static JSONObject buildConnectionModel(String xReason) {
        CmsConnection object = new CmsConnection();
        object.setXReason(xReason);
        return JSONObject.parseObject(JSONObject.toJSONString(object));
    }

}
